@@ -1,4 +1,4 @@
-/*  $Id: blast_input.hpp 615335 2020-08-31 15:36:38Z fukanchi $
+/*  $Id: blast_input.hpp 631484 2021-05-19 13:44:33Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -357,8 +357,8 @@ public:
     /// Determine if we have reached the end of the BLAST input
     bool End() { return m_Source->End(); }
 
-    int GetNumSeqsProcessed() const { return m_NumSeqs; }
-    int GetTotalLengthProcessed() const { return m_TotalLength; }
+    Int8 GetNumSeqsProcessed() const { return m_NumSeqs; }
+    Int8 GetTotalLengthProcessed() const { return m_TotalLength; }
 private:
     CRef<CBlastInputSource> m_Source;  ///< pointer to source of sequences
     TSeqPos m_BatchSize;          ///< total size of one block of sequences
@@ -373,10 +373,10 @@ private:
     void do_copy(const CBlastInput& input);
 
     // # of seqs processed
-    int m_NumSeqs;
+    Int8 m_NumSeqs;
 
     // Total length processed
-    int m_TotalLength;
+    Int8 m_TotalLength;
 };
 
 /// Auxiliary class for creating Bioseqs given SeqIds
@@ -455,6 +455,9 @@ public:
 
     bool End(void) {return m_Source->End();}
 
+    Int8 GetNumSeqsProcessed() const { return m_NumSeqs; }
+    Int8 GetTotalLengthProcessed() const { return m_TotalLength; }
+
 private:
     CBlastInputOMF(const CBlastInputOMF& rhs);
     CBlastInputOMF& operator=(const CBlastInputOMF& rhs);
@@ -463,6 +466,12 @@ private:
     TSeqPos m_BatchSize;
     TSeqPos m_MaxNumSequences;
     CRef<CBioseq_set> m_BioseqSet;
+
+    // # of seqs processed
+    Int8 m_NumSeqs;
+
+    // Total length processed
+    Int8 m_TotalLength;
 };
 
 END_SCOPE(blast)
