@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.hpp 605536 2020-04-13 11:07:50Z ivanov $
+/*  $Id: cmdline_flags.hpp 609644 2020-06-04 18:09:24Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -469,6 +469,8 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgPrintMdTag;
 NCBI_BLASTINPUT_EXPORT extern const string kArgUnalignedOutput;
 /// Argument to specify format for reporting unaligned reads
 NCBI_BLASTINPUT_EXPORT extern const string kArgUnalignedFormat;
+/// Argument to specify mt mode (split by db or split by queries)
+NCBI_BLASTINPUT_EXPORT extern const string kArgMTMode;
 
 END_SCOPE(blast)
 END_NCBI_SCOPE
