@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBI_EXPORT__H
 #define COMMON___NCBI_EXPORT__H
 
-/*  $Id: ncbi_export.h 617033 2020-09-24 18:56:05Z ivanov $
+/*  $Id: ncbi_export.h 616536 2020-09-17 15:49:39Z foleyjp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1522,6 +1522,14 @@
 #  define NCBI_XLOADER_CDD_EXPORT NCBI_DLL_IMPORT
 #endif
 
+/* Export specifier for library xxconnect2
+ */
+#ifdef NCBI_XXCONNECT2_EXPORTS
+#  define NCBI_XXCONNECT2_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_XXCONNECT2_EXPORT NCBI_DLL_IMPORT
+#endif
+
 /* STATIC LIBRARIES SECTION */
 /* This section is for static-only libraries */
 
