@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBICONF_IMPL__H
 #define COMMON___NCBICONF_IMPL__H
 
-/* $Id: ncbiconf_impl.h 609547 2020-06-03 17:21:47Z ivanov $
+/* $Id: ncbiconf_impl.h 620460 2020-11-23 14:26:46Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -188,7 +188,7 @@
 #endif
     
 #if NCBI_HAS_CPP_ATTRIBUTE(fallthrough)  &&  \
-    (!defined(__clang__)  ||  __clang_major__ > 7  ||  __cplusplus >= 201703L)
+    (!defined(__clang__)  ||  (__clang_major__ > 7 && __cplusplus >= 201703L))
 #  define NCBI_FALLTHROUGH [[fallthrough]]
 #elif NCBI_HAS_CPP_ATTRIBUTE(gcc::fallthrough)
 #  define NCBI_FALLTHROUGH [[gcc::fallthrough]]
