@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/* $Id: ncbi_connutil.h 599955 2020-01-09 20:33:13Z lavr $
+/* $Id: ncbi_connutil.h 629074 2021-04-09 14:31:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -69,7 +69,7 @@
  *       BUF_StripToPattern()
  *
  *     4.Perform URL encoding/decoding of data:
- *       URL_Encode()
+ *       URL_Encode[Ex]()
  *       URL_Decode[Ex]()
  *
  *     5.Compose or parse NCBI-specific Content-Type's:
@@ -175,6 +175,9 @@ typedef unsigned EBDebugPrintout;
  *             For encoding / decoding, one can use command-line OpenSSL:
  *             echo [-n] "password|base64value" | openssl enc {-e|-d} -base64
  *             or an online tool (search the Web for "base64 online").
+ * NOTE3:      The NCBI_CRED credentials are stored by value and are not
+ *             otherwise managed (reallocated / duplicated) by the API: they
+ *             are passed "as-is" to the lower levels of the SSL.
  */
 typedef struct {  /* NCBI_FAKE_WARNING: ICC */
     char            client_host[CONN_HOST_LEN+1]; /*client hostname('\0'=def)*/
@@ -314,9 +317,9 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
  *    then "as-is");
  * 2. Registry key "param" in the section "[service]";
  * 3. Environment setting "param" (in all upper-case);
- * 4. Registry key "param" (with leading "CONN_" stripped) in the default
+ * 4. Registry key "param" (with the leading "CONN_" stripped) in the default
  *    section "[CONN]".
- * Steps 1 & 2 skipped for "service" passed as NULL or empty ("").
+ * Steps 1 & 2 skipped for "service" passed as either NULL or empty ("").
  * Steps 3 & 4 skipped for a non-empty "service" and a "param" that already
  *             begins with "CONN_".
  * If the found match's value has enveloping quotes (either single '' or
@@ -380,7 +383,7 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_Boolean
  * of the form <service>_CONN_<NAME>; then in the current corelib registry, in
  * the section 'service' by using the CONN_<NAME> key; then in the environment
  * variable again, but using the name CONN_<NAME>; and finally in the default
- * registry section (DEF_CONN_REG_SECTION), using just <NAME>. If service
+ * registry section (DEF_CONN_REG_SECTION), using just <NAME>.  If the service
  * is NULL or empty then the first 2 steps in the above lookup are skipped.
  *
  * For default values see right above, within macros DEF_CONN_<NAME>.
@@ -493,6 +496,12 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PostOverrideArg
 /* Set up standard arguments:  service(as passed), address, and platform.
  * Also, set up the User-Agent: HTTP header (using CORE_GetAppName()) in
  * SConnNetInfo::http_user_header.  Return non-zero on success; zero on error.
+ * @note
+ *  "service" passed as NULL does not add any service argument, yet "service"
+ *  passed as empty string ("") removes the service argument.
+ * @note
+ *  The call can sacrifice the platform and address (in this order) in order to
+ *  fit the service argument (when provided non-empty).
  * @sa
  *  CORE_GetAppName, CORE_GetPlatform
  */
@@ -757,15 +766,31 @@ extern NCBI_XCONNECT_EXPORT NCBI_CONNUTIL_DEPRECATED SOCK URL_Connect
  );
 
 
-/* Discard all input data before (and including) the first occurrence of a
+/** Discard all input data before (and including) the first occurrence of a
  * "pattern".  If "discard" is not NULL then add the stripped data (including
  * the "pattern") to it.  If "n_discarded" is not NULL then "*n_discarded"
- * will return the number of actually stripped bytes.  If there was some excess
+ * will get the number of actually stripped bytes.  If there was some excess
  * read, push it back to the original source (and not count as discarded).
- * NOTE: If "pattern_size" == 0, then "pattern" is ignored (assumed to be NULL)
- * and the stripping continues until EOF;  if "pattern" is NULL and
+ * NOTE:  If "pattern_size" == 0, then "pattern" is ignored (and is assumed to
+ * be NULL), and the stripping continues until EOF;  if "pattern" is NULL and
  * "pattern_size" is not 0, then exactly "pattern_size" bytes will have
  * attempted to be stripped (unless an I/O error occurs prematurely).
+ *
+ * @return eIO_Success when the requested operation has completed successfully
+ * (pattern found, or the requested number of bytes was skipped, including when
+ * either was followed by the end-of-file condition), and if the "discard"
+ * buffer was provided, then everything skipped has been successfully stored in
+ * it.  Otherwise, return other error code, and store everything read / skipped
+ * thus far in the "discard" buffer, if provided.  Naturally, it never returns
+ * eIO_Success when requested to skip through the end of file, but eIO_Closed
+ * when the EOF has been reached.  Note that memory allocation errors (such as
+ * unable to save skipped data in the "discard" buffer) will be assigned the
+ * eIO_Unknown code -- which as well can be returned from the failing I/O).
+ *
+ * @note To distiguish the nature of eIO_Unknown (whether it is I/O or memory)
+ * one can check the grow of the buffer size after the call, and compare it to
+ * the value of "*n_discarded": if they agree, there was an I/O error, not the
+ * buffer's.
  */
 extern NCBI_XCONNECT_EXPORT EIO_Status CONN_StripToPattern
 (CONN        conn,
@@ -801,12 +826,29 @@ extern NCBI_XCONNECT_EXPORT EIO_Status BUF_StripToPattern
  * Assign "*dst_written" to the # of bytes written to buffer "dst_buf".
  */
 extern NCBI_XCONNECT_EXPORT void URL_Encode
-(const void* src_buf,    /* [in]     non-NULL */
- size_t      src_size,   /* [in]              */
- size_t*     src_read,   /* [out]    non-NULL */
- void*       dst_buf,    /* [in/out] non-NULL */
- size_t      dst_size,   /* [in]              */
- size_t*     dst_written /* [out]    non-NULL */
+(const void* src_buf,      /* [in]     non-NULL  */
+ size_t      src_size,     /* [in]               */
+ size_t*     src_read,     /* [out]    non-NULL  */
+ void*       dst_buf,      /* [in/out] non-NULL  */
+ size_t      dst_size,     /* [in]               */
+ size_t*     dst_written   /* [out]    non-NULL  */
+ );
+
+
+/* Act just like URL_Encode (see above) but caller can allow the specified
+ * non-standard URL symbols in the input buffer to be encoded "as is".
+ * The extra allowed symbols are passed in a '\0'-terminated string
+ * "allow_symbols" (it can be NULL or empty -- then this will be an exact
+ * equivalent of URL_Encode).
+ */
+extern NCBI_XCONNECT_EXPORT void URL_EncodeEx
+(const void* src_buf,      /* [in]     non-NULL  */
+ size_t      src_size,     /* [in]               */
+ size_t*     src_read,     /* [out]    non-NULL  */
+ void*       dst_buf,      /* [in/out] non-NULL  */
+ size_t      dst_size,     /* [in]               */
+ size_t*     dst_written,  /* [out]    non-NULL  */
+ const char* allow_symbols /* [in]     '\0'-term */
  );
 
 
@@ -822,12 +864,12 @@ extern NCBI_XCONNECT_EXPORT void URL_Encode
  *        "read" it.
  */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ URL_Decode
-(const void* src_buf,    /* [in]     non-NULL */
- size_t      src_size,   /* [in]              */
- size_t*     src_read,   /* [out]    non-NULL */
- void*       dst_buf,    /* [in/out] non-NULL */
- size_t      dst_size,   /* [in]              */
- size_t*     dst_written /* [out]    non-NULL */
+(const void* src_buf,      /* [in]     non-NULL  */
+ size_t      src_size,     /* [in]               */
+ size_t*     src_read,     /* [out]    non-NULL  */
+ void*       dst_buf,      /* [in/out] non-NULL  */
+ size_t      dst_size,     /* [in]               */
+ size_t*     dst_written   /* [out]    non-NULL  */
  );
 
 
