@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_HTTP_SESSION__HPP
 #define CONNECT___NCBI_HTTP_SESSION__HPP
 
-/* $Id: ncbi_http_session.hpp 599959 2020-01-09 20:37:16Z lavr $
+/* $Id: ncbi_http_session.hpp 612999 2020-07-30 19:22:55Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -121,7 +121,7 @@ CHttpResponse g_HttpPut(const CUrl&         url,
                         const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
                         THttpRetries        retries = null);
 
-class CHttpSession;
+class CHttpSession_Base;
 
 
 class NCBI_XCONNECT_EXPORT CHttpHeaders : public CObject
@@ -215,6 +215,9 @@ public:
     virtual ~CHttpHeaders(void) {}
 
 private:
+    friend class CHttpResponse;
+    friend class CHttpRequest;
+
     // Prohibit copying headers.
     CHttpHeaders(const CHttpHeaders&);
     CHttpHeaders& operator=(const CHttpHeaders&);
@@ -237,6 +240,8 @@ public:
     /// Each header line is terminated by a single HTTP_EOL.
     string GetHttpHeader(void) const;
 
+    const THeaders& Get() const { return m_Headers; }
+
     /// Get string representation of the given name.
     static const char* GetHeaderName(EHeaderName name);
 };
@@ -415,32 +420,15 @@ public:
 private:
     friend class CHttpRequest;
 
-    // CObject wrapper for CConn_HttpStream.
-    // Should not be used directly.
-    class CHttpStreamRef : public CObject
-    {
-    public:
-        CHttpStreamRef(void) {}
-        virtual ~CHttpStreamRef(void) {}
-        bool IsInitialized(void) const { return m_ConnStream.get() ? true : false; }
-        CConn_IOStream& GetConnStream(void) const { return *m_ConnStream; }
-        void SetConnStream(CConn_IOStream* stream) { m_ConnStream.reset(stream); }
-    private:
-        CHttpStreamRef(const CHttpStreamRef&);
-        CHttpStreamRef& operator=(const CHttpStreamRef&);
-        auto_ptr<CConn_IOStream> m_ConnStream;
-    };
+    CHttpResponse(CHttpSession_Base& session, const CUrl& url, shared_ptr<iostream> stream = {});
 
-    CHttpResponse(CHttpSession& session, const CUrl& url, CHttpStreamRef& stream);
+    // Update response headers and location, parse cookies and store them in the session.
+    void x_Update(CHttpHeaders::THeaders headers, int status_code, string status_text);
 
-    // Parse response headers, update location, parse cookies and
-    // store them in the session.
-    void x_ParseHeader(const char* header);
-
-    CRef<CHttpSession>      m_Session;
+    CRef<CHttpSession_Base> m_Session;
     CUrl                    m_Url;      // Original URL
     CUrl                    m_Location; // Redirection or the original URL.
-    CRef<CHttpStreamRef>    m_Stream;
+    shared_ptr<iostream>    m_Stream;
     CRef<CHttpHeaders>      m_Headers;
     int                     m_StatusCode;
     string                  m_StatusText;
@@ -519,9 +507,11 @@ public:
     }
 
 private:
-    friend class CHttpSession;
+    friend class CHttpSession_Base;
+    friend class CHttpSessionImpl1x;
+    friend class CHttpSessionImpl2;
 
-    CHttpRequest(CHttpSession& session, const CUrl& url, EReqMethod method);
+    CHttpRequest(CHttpSession_Base& session, const CUrl& url, EReqMethod method);
 
     class CAdjustUrlCallback_Base : public CObject {
     public:
@@ -540,6 +530,7 @@ private:
 
     // Open connection, initialize response.
     void x_InitConnection(bool use_form_data);
+    void x_InitConnection2(shared_ptr<iostream> stream, bool is_service);
 
     bool x_CanSendData(void) const;
 
@@ -547,6 +538,9 @@ private:
     // new values.
     void x_AddCookieHeader(const CUrl& url, bool initial);
 
+    void x_AdjustHeaders(bool use_form_data);
+    void x_UpdateResponse(CHttpHeaders::THeaders headers, int status_code, string status_text);
+
     // CConn_HttpStream callback for parsing headers.
     // 'user_data' must point to a CHttpRequest object.
     static EHTTP_HeaderParse sx_ParseHeader(const char*, void*, int);
@@ -556,15 +550,13 @@ private:
                          void*         user_data,
                          unsigned int  failure_count);
 
-    typedef CHttpResponse::CHttpStreamRef TStreamRef;
-
-    CRef<CHttpSession>  m_Session;
+    CRef<CHttpSession_Base> m_Session;
     CUrl                m_Url;
     bool                m_IsService;
     EReqMethod          m_Method;
     CRef<CHttpHeaders>  m_Headers;
     CRef<CHttpFormData> m_FormData;
-    CRef<TStreamRef>    m_Stream;
+    shared_ptr<iostream> m_Stream;
     CRef<CHttpResponse> m_Response; // current response or null
     CTimeout            m_Timeout;
     THttpRetries        m_Retries;
@@ -575,7 +567,7 @@ private:
 
 
 /// HTTP session class, holding common data for multiple requests.
-class NCBI_XCONNECT_EXPORT CHttpSession : public CObject,
+class NCBI_XCONNECT_EXPORT CHttpSession_Base : public CObject,
                                           virtual protected CConnIniter
 {
 public:
@@ -644,7 +636,8 @@ public:
     /// HTTP protocol version.
     enum EProtocol {
         eHTTP_10, ///< HTTP/1.0
-        eHTTP_11  ///< HTTP/1.1
+        eHTTP_11, ///< HTTP/1.1
+        eHTTP_2,  ///< HTTP/2
     };
 
     /// Get protocol version.
@@ -659,13 +652,16 @@ public:
     /// @sa GetHttpFlags
     void SetHttpFlags(THTTP_Flags flags) { m_HttpFlags = flags; }
 
-    CHttpSession(void);
-    virtual ~CHttpSession(void) {}
+    CHttpSession_Base(void);
+    virtual ~CHttpSession_Base(void) {}
 
 private:
     friend class CHttpRequest;
     friend class CHttpResponse;
 
+    virtual void x_StartRequest(EProtocol protocol, CHttpRequest& req, bool use_form_data) = 0;
+    virtual bool x_Downgrade(CHttpResponse& resp, EProtocol& protocol) const = 0;
+
     // Save cookies returned by a response.
     void x_SetCookies(const CHttpHeaders::THeaderValues& cookies,
                       const CUrl*                        url);
@@ -678,6 +674,42 @@ private:
 };
 
 
+template <class TImpl>
+class CHttpSessionTmpl : public CHttpSession_Base
+{
+    void x_StartRequest(EProtocol protocol, CHttpRequest& req, bool use_form_data) override
+    {
+        TImpl::StartRequest(protocol, req, use_form_data);
+    }
+
+    bool x_Downgrade(CHttpResponse& resp, EProtocol& protocol) const override
+    {
+        return TImpl::Downgrade(resp, protocol);
+    }
+};
+
+
+class CHttpSessionImpl1x
+{
+    friend class CHttpSessionTmpl<CHttpSessionImpl1x>;
+
+    static void StartRequest(CHttpSession_Base::EProtocol _DEBUG_ARG(protocol), CHttpRequest& req, bool use_form_data)
+    {
+        _ASSERT(protocol <= CHttpSession_Base::eHTTP_11);
+        req.x_InitConnection(use_form_data);
+    }
+
+    static bool Downgrade(CHttpResponse&, CHttpSession_Base::EProtocol&)
+    {
+        return false;
+    }
+};
+
+
+/// @sa CHttpSession_Base
+using CHttpSession = CHttpSessionTmpl<CHttpSessionImpl1x>;
+
+
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CHttpSessionException --
