@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_PARAM_IMPL__HPP
 #define CORELIB___NCBI_PARAM_IMPL__HPP
 
-/*  $Id: ncbi_param_impl.hpp 598520 2019-12-10 17:48:54Z grichenk $
+/*  $Id: ncbi_param_impl.hpp 606787 2020-04-27 16:15:35Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,11 +33,10 @@
  *
  */
 
-
-#include <corelib/ncbistr.hpp>
 #include <corelib/ncbistre.hpp>
 #include <corelib/ncbi_safe_static.hpp>
 
+
 BEGIN_NCBI_SCOPE
 
 
@@ -443,7 +442,7 @@ CParam<TDescription>::sx_GetDefault(bool force_reset)
             state = eState_Config; // No need to check anything else.
         }
         else {
-            int src = eSource_NotSet;
+            EParamSource src = eSource_NotSet;
             string config_value =
                 g_GetConfigString(TDescription::sm_ParamDescription.section,
                                     TDescription::sm_ParamDescription.name,
