@@ -1,7 +1,7 @@
 #ifndef NCBIEXPT__HPP
 #define NCBIEXPT__HPP
 
-/*  $Id: ncbiexpt.hpp 606470 2020-04-22 14:14:58Z ivanov $
+/*  $Id: ncbiexpt.hpp 605495 2020-04-11 04:31:07Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -67,14 +67,15 @@ BEGIN_NCBI_SCOPE
 /// Define use of C++ exception specification mechanism:
 ///   "f(void) throw();"       <==  "f(void) THROWS_NONE;"
 ///   "g(void) throw(e1,e2);"  <==  "f(void) THROWS((e1,e2));"
-#if defined(NCBI_USE_THROW_SPEC)
+#ifdef NCBI_USE_THROW_SPEC
 #  define THROWS_NONE throw()
-#  define THROWS(x) throw x
+#  define THROWS(x)   throw x
 #else
 #  define THROWS_NONE
 #  define THROWS(x)
 #endif
 
+
 /// ABORT_ON_THROW controls if program should be aborted.
 #define ABORT_ON_THROW "ABORT_ON_THROW"
 
@@ -101,78 +102,10 @@ extern void DoDbgPrint(const CDiagCompileInfo& info, const string& message);
 /// Print the specified debug messages.
 NCBI_XNCBI_EXPORT
 extern void DoDbgPrint(const CDiagCompileInfo& info,
-                       const char* msg1, const char* msg2);
+    const char* msg1, const char* msg2);
 
 #if defined(_DEBUG)
 
-/// Templated function for printing debug message.
-///
-/// Print debug message for the specified exception type.
-template<typename T>
-inline
-const T& DbgPrint(const CDiagCompileInfo& info,
-                  const T& e, const char* e_str)
-{
-    DoDbgPrint(info, e_str, e.what());
-    return e;
-}
-
-/// Print debug message for "const char*" object.
-inline
-const char* DbgPrint(const CDiagCompileInfo& info,
-                     const char* e, const char* )
-{
-    DoDbgPrint(info, e);
-    return e;
-}
-
-/// Print debug message for "char*" object.
-inline
-char* DbgPrint(const CDiagCompileInfo& info,
-               char* e, const char* )
-{
-    DoDbgPrint(info, e);
-    return e;
-}
-
-/// Print debug message for "std::string" object.
-inline
-const string& DbgPrint(const CDiagCompileInfo& info,
-                       const string& e, const char* )
-{
-    DoDbgPrint(info, e);
-    return e;
-}
-
-/// Create diagnostic stream for printing specified message and "abort()" the
-/// program if set by SetThrowTraceAbort() or $ABORT_ON_THROW.
-///
-/// @sa
-///   SetThrowTraceAbort(), DoThrowTraceAbort()
-template<typename T>
-inline
-const T& DbgPrintP(const CDiagCompileInfo& info, const T& e,const char* e_str)
-{
-    CNcbiDiag(info, eDiag_Trace) << e_str << ": " << e;
-    DoThrowTraceAbort();
-    return e;
-}
-
-/// Create diagnostic stream for printing specified message.
-///
-/// Similar to DbgPrintP except that "abort()" not executed.
-/// @sa
-///   DbgPrintP()
-template<typename T>
-inline
-const T& DbgPrintNP(const CDiagCompileInfo& info,
-                    const T& e,
-                    const char* e_str)
-{
-    DoDbgPrint(info, e_str);
-    return e;
-}
-
 /// Rethrow trace.
 ///
 /// Reason for do {...} while in macro definition is to permit a natural
@@ -184,25 +117,253 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 ///
 /// Example:
 /// -  RETHROW_TRACE;
-#  define RETHROW_TRACE do { \
-    _TRACE("EXCEPTION: re-throw"); \
-    NCBI_NS_NCBI::DoThrowTraceAbort(); \
-    throw; \
-} while(0)
+#  define RETHROW_TRACE                         \
+    do {                                        \
+        _TRACE("EXCEPTION: re-throw");          \
+        NCBI_NS_NCBI::DoThrowTraceAbort();      \
+        throw;                                  \
+    } while(0)
+
+
+/////////////////////////////////
+// New style throw-trace macros.
+// 
+
+BEGIN_NAMESPACE(ncbi_throw_trace);
+
+// Helper class for detecting exceptions with what() method defined.
+template<class T, class = T>
+struct is_exception : false_type {};
+template<class T>
+struct is_exception<T, typename enable_if<
+    is_member_function_pointer<decltype(&T::what)>::value,
+    T>::type> : true_type {};
+
+// Helper class for detecting classes with output operator defined.
+template<class T>
+struct has_no_output_operator_impl {
+    template<class V> static auto check(const V&) -> decltype(declval<ostream&>() << declval<V>());
+    template<typename> static bool check(...);
+    using type = typename is_same<bool, decltype(check<T>(declval<T>()))>::type;
+};
+template<class T>
+struct has_no_output_operator : has_no_output_operator_impl<T>::type {};
+template<class T, class = T>
+struct has_output_operator : false_type {};
+template<class T>
+struct has_output_operator<T, typename enable_if<
+    !has_no_output_operator_impl<T>::type::value,
+    T>::type> : true_type {};
+
+// Helper class for detecting printable non-exception classes.
+template<class T, class = T>
+struct is_printable : false_type {};
+template<class T>
+struct is_printable<T, typename enable_if<
+    !is_exception<T>::value &&
+    has_output_operator<T>::value,
+    T>::type> : true_type {};
+
+// Helper class for detecting non-printable classes (no what() or output operator).
+template<class T, class = T>
+struct is_non_printable : false_type {};
+template<class T>
+struct is_non_printable<T, typename enable_if<
+    !is_exception<T>::value && !is_printable<T>::value,
+    T>::type> : true_type {};
+
+END_NAMESPACE(ncbi_throw_trace);
+
+
+template<class T>
+typename enable_if<ncbi_throw_trace::is_exception<T>::value, const T&>::type DbgPrintEx(
+    const CDiagCompileInfo& info, const T& t, const char* str)
+{
+    CNcbiDiag(info, eDiag_Trace) << str << ": " << t.what();
+    DoThrowTraceAbort();
+    return t;
+}
+
+template<class T>
+typename enable_if<ncbi_throw_trace::is_printable<T>::value, const T&>::type DbgPrintEx(
+    const CDiagCompileInfo& info, const T& t, const char* str)
+{
+    CNcbiDiag(info, eDiag_Trace) << str << ": " << t;
+    DoThrowTraceAbort();
+    return t;
+}
+
+template<class T>
+typename enable_if<ncbi_throw_trace::is_non_printable<T>::value, const T&>::type DbgPrintEx(
+    const CDiagCompileInfo& info, const T& t, const char* str)
+{
+    CNcbiDiag(info, eDiag_Trace) << str;
+    DoThrowTraceAbort();
+    return t;
+}
+
+// Need overload for non-const char*? See old style DbgPrint() definitions.
+inline
+char* DbgPrintEx(const CDiagCompileInfo& info, char* str, const char*)
+{
+    CNcbiDiag(info, eDiag_Trace) << str;
+    DoThrowTraceAbort();
+    return str;
+}
+
+inline
+const char* DbgPrintEx(const CDiagCompileInfo& info, const char* str, const char*)
+{
+    CNcbiDiag(info, eDiag_Trace) << str;
+    DoThrowTraceAbort();
+    return str;
+}
+
+inline
+const string& DbgPrintEx(const CDiagCompileInfo& info, const string& str, const char*)
+{
+    CNcbiDiag(info, eDiag_Trace) << str;
+    DoThrowTraceAbort();
+    return str;
+}
+
+
+/// Throw trace.
+///
+/// Combines diagnostic message trace and exception throwing. First the
+/// diagnostic message is printed, and then exception is thrown.
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+///
+/// - If the argument has what() method defined (is an exception), the
+///   method is used for output.
+/// - If the argument has output operator defined, it's printed using the
+///   operator.
+/// - If the argument is non-printable, it's name is printed.
+///
+/// Example:
+/// -  THROW_TRACE_SIMPLE("Throw just a string");
+/// -  THROW_TRACE_SIMPLE(runtime_error("message"));
+#  define THROW_TRACE_SIMPLE(exception_object) \
+    throw NCBI_NS_NCBI::DbgPrintEx(DIAG_COMPILE_INFO, \
+                                   exception_object, #exception_object)
+
+/// Throw trace.
+///
+/// Combines diagnostic message trace and exception throwing. First the
+/// diagnostic message is printed, and then exception is thrown.
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+///
+/// Arguments can be any class with the specified initialization arguments.
+///
+/// - If the argument has what() method defined (is an exception), the
+///   method is used for output.
+/// - If the argument has output operator defined, it's printed using the
+///   operator.
+/// - If the argument is non-printable, it's name is printed.
+///
+/// Example:
+/// -  THROW_TRACE_ARGS(runtime_error, "Something is weird...");
+/// -  THROW_TRACE_ARGS(CParseException, ("Some parse error", 123));
+/// @sa
+///   THROW_TRACE_SIMPLE
+#  define THROW_TRACE_ARGS(exception_class, ...) \
+    throw NCBI_NS_NCBI::DbgPrintEx(DIAG_COMPILE_INFO, \
+                                   exception_class(__VA_ARGS__), #exception_class)
+
+
+/////////////////////////////////
+// Old style throw-trace macros.
+// 
+
+/// Templated function for printing debug message.
+///
+/// Print debug message for the specified exception type and "abort()" the
+/// program if set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+template<typename T>
+inline
+const T& DbgPrint(const CDiagCompileInfo& info,
+    const T& e, const char* e_str)
+{
+    DoDbgPrint(info, e_str, e.what());
+    return e;
+}
+
+/// Print debug message for "const char*" object and "abort()" the
+/// program if set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+inline
+const char* DbgPrint(const CDiagCompileInfo& info,
+    const char* e, const char*)
+{
+    DoDbgPrint(info, e);
+    return e;
+}
+
+/// Print debug message for "char*" object and "abort()" the
+/// program if set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+inline
+char* DbgPrint(const CDiagCompileInfo& info,
+    char* e, const char*)
+{
+    DoDbgPrint(info, e);
+    return e;
+}
+
+/// Print debug message for "std::string" object and "abort()" the
+/// program if set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+inline
+const string& DbgPrint(const CDiagCompileInfo& info,
+    const string& e, const char*)
+{
+    DoDbgPrint(info, e);
+    return e;
+}
+
+/// Print the specified printable object and "abort()" the program if set by
+/// SetThrowTraceAbort() or $ABORT_ON_THROW.
+/// The object must have stream output operator defined.
+///
+/// @sa
+///   SetThrowTraceAbort(), DoThrowTraceAbort()
+template<typename T>
+inline
+const T& DbgPrintP(const CDiagCompileInfo& info, const T& e, const char* e_str)
+{
+    CNcbiDiag(info, eDiag_Trace) << e_str << ": " << e;
+    DoThrowTraceAbort();
+    return e;
+}
+
+/// Create diagnostic stream for printing specified message and "abort()" the
+/// program if set by SetThrowTraceAbort() or $ABORT_ON_THROW.
+///
+/// @sa
+///   DbgPrintP()
+template<typename T>
+inline
+const T& DbgPrintNP(const CDiagCompileInfo& info,
+    const T& e,
+    const char* e_str)
+{
+    DoDbgPrint(info, e_str);
+    return e;
+}
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Argument can be a simple string, or an exception object.
+/// Argument can be a simple string, or an exception object printed using
+/// what() method.
+///
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
 /// Example:
 /// -  THROW0_TRACE("Throw just a string");
 /// -  THROW0_TRACE(runtime_error("message"));
 #  define THROW0_TRACE(exception_object) \
     throw NCBI_NS_NCBI::DbgPrint(DIAG_COMPILE_INFO, \
-        exception_object, #exception_object)
+                                 exception_object, #exception_object)
 
 /// Throw trace.
 ///
@@ -221,19 +382,16 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 ///   THROW0np_TRACE
 #  define THROW0p_TRACE(exception_object) \
     throw NCBI_NS_NCBI::DbgPrintP(DIAG_COMPILE_INFO, \
-        exception_object, #exception_object)
+                                  exception_object, #exception_object)
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Argument can be any printable object; that is, any object with a defined
-/// output operator.
+/// Argument can be any object; its name is printed.
 ///
-/// Similar to THROW0p_TRACE except that program is not "aborted" when
-/// exception is thrown, and argument type can be an aggregate type such as
-/// Vector<T> where T is a printable argument.
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
 /// Example:
 /// -  THROW0np_TRACE(vector<char>());
@@ -241,7 +399,7 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 ///   THROW0p_TRACE
 #  define THROW0np_TRACE(exception_object) \
     throw NCBI_NS_NCBI::DbgPrintNP(DIAG_COMPILE_INFO, \
-        exception_object, #exception_object)
+                                   exception_object, #exception_object)
 
 /// Throw trace.
 ///
@@ -249,25 +407,25 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 /// diagnostic message is printed, and then exception is thrown.
 ///
 /// Arguments can be any exception class with the specified initialization
-/// argument. The class argument need not be derived from std::exception as
-/// a new class object is constructed using the specified class name and
-/// initialization argument.
+/// argument. The class argument need not be derived from std::exception
+/// but must have what() method used for output.
+///
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
 /// Example:
 /// -  THROW1_TRACE(runtime_error, "Something is weird...");
 #  define THROW1_TRACE(exception_class, exception_arg) \
     throw NCBI_NS_NCBI::DbgPrint(DIAG_COMPILE_INFO, \
-        exception_class(exception_arg), #exception_class)
+                                 exception_class(exception_arg), #exception_class)
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Arguments can be any exception class with a the specified initialization
-/// argument. The class argument need not be derived from std::exception as
-/// a new class object is constructed using the specified class name and
-/// initialization argument.
+/// Arguments can be any exception class with the specified initialization
+/// argument. The class argument need not be derived from std::exception but
+/// must have stream output operator defined.
 ///
 /// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
@@ -277,37 +435,35 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 ///   THROW1np_TRACE
 #  define THROW1p_TRACE(exception_class, exception_arg) \
     throw NCBI_NS_NCBI::DbgPrintP(DIAG_COMPILE_INFO,    \
-        exception_class(exception_arg), #exception_class)
+                                  exception_class(exception_arg), #exception_class)
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Arguments can be any exception class with a the specified initialization
-/// argument. The class argument need not be derived from std::exception as
-/// a new class object is constructed using the specified class name and
-/// initialization argument.
+/// Arguments can be any exception class with the specified initialization
+/// argument. The class argument need not be derived from std::exception,
+/// class name is printed.
 ///
-/// Similar to THROW1p_TRACE except that program is not "aborted" when
-/// exception is thrown, and argument type can be an aggregate type such as
-/// Vector<T> where T is a printable argument.
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
 /// Example:
 /// -  THROW1np_TRACE(CUserClass, "argument");
 #  define THROW1np_TRACE(exception_class, exception_arg) \
     throw NCBI_NS_NCBI::DbgPrintNP(DIAG_COMPILE_INFO,    \
-        exception_class(exception_arg), #exception_class)
+                                   exception_class(exception_arg), #exception_class)
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Arguments can be any exception class with a the specified initialization
-/// arguments. The class argument need not be derived from std::exception as
-/// a new class object is constructed using the specified class name and
-/// initialization arguments.
+/// Arguments can be any exception class with the specified initialization
+/// argument. The class argument need not be derived from std::exception
+/// but must have what() method used for output.
+/// 
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
 /// Similar to THROW1_TRACE except that the exception class can have multiple
 /// initialization arguments instead of just one.
@@ -320,17 +476,16 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 ///   THROW1_TRACE
 #  define THROW_TRACE(exception_class, exception_args) \
     throw NCBI_NS_NCBI::DbgPrint(DIAG_COMPILE_INFO,    \
-        exception_class exception_args, #exception_class)
+                                 exception_class exception_args, #exception_class)
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Arguments can be any exception class with a the specified initialization
-/// arguments. The class argument need not be derived from std::exception as
-/// a new class object is constructed using the specified class name and
-/// initialization arguments.
+/// Arguments can be any exception class with the specified initialization
+/// argument. The class argument need not be derived from std::exception but
+/// must have stream output operator defined.
 ///
 /// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
@@ -343,53 +498,58 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 ///   THROW1p_TRACE
 #  define THROWp_TRACE(exception_class, exception_args) \
     throw NCBI_NS_NCBI::DbgPrintP(DIAG_COMPILE_INFO,    \
-        exception_class exception_args, #exception_class)
+                                  exception_class exception_args, #exception_class)
 
 /// Throw trace.
 ///
 /// Combines diagnostic message trace and exception throwing. First the
 /// diagnostic message is printed, and then exception is thrown.
 ///
-/// Arguments can be any exception class with a the specified initialization
-/// argument. The class argument need not be derived from std::exception as
-/// a new class object is constructed using the specified class name and
-/// initialization argument.
+/// Arguments can be any exception class with the specified initialization
+/// argument. The class argument need not be derived from std::exception,
+/// class name is printed.
 ///
-/// Argument type can be an aggregate type such as Vector<T> where T is a
-/// printable argument.
+/// Program may abort if so set by SetThrowTraceAbort() or $ABORT_ON_THROW.
 ///
-/// Similar to THROWp_TRACE except that program is not "aborted" when
-/// exception is thrown.
+/// Similar to THROW1np_TRACE except that the exception class can have multiple
+/// initialization arguments instead of just one.
 ///
 /// Example:
 /// -  THROWnp_TRACE(CUserClass, (arg1, arg2));
 #  define THROWnp_TRACE(exception_class, exception_args) \
     throw NCBI_NS_NCBI::DbgPrintNP(DIAG_COMPILE_INFO,    \
-        exception_class exception_args, #exception_class)
+                                   exception_class exception_args, #exception_class)
+
 
 #else  /* _DEBUG */
 
-// No trace/debug versions of these macros.
+// Release-mode versions of the above macros: no trace logging, and no abort().
 
-#  define RETHROW_TRACE \
+#  define RETHROW_TRACE                                     \
     throw
-#  define THROW0_TRACE(exception_object) \
+
+#  define THROW_TRACE_SIMPLE(exception_object)              \
     throw exception_object
-#  define THROW0p_TRACE(exception_object) \
+#  define THROW_TRACE_ARGS(exception_class, ...)            \
+    throw exception_class(__VA_ARGS__)
+
+#  define THROW0_TRACE(exception_object)                    \
     throw exception_object
-#  define THROW0np_TRACE(exception_object) \
+#  define THROW0p_TRACE(exception_object)                   \
     throw exception_object
-#  define THROW1_TRACE(exception_class, exception_arg) \
+#  define THROW0np_TRACE(exception_object)                  \
+    throw exception_object
+#  define THROW1_TRACE(exception_class, exception_arg)      \
     throw exception_class(exception_arg)
-#  define THROW1p_TRACE(exception_class, exception_arg) \
+#  define THROW1p_TRACE(exception_class, exception_arg)     \
     throw exception_class(exception_arg)
-#  define THROW1np_TRACE(exception_class, exception_arg) \
+#  define THROW1np_TRACE(exception_class, exception_arg)    \
     throw exception_class(exception_arg)
-#  define THROW_TRACE(exception_class, exception_args) \
+#  define THROW_TRACE(exception_class, exception_args)      \
     throw exception_class exception_args
-#  define THROWp_TRACE(exception_class, exception_args) \
+#  define THROWp_TRACE(exception_class, exception_args)     \
     throw exception_class exception_args
-#  define THROWnp_TRACE(exception_class, exception_args) \
+#  define THROWnp_TRACE(exception_class, exception_args)    \
     throw exception_class exception_args
 
 #endif  /* else!_DEBUG */
@@ -459,7 +619,6 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 #define NCBI_CATCH_ALL_X(err_subcode, message)                \
     NCBI_CATCH_ALL_XX(NCBI_USE_ERRCODE_X, err_subcode, message)
 
-
 /// Standard handling of "exception"-derived exceptions
 /// with given error code name and given error subcode placed in diagnostics
 ///
@@ -505,7 +664,6 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
     STD_CATCH_ALL_XX(err_name, err_subcode, message)
 
 
-
 /////////////////////////////////////////////////////////////////////////////
 // CException: useful macros
 
@@ -527,12 +685,11 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
     NCBI_EXCEPTION_VAR_EX(name, 0, exception_class, err_code, message)
 
 /// Throw an existing exception object
-#define NCBI_EXCEPTION_THROW(exception_var) \
+#define NCBI_EXCEPTION_THROW(exception_var)                          \
     throw (exception_var)
 
 #define NCBI_EXCEPTION_EMPTY_NAME
 
-// NCBI_THROW(foo).SetModule("aaa");
 /// Generic macro to make an exception, given the exception class,
 /// error code and message string.
 #define NCBI_EXCEPTION(exception_class, err_code, message)           \
@@ -559,12 +716,12 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
     NCBI_THROW(NCBI_NS_NCBI::CException, eUnknown, message)
 
 /// The same as NCBI_THROW but with message processed as output to ostream.
-#define NCBI_THROW_FMT(exception_class, err_code, message)  \
+#define NCBI_THROW_FMT(exception_class, err_code, message)      \
     NCBI_THROW(exception_class, err_code, FORMAT(message))
 
 /// Throw a "user exception" with message processed as output to ostream.
 /// See NCBI_USER_THROW for details.
-#define NCBI_USER_THROW_FMT(message)  \
+#define NCBI_USER_THROW_FMT(message)                            \
     NCBI_THROW_FMT(NCBI_NS_NCBI::CException, eUnknown, message)
 
 /// Generic macro to make an exception, given the exception class,
@@ -595,13 +752,13 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
     throw; }  while (0)
 
 /// Generate a report on the exception.
-#define NCBI_REPORT_EXCEPTION(title, ex) \
-    NCBI_NS_NCBI::CExceptionReporter::ReportDefault \
+#define NCBI_REPORT_EXCEPTION(title, ex)                        \
+    NCBI_NS_NCBI::CExceptionReporter::ReportDefault             \
         (DIAG_COMPILE_INFO, title, ex, NCBI_NS_NCBI::eDPF_Default)
 
 /// Generate a report on the exception with default error code and
 /// given subcode.
-#define NCBI_REPORT_EXCEPTION_X(err_subcode, title, ex)                  \
+#define NCBI_REPORT_EXCEPTION_X(err_subcode, title, ex)                 \
     NCBI_REPORT_EXCEPTION_XX(NCBI_USE_ERRCODE_X, err_subcode, title, ex)
 
 /// Generate a report on the exception with default error code and
@@ -613,7 +770,6 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
      DIAG_COMPILE_INFO, title, ex, NCBI_NS_NCBI::eDPF_Default)
 
 
-
 /////////////////////////////////////////////////////////////////////////////
 // CException
 
@@ -1337,6 +1493,7 @@ public:
     NCBI_EXCEPTION_DEFAULT(CCoreException, CException);
 };
 
+
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CInvalidParamException --
@@ -1364,7 +1521,7 @@ public:
 // to let them satisfy TErrorStr without a wrapper.  However, they
 // don't all agree on what form the wrapper should take. :-/
 NCBI_XNCBI_EXPORT
-extern const char*  Ncbi_strerror(int errnum);
+extern const char* Ncbi_strerror(int errnum);
 
 #ifdef NCBI_COMPILER_GCC
 inline int         NcbiErrnoCode(void)      { return errno; }
