@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 609927 2020-06-08 16:52:43Z grichenk $
+/*  $Id: ncbimisc.hpp 627621 2021-03-16 14:36:05Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -874,28 +874,106 @@ const TSeqPos kInvalidSeqPos = ((TSeqPos) (-1));
 /// Use this typedef rather than its expansion, which may change.
 typedef int TSignedSeqPos;
 
-/// Taxon id type
-typedef int TTaxId;
-# define TAX_ID_TO(T, tax_id) (static_cast<T>(tax_id))
-# define TAX_ID_FROM(T, value) (static_cast<ncbi::TTaxId>(value))
-# define TAX_ID_CONST(id) id
-#define ZERO_TAX_ID TAX_ID_CONST(0)
-#define INVALID_TAX_ID TAX_ID_CONST(-1)
+
+/// Template class for strict ID types.
+template<class TKey, class TStorage>
+class CStrictId
+{
+public:
+    typedef TStorage TId;
+    typedef CStrictId<TKey, TStorage> TThis;
+
+    CStrictId(void) : m_Id(0) {}
+    CStrictId(const TThis& other) : m_Id(other.m_Id) {}
+    TThis& operator=(const TThis& other) { m_Id = other.m_Id; return *this; }
+
+    bool operator==(const TThis& id) const { return m_Id == id.m_Id; }
+    bool operator!=(const TThis& id) const { return m_Id != id.m_Id; }
+    bool operator<(const TThis& id) const { return m_Id < id.m_Id; }
+    bool operator<=(const TThis& id) const { return m_Id <= id.m_Id; }
+    bool operator>(const TThis& id) const { return m_Id > id.m_Id; }
+    bool operator>=(const TThis& id) const { return m_Id >= id.m_Id; }
+
+    TThis& operator++(void) { m_Id++; return *this; }
+    TThis operator++(int) { TThis tmp = *this; m_Id++; return tmp; }
+    TThis& operator--(void) { m_Id--; return *this; }
+    TThis operator--(int) { TThis tmp = *this; m_Id--; return tmp; }
+
+    TThis operator+(const TThis& id) const { return TThis(m_Id + id.m_Id); }
+    TThis operator-(const TThis& id) const { return TThis(m_Id - id.m_Id); }
+    TThis operator-(void) const { return TThis(-m_Id); }
+
+    //template<typename T>
+    //operator typename enable_if<is_same<T, bool>::value, bool>::type(void) const { return m_Id != 0; }
+
+    TId Get(void) const { return m_Id; }
+    void Set(TId id) { m_Id = id; }
+    TId& Set(void) { return m_Id; }
+
+#if !defined(NCBI_TEST_APPLICATION)
+    template<typename T>
+    explicit CStrictId(T id, typename enable_if<is_same<T, TId>::value, T>::type = 0) : m_Id(id) {}
+private:
+#else
+    template<typename T>
+    CStrictId(T id, typename enable_if<is_arithmetic<T>::value, T>::type = 0) : m_Id(id) {}
+    template<typename T>
+    typename enable_if<is_arithmetic<T>::value, TThis>::type& operator=(T id) { m_Id = id; return *this; }
+    template<typename T>
+    typename enable_if<is_arithmetic<T>::value, bool>::type operator==(T id) const { return m_Id == id; }
+    template<typename T>
+    operator typename enable_if<is_arithmetic<T>::value, T>::type(void) const { return static_cast<T>(m_Id); }
+#endif
+
+private:
+    TId m_Id;
+};
+
+template<class TKey, class TStorage>
+inline
+CNcbiOstream& operator<<(CNcbiOstream& out, const CStrictId<TKey, TStorage>& id)
+{
+    return out << id.Get();
+}
+
+template<class TKey, class TStorage>
+inline
+CNcbiIstream& operator>>(CNcbiIstream& in, CStrictId<TKey, TStorage>& id)
+{
+    in >> id.Set();
+    return in;
+}
+
 
 /// Type for sequence GI.
 ///
 /// Use this typedef rather than its expansion, which may change.
 
+//#define NCBI_INT4_GI
 //#define NCBI_STRICT_GI
-//#define NCBI_INT8_GI
 //#define NCBI_STRICT_ENTREZ_ID
+//#define NCBI_STRICT_TAX_ID
 
-#ifdef NCBI_STRICT_ENTREZ_ID
-#undef NCBI_STRICT_ENTREZ_ID
+
+#if defined(NCBI_INT4_GI)
+#  ifdef NCBI_INT8_GI
+#    error "Both NCBI_INT4_GI and NCBI_INT8_GI must not be defined!"
+#  endif 
+#  ifdef NCBI_STRICT_GI
+#    error "Both NCBI_INT4_GI and NCBI_STRICT_GI must not be defined!"
+#  endif
+#elif !defined(NCBI_INT8_GI)
+#  define NCBI_INT8_GI
 #endif
 
-#ifdef NCBI_STRICT_GI
-# define NCBI_INT8_GI
+// Temporary fix: disable strict TEntrezId
+#ifdef NCBI_STRICT_ENTREZ_ID
+#  undef NCBI_STRICT_ENTREZ_ID
+#endif
+
+#ifndef NCBI_STRICT_GI
+#  undef NCBI_STRICT_ENTREZ_ID
+#  undef NCBI_STRICT_TAX_ID
 #endif
 
 #ifdef NCBI_INT8_GI
@@ -904,112 +982,6 @@ typedef int TTaxId;
 typedef Int8 TIntId;
 typedef Uint8 TUintId;
 
-#ifdef NCBI_STRICT_GI
-
-// Strict mode can be enabled only for Int8 GIs.
-
-class CStrictId64
-{
-public:
-    CStrictId64(void) : m_Gi(0) {}
-
-    bool operator==(const CStrictId64& gi) const { return m_Gi == gi.m_Gi; }
-    bool operator!=(const CStrictId64& gi) const { return m_Gi != gi.m_Gi; }
-    bool operator<(const CStrictId64& gi) const { return m_Gi < gi.m_Gi; }
-    bool operator<=(const CStrictId64& gi) const { return m_Gi <= gi.m_Gi; }
-    bool operator>(const CStrictId64& gi) const { return m_Gi > gi.m_Gi; }
-    bool operator>=(const CStrictId64& gi) const { return m_Gi >= gi.m_Gi; }
-
-    CStrictId64& operator++(void) { m_Gi++; return *this; }
-    CStrictId64 operator++(int) { CStrictId64 tmp = *this; m_Gi++; return tmp; }
-    CStrictId64& operator--(void) { m_Gi--; return *this; }
-    CStrictId64 operator--(int) { CStrictId64 tmp = *this; m_Gi--; return tmp; }
-
-    TIntId operator+(const CStrictId64& gi) const { return m_Gi + gi.m_Gi; }
-    TIntId operator-(const CStrictId64& gi) const { return m_Gi - gi.m_Gi; }
-
-    CStrictId64 operator+(TIntId offset) const { return m_Gi + offset; }
-    CStrictId64 operator-(TIntId offset) const { return m_Gi - offset; }
-#if defined(NCBI_INT8_GI)
-    CStrictId64 operator+(Int4 offset) const { return m_Gi + offset; }
-    CStrictId64 operator-(Int4 offset) const { return m_Gi - offset; }
-#endif
-
-    CStrictId64(TIntId value) : m_Gi(value) {}
-    CStrictId64& operator=(TIntId value) { m_Gi = value; return *this; }
-    operator TIntId(void) const { return m_Gi; }
-    bool operator==(TIntId value) const { return m_Gi == value; }
-
-#if defined(NCBI_INT8_GI) && defined(NCBI_TEST_APPLICATION)
-    CStrictId64(Int4 value) : m_Gi(value) {}
-    CStrictId64& operator=(Int4 value) { m_Gi = value; return *this; }
-    bool operator==(Int4 value) const { return m_Gi == value; }
-#endif
-
-    operator bool(void) const { return m_Gi != 0; }
-
-private:
-#if defined(NCBI_INT8_GI) && !defined(NCBI_TEST_APPLICATION)
-    CStrictId64(Int4);
-    CStrictId64& operator=(Int4);
-    operator Int4(void) const;
-#endif
-    CStrictId64(Int1);
-    CStrictId64(Uint1);
-    CStrictId64(Int2);
-    CStrictId64(Uint2);
-    CStrictId64(Uint4);
-    CStrictId64(Uint8);
-    CStrictId64(float);
-    CStrictId64(double);
-    CStrictId64& operator=(Int1);
-    CStrictId64& operator=(Uint1);
-    CStrictId64& operator=(Int2);
-    CStrictId64& operator=(Uint2);
-    CStrictId64& operator=(Uint4);
-    CStrictId64& operator=(Uint8);
-    CStrictId64& operator=(float);
-    CStrictId64& operator=(double);
-    operator Int1(void) const;
-    operator Uint1(void) const;
-    operator Int2(void) const;
-    operator Uint2(void) const;
-    operator Uint4(void) const;
-    operator Uint8(void) const;
-    operator float(void) const;
-    operator double(void) const;
-
-private:
-    TIntId m_Gi;
-};
-
-/// @deprecated: Use CStrictId64 instead of CStrictGi, or TGi/TEntrezId typedefs.
-NCBI_DEPRECATED typedef CStrictId64 CStrictGi;
-
-
-inline
-CNcbiOstream& operator<<(CNcbiOstream& out, const CStrictId64& gi)
-{
-    return out << TIntId(gi);
-}
-
-inline
-CNcbiIstream& operator>>(CNcbiIstream& in, CStrictId64& gi)
-{
-    TIntId id;
-    in >> id;
-    gi = id;
-    return in;
-}
-
-typedef CStrictId64 TGi;
-
-#else // NCBI_STRICT_GI
-
-typedef TIntId TGi;
-
-#endif // NCBI_STRICT_GI
-
 #else // NCBI_INT8_GI
 
 typedef int TGi;
@@ -1018,37 +990,114 @@ typedef Uint4 TUintId;
 
 #endif
 
+
+/// Key structs must always be defined - they are used in aliasinfo.cpp
+struct SStrictId_Gi {
+    typedef TIntId TId;
+};
+struct SStrictId_Entrez {
+    typedef TIntId TId;
+};
+struct SStrictId_Tax {
+    typedef int TId;
+};
+
+
+#ifdef NCBI_STRICT_GI
+
+typedef CStrictId<SStrictId_Gi, SStrictId_Gi::TId> TGi;
+
+/// @deprecated: Use TGi/TEntrezId typedefs.
+NCBI_DEPRECATED typedef TGi CStrictGi;
+
+#else // NCBI_STRICT_GI
+
+typedef SStrictId_Gi::TId TGi;
+
+#endif // NCBI_STRICT_GI
+
+
 /// TEntrezId type for entrez ids which require the same strictness as TGi.
 #ifdef NCBI_STRICT_ENTREZ_ID
-# ifndef NCBI_STRICT_GI
-#  undef NCBI_STRICT_ENTREZ_ID
-# endif
-typedef TGi TEntrezId;
+typedef CStrictId<SStrictId_Entrez, SStrictId_Entrez::TId> TEntrezId;
 #else
-typedef TIntId TEntrezId;
+typedef SStrictId_Entrez::TId TEntrezId;
 #endif
 
+/// Taxon id type.
+#ifdef NCBI_STRICT_TAX_ID
+typedef CStrictId<SStrictId_Tax, SStrictId_Tax::TId> TTaxId;
+#else
+typedef SStrictId_Tax::TId TTaxId;
+#endif
+
+
 /// a helper template to enforce constness of argument to GI_CONST macro
-template<TIntId gi>
-class CConstGIChecker {
+template<class TId, TId id>
+class CConstIdChecker {
 public:
-    static const TIntId value = gi;
+#ifndef NCBI_COMPILER_MSVC
+    enum : TId {
+        value = id
+    };
+#else
+    static const TId value = id;
+#endif
 };
-#define GI_CONST(gi) (ncbi::TGi(CConstGIChecker<gi>::value))
-#define ZERO_GI GI_CONST(0)
-#define INVALID_GI GI_CONST(-1)
 
-/// Temporary macros to convert TGi to other types (int, unsigned etc.).
-#define GI_TO(T, gi) (static_cast<T>(ncbi::TIntId(gi)))
-#define GI_FROM(T, value) (ncbi::TGi(static_cast<ncbi::TIntId>(value)))
+#ifdef NCBI_STRICT_GI
+
+/// Macros to convert TGi to other types (int, unsigned etc.).
+#define STRICT_ID_TO(TId, TInt, id) (static_cast<TInt>((id).Get()))
+#define STRICT_ID_FROM(TIdType, TIntType, id) (TIdType(static_cast<TIdType::TId>(id)))
+#define STRICT_ID_CONST(type, id) \
+    (type(static_cast<type::TId>(ncbi::CConstIdChecker<type::TId, id>::value)))
+#define STRICT_ID_ZERO(type) STRICT_ID_CONST(type, 0)
+#define STRICT_ID_INVALID(type) STRICT_ID_CONST(type, -1)
+
+#else // NCBI_STRICT_GI
+
+#define STRICT_ID_TO(TId, TInt, id) (static_cast<TInt>(id))
+#define STRICT_ID_FROM(TIdType, TIntType, id) (static_cast<TIdType>(id))
+#define STRICT_ID_CONST(type, id) (ncbi::CConstIdChecker<type, id>::value)
+#define STRICT_ID_ZERO(type) type(0)
+#define STRICT_ID_INVALID(type) type(-1)
+
+#endif // NCBI_STRICT_GI
+
+#define GI_TO(T, gi) STRICT_ID_TO(ncbi::TGi, T, gi)
+#define GI_FROM(T, value) STRICT_ID_FROM(ncbi::TGi, T, value)
+#define GI_CONST(gi) STRICT_ID_CONST(ncbi::TGi, gi)
+#define ZERO_GI STRICT_ID_ZERO(ncbi::TGi)
+#define INVALID_GI STRICT_ID_INVALID(ncbi::TGi)
+
+
+#ifdef NCBI_STRICT_ENTREZ_ID
+# define ENTREZ_ID_TO(T, entrez_id) STRICT_ID_TO(ncbi::TEntrezId, T, entrez_id)
+# define ENTREZ_ID_FROM(T, value) STRICT_ID_FROM(ncbi::TEntrezId, T, value)
+# define ENTREZ_ID_CONST(id) STRICT_ID_CONST(ncbi::TEntrezId, id)
+#else
+# define ENTREZ_ID_TO(T, entrez_id) (static_cast<T>(entrez_id))
+# define ENTREZ_ID_FROM(T, value) (static_cast<ncbi::TEntrezId>(value))
+# define ENTREZ_ID_CONST(id) id
+#endif
 
-#define ENTREZ_ID_CONST(id) (ncbi::TEntrezId(CConstGIChecker<id>::value))
 #define ZERO_ENTREZ_ID ENTREZ_ID_CONST(0)
 #define INVALID_ENTREZ_ID ENTREZ_ID_CONST(-1)
 
-/// Temporary macros to convert TEntrezId to other types (int, unsigned etc.).
-#define ENTREZ_ID_TO(T, entrez_id) (static_cast<T>(ncbi::TIntId(entrez_id)))
-#define ENTREZ_ID_FROM(T, value) (ncbi::TEntrezId(static_cast<ncbi::TIntId>(value)))
+#ifdef NCBI_STRICT_TAX_ID
+# define TAX_ID_TO(T, tax_id) STRICT_ID_TO(ncbi::TTaxId, T, tax_id)
+# define TAX_ID_FROM(T, value) STRICT_ID_FROM(ncbi::TTaxId, T, value)
+# define TAX_ID_CONST(id) STRICT_ID_CONST(ncbi::TTaxId, id)
+#else
+# define TAX_ID_TO(T, tax_id) (static_cast<T>(tax_id))
+# define TAX_ID_FROM(T, value) (static_cast<ncbi::TTaxId>(value))
+# define TAX_ID_CONST(id) id
+#endif
+
+#define ZERO_TAX_ID TAX_ID_CONST(0)
+#define INVALID_TAX_ID TAX_ID_CONST(-1)
+
 
 /// Convert gi-compatible int to/from other types.
 #define INT_ID_TO(T, id) (static_cast<T>(id))
@@ -1386,13 +1435,53 @@ unique_ptr<T, TDeleter> make_c_unique(T* p, TDeleter d)
 
 /// Overload for the above for all types of allocated memory that are to be deallocated by free()
 template <class T>
-unique_ptr<T, void(*)(T*)> make_c_unique(T* p)
+unique_ptr<T, void(*)(void*)> make_c_unique(T* p)
 {
     return make_c_unique(p, &free);
 }
 /// @}
 
 
+/// Helpers for exporting stand-alone functions from DLLs without changing the functions.
+/// Useful for exporting third-party APIs without a need to adjust third-party headers.
+/// @example:
+/// dll_header.hpp: NCBI_EXPORT_FUNC_DECLARE(XCONNECT, mbedtls_strerror);
+/// dll_source.cpp: NCBI_EXPORT_FUNC_DEFINE(XCONNECT, mbedtls_strerror);
+/// app.cpp:        NCBI_XCONNECT::mbedtls_strerror(err, buf, sizeof(buf));
+/// @{
+#define NCBI_EXPORT_FUNC_DECLARE(lib, func)                                     \
+namespace NCBI_ ## lib                                                          \
+{                                                                               \
+    namespace S ## func                                                         \
+    {                                                                           \
+        using T = decltype(func);                                               \
+                                                                                \
+        NCBI_ ## lib ## _EXPORT T* F();                                         \
+                                                                                \
+        template <class T>                                                      \
+        struct S;                                                               \
+                                                                                \
+        template <class TR, class... TArgs>                                     \
+        struct S<TR (TArgs...)>                                                 \
+        {                                                                       \
+            static TR Call(TArgs... args) { return F()(args...); }              \
+        };                                                                      \
+    }                                                                           \
+                                                                                \
+    constexpr auto func = S ## func::S<S ## func::T>::Call;                     \
+}
+
+#define NCBI_EXPORT_FUNC_DEFINE(lib, func)                                      \
+namespace NCBI_ ## lib                                                          \
+{                                                                               \
+    namespace S ## func                                                         \
+    {                                                                           \
+        T* F() { return ::func; }                                               \
+    }                                                                           \
+}
+/// @}
+
+
 END_NCBI_NAMESPACE;
 
 BEGIN_STD_NAMESPACE;
@@ -1423,17 +1512,17 @@ void swap(NCBI_NS_NCBI::AutoPtr<P,D>& ptr1,
 
 template<class Element, size_t Size>
 inline
-size_t ArraySize(const Element (&)[Size])
+constexpr size_t ArraySize(const Element (&)[Size])
 {
     return Size;
 }
 
 #ifdef NCBI_STRICT_GI
-template <> struct hash<ncbi::CStrictId64>
+template <class TKey, class TStorage> struct hash<ncbi::CStrictId<TKey, TStorage>>
 {
-    size_t operator()(const ncbi::CStrictId64 & x) const
+    size_t operator()(const ncbi::CStrictId<TKey, TStorage> & x) const
     {
-        return hash<ncbi::TIntId>()((ncbi::TIntId)x);
+        return hash<TStorage>()(x.Get());
     }
 };
 #endif /* NCBI_STRICT_GI */
