@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___EXCEPTION__HPP
 #define DBAPI_DRIVER___EXCEPTION__HPP
 
-/* $Id: exception.hpp 574980 2018-11-21 14:24:48Z ucko $
+/* $Id: exception.hpp 619521 2020-11-05 19:23:08Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -254,6 +254,21 @@ public:
         , m_RowsInBatch(0)
         NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION(CDB_Exception, CException);
 
+public:
+    template<class E>
+    NCBI_NORETURN static void MakeException(const CDiagCompileInfo& info,
+        const SMessageInContext& message,
+        EDiagSev severity,
+        int db_err_code)
+    {
+        E ex(info, nullptr, message, severity, db_err_code);
+        if (severity == eDiag_Error || severity == eDiag_Fatal)
+            ex.SetRetriable(eRetriable_No);
+        else
+            ex.SetRetriable(eRetriable_Unknown);
+        throw ex;
+    }
+
 protected:
     int     m_DBErrCode;
 
@@ -699,15 +714,9 @@ typedef CDB_UserHandler_Diag CDB_UserHandler_Default;
 /// database error code and message string.
 #define NCBI_DATABASE_THROW( exception_class, message, err_code, severity ) \
     do { \
-        exception_class ex( DIAG_COMPILE_INFO, \
-                            0, (message), severity, err_code ); \
-        if (severity == eDiag_Error || severity == eDiag_Fatal) \
-            ex.SetRetriable(eRetriable_No); \
-        else \
-            ex.SetRetriable(eRetriable_Unknown); \
-        throw ex; \
+        NCBI_NS_NCBI::CDB_Exception::MakeException<exception_class>( \
+            DIAG_COMPILE_INFO, (message), severity, err_code); \
     } while(0)
-
 #define NCBI_DATABASE_RETHROW( prev_exception, exception_class, message, \
     err_code, severity ) \
     throw exception_class( DIAG_COMPILE_INFO, \
