@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___INTERFACES__HPP
 #define DBAPI_DRIVER___INTERFACES__HPP
 
-/* $Id: interfaces.hpp 582824 2019-03-20 17:28:18Z ucko $
+/* $Id: interfaces.hpp 626337 2021-02-25 18:42:32Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
  */
 
 #include <corelib/ncbi_param.hpp>
-#include <corelib/impl/ncbi_dbsvcmapper.hpp>
+#include <corelib/ncbi_pool_balancer.hpp>
 
 #include <dbapi/driver/types.hpp>
 #include <dbapi/driver/exception.hpp>
