@@ -1,7 +1,7 @@
 #ifndef MISC_JSONWRAPP___JSONWRAPP11__HPP
 #define MISC_JSONWRAPP___JSONWRAPP11__HPP
 
-/*  $Id: jsonwrapp11.hpp 606564 2020-04-23 15:23:08Z sadyrovr $
+/*  $Id: jsonwrapp11.hpp 619053 2020-10-29 19:09:26Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -184,6 +184,9 @@ protected:
     }
     CJson_ConstNode(_Impl* impl) : m_Impl(impl) {
     }
+    void x_Assign(const CJson_ConstNode& n) {
+        m_Impl = n.m_Impl;
+    }
     _Impl* m_Impl;
     static _Impl*& x_Impl(CJson_ConstNode& v){
         return v.m_Impl;
@@ -330,6 +333,7 @@ protected:
     CJson_Value(void) {
     }
     CJson_Value( _Impl* impl) : CJson_ConstValue(impl), CJson_Node(impl) {
+        m_Impl = impl;
     }
     friend class CJson_Node;
 };
@@ -579,6 +583,7 @@ protected:
     CJson_Array(void)  {
     }
     CJson_Array(_Impl* impl) : CJson_ConstArray(impl), CJson_Node(impl) {
+        m_Impl = impl;
     }
     friend class CJson_Node;
     friend class CJson_Object;
@@ -842,6 +847,7 @@ protected:
     CJson_Object(void) {
     }
     CJson_Object(_Impl* impl) : CJson_ConstObject(impl), CJson_Node(impl) {
+        m_Impl = impl;
     }
     friend class CJson_Node;
     friend class CJson_Array;
@@ -1313,7 +1319,8 @@ CJson_ConstValue::operator=(const CJson_ConstValue& n) {
     CJson_ConstNode::operator=(n); return *this;
 }
 inline CJson_Value::CJson_Value( const CJson_Value& n)
-    :  CJson_ConstNode(), CJson_ConstValue(n), CJson_Node(n) {
+    : CJson_ConstNode(n), CJson_ConstValue(n), CJson_Node(n) {
+    x_Assign(n);
 }
 inline CJson_Value&
 CJson_Value::operator=(const CJson_Value& n) {
@@ -1403,7 +1410,8 @@ CJson_ConstArray::operator=(const CJson_ConstArray& n) {
     CJson_ConstNode::operator=(n); return *this;
 }
 inline CJson_Array::CJson_Array( const CJson_Array& n)
-    :  CJson_ConstNode(), CJson_ConstArray(n), CJson_Node(n) {
+    : CJson_ConstNode(n), CJson_ConstArray(n), CJson_Node(n) {
+    x_Assign(n);
 }
 inline CJson_Array&
 CJson_Array::operator=(const CJson_Array& n) {
@@ -1731,7 +1739,8 @@ CJson_ConstObject::operator=(const CJson_ConstObject& n) {
     CJson_ConstNode::operator=(n); return *this;
 }
 inline CJson_Object::CJson_Object( const CJson_Object& n)
-    :  CJson_ConstNode(), CJson_ConstObject(n), CJson_Node(n) {
+    : CJson_ConstNode(n), CJson_ConstObject(n), CJson_Node(n) {
+    x_Assign(n);
 }
 inline CJson_Object&
 CJson_Object::operator=(const CJson_Object& n) {
