@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___DATA_SOURCE__HPP
 #define OBJECTS_OBJMGR_IMPL___DATA_SOURCE__HPP
 
-/*  $Id: data_source.hpp 586788 2019-05-22 19:25:58Z vasilche $
+/*  $Id: data_source.hpp 610968 2020-06-26 12:55:17Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -243,7 +243,7 @@ public:
     string GetLabel(const CSeq_id_Handle& idh);
     // -1 if sequence not found
     // 0 if sequence doesn't have TaxID
-    int GetTaxId(const CSeq_id_Handle& idh);
+    TTaxId GetTaxId(const CSeq_id_Handle& idh);
     // kInvalidSeqPos if sequence not found
     TSeqPos GetSequenceLength(const CSeq_id_Handle& idh);
     typedef CDataLoader::STypeFound STypeFound;
@@ -257,7 +257,7 @@ public:
     typedef vector<bool> TLoaded;
     typedef vector<TGi> TGis;
     typedef vector<string> TLabels;
-    typedef vector<int> TTaxIds;
+    typedef vector<TTaxId> TTaxIds;
     typedef vector<TSeqPos> TSequenceLengths;
     typedef vector<CSeq_inst::TMol> TSequenceTypes;
     typedef vector<int> TSequenceStates;
