@@ -1,7 +1,7 @@
 #ifndef OBJMGR_IMPL_SCOPE_IMPL__HPP
 #define OBJMGR_IMPL_SCOPE_IMPL__HPP
 
-/*  $Id: scope_impl.hpp 610058 2020-06-10 16:19:48Z ivanov $
+/*  $Id: scope_impl.hpp 610968 2020-06-26 12:55:17Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -338,7 +338,7 @@ public:
     CSeq_id_Handle GetAccVer(const CSeq_id_Handle& idh, TGetFlags flags);
     TGi GetGi(const CSeq_id_Handle& idh, TGetFlags flags);
     string GetLabel(const CSeq_id_Handle& idh, TGetFlags flags);
-    int GetTaxId(const CSeq_id_Handle& idh, TGetFlags flags);
+    TTaxId GetTaxId(const CSeq_id_Handle& idh, TGetFlags flags);
 
     /// Bulk retrieval methods
 
@@ -358,7 +358,7 @@ public:
     void GetLabels(TLabels& ret, const TIds& idhs, TGetFlags flags);
 
     // Get a set of taxids
-    typedef vector<int> TTaxIds;
+    typedef vector<TTaxId> TTaxIds;
     void GetTaxIds(TTaxIds& ret, const TIds& idhs, TGetFlags flags);
 
     // Get bioseq synonyms, resolving to the bioseq in this scope.
