@@ -483,6 +483,7 @@ public:
     bool IsNC (void) const { return m_IsNC; }
     bool IsNM (void) const { return m_IsNM; }
     bool IsNR (void) const { return m_IsNR; }
+    bool IsNZ (void) const { return m_IsNZ; }
     bool IsPatent (void) const { return m_IsPatent; }
     bool IsPDB (void) const { return m_IsPDB; }
     bool IsWP (void) const { return m_IsWP; }
@@ -499,6 +500,7 @@ public:
     int GetPatentSequence (void) const { return m_PatentSequence; }
 
     int GetPDBChain (void) const { return m_PDBChain; }
+    string GetPDBChainID (void) const { return m_PDBChainID; }
 
     // Most important descriptor fields
 
@@ -554,6 +556,7 @@ public:
     CTempString GetIsolate (void);
     CTempString GetStrain (void);
     CTempString GetSubstrain (void);
+    CTempString GetMetaGenomeSource (void);
 
     bool IsHTGSCancelled (void);
     bool IsHTGSDraft (void);
@@ -661,6 +664,7 @@ private:
     bool m_IsNC;
     bool m_IsNM;
     bool m_IsNR;
+    bool m_IsNZ;
     bool m_IsPatent;
     bool m_IsPDB;
     bool m_IsWP;
@@ -677,6 +681,7 @@ private:
     int m_PatentSequence;
 
     int m_PDBChain;
+    string m_PDBChainID;
 
     // Instantiated title
     string m_Title;
@@ -736,6 +741,7 @@ private:
     CTempString m_Isolate;
     CTempString m_Strain;
     CTempString m_Substrain;
+    CTempString m_MetaGenomeSource;
 
     // Keyword fields (genbank or embl blocks)
     bool m_HTGSCancelled;
@@ -1031,7 +1037,7 @@ size_t CBioseqIndex::IterateGaps (Fnc m)
         }
     }
     catch (CException& e) {
-        LOG_POST(Error << "Error in CBioseqIndex::IterateGaps: " << e.what());
+        ERR_POST(Error << "Error in CBioseqIndex::IterateGaps: " << e.what());
     }
     return count;
 }
@@ -1055,7 +1061,7 @@ size_t CBioseqIndex::IterateDescriptors (Fnc m)
         }
     }
     catch (CException& e) {
-        LOG_POST(Error << "Error in CBioseqIndex::IterateDescriptors: " << e.what());
+        ERR_POST(Error << "Error in CBioseqIndex::IterateDescriptors: " << e.what());
     }
     return count;
 }
@@ -1079,7 +1085,7 @@ size_t CBioseqIndex::IterateFeatures (Fnc m)
         }
     }
     catch (CException& e) {
-        LOG_POST(Error << "Error in CBioseqIndex::IterateFeatures: " << e.what());
+        ERR_POST(Error << "Error in CBioseqIndex::IterateFeatures: " << e.what());
     }
     return count;
 }
@@ -1100,7 +1106,7 @@ size_t CBioseqIndex::IterateFeatures (CSeq_loc& slp, Fnc m)
         }
     }
     catch (CException& e) {
-        LOG_POST(Error << "Error in CBioseqIndex::IterateFeatures: " << e.what());
+        ERR_POST(Error << "Error in CBioseqIndex::IterateFeatures: " << e.what());
     }
     return count;
 }
