@@ -1,4 +1,4 @@
-/*  $Id: align_format_util.hpp 599944 2020-01-09 18:04:01Z zaretska $
+/*  $Id: align_format_util.hpp 620577 2020-11-24 14:41:21Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -102,7 +102,9 @@ static const string kUnigeneDispl =  "<div><@lnk@>-<span class=\"rlLink\">cluste
 
 ///structure
 // .ncbirc alias: STRUCTURE_URLhttps://www.ncbi.nlm.nih.gov/Structure/pdb/1AJ9 
-static const char kStructureUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/Structure/pdb/<@molid@>?log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kStructureUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/Structure/icn3d/full.html?from=blast&blast_rep_id=<@label@>&query_id=<@queryID@>&command=view+annotations;set+annotation+cdd;set+annotation+site;set+view+detailed+view;select+chain+<@label@>;show+selection&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+
+
 //substitues <@lnk_displ@>
 static const char kStructureImg[] = "<img border=0 height=16 width=16 src=\"<@protocol@>//www.ncbi.nlm.nih.gov/Structure/cblast/str_link.gif\" alt=\"Structure related to <@label@>\">";
 //For text link <@lnk@> is substituted by formatted url
@@ -394,7 +396,7 @@ public:
         bool new_win;           ///< bool indicating if click of the url will open a new window
         CRange<TSeqPos> seqRange;///< sequence range
         bool flip;              ///< flip sequence in case of opposite strands
-        int taxid;              ///< taxid
+        TTaxId taxid;           ///< taxid
         bool addCssInfo;        ///< bool indicating that css info should be added
         string segs;            ///< string containing align segments in the the following format seg1Start-seg1End,seg2Start-seg2End
         string resourcesUrl;    ///< URL(s) to other resources from .ncbirc
@@ -406,7 +408,8 @@ public:
         
         /// Constructor        
         SSeqURLInfo(string usurl,string bt, bool isnuc,string db, string rid_in,int qn, 
-                    TGi gi_in,  string acc, int lnk, int blrk,bool alnLink, bool nw, CRange<TSeqPos> range = CRange<TSeqPos>(0,0),bool flp = false, int txid = -1,bool addCssInf = false,string seqSegs = "",string resUrl = "",bool useTmpl = false, bool advView = false) 
+                    TGi gi_in,  string acc, int lnk, int blrk,bool alnLink, bool nw, CRange<TSeqPos> range = CRange<TSeqPos>(0,0),bool flp = false,
+                    TTaxId txid = INVALID_TAX_ID,bool addCssInf = false,string seqSegs = "",string resUrl = "",bool useTmpl = false, bool advView = false) 
                     : user_url(usurl),blastType(bt), isDbNa(isnuc), database(db),rid(rid_in), 
                     queryNumber(qn), gi(gi_in), accession(acc), linkout(lnk),blast_rank(blrk),isAlignLink(alnLink),
                     new_win(nw),seqRange(range),flip(flp),taxid (txid),addCssInfo(addCssInf),segs(seqSegs),
@@ -427,7 +430,7 @@ public:
         bool structure_linkout_as_group;
         bool for_alignment;
 
-        int taxid;        
+        TTaxId taxid;        
         int cur_align;        
         string taxName;
         string gnl;
@@ -894,7 +897,7 @@ public:
     ///@param id: seq id
     ///@param scope: scope to fetch this sequence
     ///
-    static int GetTaxidForSeqid(const objects::CSeq_id& id, objects::CScope& scope);
+    static TTaxId GetTaxidForSeqid(const objects::CSeq_id& id, objects::CScope& scope);
     
     ///return the frame for a given strand
     ///Note that start is zero bases.  It returns frame +/-(1-3).
@@ -948,7 +951,7 @@ public:
     ///@param query_number: the blast query number.
     ///@param for_alignment: is the URL generated for an alignment or a top defline?
     ///
-    static string BuildUserUrl(const objects::CBioseq::TId& ids, int taxid, string user_url,
+    static string BuildUserUrl(const objects::CBioseq::TId& ids, TTaxId taxid, string user_url,
                                string database, bool db_is_na, string rid,
                                int query_number, bool for_alignment);
 
@@ -1154,6 +1157,16 @@ public:
     /// CRef<CSeq_align_set> - filtered seq align
     static CRef<objects::CSeq_align_set> FilterSeqalignBySeqList(objects::CSeq_align_set& source_aln,
                                                                vector <string> &seqList);
+
+    ///function to remove sequences of accesionType from use_this_seq list    
+    ///@param use_this_seq
+    /// list <string> of seqIDs
+    ///@param accesionType
+    /// CSeq_id::EAccessionInfo accession type to check    
+    ///@return
+    /// bool true if list changed
+    static bool RemoveSeqsOfAccessionTypeFromSeqInUse(list<string> &use_this_seq, objects::CSeq_id::EAccessionInfo accesionType);
+
     ///function for Limitting seqalign by hsps number
     ///(by default results are not cut off within the query)
     ///@param source_aln
@@ -1261,7 +1274,7 @@ public:
                                                  bool for_alignment, 
                                                  int cur_align,
                                                  string& linkoutOrder,
-                                                 int taxid,
+                                                 TTaxId taxid,
                                                  string &database,
                                                  int query_number,                                                 
                                                  string &user_url,
@@ -1295,7 +1308,7 @@ public:
                                                  bool for_alignment, 
                                                  int cur_align,
                                                  string& linkoutOrder,
-                                                 int taxid,
+                                                 TTaxId taxid,
                                                  string &database,
                                                  int query_number,                                                 
                                                  string &user_url,
@@ -1465,6 +1478,7 @@ public:
     static CAlignFormatUtil::SSeqAlignSetCalcParams* GetSeqAlignSetCalcParams(const objects::CSeq_align_set& aln,int queryLength,bool do_translation);
 
     static CAlignFormatUtil::SSeqAlignSetCalcParams* GetSeqAlignSetCalcParamsFromASN(const objects::CSeq_align_set& alnSet);
+    static double GetSeqAlignSetCalcPercentIdent(const objects::CSeq_align_set& aln, bool do_translation);
 
     static map < string, CRef<objects::CSeq_align_set>  >  HspListToHitMap(vector <string> seqIdList, const objects::CSeq_align_set& source);
 
@@ -1479,7 +1493,7 @@ public:
                                                 const objects::CSeq_id& aln_id,
                                                 list<TGi>& use_this_gi,
                                                 TGi& gi,
-                                                int& taxid);
+                                                TTaxId& taxid);
     ///Scan the the list of blast deflines and find seqID to be use in display    
     ///@param handle: CBioseq_Handle [in]
     ///@param aln_id: CSeq_id object for alignment seq [in]
@@ -1510,7 +1524,7 @@ public:
                                 const objects::CSeq_id& aln_id,
                                 list<string>& use_this_seq,
                                 TGi *gi = NULL,                                
-                                int *taxid  = NULL,
+                                TTaxId *taxid  = NULL,
                                 string *textSeqID = NULL);
 
 
