@@ -1,4 +1,4 @@
-/*  $Id: showdefline.hpp 593346 2019-09-17 15:55:43Z zaretska $
+/*  $Id: showdefline.hpp 620198 2020-11-17 18:35:06Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,7 +97,8 @@ public:
         eLongSeqId = (1 << 10),         //print long sequences id (with bars)
         eRealtedInfoLinks = (1 << 11),  //Related information links - Linkout links with descriptions + others
         eShowTotalScore = (1 << 12),    //show total score column
-        eShowQueryCoverage = (1 << 13)  //show query coverage column
+        eShowQueryCoverage = (1 << 13),  //show query coverage column
+        eShowCSVDescr = (1 << 14)       //show comma separated descriptions table
     };
 
     ///Data Representing each defline
@@ -113,7 +114,7 @@ public:
         bool is_new;                   //is this sequence new (for psiblast)?
         bool was_checked;              //was this sequence checked before?
 	    string fullDefline;            //defline, containing all seq defines (PIG for example)
-        int    taxid;
+        TTaxId taxid;
         string  textSeqID;      
     };
 
@@ -488,6 +489,9 @@ protected:
     //Coverage header size
     size_t m_MaxQueryCoverLen;  
     size_t m_MaxAccLength;
+    size_t m_MaxTaxonomyNameLength;
+    size_t m_MaxTaxidLength;
+    size_t m_AccLenLength;
     //Query Lenghth
     int m_QueryLength;
     ///Indicates if 'Related structures' link should show
@@ -597,6 +601,7 @@ protected:
     ///
     void x_DisplayDeflineTableTemplate(CNcbiOstream & out);
     void x_DisplayDeflineTableTemplateText(CNcbiOstream & out);
+    void x_DisplayDeflineTableTemplateCSV(CNcbiOstream & out);
 
     void x_InitFormattingInfo(SScoreInfo* sci);    
     //For internal test
