@@ -1,4 +1,4 @@
-/* $Id: tabular.hpp 598778 2019-12-13 18:23:44Z jianye $ 
+/* $Id: tabular.hpp 616915 2020-09-22 19:19:18Z jianye $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -347,12 +347,12 @@ private:
     string m_BTOP;            /// Blast-traceback-operations.
 
     //TaxInfo
-    set<int>			m_SubjectTaxIds;
+    set<TTaxId> 		m_SubjectTaxIds;
     vector<string>		m_SubjectSciNames;
     vector<string>		m_SubjectCommonNames;
     set<string>		m_SubjectBlastNames;
     set<string>		m_SubjectSuperKingdoms;
-    int				m_SubjectTaxId;
+    TTaxId			m_SubjectTaxId;
     string			m_SubjectSciName;
     string			m_SubjectCommonName;
     string			m_SubjectBlastName;
@@ -704,17 +704,20 @@ private:
     string m_Query;
     bool m_IsNucl;
     bool m_IsMinusStrand;
-    string m_FrameInfo;                                                           
+    string m_FrameInfo; 
+    string m_VFrameShift; 
     string m_ChainType;
     string m_MasterChainTypeToShow;
     SIgGene m_VGene;
     SIgGene m_DGene;
     SIgGene m_JGene;
     vector<SIgDomain *> m_IgDomains; 
+
+    //index 0-2, not currently being used
     // index 4, productive/non-productive
     // index 3, stop codon or not
-
-    vector<string> m_OtherInfo;
+    static const int num_otherinfo = 5; 
+    string m_OtherInfo[num_otherinfo];
     int m_Cdr3Start;
     int m_Cdr3End;
     int m_Fwr4Start;
