@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBATLAS_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBATLAS_HPP
 
-/*  $Id: seqdbatlas.hpp 591546 2019-08-16 16:59:06Z vasilche $
+/*  $Id: seqdbatlas.hpp 631567 2021-05-19 13:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -204,7 +204,7 @@ private:
     class CSeqDBAtlas & m_Atlas;
 
     /// If this is true, this thread owns the atlas lock.
-    bool m_Locked;
+    volatile bool m_Locked;
 };
 
 
@@ -335,7 +335,7 @@ public:
     ///   The lock hold object for this thread.
     /// @return
     ///   True if the file exists.
-    bool DoesFileExist(const string & fname, CSeqDBLockHold & locked);
+    bool DoesFileExist(const string & fname);
 
     /// Check if file exists.
     ///
@@ -347,23 +347,11 @@ public:
     ///   The lock hold object for this thread.
     /// @return
     ///   True if the file exists.
-    bool DoesFileExist(const CSeqDB_Path & fname, CSeqDBLockHold & locked)
-    {
-        return DoesFileExist(fname.GetPathS(), locked);
-    }
-
-    bool DoesFileExist(const string & fname)
-    {
-        TIndx length(0);
-        return GetFileSizeL(fname, length);
-    }
-
     bool DoesFileExist(const CSeqDB_Path & fname)
     {
-        TIndx length(0);
-        return GetFileSizeL(fname.GetPathS(), length);
+        return DoesFileExist(fname.GetPathS());
     }
-  
+
     /// Get size of a file.
     ///
     /// Check whether a file exists and get the file's size.
@@ -377,8 +365,7 @@ public:
     /// @return
     ///   true if the file exists.
     bool GetFileSize(const string   & fname,
-                     TIndx          & length,
-                     CSeqDBLockHold & locked);
+                     TIndx          & length);
 
     /// Get size of a file.
     ///
@@ -402,12 +389,7 @@ public:
     ///
     /// @param datap
     ///   Pointer to the data to release or deallocate.
-    void RetRegion(const char * datap)
-    {
-        x_RetRegion(datap);
-    }
-
-    
+    static void RetRegion(const char * datap);
 
     /// Allocate memory that atlas will keep track of.
     ///
@@ -435,20 +417,8 @@ public:
     ///   Specify true to zero out memory contents.
     /// @return
     ///   A pointer to the allocation region of memory.
-    char * Alloc(size_t length, CSeqDBLockHold & locked, bool clear = true);
+    static char * Alloc(size_t length, bool clear = true);
 
-    /// Return allocated memory.
-    ///
-    /// This method returns memory acquired from the Alloc() method.
-    /// Dynamically allocated memory from other sources should not be
-    /// freed with this method, and memory allocated via Alloc()
-    /// should not be freed by any means other than this method    
-    ///
-    /// @param freeme
-    ///   A pointer to memory allocated via Alloc().
-    /// @param locked
-    ///   The lock hold object for this thread.
-    void Free(const char * freeme, CSeqDBLockHold & locked);
 
     /// Register externally allocated memory.
     ///
@@ -531,19 +501,6 @@ public:
         return sliceSize;
     }
     
-    /// Return the current number of bytes allocated.
-    ///
-    /// This returns the number of bytes currently allocated by the
-    /// atlas code.  It does not include overhead or meta-data such as
-    /// the CRegionMap objects or the atlas object itself.
-    ///
-    /// @return
-    ///   The amount of memory allocated in bytes.
-    TIndx GetCurrentAllocationTotal()
-    {
-        return m_CurAlloc;
-    }
-    
     /// Get BlastDB search path.
     const string GetSearchPath() const
     {
@@ -586,6 +543,8 @@ public:
         eFileCounterDecrement
     };
 
+    CMemoryFile* ReturnMemoryFile(const string& fileName);
+
     int ChangeOpenedFilseCount(EFilesCount fc) 
     {
         switch(fc) {
@@ -607,29 +566,27 @@ public:
     int GetOpenedFilseCount(void) { return m_OpenedFilesCount;}
     
 private:
+
+    class CAtlasMappedFile : public CMemoryFile {
+    public:
+    	CAtlasMappedFile(const string & filename): CMemoryFile(filename),m_Count(1){
+    		const string exts="hd|hi|nd|ni|pd|pi|si|sd|ti|td";
+    		string ext = filename.substr(filename.length()-2);
+    		if (exts.find(ext) != NPOS) {
+    			m_isIsam = true;
+    		}
+    		else {
+    			m_isIsam = false;
+    		}
+    	}
+    	~CAtlasMappedFile() {
+    		_ASSERT(m_Count == 0);
+    	}
+    	int m_Count;
+    	bool m_isIsam;
+    };
     /// Private method to prevent copy construction.
     CSeqDBAtlas(const CSeqDBAtlas &);
-
-    /// Try to find the region and free it.
-    ///
-    /// This method looks for the region in the memory pool (m_Pool),
-    /// which means it must have been allocated with Alloc().
-    ///
-    /// @param freeme
-    ///   The pointer to free.
-    /// @return
-    ///   true, if the memory block was found and freed.
-    bool x_Free(const char * freeme);
-
-    
-    /// Releases or deletes a memory region.
-    ///
-    /// It searches forallocated block that owns the specified address,
-    /// and releases it, if found. 
-    ///
-    /// @param datap
-    ///   Pointer to the area to delete or release a hold on.
-    void x_RetRegion(const char * datap);
    
     /// Protects most of the critical regions of the SeqDB library.
     CMutex m_Lock;
@@ -638,15 +595,6 @@ private:
     /// not needed if each thread access different database volume.
     bool m_UseLock;
     
-    /// Maps from pointers to dynamically allocated blocks to the byte
-    /// size of the allocation.
-    map<const char *, size_t> m_Pool;
-    /// Bytes of "data" currently known to SeqDBAtlas.  This does not
-    /// include metadata
-    TIndx m_CurAlloc;
-    //m_pool was used for mrmory allocation
-    bool m_Alloc;
-
     enum {e_MaxSlice64 = 1 << 30};
     
     /// Cache of file existence and length.
@@ -656,7 +604,7 @@ private:
     Uint8 m_MaxFileSize;
 
     std::mutex m_FileMemMapMutex;
-    map<string, unique_ptr<CMemoryFile> > m_FileMemMap;    
+    map<string, unique_ptr<CAtlasMappedFile> > m_FileMemMap;
     int m_OpenedFilesCount;
     int m_MaxOpenedFilesCount;
 
@@ -753,29 +701,25 @@ public:
     /// @param filename
     ///   file to memory map    
     void Init(const string filename) {                
+    	 CSeqDBLockHold locked(m_Atlas);
+    	 m_Atlas.Lock(locked);
         if(!m_MappedFile || m_Filename != filename)
         {
+        	Clear();
             m_Filename = filename;
             Init();
         }
+        m_Atlas.Unlock(locked);
     }
 
     //m_Filename is set
     void Init(void)
     {
         try {
-            if (IsIndexFile()) {
-                m_MappedFile = m_Atlas.GetMemoryFile(m_Filename);
-            }
-            else {
-                m_MappedFile = new CMemoryFile(m_Filename);
-                m_Atlas.ChangeOpenedFilseCount(CSeqDBAtlas::eFileCounterIncrement);
-                x_LogMessage(eMapNew);
-            }
+            m_MappedFile = m_Atlas.GetMemoryFile(m_Filename);
             m_Mapped = true;
         }
         catch (const std::exception&) {
-            x_LogMessage(eMapError);
             NCBI_THROW(CSeqDBException,
                 eFileErr,
                 "Cannot memory map " + m_Filename + ". Number of files opened: " + NStr::IntToString(m_Atlas.GetOpenedFilseCount()));
@@ -790,13 +734,9 @@ public:
     void Clear()
     {
         
-        if(m_MappedFile && m_Mapped && !IsIndexFile()) { 
-                m_MappedFile->Unmap();		                                                        
-                m_Atlas.ChangeOpenedFilseCount(CSeqDBAtlas::eFileCounterDecrement);                        
-                x_LogMessage(eUnmap);
-                delete m_MappedFile;
-                m_MappedFile = NULL;
-                m_Mapped = false;
+        if(m_MappedFile && m_Mapped ) {
+        	m_MappedFile = m_Atlas.ReturnMemoryFile(m_Filename);
+            m_Mapped = false;
         }        
     }
 
@@ -825,74 +765,7 @@ public:
         return(const char *)(m_DataPtr + offset);    
     }
 
-    bool IsIndexFile() {
-        bool isIndex = (NStr::Find(m_Filename,".pin") != NPOS ||  NStr::Find(m_Filename,".nin") != NPOS) ? true : false;      
-        return isIndex;
-    }
-
 private:
-    enum ELogMessage {
-        eNone,
-        eMapNew,
-        eMapNewLocked,
-        eMapExists,
-        eMapExistsLocked,
-        eUnmap,
-        eMapError
-    };
-
-    void x_LogMessage(ELogMessage em,CSeqDBAtlas::EFilesCount fc = CSeqDBAtlas::eFileCounterNoChange)
-    {
-        //Disable logging
-        if(em != eMapError) return; 
-        int openedFilesCount;
-        if(fc != CSeqDBAtlas::eFileCounterNoChange)
-        {
-            CSeqDBLockHold locked(m_Atlas);
-            m_Atlas.Lock(locked);          
-            openedFilesCount = m_Atlas.ChangeOpenedFilseCount(fc);
-        }
-        else {
-            openedFilesCount = m_Atlas.GetOpenedFilseCount();        
-        }
-        int threadID = CThread::GetSelf();
-                
-        string logMessage;
-        switch (em) {
-            case eMapNew: 
-                logMessage = "Map    new   :";
-                break;
-
-            case eMapNewLocked: 
-                logMessage = "Map    new(l):";
-                break;
-
-            case eMapExists:
-                logMessage = "Map exists   :";                
-                break;
-
-            case eMapExistsLocked:
-                logMessage = "Map exists(l):";               
-                break;
-
-            case eUnmap:
-                logMessage = "Unmap        :";
-                break;
-
-            case eMapError:
-                logMessage = "Error memory mapping:";                
-                break;
-
-            case eNone:
-                break;
-
-            default:
-                break;
-        }
-        if(!logMessage.empty()) {
-            cerr << logMessage << m_Filename  << " openedFilesCount=" << openedFilesCount << " threadID=" << threadID << endl;
-        }
-    }
     CSeqDBAtlas & m_Atlas;    
      /// Points to the beginning of the data area.
     const char * m_DataPtr;
