@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBISAM_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBISAM_HPP
 
-/*  $Id: seqdbisam.hpp 571379 2018-09-25 18:59:24Z fongah2 $
+/*  $Id: seqdbisam.hpp 631527 2021-05-19 13:49:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -491,7 +491,8 @@ private:
         eNoError         =  0,   /// Lookup was successful
         eBadVersion      =  -10, /// The format version of the ISAM file is unsupported.
         eBadType         =  -11, /// The requested ISAM type did not match the file.
-        eWrongFile       =  -12  /// The file was not found, or was the wrong length.
+        eWrongFile       =  -12, /// The file was not found, or was the wrong length.
+        eInitFailed      =  -13  /// The file was not found, or was the wrong length.
     };
 
     /// Load and extract all index samples into array at once
@@ -554,23 +555,12 @@ private:
 
         gis.InsureOrder(CSeqDBGiList::eGi);
 
-        x_InitLease();//Map files if needed
         if(m_Initialized == false) {
-            EErrorCode error = x_InitSearch();
-
-            if(error != eNoError) {
-                // Most ordinary errors (missing GIs for example) are
-                // ignored for "multi" mode searches.  But if a GI list is
-                // specified, and cannot be interpreted, it is an error.
-
                 NCBI_THROW(CSeqDBException,
                        eArgErr,
                        "Error: Unable to use ISAM index in batch mode.");
-            }
         }
 
-        //CSeqDBMemLease lease(m_Atlas);
-
         vector<T> sample_keys;
         vector<TIndx> page_offs;
         vector<T> keys;
@@ -1184,11 +1174,6 @@ private:
                                 string       & index_name,
                                 string       & data_name);
 
-    void x_InitLease(void) {
-        if(!m_IndexLease.IsMapped()) m_IndexLease.Init();
-        if(!m_DataLease.IsMapped()) m_DataLease.Init();
-    }
-
     // Data
 
     /// The memory management layer
