@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 
-/*  $Id: seqdbvol.hpp 605340 2020-04-09 16:06:43Z ivanov $
+/*  $Id: seqdbvol.hpp 631537 2021-05-19 13:50:49Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -101,7 +101,7 @@ class CSeqDBRangeList : public CObject {
 public:
     /// Constructor.
     /// @param atlas The SeqDB memory management layer. [in]
-    CSeqDBRangeList(CSeqDBAtlas & atlas)
+    CSeqDBRangeList()
       : m_CacheData (false)
     {
         // Sequence caching is not implemented yet.  It would increase
@@ -222,7 +222,7 @@ public:
     ///     The lock holder object for this thread. [in]
     /// @return
     ///   The length in bases of the sequence.
-    int GetSeqLengthProt(int oid, CSeqDBLockHold & locked) const;
+    int GetSeqLengthProt(int oid) const;
 
     /// Approximate sequence length for nucleotide databases.
     ///
@@ -239,7 +239,7 @@ public:
     ///     The lock holder object for this thread. [in]
     /// @return
     ///   The approximate length in bases of the sequence.
-    int GetSeqLengthApprox(int oid, CSeqDBLockHold & locked) const;
+    int GetSeqLengthApprox(int oid) const;
 
     /// Exact sequence length for nucleotide databases.
     ///
@@ -254,7 +254,7 @@ public:
     ///     The lock holder object for this thread. [in]
     /// @return
     ///   The length in bases of the sequence.
-    int GetSeqLengthExact(int oid, CSeqDBLockHold & locked) const;
+    int GetSeqLengthExact(int oid) const;
 
     /// Get filtered sequence header information.
     ///
@@ -351,11 +351,9 @@ public:
     ///   within the previous lease [in]
     /// @return
     ///   The length of this sequence in bases.
-    int GetSequence(int oid, const char ** buffer,
-                    CSeqDBLockHold & locked,
-                    bool in_lease = false) const
+    int GetSequence(int oid, const char ** buffer) const
     {
-        return x_GetSequence(oid, buffer, true, false, in_lease);
+        return x_GetSequence(oid, buffer);
     }
 
     /// Get a sequence with ambiguous regions.
@@ -388,16 +386,14 @@ public:
                     int               nucl_code,
                     ESeqDBAllocType   alloc_type,
                     SSeqDBSlice     * region,
-                    CSeqDB::TSequenceRanges * masks,
-                    CSeqDBLockHold  & locked) const;
+                    CSeqDB::TSequenceRanges * masks) const;
 
     int GetAmbigPartialSeq(int                oid,
                            char            ** buffer,
                            int                nucl_code,
                            ESeqDBAllocType    alloc_type,
                            CSeqDB::TSequenceRanges  * partial_ranges,
-                           CSeqDB::TSequenceRanges  * masks,
-                           CSeqDBLockHold  & locked) const;
+                           CSeqDB::TSequenceRanges  * masks) const;
 
     /// Get the Seq-ids associated with a sequence.
     ///
@@ -411,6 +407,8 @@ public:
     /// @return
     ///   The list of Seq-id objects for this sequences.
     list< CRef<CSeq_id> > GetSeqIDs(int  oid) const;
+    // same as above version with cached CObjectIStreamAsnBinary
+    list< CRef<CSeq_id> > GetSeqIDs(int  oid, CObjectIStreamAsnBinary  *inpstr) const;
 
     /// Get the GI of a sequence
     /// This method returns the gi of the sequence
@@ -477,7 +475,7 @@ public:
     ///   The lock holder object for this thread. [in]
     /// @return
     ///   True if the PIG was found.
-    bool PigToOid(int pig, int & oid, CSeqDBLockHold & locked) const;
+    bool PigToOid(int pig, int & oid) const;
 
     /// Find the PIG given an OID.
     ///
@@ -690,8 +688,7 @@ public:
     void GetRawSeqAndAmbig(int              oid,
                            const char    ** buffer,
                            int            * seq_length,
-                           int            * ambig_length,
-                           CSeqDBLockHold & locked) const;
+                           int            * ambig_length) const;
 
     /// Get GI Bounds.
     ///
@@ -733,8 +730,7 @@ public:
     /// @param locked Lock holder object for this thread. [in]
     void GetStringBounds(string         & low_id,
                          string         & high_id,
-                         int            & count,
-                         CSeqDBLockHold & locked) const;
+                         int            & count) const;
 
     /// List of sequence offset ranges.
     typedef set< pair<int, int> > TRangeList;
@@ -773,12 +769,11 @@ public:
     void SetOffsetRanges(int                oid,
                          const TRangeList & offset_ranges,
                          bool               append_ranges,
-                         bool               cache_data,
-                         CSeqDBLockHold   & locked) const;
+                         bool               cache_data) const;
 
     /// Flush all offset ranges cached
     /// @param locked        Lock holder object for this thread. [in]
-    void FlushOffsetRangeCache(CSeqDBLockHold& locked);
+    void FlushOffsetRangeCache();
 
     /// Get the sequence hash for a given OID.
     ///
@@ -1030,6 +1025,11 @@ private:
                  bool             adjust_oids,
                  bool           * changed) const;
 
+    CRef<CBlast_def_line_set>
+    x_GetHdrAsn1(int              oid,
+                 bool             adjust_oids,
+                 bool           * changed,
+		 CObjectIStreamAsnBinary  *inpstr) const;
     /// Get sequence header binary data.
     ///
     /// This method returns the sequence header information as a
@@ -1081,6 +1081,11 @@ private:
     x_GetFilteredHeader(int                    oid,
                         bool                 * changed) const;
 
+    CRef<CBlast_def_line_set>
+    x_GetFilteredHeader(int                    oid,
+                        bool                 * changed,
+			CObjectIStreamAsnBinary *inpstr ) const;
+
     /// Get sequence header information structures.
     ///
     /// This method reads the sequence header information and returns
@@ -1149,8 +1154,7 @@ private:
                       int                nucl_code,
                       ESeqDBAllocType    alloc_type,
                       SSeqDBSlice      * region,
-                      CSeqDB::TSequenceRanges *masks,
-                      CSeqDBLockHold   & locked) const;
+                      CSeqDB::TSequenceRanges *masks) const;
 
     /// Allocate memory in one of several ways.
     ///
@@ -1168,8 +1172,7 @@ private:
     /// @return
     ///     A pointer to the allocated memory.
     char * x_AllocType(size_t            length,
-                       ESeqDBAllocType   alloc_type,
-                       CSeqDBLockHold  & locked) const;
+                       ESeqDBAllocType   alloc_type) const;
 
     /// Get sequence data.
     ///
@@ -1197,10 +1200,7 @@ private:
     /// @return
     ///     The length of the sequence in bases.
     int x_GetSequence(int              oid,
-                      const char    ** buffer,
-                      bool             keep,                      
-                      bool             can_release,
-                      bool             in_lease = false) const;
+                      const char    ** buffer) const;
 
     /// Get partial sequence data.
     ///
@@ -1336,12 +1336,15 @@ private:
     void x_OpenSeqFile(void) const;
     void x_OpenHdrFile(void) const;
     void x_OpenPigFile(void) const;
+    void x_UnleasePigFile(void) const;
     void x_OpenGiFile(void) const;
+    void x_UnleaseGiFile(void) const;
     void x_OpenStrFile(void) const;
+    void x_UnleaseStrFile(void) const;
     void x_OpenTiFile(void) const;
+    void x_UnleaseTiFile(void) const;
     void x_OpenHashFile(void) const;
     void x_OpenOidFile(void) const;
-    void x_UnLeaseIsam(void) const;
 
     /// The memory management layer.
     CSeqDBAtlas & m_Atlas;
@@ -1420,13 +1423,17 @@ private:
     /// True if the volume file has been (at least tried to) opened
     mutable bool m_SeqFileOpened;
     mutable bool m_HdrFileOpened;
-    mutable bool m_PigFileOpened;
-    mutable bool m_GiFileOpened;
-    mutable bool m_StrFileOpened;
-    mutable bool m_TiFileOpened;
     mutable bool m_HashFileOpened;
     mutable bool m_OidFileOpened;
 
+    mutable CFastMutex m_MtxGi;
+    mutable CFastMutex m_MtxPig;
+    mutable CFastMutex m_MtxStr;
+    mutable CFastMutex m_MtxTi;
+    mutable CFastMutex m_MtxSeq;
+    mutable CFastMutex m_MtxHdr;
+    mutable CFastMutex m_MtxCachedRange;
+
 #if ((!defined(NCBI_COMPILER_WORKSHOP) || (NCBI_COMPILER_VERSION  > 550)) && \
      (!defined(NCBI_COMPILER_MIPSPRO)) )
     /// Set of columns defined for this volume.
