@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 
-/*  $Id: seqdbcommon.hpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: seqdbcommon.hpp 610974 2020-06-26 12:59:33Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -446,7 +446,7 @@ public:
     template <class T>
     T GetKey(int index) const
     {
-        return m_GisOids[index].gi;
+        return GI_TO(T, m_GisOids[index].gi);
     }
 
     template <class T>
