@@ -1,4 +1,4 @@
-/*  $Id: build_db.hpp 590117 2019-07-23 19:26:44Z camacho $
+/*  $Id: build_db.hpp 631508 2021-05-19 13:47:28Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -451,6 +451,7 @@ public:
     ///
     /// @param dbname output blast db name (with path)
     static void CreateDirectories(const string& dbname);
+
 private:
     /// Get a scope for remote loading of objects.
     objects::CScope & x_GetScope();
@@ -650,6 +651,9 @@ private:
 
     /// If set to true, when copying BLASTDBs, skip the GIs
     bool m_SkipCopyingGis;
+
+    /// If set to true, skip GIs with value > 0x7FFFFFFF
+    bool m_SkipLargeGis;
 };
 
 END_NCBI_SCOPE
