@@ -1,4 +1,4 @@
-/*  $Id: feattable_edit.hpp 612521 2020-07-23 11:23:16Z ivanov $
+/*  $Id: feattable_edit.hpp 632623 2021-06-03 17:38:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -57,6 +57,15 @@ public:
         unsigned int = 1, //starting locus tag
         unsigned int = 1, //starting feature id
         IObjtoolsListener* = nullptr);
+
+    CFeatTableEdit(
+        CSeq_annot&,
+        unsigned int sequenceSize,
+        const string& = "",
+        unsigned int = 1, //starting locus tag
+        unsigned int = 1, //starting feature id
+        IObjtoolsListener* = nullptr);
+
     ~CFeatTableEdit();
 
     void GenerateLocusTags();
@@ -77,6 +86,10 @@ public:
     void GenerateMissingParentFeaturesForEukaryote();
     void GenerateMissingParentFeaturesForProkaryote();
     void ProcessCodonRecognized();
+
+    void MergeFeatures(
+        const CSeq_annot::TData::TFtable& other);
+
     unsigned int PendingLocusTagNumber() const {
         return mLocusTagNumber;
     }
@@ -86,15 +99,9 @@ public:
 
     bool AnnotHasAllLocusTags() const;
 
-    bool m_use_hypothetic_protein = true;
-    string mLocusTagPrefix;
-    unsigned int mLocusTagNumber = 1;
-    unsigned int mNextFeatId     = 1;
-
 protected:
     void xGenerateLocusIdsUseExisting();
     void xGenerateLocusIdsRegenerate();
-
     string xNextFeatId();
     string xNextLocusTag();
     string xNextProteinId(
@@ -161,6 +168,9 @@ protected:
     bool xAdjustExistingParentGene(
         CMappedFeat);
 
+    CRef<CSeq_loc> xGetGeneLocation(
+        const CSeq_loc&);
+
     static std::string xGetIdStr(
         CMappedFeat);
     std::string xGetCurrentLocusTagPrefix(
@@ -178,14 +188,25 @@ protected:
     void xGenerate_mRNA_Product(CSeq_feat& cd_feature);
     CConstRef<CSeq_feat> xGetLinkedFeature(const CSeq_feat& cd_feature, bool gene);
 
+    using FeatMap = map<string, CRef<CSeq_feat>>;
+    void xRenameFeatureId(
+        const CObject_id&,
+        FeatMap&);
 
     CSeq_annot& mAnnot;
+    unsigned int mSequenceSize;
     CRef<CScope> mpScope;
     CSeq_annot_Handle mHandle;
     feature::CFeatTree mTree;
     CSeq_annot_EditHandle mEditHandle;
     IObjtoolsListener* mpMessageListener;
 
+    bool m_use_hypothetic_protein = true;
+    string mLocusTagPrefix;
+    unsigned int mLocusTagNumber = 1;
+    unsigned int mNextFeatId     = 1;
+
+protected:
     map<string, int> mMapProtIdCounts;
 
     using TFeatQualMap = map<CMappedFeat, string>;
