@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_GENERATOR__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_GENERATOR__HPP
 
-/*  $Id: flat_file_generator.hpp 613149 2020-08-03 15:02:23Z ivanov $
+/*  $Id: flat_file_generator.hpp 632193 2021-05-27 13:43:02Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -101,10 +101,18 @@ public:
         ENa_strand strand, CScope& scope, CFlatItemOStream& item_os);
 
     // Versions that loop through Bioseq components
-    void Generate(const CSeq_entry_Handle& entry, CNcbiOstream& os, bool useSeqEntryIndexing);
-    void Generate(const CBioseq_Handle& bsh, CNcbiOstream& os, bool useSeqEntryIndexing);
-    void Generate(const CSeq_entry_Handle& entry, CFlatItemOStream& item_os, bool useSeqEntryIndexing);
-    void Generate(const CSeq_loc& loc, CScope& scope, CNcbiOstream& os, bool useSeqEntryIndexing);
+    void Generate(const CSeq_entry_Handle& entry, CNcbiOstream& os, bool useSeqEntryIndexing,
+                  CNcbiOstream* m_Os = 0, CNcbiOstream* m_On = 0, CNcbiOstream* m_Og = 0,
+                  CNcbiOstream* m_Or = 0, CNcbiOstream* m_Op = 0, CNcbiOstream* m_Ou = 0);
+    void Generate(const CBioseq_Handle& bsh, CNcbiOstream& os, bool useSeqEntryIndexing,
+                  CNcbiOstream* m_Os = 0, CNcbiOstream* m_On = 0, CNcbiOstream* m_Og = 0,
+                  CNcbiOstream* m_Or = 0, CNcbiOstream* m_Op = 0, CNcbiOstream* m_Ou = 0);
+    void Generate(const CSeq_entry_Handle& entry, CFlatItemOStream& item_os, bool useSeqEntryIndexing,
+                  CNcbiOstream* m_Os = 0, CNcbiOstream* m_On = 0, CNcbiOstream* m_Og = 0,
+                  CNcbiOstream* m_Or = 0, CNcbiOstream* m_Op = 0, CNcbiOstream* m_Ou = 0);
+    void Generate(const CSeq_loc& loc, CScope& scope, CNcbiOstream& os, bool useSeqEntryIndexing,
+                  CNcbiOstream* m_Os = 0, CNcbiOstream* m_On = 0, CNcbiOstream* m_Og = 0,
+                  CNcbiOstream* m_Or = 0, CNcbiOstream* m_Op = 0, CNcbiOstream* m_Ou = 0);
 
     // for use when generating a range of a Seq-submit
     void SetSubmit(const CSubmit_block& sub) { m_Ctx->SetSubmit(sub); }
