@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 
-/*  $Id: feature_item.hpp 604098 2020-03-23 12:19:42Z ivanov $
+/*  $Id: feature_item.hpp 613862 2020-08-13 17:00:53Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -240,6 +240,8 @@ protected:
     void x_AddQualsRegion( CBioseqContext& );
     void x_AddQualsProt( CBioseqContext&, bool );
     void x_AddQualsPsecStr( CBioseqContext& );
+    void x_AddQualsNonStd(CBioseqContext& ctx );
+
     void x_AddQualsHet( CBioseqContext& ctx );
     void x_AddQualsVariation( CBioseqContext& ctx );
 
@@ -277,6 +279,7 @@ protected:
     void x_AddFTableSiteQuals(const CSeqFeatData::TSite& site);
     void x_AddFTablePsecStrQuals(const CSeqFeatData::TPsec_str& psec_str);
     void x_AddFTablePsecStrQuals(const CSeqFeatData::THet& het);
+    void x_AddFTableNonStdQuals(const CSeqFeatData::TNon_std_residue& res);
     void x_AddFTableBiosrcQuals(const CBioSource& src);
     void x_AddFTableDbxref(const CSeq_feat::TDbxref& dbxref);
     void x_AddFTableExtQuals(const CSeq_feat::TExt& ext);
