@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_HPP
 #define OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_HPP
 
-/*  $Id: psg_client.hpp 612393 2020-07-21 13:51:24Z ivanov $
+/*  $Id: psg_client.hpp 628921 2021-04-07 18:46:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,6 +30,7 @@
  *
  */
 
+#include <corelib/ncbimisc.hpp>
 #include <corelib/ncbitime.hpp>
 #include <corelib/ncbi_url.hpp>
 #include <corelib/request_ctx.hpp>
@@ -38,6 +39,7 @@
 #include <objects/seqloc/Seq_loc.hpp>
 #include <objects/seqset/Bioseq_set.hpp>
 #include <objects/seq/Seq_annot.hpp>
+#include <objects/seqsplit/ID2S_Seq_annot_Info.hpp>
 
 
 #if defined(NCBI_THREADS) && defined(HAVE_LIBNGHTTP2) && defined(HAVE_LIBUV)
@@ -99,8 +101,6 @@ protected:
 
     virtual ~CPSG_Request() = default;
 
-    struct x_GetBioIdParams;
-
 private:
     virtual string x_GetType() const = 0;
     virtual string x_GetId() const = 0;
@@ -126,37 +126,96 @@ public:
     ///  Bio ID (like accession)
     CPSG_BioId(string id, TType type = {}) : m_Id(move(id)), m_Type(type) {}
 
-    const string& Get()     const { return m_Id; }
+    /// Get tilde-separated string representation of this bio ID (e.g. for logging)
+    string Repr() const;
+
+    /// Get ID
+    const string& GetId() const { return m_Id; }
+
+    /// Get type
+    TType GetType() const { return m_Type; }
 
 private:
     string m_Id;
     TType  m_Type;
-
-    friend class CPSG_Request;
 };
 
 
 
-/// Data blob unique ID
+/// Blob data unique ID
 ///
-class CPSG_BlobId
+class CPSG_DataId
 {
 public:
+    virtual ~CPSG_DataId() = default;
+
+    /// Get tilde-separated string representation of this data ID (e.g. for logging)
+    virtual string Repr() const = 0;
+};
+
+
+
+/// Blob unique ID
+///
+class CPSG_BlobId : public CPSG_DataId
+{
+public:
+    using TLastModified = CNullable<Int8>;
+
     /// Mainstream blob ID ctor - from a string ID
     /// @param id
     ///  Blob ID
-    CPSG_BlobId(string id) : m_Id(move(id)) {}
+    CPSG_BlobId(string id, TLastModified last_modified = {})
+        : m_Id(move(id)),
+          m_LastModified(move(last_modified))
+    {}
 
     /// Historical blob ID system -- based on the "satellite" and the "key"
     /// inside it. It'll be translated into "<sat>.<sat_key>" string.
     /// @sa  objects::CID2_Blob_Id::TSat, objects::CID2_Blob_Id::TSat_key
-    CPSG_BlobId(int sat, int sat_key) : m_Id(to_string(sat) + "." + to_string(sat_key)) {}
+    CPSG_BlobId(int sat, int sat_key, TLastModified last_modified = {})
+        : m_Id(to_string(sat) + "." + to_string(sat_key)),
+          m_LastModified(move(last_modified))
+    {}
 
-    /// Get the blob ID
-    const string& Get() const { return m_Id; }
+    /// Get tilde-separated string representation of this blob ID (e.g. for logging)
+    string Repr() const override;
+
+    /// Get ID
+    const string& GetId() const { return m_Id; }
+
+    /// Get last modified
+    const TLastModified& GetLastModified() const { return m_LastModified; }
 
 private:
     string m_Id;
+    TLastModified m_LastModified;
+};
+
+
+
+/// Chunk unique ID
+///
+class CPSG_ChunkId : public CPSG_DataId
+{
+public:
+    CPSG_ChunkId(Uint8 id2_chunk, string id2_info)
+        : m_Id2Chunk(id2_chunk),
+          m_Id2Info(move(id2_info))
+    {}
+
+    /// Get tilde-separated string representation of this chunk ID (e.g. for logging)
+    string Repr() const override;
+
+    /// Get ID2 chunk number
+    Uint8 GetId2Chunk() const { return m_Id2Chunk; }
+
+    /// Get ID2 info
+    const string& GetId2Info() const { return m_Id2Info; }
+
+private:
+    Uint8 m_Id2Chunk;
+    string m_Id2Info;
 };
 
 
@@ -223,15 +282,19 @@ public:
     /// Set substitution policy for version-less primary seq-ids
     void SetAccSubstitution(EPSG_AccSubstitution acc_substitution) { m_AccSubstitution = acc_substitution; }
 
+    /// Enable/disable auto blob skipping on server for this request
+    void SetAutoBlobSkipping(bool auto_blob_skipping) { m_AutoBlobSkipping = auto_blob_skipping ? eOn : eOff; }
+
 private:
     string x_GetType() const override { return "biodata"; }
-    string x_GetId() const override { return GetBioId().Get(); }
+    string x_GetId() const override { return GetBioId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
     CPSG_BioId    m_BioId;
     EIncludeData  m_IncludeData = EIncludeData::eDefault;
     TExcludeTSEs  m_ExcludeTSEs;
     EPSG_AccSubstitution m_AccSubstitution = EPSG_AccSubstitution::Default;
+    ESwitch m_AutoBlobSkipping = ESwitch::eDefault;
 };
 
 
@@ -279,11 +342,11 @@ public:
 
 private:
     string x_GetType() const override { return "resolve"; }
-    string x_GetId() const override { return GetBioId().Get(); }
+    string x_GetId() const override { return GetBioId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
     CPSG_BioId    m_BioId;
-    TIncludeInfo  m_IncludeInfo = 0;
+    TIncludeInfo  m_IncludeInfo = TIncludeInfo(0);
     EPSG_AccSubstitution m_AccSubstitution = EPSG_AccSubstitution::Default;
 };
 
@@ -296,17 +359,14 @@ class CPSG_Request_Blob : public CPSG_Request
 {
 public:
     /// 
-    CPSG_Request_Blob(CPSG_BlobId      blob_id,
-                      string           last_modified = {},
-                      shared_ptr<void> user_context = {},
+    CPSG_Request_Blob(CPSG_BlobId           blob_id,
+                      shared_ptr<void>      user_context = {},
                       CRef<CRequestContext> request_context = {})
-        : CPSG_Request(user_context, request_context),
-          m_BlobId(blob_id),
-          m_LastModified(last_modified)
+        : CPSG_Request(move(user_context), move(request_context)),
+          m_BlobId(move(blob_id))
     {}
 
     const CPSG_BlobId& GetBlobId()       const { return m_BlobId; }
-    const string&      GetLastModified() const { return m_LastModified; }
 
     /// Specify which data is needed (info is always returned)
     using EIncludeData = CPSG_Request_Biodata::EIncludeData;
@@ -316,11 +376,10 @@ public:
 
 private:
     string x_GetType() const override { return "blob"; }
-    string x_GetId() const override { return GetBlobId().Get(); }
+    string x_GetId() const override { return GetBlobId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
     CPSG_BlobId  m_BlobId;
-    string       m_LastModified;
     EIncludeData m_IncludeData = EIncludeData::eDefault;
 };
 
@@ -355,51 +414,46 @@ public:
     /// Set substitution policy for version-less primary seq-ids
     void SetAccSubstitution(EPSG_AccSubstitution acc_substitution) { m_AccSubstitution = acc_substitution; }
 
+    /// Specify which data is needed (info is always returned)
+    using EIncludeData = CPSG_Request_Biodata::EIncludeData;
+    void IncludeData(EIncludeData include) { m_IncludeData = include; }
+
+    EIncludeData GetIncludeData() const { return m_IncludeData; }
+
 private:
     string x_GetType() const override { return "annot"; }
-    string x_GetId() const override { return GetBioId().Get(); }
+    string x_GetId() const override { return GetBioId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
     CPSG_BioId  m_BioId;
     TAnnotNames m_AnnotNames;
     EPSG_AccSubstitution m_AccSubstitution = EPSG_AccSubstitution::Default;
+    EIncludeData m_IncludeData = EIncludeData::eDefault;
 };
 
 
 
-/// Request to the PSG server
-/// (by TSE blob-id and chunk number, for a particular version of split)
+/// Request blob data chunk
 ///
 
-class CPSG_Request_TSE_Chunk : public CPSG_Request
+class CPSG_Request_Chunk : public CPSG_Request
 {
 public:
-    using TChunkNo = unsigned;
-    using TSplitVersion = int;
-
-    CPSG_Request_TSE_Chunk(CPSG_BlobId      tse_blob_id,
-                           TChunkNo         chunk_no,
-                           TSplitVersion    split_version,
-                           shared_ptr<void> user_context = {},
-                           CRef<CRequestContext> request_context = {})
-        : CPSG_Request(user_context, request_context),
-          m_TSE_BlobId(tse_blob_id),
-          m_ChunkNo(chunk_no),
-          m_SplitVersion(split_version)
+    CPSG_Request_Chunk(CPSG_ChunkId          chunk_id,
+                       shared_ptr<void>      user_context = {},
+                       CRef<CRequestContext> request_context = {})
+        : CPSG_Request(move(user_context), move(request_context)),
+          m_ChunkId(move(chunk_id))
     {}
 
-    const CPSG_BlobId&  GetTSE_BlobId()   const { return m_TSE_BlobId;   }
-    TChunkNo            GetChunkNo()      const { return m_ChunkNo;      }
-    TSplitVersion       GetSplitVersion() const { return m_SplitVersion; }
+    const CPSG_ChunkId& GetChunkId() const { return m_ChunkId; }
 
 private:
-    string x_GetType() const override { return "tse_chunk"; }
-    string x_GetId() const override { return GetTSE_BlobId().Get(); }
+    string x_GetType() const override { return "chunk"; }
+    string x_GetId() const override { return GetChunkId().Repr(); }
     void x_GetAbsPathRef(ostream&) const override;
 
-    CPSG_BlobId    m_TSE_BlobId;
-    TChunkNo       m_ChunkNo;
-    TSplitVersion  m_SplitVersion;
+    CPSG_ChunkId m_ChunkId;
 };
 
 
@@ -441,6 +495,8 @@ public:
         eSkippedBlob,
         eBioseqInfo,
         eNamedAnnotInfo,
+        ePublicComment,
+        eProcessor,
         eEndOfReply,    ///< No more items expected in the (overall!) reply
     };
 
@@ -459,6 +515,9 @@ public:
     /// Get the reply that contains this item
     shared_ptr<CPSG_Reply> GetReply() const { return m_Reply; }
 
+    /// Get processor ID
+    const string& GetProcessorId() { return m_ProcessorId; }
+
     virtual ~CPSG_ReplyItem();
 
 protected:
@@ -469,28 +528,30 @@ private:
     unique_ptr<SImpl>      m_Impl;
     shared_ptr<CPSG_Reply> m_Reply;
     const EType            m_Type;
+    string                 m_ProcessorId;
 
     friend class CPSG_Reply;
 };
 
 
 
-/// Data blob.
+/// Blob data.
 
 class CPSG_BlobData : public CPSG_ReplyItem
 {
 public:
-    /// Get blob ID
-    const CPSG_BlobId& GetId() const { return m_Id; }
+    /// Get data ID
+    template <class TDataId = CPSG_DataId>
+    const TDataId* GetId() const { return dynamic_cast<const TDataId*>(m_Id.get()); }
 
     /// Get the stream from which to read the item's content.
     /// @note  If no content, then reading from the stream will result in EOF.
     istream& GetStream() const { return *m_Stream; }
 
 private:
-    CPSG_BlobData(CPSG_BlobId id);
+    CPSG_BlobData(unique_ptr<CPSG_DataId> id);
 
-    CPSG_BlobId         m_Id;
+    unique_ptr<CPSG_DataId> m_Id;
     unique_ptr<istream> m_Stream;
 
     friend class CPSG_Reply;
@@ -498,76 +559,66 @@ private:
 
 
 
-/// Data blob meta information
-///
-/// @note  Most of the data comes from table "BIOSEQ_BLOB" or "ANNOT_BLOB".
+/// Blob data meta information
 
 class CPSG_BlobInfo : public CPSG_ReplyItem
 {
 public:
-    /// Get blob ID
-    const CPSG_BlobId& GetId() const { return m_Id; }
+    /// Get data ID
+    template <class TDataId = CPSG_DataId>
+    const TDataId* GetId() const { return dynamic_cast<const TDataId*>(m_Id.get()); }
 
     /// Get data compression algorithm: gzip, bzip2, zip, compress, nlmzip, ...
-    /// Return empty string if the blob is not compressed
+    /// Return empty string if the blob data is not compressed
     string GetCompression() const;
 
     /// Get data serialization format:  asn.1, asn1-text, json, xml, ...
     string GetFormat() const;
 
-    /// Get blob version (the larger the version the fresher the blob data)
-    Uint8 GetVersion() const;
-
-    /// Get size of the blob (as it is stored)
+    /// Get size of the blob data (as it is stored)
     Uint8 GetStorageSize() const;
 
     /// Get size of the real (before any compression or encryption) blob data
     Uint8 GetSize() const;
 
-    /// Return TRUE if the blob is "dead"
+    /// Return TRUE if the blob data is "dead"
     bool IsDead() const;
 
-    /// Return TRUE if the blob is "suppressed"
+    /// Return TRUE if the blob data is "suppressed"
     bool IsSuppressed() const;
 
-    /// Return TRUE if the blob is "withdrawn"
+    /// Return TRUE if the blob data is "withdrawn"
     bool IsWithdrawn() const;
 
-    /// Date when the blob will be released for public use.
-    /// If the blob is already released, then return "empty" (IsEmpty()) time
+    /// Date when the blob data will be released for public use.
+    /// If the blob data is already released, then return "empty" (IsEmpty()) time
     CTime GetHupReleaseDate() const;
 
-    /// Blob owner's ID
+    /// Blob data owner's ID
     Uint8 GetOwner() const;
 
-    /// Date when the blob was first loaded into the database
+    /// Date when the blob data was first loaded into the database
     CTime GetOriginalLoadDate() const;
 
-    /// Class of this blob
+    /// Class of this blob data
     objects::CBioseq_set::EClass GetClass() const;
 
     /// Internal division value (used by various dumpers)
     string GetDivision() const;
 
-    /// Name of the user who loaded this blob
+    /// Name of the user who loaded this blob data
     string GetUsername() const;
 
-    /// Get coordinates of the blob that contains the specified ID2 split info.
-    /// If the blob is not split, then return an empty blob id.
-    CPSG_BlobId GetSplitInfoBlobId() const;
+    /// Get ID2 info
+    string GetId2Info() const;
 
-    /// Get coordinates of a chunk blob -- from the chunk's serial number.
-    /// @throw  If the blob has not been splitted.
-    CPSG_BlobId GetChunkBlobId(unsigned split_chunk_no) const;
-
-    /// Return ID2 split version (or zero, if not available)
-    using TSplitVersion = int;
-    TSplitVersion GetSplitVersion() const;
+    /// Get number of chunks
+    Uint8 GetNChunks() const;
 
 private:
-    CPSG_BlobInfo(CPSG_BlobId id);
+    CPSG_BlobInfo(unique_ptr<CPSG_DataId> id);
 
-    CPSG_BlobId m_Id;
+    unique_ptr<CPSG_DataId> m_Id;
     CJsonNode m_Data;
 
     friend class CPSG_Reply;
@@ -692,15 +743,15 @@ public:
     /// Name of the annotation
     const string& GetName() const { return m_Name; }
 
+    /// Annotated bio-id
+    CPSG_BioId GetAnnotatedId() const;
+
     /// Range where the feature(s) from this NA appear on the bio-sequence
     CRange<TSeqPos> GetRange() const;
 
     /// Coordinates of the blob that contains the NA data
     CPSG_BlobId GetBlobId() const;
 
-    /// Get NA version (the larger the version the fresher the NA data)
-    Uint8 GetVersion() const;
-
     /// Available zoom levels
     using TZoomLevel  = unsigned int;
     using TZoomLevels = vector<TZoomLevel>;
@@ -719,6 +770,17 @@ public:
     using TAnnotInfoList = list<SAnnotInfo>;
     TAnnotInfoList GetAnnotInfoList() const;
 
+    /// Base64 encoded asn.1 of ID2-Seq-annot-Info
+    string GetId2AnnotInfo() const;
+
+    /// Detailed ID2-Seq-annot-Info structures (from GetId2AnnotInfo, decoded)
+    /// @sa GetId2AnnotInfo
+    /// @{
+    using TId2AnnotInfo = objects::CID2S_Seq_annot_Info;
+    using TId2AnnotInfoList = list<CRef<TId2AnnotInfo>>;
+    TId2AnnotInfoList GetId2AnnotInfoList() const;
+    /// @}
+    
 private:
     CPSG_NamedAnnotInfo(string name);
 
@@ -730,6 +792,29 @@ private:
 
 
 
+/// Public comment
+
+class CPSG_PublicComment : public CPSG_ReplyItem
+{
+public:
+    /// Get data ID for this public comment
+    template <class TDataId = CPSG_DataId>
+    const TDataId* GetId() const { return dynamic_cast<const TDataId*>(m_Id.get()); }
+
+    /// Get text
+    const string& GetText() const { return m_Text; }
+
+private:
+    CPSG_PublicComment(unique_ptr<CPSG_DataId> id, string text);
+
+    unique_ptr<CPSG_DataId> m_Id;
+    string m_Text;
+
+    friend class CPSG_Reply;
+};
+
+
+
 /// PSG reply -- corresponds to a PSG request. It is used to retrieve data 
 /// (accession resolution; bio-sequence; annotation blobs) from the storage.
 /// 
@@ -867,6 +952,10 @@ public:
     bool IsInitialized() const { return static_cast<bool>(m_Impl); }
 
 
+    /// Is the queue in a state (possibly temporary) when requests get immediately rejected.
+    bool RejectsRequests() const;
+
+
     /// Get an API lock.
     /// Holding this API lock is essential if numerous short-lived queue instances are used.
     /// It prevents an internal I/O implementation (threads, TCP connections, HTTP sessions, etc)
