@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___FASTA__HPP
 #define OBJTOOLS_READERS___FASTA__HPP
 
-/*  $Id: fasta.hpp 612524 2020-07-23 11:37:59Z ivanov $
+/*  $Id: fasta.hpp 632526 2021-06-02 17:25:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -103,8 +103,8 @@ public:
         fAddMods              = 1<<17, ///< Parse defline mods and add to SeqEntry
         fLetterGaps           = 1<<18, ///< Parse runs of Ns when splitting data
         fNoUserObjs           = 1<<19, ///< Don't save raw deflines in User-objects
-        fBadModThrow  NCBI_STD_DEPRECATED("This flag is redundant as CFastaReader no longer utilizes CSourceModParser.")    = 1<<20, 
-        fUnknModThrow NCBI_STD_DEPRECATED("This flag is redundant as CFastaReader no longer utilizes CSourceModParser.")    = 1<<21, 
+        fBadModThrow  NCBI_STD_DEPRECATED("This flag is redundant as CFastaReader no longer utilizes CSourceModParser.")    = 1<<20,
+        fUnknModThrow NCBI_STD_DEPRECATED("This flag is redundant as CFastaReader no longer utilizes CSourceModParser.")    = 1<<21,
         fLeaveAsText          = 1<<22, ///< Don't reencode at all, just parse
         fQuickIDCheck         = 1<<23, ///< Just check local IDs' first characters
         fUseIupacaa           = 1<<24, ///< If Prot, use iupacaa instead of the default ncbieaa.
@@ -123,8 +123,8 @@ public:
     CFastaReader(ILineReader& reader, TFlags flags = 0, FIdCheck f_idcheck = CSeqIdCheck());
     CFastaReader(CNcbiIstream& in,    TFlags flags = 0, FIdCheck f_idcheck = CSeqIdCheck());
     CFastaReader(const string& path,  TFlags flags = 0, FIdCheck f_idcheck = CSeqIdCheck());
-    CFastaReader(CReaderBase::TReaderFlags fBaseFlags,  
-                 TFlags flags = 0, 
+    CFastaReader(CReaderBase::TReaderFlags fBaseFlags,
+                 TFlags flags = 0,
                  FIdCheck f_idcheck = CSeqIdCheck()
                  );
     virtual ~CFastaReader(void);
@@ -173,14 +173,6 @@ public:
     /// Re-allow previously seen IDs even if fUniqueIds is on.
     //void ResetIDTracker(void) { m_IDTracker.clear(); }
     void ResetIDTracker(void) { m_IDHandler->ClearIdCache(); }
-    
-    NCBI_DEPRECATED const CSourceModParser::TMods & GetUnusedMods(void) const { return m_UnusedMods; }
-    NCBI_DEPRECATED const CSourceModParser::TMods & GetBadMods(void) const { return m_BadMods; }
-    NCBI_DEPRECATED void  ClearBadMods(void) {}
-
-    NCBI_DEPRECATED void SetModFilter( CRef<CSourceModParser::CModFilter> /*pModFilter*/ ) {}
-
-    NCBI_DEPRECATED void SetModFilter(FModFilter) {}
 
     void SetExcludedMods(const vector<string>& excluded_mods);
 
@@ -205,11 +197,11 @@ public:
 
     void SetGapLinkageEvidence(
             CSeq_gap::EType type,
-            const set<int>& defaultEvidence, 
+            const set<int>& defaultEvidence,
             const map<TSeqPos, set<int>>& countToEvidenceMap);
 
     void SetGapLinkageEvidences(
-        CSeq_gap::EType type, 
+        CSeq_gap::EType type,
         const set<int>& evidences);
 
     void IgnoreProblem(ILineError::EProblem problem);
@@ -220,14 +212,14 @@ public:
     using TSeqTitles = vector<SLineTextAndLoc>;
     typedef CTempString TStr;
 
-    static void ParseDefLine(const TStr& defLine, 
+    static void ParseDefLine(const TStr& defLine,
         const SDefLineParseInfo& info,
         const TIgnoredProblems& ignoredErrors,
-        list<CRef<CSeq_id>>& ids, 
+        list<CRef<CSeq_id>>& ids,
         bool& hasRange,
         TSeqPos& rangeStart,
         TSeqPos& rangeEnd,
-        TSeqTitles& seqTitles, 
+        TSeqTitles& seqTitles,
         ILineErrorListener* pMessageListener);
 
 protected:
@@ -237,26 +229,14 @@ protected:
     };
 
 
-    NCBI_DEPRECATED virtual CRef<CSeq_entry> x_ReadSegSet(ILineErrorListener * pMessageListener);
-
     virtual void ParseDefLine (const TStr& s, ILineErrorListener * pMessageListener);
 
-    NCBI_STD_DEPRECATED("Superseded by CFastaDeflineReader::ParseIDs(). Not used by non-static CFastaReader::ParseDefLine().") 
-    virtual bool ParseIDs(const TStr& s, ILineErrorListener * pMessageListener);
-
-    virtual void PostProcessIDs(const CBioseq::TId& defline_ids, 
+    virtual void PostProcessIDs(const CBioseq::TId& defline_ids,
         const string& defline,
         bool has_range=false,
         TSeqPos range_start=kInvalidSeqPos,
         TSeqPos range_end=kInvalidSeqPos);
 
-    NCBI_STD_DEPRECATED("Superseded by CFastaDeflineReader::ParseIDs(). Not used by static CFastaReader::ParseDefLine().") 
-    static bool ParseIDs (const TStr& s, 
-        const SDefLineParseInfo& info,
-        const TIgnoredProblems& ignoredErrors,
-        list<CRef<CSeq_id>>& ids, 
-        ILineErrorListener* pMessageListener);
-
     static size_t ParseRange    (const TStr& s, TSeqPos& start, TSeqPos& end, ILineErrorListener * pMessageListener);
     virtual void   ParseTitle    (const SLineTextAndLoc & lineInfo, ILineErrorListener * pMessageListener);
     virtual bool   IsValidLocalID(const TStr& s) const;
@@ -271,20 +251,15 @@ protected:
     virtual void   AssembleSeq   (ILineErrorListener * pMessageListener);
     virtual void   AssignMolType (ILineErrorListener * pMessageListener);
     virtual bool   CreateWarningsForSeqDataInTitle(
-        const TStr& sLineText, 
+        const TStr& sLineText,
         TSeqPos iLineNum,
         ILineErrorListener * pMessageListener) const;
-    NCBI_STD_DEPRECATED("") static bool ExcessiveSeqDataInTitle(const string& title, 
-                                        TFlags fFastaFlags);
     virtual void   PostWarning(ILineErrorListener * pMessageListener,
-            EDiagSev _eSeverity, size_t _uLineNum, CTempString _MessageStrmOps, 
-            CObjReaderParseException::EErrCode _eErrCode, 
-            ILineError::EProblem _eProblem, 
+            EDiagSev _eSeverity, size_t _uLineNum, CTempString _MessageStrmOps,
+            CObjReaderParseException::EErrCode _eErrCode,
+            ILineError::EProblem _eProblem,
             CTempString _sFeature, CTempString _sQualName, CTempString _sQualValue) const;
 
-    NCBI_STD_DEPRECATED("CFastaReader no longer utilizes CSourceModParser.") 
-    CSourceModParser* xCreateSourceModeParser(ILineErrorListener* pErrorListener);
-
     typedef int                         TRowNum;
     typedef map<TRowNum, TSignedSeqPos> TSubMap;
     // align coord -> row -> seq coord
@@ -296,7 +271,7 @@ protected:
                                              TRowNum reference_row);
     void             x_AddMultiwayAlignment(CSeq_annot& annot, const TIds& ids);
 
-    // inline utilities 
+    // inline utilities
     void CloseGap(bool atStartOfLine=true, ILineErrorListener * pMessageListener = 0) {
         if (m_CurrentGapLength > 0) {
             x_CloseGap(m_CurrentGapLength, atStartOfLine, pMessageListener);
@@ -317,7 +292,7 @@ protected:
     bool          TestFlag(EInternalFlags flag) const
         { return (GetFlags() & flag) != 0; }
     CBioseq::TId& SetIDs(void)                { return m_CurrentSeq->SetId(); }
- 
+
     // NB: Not necessarily fully-formed!
     CBioseq& SetCurrentSeq(void)              { return *m_CurrentSeq; }
 
@@ -328,16 +303,12 @@ protected:
     };
     TSeqPos GetCurrentPos(EPosType pos_type);
 
-    NCBI_DEPRECATED void x_ApplyAllMods(CBioseq & /*bioseq*/, 
-        TSeqPos /*iLineNum*/, 
-        ILineErrorListener * /*pMessageListener*/) {}
-
     std::string x_NucOrProt(void) const;
 
-private: 
+private:
     CModHandler m_ModHandler;
 
-    void x_ApplyMods(const string& title, 
+    void x_ApplyMods(const string& title,
                      TSeqPos line_number,
                      CBioseq& bioseq,
                      ILineErrorListener* pMessageListener);
@@ -365,15 +336,15 @@ protected:
             EKnownSize eKnownSize,
             TSeqPos uLineNumber,
             TNullableGapType pGapType = TNullableGapType(),
-            const set<CLinkage_evidence::EType>& setOfLinkageEvidence = 
+            const set<CLinkage_evidence::EType>& setOfLinkageEvidence =
                 set<CLinkage_evidence::EType>());
         // immutable once created
 
         // 0-based, and NOT counting previous gaps
-        const TSeqPos m_uPos; 
+        const TSeqPos m_uPos;
         // 0: unknown, negative: negated nominal length of unknown gap size.
         // positive: known gap size
-        const TSeqPos m_uLen; 
+        const TSeqPos m_uLen;
         const EKnownSize m_eKnownSize;
         const TSeqPos m_uLineNumber;
         // NULL means "no gap type specified"
@@ -385,9 +356,9 @@ protected:
         SGap & operator = (const SGap & );
         SGap(const SGap & rhs);
     };
-        
-    CRef<CSeq_align> xCreateAlignment(CRef<CSeq_id> old_id, 
-        CRef<CSeq_id> new_id, 
+
+    CRef<CSeq_align> xCreateAlignment(CRef<CSeq_id> old_id,
+        CRef<CSeq_id> new_id,
         TSeqPos range_start,
         TSeqPos range_end);
 
@@ -396,7 +367,6 @@ protected:
     typedef CRef<SGap> TGapRef;
     typedef vector<TGapRef>     TGaps;
     typedef set<CSeq_id_Handle> TIDTracker;
-   
 
     CRef<ILineReader>       m_LineReader;
     stack<TFlags>           m_Flags;
@@ -404,12 +374,10 @@ protected:
     TMask                   m_CurrentMask;
     TMask                   m_NextMask;
     TMasks *                m_MaskVec;
-    NCBI_DEPRECATED CRef<CSeqIdGenerator>   m_IDGenerator;
     CRef<CFastaIdHandler>   m_IDHandler;
     string                  m_SeqData;
     TGaps                   m_Gaps;
     TSeqPos                 m_CurrentPos; // does not count gaps
-    NCBI_DEPRECATED TSeqPos m_ExpectedEnd;
     TSeqPos                 m_MaskRangeStart;
     TSeqPos                 m_SegmentBase;
     TSeqPos                 m_CurrentGapLength;
@@ -430,7 +398,6 @@ protected:
 
     using TCountToLinkEvidMap = map<TSeqPos, SGap::TLinkEvidSet>;
     TCountToLinkEvidMap     m_GapsizeToLinkageEvidence;
-    NCBI_STD_DEPRECATED("") SGap::TLinkEvidSet      m_gap_linkage_evidence;
 private:
     SGap::TLinkEvidSet      m_DefaultLinkageEvidence;
 protected:
@@ -443,37 +410,13 @@ protected:
 };
 
 
-enum NCBI_STD_DEPRECATED("") EReadFastaFlags {
-    fReadFasta_AssumeNuc  = CFastaReader::fAssumeNuc,
-    fReadFasta_AssumeProt = CFastaReader::fAssumeProt,
-    fReadFasta_ForceType  = CFastaReader::fForceType,
-    fReadFasta_NoParseID  = CFastaReader::fNoParseID,
-    fReadFasta_ParseGaps  = CFastaReader::fParseGaps,
-    fReadFasta_OneSeq     = CFastaReader::fOneSeq,
-//    fReadFasta_AllSeqIds  = CFastaReader::fAllSeqIds,
-    fReadFasta_NoSeqData  = CFastaReader::fNoSeqData,
-    fReadFasta_RequireID  = CFastaReader::fRequireID
-};
 typedef CFastaReader::TFlags TReadFastaFlags;
 
 
-/// Traditional interface for reading FASTA files which is not
-/// just deprecated but dangerous due to an unsafe reinterpret_cast that
-/// occurs within and affects the caller's params.
-///
-/// @deprecated
-/// @sa CFastaReader
-NCBI_DEPRECATED
-NCBI_XOBJREAD_EXPORT
-CRef<CSeq_entry> ReadFasta(CNcbiIstream& in, TReadFastaFlags flags = 0,
-                           int* counter = 0,
-                           vector<CConstRef<CSeq_loc> >* lcv = 0,
-                           ILineErrorListener * pMessageListener = 0);
-
 /// A const-correct replacement for the deprecated ReadFasta function
 /// @sa CFastaReader
 NCBI_XOBJREAD_EXPORT
-CRef<CSeq_entry> ReadFasta(CNcbiIstream& in, TReadFastaFlags flags=0,
+CRef<CSeq_entry> ReadFasta(CNcbiIstream& in, CFastaReader::TFlags flags=0,
                            int* counter = nullptr,
                            CFastaReader::TMasks* lcv=nullptr,
                            ILineErrorListener* pMessageListener=nullptr);
@@ -509,7 +452,7 @@ public:
     virtual ~IFastaEntryScan();
 
     /// Callback function, called after reading the fasta entry
-    virtual void EntryFound(CRef<CSeq_entry> se, 
+    virtual void EntryFound(CRef<CSeq_entry> se,
                             CNcbiStreampos stream_position) = 0;
 };
 
@@ -518,13 +461,13 @@ public:
 /// molecule entry after another filling the map structure describing and
 /// pointing on molecule entries. Fasta map can be used later for quick
 /// CSeq_entry retrival
-void NCBI_XOBJREAD_EXPORT ReadFastaFileMap(SFastaFileMap* fasta_map, 
+void NCBI_XOBJREAD_EXPORT ReadFastaFileMap(SFastaFileMap* fasta_map,
                                            CNcbiIfstream& input);
 
 /// Scan FASTA files, call IFastaEntryScan::EntryFound (payload function)
-void NCBI_XOBJREAD_EXPORT ScanFastaFile(IFastaEntryScan* scanner, 
+void NCBI_XOBJREAD_EXPORT ScanFastaFile(IFastaEntryScan* scanner,
                                         CNcbiIfstream&   input,
-                                        TReadFastaFlags  fread_flags);
+                                        CFastaReader::TFlags fread_flags);
 
 
 /////////////////// CFastaReader inline methods
