@@ -1,4 +1,4 @@
- /*  $Id: gtf_reader.hpp 610647 2020-06-22 11:31:17Z ivanov $
+ /*  $Id: gtf_reader.hpp 632526 2021-06-02 17:25:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,8 @@ BEGIN_NCBI_SCOPE
 
 BEGIN_SCOPE(objects) // namespace ncbi::objects::
 
+class CGtfLocationMerger;
+
 //  ============================================================================
 class CGtfAttributes
 //  ============================================================================
@@ -123,11 +125,12 @@ class CGtfReadRecord
     : public CGff2Record
 {
 public:
-    CGtfReadRecord(): CGff2Record() {};
+    CGtfReadRecord(): CGff2Record() {
+    };
     ~CGtfReadRecord() {};
 
     const CGtfAttributes&
-    GtfAttributes() const 
+    GtfAttributes() const
     {
         return mAttributes;
     };
@@ -175,35 +178,35 @@ public:
         fGenerateChildXrefs = 1<<8,
     };
 
-    CGtfReader( 
-        unsigned int =0, 
-        const string& = "", 
+    CGtfReader(
+        unsigned int =0,
+        const string& = "",
         const string& = "",
         SeqIdResolver = CReadUtil::AsSeqId,
         CReaderListener* = nullptr);
 
-    CGtfReader( 
-        unsigned int, 
+    CGtfReader(
+        unsigned int,
         CReaderListener*);
 
     virtual ~CGtfReader();
-    
-    virtual CRef< CSeq_annot >
+
+    CRef< CSeq_annot >
     ReadSeqAnnot(
         ILineReader& lr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors=nullptr) override;
 
 protected:
-    virtual void xProcessData(
+    void xProcessData(
         const TReaderData&,
-        CSeq_annot&);
+        CSeq_annot&) override;
 
-    virtual CGff2Record* x_CreateRecord() { return new CGtfReadRecord(); };    
+    CGff2Record* x_CreateRecord() override { return new CGtfReadRecord(); }
 
-    virtual bool xUpdateAnnotFeature(
+    bool xUpdateAnnotFeature(
         const CGff2Record&,
         CSeq_annot&,
-        ILineErrorListener* =0);
+        ILineErrorListener* =nullptr) override;
 
     virtual bool xUpdateAnnotCds(
         const CGtfReadRecord&,
@@ -213,43 +216,18 @@ protected:
         const CGtfReadRecord&,
         CSeq_annot&);
 
+    void xPostProcessAnnot(
+        CSeq_annot&) override;
+
     bool xCreateFeatureId(
         const CGtfReadRecord&,
         const string&,
         CSeq_feat&);
 
-    bool xCreateFeatureLocation(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-    
-    bool xCreateGeneXrefs(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-    
-    bool xCreateMrnaXrefs(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-    
-    bool xCreateCdsXrefs(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-    
-    bool xMergeFeatureLocationSingleInterval(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-    
-    bool xMergeFeatureLocationMultiInterval(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-
     bool xCreateParentGene(
         const CGtfReadRecord&,
         CSeq_annot&);
-        
-    bool xMergeParentGene(
-        const CGtfReadRecord&,
-        CSeq_feat&);
-            
+
     bool xFeatureSetQualifiersGene(
         const CGtfReadRecord& record,
         CSeq_feat&);
@@ -265,11 +243,11 @@ protected:
     bool xCreateParentCds(
         const CGtfReadRecord&,
         CSeq_annot&);
-        
+
     bool xCreateParentMrna(
         const CGtfReadRecord&,
         CSeq_annot&);
-        
+
     bool xFeatureSetDataGene(
         const CGtfReadRecord&,
         CSeq_feat&);
@@ -291,14 +269,8 @@ protected:
         const CGtfReadRecord&,
         CSeq_feat&);
 
-    CRef<CSeq_feat> xFindParentGene(
-        const CGtfReadRecord&);
-
-    CRef<CSeq_feat> xFindParentCds(
-        const CGtfReadRecord&);
-
-    CRef<CSeq_feat> xFindParentMrna(
-        const CGtfReadRecord&);
+    CRef<CSeq_feat> xFindFeatById(
+        const string&);
 
     bool xProcessQualifierSpecialCase(
         const string&,
@@ -310,16 +282,11 @@ protected:
         const CGtfAttributes::MultiValue&,
         CSeq_feat&);
 
-    NCBI_DEPRECATED bool xCdsIsPartial(
-        const CGtfReadRecord& );
+    void xSetAncestorXrefs(
+        CSeq_feat&,
+        CSeq_feat&) override;
 
-    typedef map< string, CRef< CSeq_feat > > TIdToFeature;
-    TIdToFeature m_GeneMap;
-    TIdToFeature m_CdsMap;
-    TIdToFeature m_MrnaMap;
-
-    typedef map<string, string> TGeneIdToLocusTag;
-    TGeneIdToLocusTag m_LocusTags;
+    unique_ptr<CGtfLocationMerger> mpLocations;
 };
 
 END_SCOPE(objects)
