@@ -1,7 +1,7 @@
 #ifndef SERIAL_GRPC_INTEGRATION___GRPC_INTEGRATION__HPP
 #define SERIAL_GRPC_INTEGRATION___GRPC_INTEGRATION__HPP
 
-/*  $Id: grpc_integration.hpp 608310 2020-05-14 12:35:38Z ivanov $
+/*  $Id: grpc_integration.hpp 609619 2020-06-04 14:14:58Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 
 
 #include <serial/grpc_integration/impl/grpc_support.hpp>
+#include <corelib/ncbi_param.hpp>
 #ifdef HAVE_LIBGRPC
 #  include <grpc++/client_context.h>
 #else
@@ -92,6 +93,14 @@ public:
 
 /// BEGIN_NCBI_GRPC_REQUEST -- Set up AppLog logging for the current
 /// (server-side) request handler.
+///
+/// The third argument may be either a reply google::protobuf::Message
+/// or a streaming grpc::ServerWriter<>.  In the case of a reply
+/// message, automatic wrap-up actions will include propagating its
+/// byte size to the request context's count of bytes written.  In the
+/// case of a streaming writer, however, there's no automatic way to
+/// determine that count, so callers are responsible for supplying it
+/// themselves.
 #define BEGIN_NCBI_GRPC_REQUEST(sctx, request, reply) \
     CGRPCRequestLogger request_logger##__LINE__       \
         (sctx, NCBI_CURRENT_FUNCTION, request, reply)
@@ -120,7 +129,7 @@ public:
 string g_NCBI_GRPC_GetAddress(const char* section,
                               const char* variable,
                               const char* env_var_name = nullptr,
-                              int* value_source = nullptr);
+                              CParamBase::EParamSource* value_source = nullptr);
 
 END_NCBI_SCOPE
 
