@@ -1,7 +1,7 @@
 #ifndef SERIAL_GRPC_INTEGRATION_IMPL___GRPC_SUPPORT__HPP
 #define SERIAL_GRPC_INTEGRATION_IMPL___GRPC_SUPPORT__HPP
 
-/*  $Id: grpc_support.hpp 608345 2020-05-14 18:36:54Z ivanov $
+/*  $Id: grpc_support.hpp 623780 2021-01-19 20:24:54Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -73,6 +73,11 @@ namespace grpc {
 }
 #endif
 
+namespace grpc_impl {
+    namespace internal {
+        class ServerStreamingInterface;
+    }
+}
 
 /** @addtogroup Miscellaneous
  *
@@ -98,13 +103,19 @@ public:
     typedef google::protobuf::Message TMessage;
     CGRPCRequestLogger(TGRPCServerContext* sctx, CTempString method_name,
                        const TMessage& request, const TMessage& reply);
+    CGRPCRequestLogger(TGRPCServerContext* sctx, CTempString method_name,
+                       const TMessage& request,
+                       const grpc_impl::internal::ServerStreamingInterface&);
     ~CGRPCRequestLogger();
 
 private:
+    void x_Init(TGRPCServerContext* sctx, CTempString method_name,
+                const TMessage& request);
+    
     CDiagContext&    m_DiagContext;
     CRequestContext& m_RequestContext;
 #ifdef HAVE_LIBGRPC // HAVE_LIBPROTOBUF
-    const TMessage&  m_Reply;
+    const TMessage*  m_Reply;
 #endif
     bool             m_ManagingRequest;
 };
@@ -170,9 +181,31 @@ CGRPCRequestLogger::CGRPCRequestLogger(TGRPCServerContext* sctx,
     : m_DiagContext(GetDiagContext()),
       m_RequestContext(m_DiagContext.GetRequestContext()),
 #ifdef HAVE_LIBGRPC // HAVE_LIBPROTOBUF
-      m_Reply(reply),
+      m_Reply(&reply),
 #endif
       m_ManagingRequest(false)
+{
+    x_Init(sctx, method_name, request);
+}
+
+inline
+CGRPCRequestLogger::CGRPCRequestLogger(
+    TGRPCServerContext* sctx, CTempString method_name, const TMessage& request,
+    const grpc_impl::internal::ServerStreamingInterface&)
+    : m_DiagContext(GetDiagContext()),
+      m_RequestContext(m_DiagContext.GetRequestContext()),
+#ifdef HAVE_LIBGRPC // HAVE_LIBPROTOBUF
+      m_Reply(nullptr),
+#endif
+      m_ManagingRequest(false)
+{
+    x_Init(sctx, method_name, request);
+}
+
+inline
+void CGRPCRequestLogger::x_Init(TGRPCServerContext* sctx,
+                                CTempString method_name,
+                                const TMessage& request)
 {
     if (m_DiagContext.GetAppState() < eDiagAppState_RequestBegin) {
         m_ManagingRequest = true;
@@ -190,7 +223,11 @@ CGRPCRequestLogger::~CGRPCRequestLogger()
         m_RequestContext.SetRequestStatus
             (CRequestStatus::e500_InternalServerError);
     }
-    m_RequestContext.SetBytesWr(NCBI_GRPC_GET_BYTE_SIZE(m_Reply));
+#ifdef HAVE_LIBGRPC // HAVE_LIBPROTOBUF
+    if (m_Reply != nullptr) {
+        m_RequestContext.SetBytesWr(NCBI_GRPC_GET_BYTE_SIZE(*m_Reply));
+    }
+#endif
     if (m_ManagingRequest) {
         m_DiagContext.PrintRequestStop();
     }
