@@ -1,7 +1,7 @@
 #ifndef SERIAL___OBJISTR__HPP
 #define SERIAL___OBJISTR__HPP
 
-/*  $Id: objistr.hpp 574303 2018-11-08 19:59:04Z gouriano $
+/*  $Id: objistr.hpp 607093 2020-04-30 12:29:36Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -776,10 +776,17 @@ public:
 
     virtual void SkipPointer(TTypeInfo declaredType);
 
-#ifdef NCBI_STRICT_GI
-    void ReadStd(TGi& data);
-    void SkipStd(TGi& data);
+    template<class TKey, class TStorage> void ReadStd(CStrictId<TKey, TStorage>& data)
+    {
+        ReadStd(data.Set());
+    }
 
+    template<class TKey, class TStorage> void SkipStd(CStrictId<TKey, TStorage>& data)
+    {
+        SkipStd(data.Set());
+    }
+
+#ifdef NCBI_STRICT_GI
     virtual void ReadGi(TGi& obj);
     virtual void SkipGi(void);
 #endif
