@@ -179,10 +179,18 @@ public:
 
         \return true if anything was found
 
-        @sa add, reset
+        @sa add, reset, find_first_and_sub
     */
     bool combine_and_sub(bvector_type& bv_target, bool any);
-    
+
+    /**
+        Aggregate added group of vectors using fused logical AND-SUB,
+        find the first match
+
+        \param idx - [out] index of the first occurence
+        \return true if anything was found
+        @sa combine_and_sub
+     */
     bool find_first_and_sub(size_type& idx);
 
 
@@ -883,7 +891,9 @@ bool aggregator<BV>::find_first_and_sub(size_type& idx,
             {
                 unsigned set_array_max2 =
                         find_effective_sub_block_size(i, bv_src_sub, src_sub_size, false);
-                if (set_array_max2 > set_array_max)
+                // TODO: should it be set_array_max2 < set_array_max ????
+                //if (set_array_max2 > set_array_max)
+                if (set_array_max2 < set_array_max)
                     set_array_max = set_array_max2;
             }
         }
@@ -1959,6 +1969,5 @@ aggregator<BV>::run_step(unsigned i, unsigned j)
 
 } // bm
 
-#include "bmundef.h"
 
 #endif
