@@ -59,10 +59,10 @@ public:
     public:
         typedef id_type size_type;
         
-        bm_func_base(blocks_manager& bman) : bm_(bman) {}
+        bm_func_base(blocks_manager& bman) BMNOEXCEPT : bm_(bman) {}
 
-        void on_empty_top(unsigned /* top_block_idx*/ ) {}
-        void on_empty_block(block_idx_type /* block_idx*/ ) {}
+        void on_empty_top(unsigned /* top_block_idx*/ ) BMNOEXCEPT {}
+        void on_empty_block(block_idx_type /* block_idx*/ )BMNOEXCEPT {}
     private:
         bm_func_base(const bm_func_base&);
         bm_func_base& operator=(const bm_func_base&);
@@ -76,13 +76,13 @@ public:
     {
     public:
         typedef id_type size_type;
-        bm_func_base_const(const blocks_manager& bman) : bm_(bman) {}
+        bm_func_base_const(const blocks_manager& bman) BMNOEXCEPT : bm_(bman) {}
 
-        void on_empty_top(unsigned /* top_block_idx*/ ) {}
-        void on_empty_block(block_idx_type /* block_idx*/ ) {}
+        void on_empty_top(unsigned /* top_block_idx*/ ) BMNOEXCEPT {}
+        void on_empty_block(block_idx_type /* block_idx*/ ) BMNOEXCEPT {}
     private:
-        bm_func_base_const(const bm_func_base_const&);
-        bm_func_base_const& operator=(const bm_func_base_const&);
+        bm_func_base_const(const bm_func_base_const&) BMNOEXCEPT;
+        bm_func_base_const& operator=(const bm_func_base_const&) BMNOEXCEPT;
     protected:
         const blocks_manager&  bm_;
     };
@@ -92,10 +92,10 @@ public:
     class block_count_base : public bm_func_base_const
     {
     protected:
-        block_count_base(const blocks_manager& bm) 
+        block_count_base(const blocks_manager& bm) BMNOEXCEPT
             : bm_func_base_const(bm) {}
 
-        bm::id_t block_count(const bm::word_t* block) const
+        bm::id_t block_count(const bm::word_t* block) const BMNOEXCEPT
         {
             return this->bm_.block_bitcount(block);
         }
@@ -108,17 +108,17 @@ public:
     public:
         typedef id_type size_type;
 
-        block_count_func(const blocks_manager& bm) 
+        block_count_func(const blocks_manager& bm) BMNOEXCEPT
             : block_count_base(bm), count_(0) {}
 
-        id_type count() const { return count_; }
+        id_type count() const BMNOEXCEPT { return count_; }
 
-        void operator()(const bm::word_t* block)
+        void operator()(const bm::word_t* block) BMNOEXCEPT
         {
             count_ += this->block_count(block);
         }
-        void add_full(id_type c) { count_ += c; }
-        void reset() { count_ = 0; }
+        void add_full(id_type c) BMNOEXCEPT { count_ += c; }
+        void reset() BMNOEXCEPT { count_ = 0; }
 
     private:
         id_type count_;
@@ -131,24 +131,22 @@ public:
     public:
         typedef id_type size_type;
 
-        block_count_arr_func(const blocks_manager& bm, unsigned* arr) 
+        block_count_arr_func(const blocks_manager& bm, unsigned* arr) BMNOEXCEPT
             : block_count_base(bm), arr_(arr), last_idx_(0) 
         {
             arr_[0] = 0;
         }
 
-        void operator()(const bm::word_t* block, id_type idx)
+        void operator()(const bm::word_t* block, id_type idx) BMNOEXCEPT
         {
             while (++last_idx_ < idx)
-            {
                 arr_[last_idx_] = 0;
-            }
             arr_[idx] = this->block_count(block);
             last_idx_ = idx;
         }
 
-        id_type last_block() const { return last_idx_; }
-        void on_non_empty_top(unsigned) {}
+        id_type last_block() const BMNOEXCEPT { return last_idx_; }
+        void on_non_empty_top(unsigned) BMNOEXCEPT {}
 
     private:
         unsigned*  arr_;
@@ -161,13 +159,14 @@ public:
     public:
         typedef id_type size_type;
 
-        block_count_change_func(const blocks_manager& bm) 
+        block_count_change_func(const blocks_manager& bm) BMNOEXCEPT
             : bm_func_base_const(bm),
                 count_(0),
                 prev_block_border_bit_(0)
         {}
 
-        block_idx_type block_count(const bm::word_t* block, block_idx_type idx)
+        block_idx_type block_count(const bm::word_t* block,
+                                   block_idx_type idx) BMNOEXCEPT
         {
             block_idx_type cnt = 0;
             id_type first_bit;
@@ -187,7 +186,7 @@ public:
                 if (BM_IS_GAP(block))
                 {
                     gap_word_t* gap_block = BMGAP_PTR(block);
-                    cnt = gap_length(gap_block) - 1;
+                    cnt = bm::gap_length(gap_block) - 1;
                     if (idx)
                     {
                         first_bit = bm::gap_test_unr(gap_block, 0);
@@ -213,9 +212,9 @@ public:
             return cnt;
         }
         
-        id_type count() const { return count_; }
+        id_type count() const BMNOEXCEPT { return count_; }
 
-        void operator()(const bm::word_t* block, block_idx_type idx)
+        void operator()(const bm::word_t* block, block_idx_type idx) BMNOEXCEPT
         {
             count_ += block_count(block, idx);
         }
@@ -232,11 +231,12 @@ public:
     public:
         typedef id_type size_type;
 
-        block_any_func(const blocks_manager& bm) 
+        block_any_func(const blocks_manager& bm) BMNOEXCEPT
             : bm_func_base_const(bm) 
         {}
 
-        bool operator()(const bm::word_t* block, block_idx_type /*idx*/)
+        bool operator()
+                (const bm::word_t* block, block_idx_type /*idx*/) BMNOEXCEPT
         {
             if (BM_IS_GAP(block)) // gap block
                 return (!gap_is_all_zero(BMGAP_PTR(block)));
@@ -250,9 +250,9 @@ public:
     class gap_level_func : public bm_func_base
     {
     public:
-        gap_level_func(blocks_manager& bm, const gap_word_t* glevel_len)
-            : bm_func_base(bm),
-                glevel_len_(glevel_len)
+        gap_level_func(blocks_manager& bm,
+                       const gap_word_t* glevel_len) BMNOEXCEPT
+            : bm_func_base(bm), glevel_len_(glevel_len)
         {
             BM_ASSERT(glevel_len);
         }
@@ -282,19 +282,18 @@ public:
                 return;
             }
 
-            unsigned len = gap_length(gap_blk);
-            int level = gap_calc_level(len, glevel_len_);
+            unsigned len = bm::gap_length(gap_blk);
+            int level = bm::gap_calc_level(len, glevel_len_);
             if (level == -1)
             {
-                bm::word_t* blk = 
-                    bman.get_allocator().alloc_bit_block();
+                bm::word_t* blk = bman.get_allocator().alloc_bit_block();
                 bman.set_block_ptr(idx, blk);
                 bm::gap_convert_to_bitset(blk, gap_blk);
             }
             else
             {
                 gap_word_t* gap_blk_new = 
-                    bman.allocate_gap_block(unsigned(level), gap_blk, glevel_len_);
+                bman.allocate_gap_block(unsigned(level), gap_blk, glevel_len_);
 
                 bm::word_t* p = (bm::word_t*) gap_blk_new;
                 BMSET_PTRGAP(p);
@@ -312,7 +311,7 @@ public:
     class block_one_func : public bm_func_base
     {
     public:
-        block_one_func(blocks_manager& bm) : bm_func_base(bm) {}
+        block_one_func(blocks_manager& bm) BMNOEXCEPT : bm_func_base(bm) {}
 
         void operator()(bm::word_t* block, block_idx_type idx)
         {
@@ -357,7 +356,7 @@ public:
     }
     
 #ifndef BM_NO_CXX11
-    blocks_manager(blocks_manager&& blockman) BMNOEXEPT
+    blocks_manager(blocks_manager&& blockman) BMNOEXCEPT
         : max_bits_(blockman.max_bits_),
           top_blocks_(0),
           top_block_size_(blockman.top_block_size_),
@@ -369,7 +368,7 @@ public:
     }
 #endif
 
-    ~blocks_manager() BMNOEXEPT
+    ~blocks_manager() BMNOEXCEPT
     {
         if (temp_block_)
             alloc_.free_bit_block(temp_block_);
@@ -379,7 +378,7 @@ public:
     /*! \brief Swaps content 
         \param bm  another blocks manager
     */
-    void swap(blocks_manager& bm) BMNOEXEPT
+    void swap(blocks_manager& bm) BMNOEXCEPT
     {
         BM_ASSERT(this != &bm);
 
@@ -399,7 +398,7 @@ public:
     
     /*! \brief implementation of moving semantics
     */
-    void move_from(blocks_manager& bm) BMNOEXEPT
+    void move_from(blocks_manager& bm) BMNOEXCEPT
     {
         deinit_tree();
         swap(bm);
@@ -412,9 +411,9 @@ public:
     }
     
 
-    void free_ptr(bm::word_t** ptr)
+    void free_ptr(bm::word_t** ptr) BMNOEXCEPT
     {
-        if (ptr) alloc_.free_ptr(ptr);
+        alloc_.free_ptr(ptr);
     }
 
     /**
@@ -422,7 +421,7 @@ public:
         \param bits_to_store - supposed capacity (number of bits)
         \return size of the top level block
     */
-    unsigned compute_top_block_size(id_type bits_to_store)
+    unsigned compute_top_block_size(id_type bits_to_store) const BMNOEXCEPT
     {
         if (bits_to_store >= bm::id_max)  // working in full-range mode
             return bm::set_top_array_size;
@@ -456,7 +455,8 @@ public:
         \param no_more_blocks - 1 if there are no more blocks at all
         \return block adress or NULL if not yet allocated
     */
-    bm::word_t* get_block(block_idx_type nb, int* no_more_blocks) const
+    const bm::word_t*
+    get_block(block_idx_type nb, int* no_more_blocks) const BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_);
         unsigned i = unsigned(nb >> bm::set_array_shift);
@@ -489,7 +489,7 @@ public:
     @return bm::set_total_blocks - no more blocks
     */
     block_idx_type
-    find_next_nz_block(block_idx_type nb, bool deep_scan = true) const
+    find_next_nz_block(block_idx_type nb, bool deep_scan=true) const BMNOEXCEPT
     {
         if (is_init())
         {
@@ -521,7 +521,7 @@ public:
         \param j - second level block index
         \return block adress or NULL if not yet allocated
     */
-    const bm::word_t* get_block(unsigned i, unsigned j) const
+    const bm::word_t* get_block(unsigned i, unsigned j) const BMNOEXCEPT
     {
         if (!top_blocks_ || i >= top_block_size_) return 0;
         const bm::word_t* const* blk_blk = top_blocks_[i];
@@ -537,7 +537,7 @@ public:
         \param j - second level block index
         \return block adress or NULL if not yet allocated
     */
-    const bm::word_t* get_block_ptr(unsigned i, unsigned j) const
+    const bm::word_t* get_block_ptr(unsigned i, unsigned j) const BMNOEXCEPT
     {
         if (!top_blocks_ || i >= top_block_size_) return 0;
 
@@ -553,9 +553,10 @@ public:
         \param j - second level block index
         \return block adress or NULL if not yet allocated
     */
-    bm::word_t* get_block_ptr(unsigned i, unsigned j)
+    bm::word_t* get_block_ptr(unsigned i, unsigned j) BMNOEXCEPT
     {
-        if (!top_blocks_ || i >= top_block_size_) return 0;
+        if (!top_blocks_ || i >= top_block_size_)
+            return 0;
         bm::word_t* const* blk_blk = top_blocks_[i];
         if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
             return FULL_BLOCK_FAKE_ADDR;
@@ -569,7 +570,7 @@ public:
         \param i - top level block index
         \return block adress or NULL if not yet allocated
     */
-    const bm::word_t* const * get_topblock(unsigned i) const
+    const bm::word_t* const * get_topblock(unsigned i) const BMNOEXCEPT
     {
         return (!top_blocks_ || i >= top_block_size_) ? 0 : top_blocks_[i];
     }
@@ -577,7 +578,7 @@ public:
     /** 
         \brief Returns root block in the tree.
     */
-    bm::word_t*** top_blocks_root() const
+    bm::word_t*** top_blocks_root() const BMNOEXCEPT
     {
         blocks_manager* bm = 
             const_cast<blocks_manager*>(this);
@@ -837,7 +838,7 @@ public:
         {
             gap_res = true;
             new_block = (bm::word_t*)
-                    get_allocator().alloc_gap_block(unsigned(new_level), glen());
+                get_allocator().alloc_gap_block(unsigned(new_level), glen());
             ::memcpy(new_block, gap_block, len * sizeof(bm::gap_word_t));
             bm::set_gap_level(new_block, new_level);
         }
@@ -1096,7 +1097,6 @@ public:
         unsigned i, j;
         bm::get_block_coord(nb, i, j);
         bm::word_t* block = this->get_block_ptr(i, j);
-
         if (!IS_VALID_ADDR(block)) // NULL block or ALLSET
         {
             // if we wanted ALLSET and requested block is ALLSET return NULL
@@ -1119,12 +1119,12 @@ public:
         }
         return block;
     }
-    
+
 
     /*! @brief Fills all blocks with 0.
         @param free_mem - if true function frees the resources (obsolete)
     */
-    void set_all_zero(bool /*free_mem*/)
+    void set_all_zero(bool /*free_mem*/) BMNOEXCEPT
     {
         if (!is_init()) return;
         deinit_tree(); // TODO: optimization of top-level realloc
@@ -1141,7 +1141,7 @@ public:
                                 bm::set_sub_array_size, func);
     }
     
-    void free_top_subblock(unsigned nblk_blk)
+    void free_top_subblock(unsigned nblk_blk) BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_[nblk_blk]);
         if ((bm::word_t*)top_blocks_[nblk_blk] != FULL_BLOCK_FAKE_ADDR)
@@ -1165,6 +1165,7 @@ public:
     */
     bm::word_t** check_alloc_top_subblock(unsigned nblk_blk)
     {
+        
         if (!top_blocks_[nblk_blk])
             return alloc_top_subblock(nblk_blk);
         if (top_blocks_[nblk_blk] == (bm::word_t**)FULL_BLOCK_FAKE_ADDR)
@@ -1205,7 +1206,6 @@ public:
 
         // NOTE: block will be replaced without freeing, potential memory leak?
         top_blocks_[nblk_blk][nb & bm::set_array_mask] = block;
-
         return old_block;
     }
 
@@ -1257,7 +1257,6 @@ public:
     bm::word_t* set_block(unsigned i, unsigned j, bm::word_t* block, bool gap)
     {
         BM_ASSERT(i < top_block_size_);
-     
         bm::word_t* old_block;
         if (block)
         {
@@ -1291,15 +1290,16 @@ public:
     /**
         Allocate subblock and fill based on
     */
-    bm::word_t** alloc_top_subblock(unsigned i, bm::word_t* addr_to_set)
+    bm::word_t** alloc_top_subblock(unsigned i, bm::word_t* addr)
     {
-        BM_ASSERT(addr_to_set == 0 || addr_to_set == FULL_BLOCK_FAKE_ADDR);
-        BM_ASSERT(top_blocks_[i] == 0 || top_blocks_[i] == (bm::word_t**)FULL_BLOCK_FAKE_ADDR);
-        
-        bm::word_t** blk_blk = (bm::word_t**)alloc_.alloc_ptr(bm::set_sub_array_size);
+        BM_ASSERT(addr == 0 || addr == FULL_BLOCK_FAKE_ADDR);
+        BM_ASSERT(top_blocks_[i] == 0 ||
+                  top_blocks_[i] == (bm::word_t**)FULL_BLOCK_FAKE_ADDR);
+        bm::word_t** blk_blk =
+            (bm::word_t**)alloc_.alloc_ptr(bm::set_sub_array_size);
         top_blocks_[i] = blk_blk;
-        for (unsigned j = 0; j < bm::set_sub_array_size; ++j)
-            blk_blk[j] = addr_to_set;
+        for (unsigned j = 0; j < bm::set_sub_array_size; j+=4)
+            blk_blk[j+0] = blk_blk[j+1] = blk_blk[j+2] = blk_blk[j+3] = addr;
         return blk_blk;
     }
     
@@ -1452,7 +1452,7 @@ public:
         Places new block into blocks table.
     */
     BMFORCEINLINE
-    void set_block_ptr(unsigned i, unsigned j, bm::word_t* block)
+    void set_block_ptr(unsigned i, unsigned j, bm::word_t* block) BMNOEXCEPT
     {
         BM_ASSERT(is_init());
         BM_ASSERT(i < top_block_size_);
@@ -1519,29 +1519,52 @@ public:
     bm::word_t* deoptimize_block(block_idx_type nb)
     {
         unsigned i, j;
-        get_block_coord(nb, i, j);
+        bm::get_block_coord(nb, i, j);
+        return deoptimize_block(i, j, false);
+    }
+
+    /** deoptimize block and return bit-block ptr
+        can return NULL if block does not exists or allocate (if requested)
+    */
+    bm::word_t* deoptimize_block(unsigned i, unsigned j, bool alloc)
+    {
         bm::word_t* block = this->get_block_ptr(i, j);
+        if (!block && alloc)
+        {
+            reserve_top_blocks(i+1);
+            if (!top_blocks_[i])
+            {
+                alloc_top_subblock(i, 0);
+            }
+            bm::word_t* new_block = alloc_.alloc_bit_block();
+            bm::bit_block_set(new_block, 0u);
+            set_block_ptr(i, j, new_block);
+            return new_block;
+        }
         if (BM_IS_GAP(block))
         {
             gap_word_t* gap_block = BMGAP_PTR(block);
-            
             bm::word_t* new_block = alloc_.alloc_bit_block();
             bm::gap_convert_to_bitset(new_block, gap_block);
             alloc_.free_gap_block(gap_block, this->glen());
-            
-            set_block_ptr(nb, new_block);
+
+            set_block_ptr(i, j, new_block);
             return new_block;
         }
-        if (IS_FULL_BLOCK(block)) 
+        if (IS_FULL_BLOCK(block))
         {
+            if (top_blocks_[i] == (bm::word_t**)FULL_BLOCK_FAKE_ADDR)
+                alloc_top_subblock(i, FULL_BLOCK_FAKE_ADDR);
+
             bm::word_t* new_block = alloc_.alloc_bit_block();
             bm::bit_block_set(new_block, ~0u);
-            set_block_ptr(nb, new_block);
+            set_block_ptr(i, j, new_block);
             return new_block;
         }
         return block;
     }
 
+
     /**
         Free block, make it zero pointer in the tree
     */
@@ -1597,7 +1620,7 @@ public:
     /**
     Free block, make it zero pointer in the tree
     */
-    void zero_gap_block_ptr(unsigned i, unsigned j)
+    void zero_gap_block_ptr(unsigned i, unsigned j) BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_ && i < top_block_size_);
         
@@ -1616,22 +1639,84 @@ public:
         Count number of bits ON in the block
     */
     static
-    bm::id_t block_bitcount(const bm::word_t* block)
+    bm::id_t block_bitcount(const bm::word_t* block) BMNOEXCEPT
     {
         BM_ASSERT(block);
         id_t count;
         if (BM_IS_GAP(block))
-        {
             count = bm::gap_bit_count_unr(BMGAP_PTR(block));
-        }
         else // bitset
-        {
             count = (IS_FULL_BLOCK(block)) ? bm::bits_in_block
                                            : bm::bit_block_count(block);
-        }
         return count;
     }
 
+    /**
+        Bit count all blocks to determine if it is very sparse
+    */
+    bool is_sparse_sblock(unsigned i, unsigned sparse_cut_off) const BMNOEXCEPT
+    {
+        if (!sparse_cut_off)
+            return false;
+        const unsigned non_sparse_cut_off = sparse_cut_off * bm::set_sub_array_size;
+
+        BM_ASSERT(i < top_block_size());
+        word_t*** blk_root = top_blocks_root();
+        if (!blk_root)
+            return false;
+        bm::word_t** blk_blk = blk_root[i];
+        if (!blk_blk || (bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
+            return false;
+        bm::id_t cnt_sum(0), effective_blocks(0), gap_len_sum(0);
+        for (unsigned j = 0; j < bm::set_sub_array_size; ++j)
+        {
+            const bm::word_t* blk = blk_blk[j]; //  this->get_block(i, j);
+            if (blk == FULL_BLOCK_FAKE_ADDR)
+                return false;
+            if (blk)
+            {
+                bm::id_t cnt;
+                if (BM_IS_GAP(blk))
+                {
+                    const bm::gap_word_t* gp = BMGAP_PTR(blk);
+                    cnt = bm::gap_bit_count_unr(gp);
+                    gap_len_sum += bm::gap_length(gp);
+                }
+                else // bitset
+                {
+                    cnt = bm::bit_block_count(blk);
+                }
+                if (cnt)
+                {
+                    ++effective_blocks;
+                    cnt_sum += cnt;
+                    if (cnt_sum > non_sparse_cut_off) // too many bits set
+                        return false;
+                }
+            }
+        } // for j
+
+        BM_ASSERT(effective_blocks <= bm::set_sub_array_size);
+        if (effective_blocks > 1)
+        {
+            if (cnt_sum < 5) // super-duper sparse ...
+                return false;
+            
+            bm::id_t blk_avg = cnt_sum / effective_blocks;
+            if (blk_avg <= sparse_cut_off)
+            {
+                if (gap_len_sum)
+                {
+                    gap_len_sum += effective_blocks * 3;
+                    if (gap_len_sum < cnt_sum)
+                        return false;
+                }
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
         \brief Extends GAP block to the next level or converts it to bit block.
         \param nb - Block's linear index.
@@ -1678,7 +1763,7 @@ public:
     }
     
     /*! deallocate temp block */
-    void free_temp_block()
+    void free_temp_block() BMNOEXCEPT
     {
         if (temp_block_)
         {
@@ -1686,6 +1771,7 @@ public:
             temp_block_ = 0;
         }
     }
+
     /*! Detach and return temp block.
         if temp block is NULL allocates a bit-block
         caller is responsible for returning
@@ -1705,7 +1791,7 @@ public:
     /*! Return temp block
         if temp block already exists - block gets deallocated
     */
-    void return_tempblock(bm::word_t* block)
+    void return_tempblock(bm::word_t* block) BMNOEXCEPT
     {
         BM_ASSERT(block != temp_block_);
         BM_ASSERT(IS_VALID_ADDR(block));
@@ -1717,7 +1803,7 @@ public:
     }
 
     /*! Assigns new GAP lengths vector */
-    void set_glen(const gap_word_t* glevel_len)
+    void set_glen(const gap_word_t* glevel_len) BMNOEXCEPT
     {
         ::memcpy(glevel_len_, glevel_len, sizeof(glevel_len_));
     }
@@ -1745,7 +1831,7 @@ public:
     
     /** Returns true if second level block pointer is 0.
     */
-    bool is_subblock_null(unsigned nsub) const
+    bool is_subblock_null(unsigned nsub) const BMNOEXCEPT
     {
         BM_ASSERT(top_blocks_);
         if (nsub >= top_block_size_)
@@ -1753,14 +1839,14 @@ public:
         return top_blocks_[nsub] == NULL;
     }
 
-    bm::word_t*** top_blocks_root()
+    bm::word_t*** top_blocks_root() BMNOEXCEPT
     {
         return top_blocks_;
     }
 
     /*! \brief Returns current GAP level vector
     */
-    const gap_word_t* glen() const
+    const gap_word_t* glen() const BMNOEXCEPT
     {
         return glevel_len_;
     }
@@ -1768,14 +1854,14 @@ public:
     /*! \brief Returns GAP level length for specified level
         \param level - level number
     */
-    unsigned glen(unsigned level) const
+    unsigned glen(unsigned level) const BMNOEXCEPT
     {
         return glevel_len_[level];
     }
     
     /*! \brief Returns size of the top block array in the tree 
     */
-    unsigned top_block_size() const
+    unsigned top_block_size() const BMNOEXCEPT
     {
         return top_block_size_;
     }
@@ -1829,21 +1915,20 @@ public:
     
     /** \brief Returns reference on the allocator
     */
-    allocator_type& get_allocator() { return alloc_; }
+    allocator_type& get_allocator() BMNOEXCEPT { return alloc_; }
 
     /** \brief Returns allocator
     */
-    allocator_type get_allocator() const { return alloc_; }
+    allocator_type get_allocator() const BMNOEXCEPT { return alloc_; }
     
     
     /// if tree of blocks already up
-    bool is_init() const { return top_blocks_ != 0; }
+    bool is_init() const BMNOEXCEPT { return top_blocks_ != 0; }
     
     /// allocate first level of descr. of blocks 
     void init_tree()
     {
         BM_ASSERT(top_blocks_ == 0);
-        
         if (top_block_size_)
         {
             top_blocks_ = (bm::word_t***) alloc_.alloc_ptr(top_block_size_);
@@ -1865,7 +1950,7 @@ public:
                 alloc_.free_bit_block(blk); \
         }
     
-    void deallocate_top_subblock(unsigned nblk_blk)
+    void deallocate_top_subblock(unsigned nblk_blk) BMNOEXCEPT
     {
         if (!top_blocks_[nblk_blk])
             return;
@@ -1907,7 +1992,7 @@ public:
     /** destroy tree, free memory in all blocks and control structures
         Note: pointers are NOT assigned to zero(!)
     */
-    void destroy_tree() BMNOEXEPT
+    void destroy_tree() BMNOEXCEPT
     {
         if (!top_blocks_)
             return;
@@ -1937,7 +2022,7 @@ public:
     }
     #undef BM_FREE_OP
 
-    void deinit_tree() BMNOEXEPT
+    void deinit_tree() BMNOEXCEPT
     {
         destroy_tree();
         top_blocks_ = 0; top_block_size_ = 0;
@@ -1946,7 +2031,7 @@ public:
     // ----------------------------------------------------------------
     
     /// calculate top blocks which are not NULL and not FULL
-    unsigned find_real_top_blocks() const
+    unsigned find_real_top_blocks() const BMNOEXCEPT
     {
         unsigned cnt = 0;
         unsigned top_blocks = top_block_size();
@@ -1964,7 +2049,7 @@ public:
     // ----------------------------------------------------------------
 
     /// calculate max top blocks size whithout NULL-tail
-    unsigned find_max_top_blocks() const
+    unsigned find_max_top_blocks() const BMNOEXCEPT
     {
         unsigned top_blocks = top_block_size();
         if (!top_blocks)
@@ -1981,11 +2066,11 @@ public:
 
     // ----------------------------------------------------------------
 
-    void validate_top_zero(unsigned i)
+    void validate_top_zero(unsigned i) BMNOEXCEPT
     {
         BM_ASSERT(i < top_block_size());
         bm::word_t** blk_blk = top_blocks_[i];
-        // TODO: SIMD
+        // TODO: SIMD or unroll
         for (unsigned j = 0; j < bm::set_sub_array_size; ++j)
         {
             if (blk_blk[j])
@@ -1997,7 +2082,7 @@ public:
 
     // ----------------------------------------------------------------
 
-    void validate_top_full(unsigned i)
+    void validate_top_full(unsigned i) BMNOEXCEPT
     {
         BM_ASSERT(i < top_block_size());
         bm::word_t** blk_blk = top_blocks_[i];
@@ -2015,7 +2100,7 @@ public:
         Calculate approximate memory needed to serialize big runs
         of 0000s and 111s (as blocks)
     */
-    size_t calc_serialization_null_full() const
+    size_t calc_serialization_null_full() const BMNOEXCEPT
     {
         size_t s_size = sizeof(unsigned);
         if (!top_blocks_)
@@ -2041,6 +2126,9 @@ public:
                 }
                 nb_empty += (i - nb_prev) * bm::set_sub_array_size;
                 blk_blk = top_blocks_[i];
+                BM_ASSERT(blk_blk);
+                if (!blk_blk)
+                    break;
             }
             if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
             {
@@ -2357,7 +2445,7 @@ template<class BlocksManager>
 class bit_block_guard
 {
 public:
-    bit_block_guard(BlocksManager& bman, bm::word_t* blk=0) 
+    bit_block_guard(BlocksManager& bman, bm::word_t* blk=0) BMNOEXCEPT
         : bman_(bman), 
           block_(blk)
     {}
@@ -2366,18 +2454,20 @@ public:
         if (IS_VALID_ADDR(block_))
             bman_.get_allocator().free_bit_block(block_, 3);
     }
-    void attach(bm::word_t* blk)
+
+    void attach(bm::word_t* blk) BMNOEXCEPT
     {
         if (IS_VALID_ADDR(block_))
             bman_.get_allocator().free_bit_block(block_);
         block_ = blk;
     }
+
     bm::word_t* allocate()
     {
         attach(bman_.get_allocator().alloc_bit_block(3));
         return block_;
     }
-    bm::word_t* get() { return block_; }
+    bm::word_t* get() BMNOEXCEPT { return block_; }
 
 private:
     bit_block_guard(const bit_block_guard&);
