@@ -137,7 +137,12 @@ public:
     /*! Make sure row is copy-constructed, return bit-vector */
     bvector_type_ptr construct_row(size_type row, const bvector_type& bv);
 
+    /*! destruct/deallocate row */
     void destruct_row(size_type row);
+
+    /*! clear row bit-vector */
+    void clear_row(size_type row, bool free_mem);
+
     ///@}
     
     
@@ -253,10 +258,10 @@ template<typename Val, typename BV, unsigned MAX_SIZE>
 class base_sparse_vector
 {
 public:
-    enum bit_plains
+    enum bit_planes
     {
-        sv_plains = (sizeof(Val) * 8 * MAX_SIZE + 1),
-        sv_value_plains = (sizeof(Val) * 8 * MAX_SIZE)
+        sv_planes = (sizeof(Val) * 8 * MAX_SIZE + 1),
+        sv_value_planes = (sizeof(Val) * 8 * MAX_SIZE)
     };
 
     enum vector_capacity
@@ -292,7 +297,7 @@ public:
     {
         bmatr_.swap(bsv.bmatr_);
         size_ = bsv.size_;
-        effective_plains_ = bsv.effective_plains_;
+        effective_planes_ = bsv.effective_plane_;
         bsv.size_ = 0;
     }
 #endif
@@ -302,11 +307,16 @@ public:
     size_type size() const BMNOEXCEPT { return size_; }
     
     void resize(size_type new_size);
-    
+
     void clear_range(size_type left, size_type right, bool set_null);
 
-    /*! \brief resize to zero, free memory */
-    void clear() BMNOEXCEPT;
+    void keep_range_no_check(size_type left, size_type right,
+                             bm::null_support splice_null);
+
+    /*! \brief resize to zero, free memory
+        @param free_mem - fully destroys the plane vectors if true
+    */
+    void clear_all(bool free_mem = true) BMNOEXCEPT;
     
     /*! return true if empty */
     bool empty() const BMNOEXCEPT { return size() == 0; }
@@ -315,19 +325,25 @@ public:
 
     // ------------------------------------------------------------
     /*! @name Various traits                                     */
-    //@{
+    ///@{
     /**
         \brief check if container supports NULL(unassigned) values
     */
     bool is_nullable() const BMNOEXCEPT
-        { return bmatr_.get_row(this->null_plain()) != 0; }
+        { return bmatr_.get_row(this->null_plane()) != 0; }
+
+    /**
+        \brief check if container supports NULL(unassigned) values
+    */
+    bm::null_support get_null_support() const BMNOEXCEPT
+        { return is_nullable() ? bm::use_null : bm::no_null; }
 
     /**
         \brief Get bit-vector of assigned values or NULL
         (if not constructed that way)
     */
     const bvector_type* get_null_bvector() const BMNOEXCEPT
-        { return bmatr_.get_row(this->null_plain()); }
+        { return bmatr_.get_row(this->null_plane()); }
     
     /** \brief test if specified element is NULL
         \param idx - element index
@@ -345,60 +361,65 @@ public:
     ///@{
 
     /*!
-        \brief get access to bit-plain, function checks and creates a plain
-        \return bit-vector for the bit plain
+        \brief get access to bit-plane, function checks and creates a plane
+        \return bit-vector for the bit plane
     */
-    bvector_type_ptr get_plain(unsigned i);
+    bvector_type_ptr get_plane(unsigned i);
 
     /*!
-        \brief get read-only access to bit-plain
-        \return bit-vector for the bit plain or NULL
+        \brief get read-only access to bit-plane
+        \return bit-vector for the bit plane or NULL
     */
     bvector_type_const_ptr
-    get_plain(unsigned i) const BMNOEXCEPT { return bmatr_.row(i); }
+    get_plane(unsigned i) const BMNOEXCEPT { return bmatr_.row(i); }
 
     /*!
-        \brief get total number of bit-plains in the vector
+        \brief get total number of bit-planes in the vector
     */
-    static unsigned plains() BMNOEXCEPT { return value_bits(); }
+    static unsigned planes() BMNOEXCEPT { return value_bits(); }
 
-    /** Number of stored bit-plains (value plains + extra */
-    static unsigned stored_plains() BMNOEXCEPT { return value_bits()+1; }
+    /** Number of stored bit-planes (value planes + extra */
+    static unsigned stored_planes() BMNOEXCEPT { return value_bits()+1; }
 
 
-    /** Number of effective bit-plains in the value type */
-    unsigned effective_plains() const BMNOEXCEPT
-                                { return effective_plains_ + 1; }
+    /** Number of effective bit-planes in the value type */
+    unsigned effective_planes() const BMNOEXCEPT
+                                { return effective_planes_ + 1; }
 
     /*!
-        \brief get access to bit-plain as is (can return NULL)
+        \brief get access to bit-plane as is (can return NULL)
     */
-    bvector_type_ptr plain(unsigned i) BMNOEXCEPT { return bmatr_.get_row(i); }
-    bvector_type_const_ptr plain(unsigned i) const BMNOEXCEPT
+    bvector_type_ptr plane(unsigned i) BMNOEXCEPT { return bmatr_.get_row(i); }
+    bvector_type_const_ptr plane(unsigned i) const BMNOEXCEPT
                                     { return bmatr_.get_row(i); }
 
-    bvector_type* get_null_bvect() { return bmatr_.get_row(this->null_plain());}
+    bvector_type* get_null_bvect() { return bmatr_.get_row(this->null_plane());}
 
     /*!
-        \brief free memory in bit-plain
+        \brief free memory in bit-plane
     */
-    void free_plain(unsigned i) { bmatr_.destruct_row(i); }
+    void free_plane(unsigned i) { bmatr_.destruct_row(i); }
     
     /*!
-        return mask of allocated bit-plains
-        1 in the mask - means bit-plain N is present
+        return mask of allocated bit-planes
+        1 in the mask - means bit-plane N is present
         returns 64-bit unsigned mask for sub 64-bit types (like int)
         unallocated mask bits will be zero extended
      
         @return 64-bit mask
         @internal
     */
-    bm::id64_t get_plains_mask(unsigned element_idx) const BMNOEXCEPT;
+    bm::id64_t get_planes_mask(unsigned element_idx) const BMNOEXCEPT;
 
     /*!
         get read-only access to inetrnal bit-matrix
     */
     const bmatrix_type& get_bmatrix() const BMNOEXCEPT { return bmatr_; }
+
+    /**
+        access to internal bit-matrix
+     */
+    bmatrix_type& get_bmatrix() BMNOEXCEPT { return bmatr_; }
     ///@}
     
     /*!
@@ -429,7 +450,7 @@ public:
      
         \param sv        - sparse vector for comparison
         \param null_able - flag to consider NULL vector in comparison (default)
-                           or compare only value content plains
+                           or compare only value content planes
      
         \return true, if it is the same
     */
@@ -439,22 +460,28 @@ public:
 protected:
     void copy_from(const base_sparse_vector<Val, BV, MAX_SIZE>& bsv);
 
-    /*!
-        clear column in all value plains
-        \param plain_idx - row (plain index to start from)
-        \param idx       - bit (column) to clear
-    */
-    void clear_value_plains_from(unsigned plain_idx, size_type idx);
+    /**
+        Merge plane bvectors from an outside base matrix
+        Note: outside base matrix gets destroyed
+     */
+    void merge_matr(bmatrix_type& bmatr, unsigned psize);
 
     /*!
-        insert false (clear) column in all value plains
-        \param plain_idx - row (plain index to start from)
+        clear column in all value planes
+        \param plane_idx - row (plane index to start from)
+        \param idx       - bit (column) to clear
+    */
+    void clear_value_planes_from(unsigned plane_idx, size_type idx);
+
+    /*!
+        insert false (clear) column in all value planes
+        \param plane_idx - row (plane index to start from)
         \param idx       - bit (column) to clear insert
     */
-    void insert_clear_value_plains_from(unsigned plain_idx, size_type idx);
+    void insert_clear_value_planes_from(unsigned plane_idx, size_type idx);
     
     /*!
-        erase bit (column) from all plains
+        erase bit (column) from all planes
         \param idx - bit (column) to erase
     */
     void erase_column(size_type idx);
@@ -464,28 +491,29 @@ protected:
     */
     void insert_null(size_type idx, bool not_null);
 
+
 protected:
     typedef typename bvector_type::block_idx_type block_idx_type;
 
-    /** Number of total bit-plains in the value type*/
+    /** Number of total bit-planes in the value type*/
     static unsigned value_bits() BMNOEXCEPT
     {
-        return base_sparse_vector<Val, BV, MAX_SIZE>::sv_value_plains;
+        return base_sparse_vector<Val, BV, MAX_SIZE>::sv_value_planes;
     }
     
-    /** plain index for the "NOT NULL" flags plain */
-    static unsigned null_plain() BMNOEXCEPT { return value_bits(); }
+    /** plane index for the "NOT NULL" flags plane */
+    static unsigned null_plane() BMNOEXCEPT { return value_bits(); }
     
-    /** optimize block in all matrix plains */
+    /** optimize block in all matrix planes */
     void optimize_block(block_idx_type nb)
     {
         bmatr_.optimize_block(nb);
     }
 
     /**
-        Perform copy_range() on a set of plains
+        Perform copy_range() on a set of planes
     */
-    void copy_range_plains(
+    void copy_range_planes(
         const base_sparse_vector<Val, BV, MAX_SIZE>& bsv,
         typename base_sparse_vector<Val, BV, MAX_SIZE>::size_type left,
         typename base_sparse_vector<Val, BV, MAX_SIZE>::size_type right,
@@ -494,7 +522,7 @@ protected:
 protected:
     bmatrix_type             bmatr_;              ///< bit-transposed matrix
     size_type                size_;               ///< array size
-    unsigned                 effective_plains_;
+    unsigned                 effective_planes_;
 
 };
 
@@ -636,7 +664,6 @@ void basic_bmatrix<BV>::copy_from(const basic_bmatrix<BV>& bbm)
 
 }
 
-
 //---------------------------------------------------------------------
 
 template<typename BV>
@@ -757,6 +784,27 @@ void basic_bmatrix<BV>::destruct_row(size_type row)
     }
 }
 
+//---------------------------------------------------------------------
+
+template<typename BV>
+void basic_bmatrix<BV>::clear_row(size_type row, bool free_mem)
+{
+    BM_ASSERT(row < rsize_);
+    bvector_type_ptr bv = bv_rows_[row];
+    if (bv)
+    {
+        if (free_mem)
+        {
+            destruct_bvector(bv);
+            bv_rows_[row] = 0;
+        }
+        else
+        {
+            bv->clear(true);
+        }
+    }
+}
+
 
 //---------------------------------------------------------------------
 
@@ -1086,7 +1134,7 @@ void basic_bmatrix<BV>::optimize(bm::word_t* temp_block,
     if (st)
         st->reset();
 
-    BM_DECLARE_TEMP_BLOCK(tb);
+    BM_DECLARE_TEMP_BLOCK(tb)
     if (!temp_block)
         temp_block = tb;
 
@@ -1133,9 +1181,9 @@ void basic_bmatrix<BV>::optimize_block(block_idx_type nb)
 
 template<class Val, class BV, unsigned MAX_SIZE>
 base_sparse_vector<Val, BV, MAX_SIZE>::base_sparse_vector()
-: bmatr_(sv_plains, allocation_policy_type(), bm::id_max, allocator_type()),
+: bmatr_(sv_planes, allocation_policy_type(), bm::id_max, allocator_type()),
   size_(0),
-  effective_plains_(0)
+  effective_planes_(0)
 {
 }
 
@@ -1147,13 +1195,13 @@ base_sparse_vector<Val, BV, MAX_SIZE>::base_sparse_vector(
         allocation_policy_type  ap,
         size_type               bv_max_size,
         const allocator_type&       alloc)
-: bmatr_(sv_plains, ap, bv_max_size, alloc),
+: bmatr_(sv_planes, ap, bv_max_size, alloc),
   size_(0),
-  effective_plains_(0)
+  effective_planes_(0)
 {
     if (null_able == bm::use_null)
     {
-        unsigned i = null_plain();
+        unsigned i = null_plane();
         bmatr_.construct_row(i)->init();
     }
 }
@@ -1165,7 +1213,7 @@ base_sparse_vector<Val, BV, MAX_SIZE>::base_sparse_vector(
                         const base_sparse_vector<Val, BV, MAX_SIZE>& bsv)
 : bmatr_(bsv.bmatr_),
   size_(bsv.size_),
-  effective_plains_(bsv.effective_plains_)
+  effective_planes_(bsv.effective_planes_)
 {
 }
 
@@ -1176,16 +1224,16 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::copy_from(
                 const base_sparse_vector<Val, BV, MAX_SIZE>& bsv)
 {
     resize(bsv.size());
-    effective_plains_ = bsv.effective_plains_;
+    effective_planes_ = bsv.effective_planes_;
 
-    unsigned ni = this->null_plain();
-    unsigned plains = bsv.stored_plains();
-    for (size_type i = 0; i < plains; ++i)
+    unsigned ni = this->null_plane();
+    unsigned planes = bsv.stored_planes();
+    for (size_type i = 0; i < planes; ++i)
     {
         bvector_type* bv = bmatr_.get_row(i);
         const bvector_type* bv_src = bsv.bmatr_.row(i);
         
-        if (i == ni) // NULL plain copy
+        if (i == ni) // NULL plane copy
         {
             if (bv && !bv_src) // special case (copy from not NULL)
             {
@@ -1204,6 +1252,25 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::copy_from(
 
 //---------------------------------------------------------------------
 
+template<class Val, class BV, unsigned MAX_SIZE>
+void base_sparse_vector<Val, BV, MAX_SIZE>::merge_matr(
+                            bmatrix_type& bmatr, unsigned psize)
+{
+    for (unsigned j = 0; j < psize; ++j)
+    {
+        bvector_type* arg_bv = bmatr.get_row(j);
+        if (arg_bv)
+        {
+            bvector_type* bv = this->bmatr_.get_row(j);
+            if (!bv) // TODO: borrow the whole arg_bv vector
+                bv = this->get_plane(j);
+            bv->merge(*arg_bv);
+        }
+    } // for j
+}
+
+//---------------------------------------------------------------------
+
 template<class Val, class BV, unsigned MAX_SIZE>
 void base_sparse_vector<Val, BV, MAX_SIZE>::swap(
                  base_sparse_vector<Val, BV, MAX_SIZE>& bsv) BMNOEXCEPT
@@ -1213,24 +1280,22 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::swap(
         bmatr_.swap(bsv.bmatr_);
 
         bm::xor_swap(size_, bsv.size_);
-        bm::xor_swap(effective_plains_, bsv.effective_plains_);
+        bm::xor_swap(effective_planes_, bsv.effective_planes_);
     }
 }
 
 //---------------------------------------------------------------------
 
 template<class Val, class BV, unsigned MAX_SIZE>
-void base_sparse_vector<Val, BV, MAX_SIZE>::clear() BMNOEXCEPT
+void base_sparse_vector<Val, BV, MAX_SIZE>::clear_all(bool free_mem) BMNOEXCEPT
 {
-    unsigned plains = value_bits();
-    for (size_type i = 0; i < plains; ++i)
-    {
-        bmatr_.destruct_row(i);
-    }
+    unsigned planes = value_bits();
+    for (size_type i = 0; i < planes; ++i)
+        bmatr_.clear_row(i, free_mem);
     size_ = 0;
     bvector_type* bv_null = get_null_bvect();
     if (bv_null)
-        bv_null->clear();
+        bv_null->clear(true);
 }
 
 //---------------------------------------------------------------------
@@ -1245,8 +1310,8 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::clear_range(
     {
         return clear_range(right, left, set_null);
     }
-    unsigned plains = value_bits();
-    for (unsigned i = 0; i < plains; ++i)
+    unsigned planes = value_bits();
+    for (unsigned i = 0; i < planes; ++i)
     {
         bvector_type* bv = this->bmatr_.get_row(i);
         if (bv)
@@ -1263,6 +1328,20 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::clear_range(
 
 //---------------------------------------------------------------------
 
+template<class Val, class BV, unsigned MAX_SIZE>
+void base_sparse_vector<Val, BV, MAX_SIZE>::keep_range_no_check(
+                        size_type left, size_type right,
+                        bm::null_support splice_null)
+{
+    if (left)
+        this->clear_range(0, left-1, (splice_null == bm::use_null));
+    size_type sz = size();
+    if (right < sz)
+        this->clear_range(right + 1, sz, (splice_null == bm::use_null));
+}
+
+//---------------------------------------------------------------------
+
 template<class Val, class BV, unsigned MAX_SIZE>
 void base_sparse_vector<Val, BV, MAX_SIZE>::resize(size_type sz)
 {
@@ -1270,7 +1349,7 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::resize(size_type sz)
         return;
     if (!sz) // resize to zero is an equivalent of non-destructive deallocation
     {
-        clear();
+        clear_all();
         return;
     }
     if (sz < size()) // vector shrink
@@ -1304,15 +1383,15 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::insert_null(size_type idx,
 
 template<class Val, class BV, unsigned MAX_SIZE>
 typename base_sparse_vector<Val, BV, MAX_SIZE>::bvector_type_ptr
-                base_sparse_vector<Val, BV, MAX_SIZE>::get_plain(unsigned i)
+                base_sparse_vector<Val, BV, MAX_SIZE>::get_plane(unsigned i)
 {
     bvector_type_ptr bv = bmatr_.get_row(i);
     if (!bv)
     {
         bv = bmatr_.construct_row(i);
         bv->init();
-        if (i > effective_plains_ && i < value_bits())
-            effective_plains_ = i;
+        if (i > effective_planes_ && i < value_bits())
+            effective_planes_ = i;
     }
     return bv;
 }
@@ -1320,20 +1399,20 @@ typename base_sparse_vector<Val, BV, MAX_SIZE>::bvector_type_ptr
 //---------------------------------------------------------------------
 
 template<class Val, class BV, unsigned MAX_SIZE>
-bm::id64_t base_sparse_vector<Val, BV, MAX_SIZE>::get_plains_mask(
+bm::id64_t base_sparse_vector<Val, BV, MAX_SIZE>::get_planes_mask(
                                         unsigned element_idx) const BMNOEXCEPT
 {
     BM_ASSERT(element_idx < MAX_SIZE);
     bm::id64_t mask = 0;
     bm::id64_t mask1 = 1;
-    const unsigned plains = sizeof(value_type) * 8;
+    const unsigned planes = sizeof(value_type) * 8;
     unsigned bidx = 0;
-    for (unsigned i = element_idx * plains; i < (element_idx+1) * plains; i+=4)
+    for (unsigned i = element_idx * planes; i < (element_idx+1) * planes; i+=4)
     {
-        mask |= get_plain(i+0) ? (mask1 << (bidx+0)) : 0ull;
-        mask |= get_plain(i+1) ? (mask1 << (bidx+1)) : 0ull;
-        mask |= get_plain(i+2) ? (mask1 << (bidx+2)) : 0ull;
-        mask |= get_plain(i+3) ? (mask1 << (bidx+3)) : 0ull;
+        mask |= get_plane(i+0) ? (mask1 << (bidx+0)) : 0ull;
+        mask |= get_plane(i+1) ? (mask1 << (bidx+1)) : 0ull;
+        mask |= get_plane(i+2) ? (mask1 << (bidx+2)) : 0ull;
+        mask |= get_plane(i+3) ? (mask1 << (bidx+3)) : 0ull;
         bidx += 4;
     } // for i
     return mask;
@@ -1353,8 +1432,8 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::optimize(bm::word_t* temp_block,
     
     bvector_type* bv_null = this->get_null_bvect();
     
-    unsigned stored_plains = this->stored_plains();
-    for (unsigned j = 0; j < stored_plains; ++j)
+    unsigned stored_planes = this->stored_planes();
+    for (unsigned j = 0; j < stored_planes; ++j)
     {
         bvector_type* bv = this->bmatr_.get_row(j);
         if (bv && (bv != bv_null)) // protect the NULL vector from de-allocation
@@ -1379,8 +1458,8 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::calc_stat(
     
     st->reset();
 
-    unsigned stored_plains = this->stored_plains();
-    for (unsigned j = 0; j < stored_plains; ++j)
+    unsigned stored_planes = this->stored_planes();
+    for (unsigned j = 0; j < stored_planes; ++j)
     {
         const bvector_type* bv = this->bmatr_.row(j);
         if (bv)
@@ -1396,17 +1475,17 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::calc_stat(
     } // for j
     
     // header accounting
-    st->max_serialize_mem += 1 + 1 + 1 + 1 + 8 + (8 * this->stored_plains());
+    st->max_serialize_mem += 1 + 1 + 1 + 1 + 8 + (8 * this->stored_planes());
     st->max_serialize_mem += 1 + 8; // extra header fields for large bit-matrixes
 }
 
 //---------------------------------------------------------------------
 
 template<class Val, class BV, unsigned MAX_SIZE>
-void base_sparse_vector<Val, BV, MAX_SIZE>::clear_value_plains_from(
-                                    unsigned plain_idx, size_type idx)
+void base_sparse_vector<Val, BV, MAX_SIZE>::clear_value_planes_from(
+                                    unsigned plane_idx, size_type idx)
 {
-    for (unsigned i = plain_idx; i < sv_value_plains; ++i)
+    for (unsigned i = plane_idx; i < sv_value_planes; ++i)
     {
         bvector_type* bv = this->bmatr_.get_row(i);
         if (bv)
@@ -1417,10 +1496,10 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::clear_value_plains_from(
 //---------------------------------------------------------------------
 
 template<class Val, class BV, unsigned MAX_SIZE>
-void base_sparse_vector<Val, BV, MAX_SIZE>::insert_clear_value_plains_from(
-                                        unsigned plain_idx, size_type idx)
+void base_sparse_vector<Val, BV, MAX_SIZE>::insert_clear_value_planes_from(
+                                        unsigned plane_idx, size_type idx)
 {
-    for (unsigned i = plain_idx; i < sv_value_plains; ++i)
+    for (unsigned i = plane_idx; i < sv_value_planes; ++i)
     {
         bvector_type* bv = this->bmatr_.get_row(i);
         if (bv)
@@ -1433,7 +1512,7 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::insert_clear_value_plains_from(
 template<class Val, class BV, unsigned MAX_SIZE>
 void base_sparse_vector<Val, BV, MAX_SIZE>::erase_column(size_type idx)
 {
-    for (unsigned i = 0; i < sv_value_plains; ++i)
+    for (unsigned i = 0; i < sv_value_planes; ++i)
     {
         bvector_type* bv = this->bmatr_.get_row(i);
         if (bv)
@@ -1453,8 +1532,8 @@ bool base_sparse_vector<Val, BV, MAX_SIZE>::equal(
     {
         return false;
     }
-    unsigned plains = this->plains();
-    for (unsigned j = 0; j < plains; ++j)
+    unsigned planes = this->planes();
+    for (unsigned j = 0; j < planes; ++j)
     {
         const bvector_type* bv = this->bmatr_.get_row(j);
         const bvector_type* arg_bv = sv.bmatr_.get_row(j);
@@ -1488,7 +1567,9 @@ bool base_sparse_vector<Val, BV, MAX_SIZE>::equal(
         if (bv_null == bv_null_arg)
             return true;
         if (!bv_null || !bv_null_arg)
-            return false;
+        {
+            return false; // TODO: this may need an improvement when one is null, others not null
+        }
         BM_ASSERT(bv_null);
         BM_ASSERT(bv_null_arg);
         bool eq = bv_null->equal(*bv_null_arg);
@@ -1501,7 +1582,7 @@ bool base_sparse_vector<Val, BV, MAX_SIZE>::equal(
 //---------------------------------------------------------------------
 
 template<class Val, class BV, unsigned MAX_SIZE>
-void base_sparse_vector<Val, BV, MAX_SIZE>::copy_range_plains(
+void base_sparse_vector<Val, BV, MAX_SIZE>::copy_range_planes(
         const base_sparse_vector<Val, BV, MAX_SIZE>& bsv,
         typename base_sparse_vector<Val, BV, MAX_SIZE>::size_type left,
         typename base_sparse_vector<Val, BV, MAX_SIZE>::size_type right,
@@ -1509,25 +1590,25 @@ void base_sparse_vector<Val, BV, MAX_SIZE>::copy_range_plains(
 {
     bvector_type* bv_null = get_null_bvect();
     const bvector_type* bv_null_arg = bsv.get_null_bvector();
-    unsigned plains;
+    unsigned planes;
     if (bv_null)
     {
-        plains = this->stored_plains();
+        planes = this->stored_planes();
         if (bv_null_arg && (splice_null == bm::use_null))
             bv_null->copy_range(*bv_null_arg, left, right);
-        --plains;
+        --planes;
     }
     else
-        plains = this->plains();
+        planes = this->planes();
 
-    for (unsigned j = 0; j < plains; ++j)
+    for (unsigned j = 0; j < planes; ++j)
     {
         const bvector_type* arg_bv = bsv.bmatr_.row(j);
         if (arg_bv)
         {
             bvector_type* bv = this->bmatr_.get_row(j);
             if (!bv)
-                bv = this->get_plain(j);
+                bv = this->get_plane(j);
             bv->copy_range(*arg_bv, left, right);
         }
     } // for j
