@@ -52,7 +52,7 @@ namespace bm
  */
 
 
-/** \defgroup sv sparse_vector<>
+/** \defgroup sv bit-sliced (bitwise transposition) succinct sparse vectors 
     Sparse vector for integer types using bit transposition transform
  
     @ingroup bmagic
@@ -60,7 +60,7 @@ namespace bm
 
 
 /*!
-   \brief sparse vector with runtime compression using bit transposition method
+   \brief succinct sparse vector with runtime compression using bit transposition method
  
    Sparse vector implements variable bit-depth storage model.
    Initial data is bit-transposed into bit-planes so each element
@@ -73,7 +73,7 @@ namespace bm
    will save memory.
  
    Overall it provides variable bit-depth compression, sparse compression in
-   bit-plains.
+   bit-planes.
  
    @ingroup sv
 */
@@ -195,7 +195,8 @@ public:
         const_iterator& operator++() BMNOEXCEPT { this->advance(); return *this; }
 
         /// \brief Advance to the next available value
-        const_iterator& operator++(int)
+        ///
+        const_iterator operator++(int)
             { const_iterator tmp(*this);this->advance(); return tmp; }
 
 
@@ -222,11 +223,6 @@ public:
         bool advance() BMNOEXCEPT;
         
         void skip_zero_values() BMNOEXCEPT;
-    private:
-        enum buf_size_e
-        {
-            n_buf_size = 1024 * 8
-        };
         
     private:
         const sparse_vector_type*         sv_;      ///!< ptr to parent
@@ -317,24 +313,18 @@ public:
         
         /** add value to the buffer without changing the NULL vector
             @param v - value to push back
-            @return index of added value in the internal buffer
             @internal
         */
-        size_type add_value_no_null(value_type v);
+        void add_value_no_null(value_type v);
         
         /**
-            Reconfшпгку back inserter not to touch the NULL vector
+            Reconfigure back inserter not to touch the NULL vector
         */
         void disable_set_null() BMNOEXCEPT { set_not_null_ = false; }
         // ---------------------------------------------------------------
         
     protected:
         typedef typename bvector_type::block_idx_type     block_idx_type;
-    private:
-        enum buf_size_e
-        {
-            n_buf_size = 1024 * 8
-        };
 
     private:
         bm::sparse_vector<Val, BV>* sv_;      ///!< pointer on the parent vector
@@ -395,7 +385,7 @@ public:
     {
         if (this != &sv)
         {
-            clear();
+            clear_all(true);
             swap(sv);
         }
         return *this;
@@ -478,13 +468,13 @@ public:
         \param idx - element index
         \param set_null - if true the value receives NULL (unassigned) value
     */
-    void clear(size_type idx, bool set_null = false);
+    void clear(size_type idx, bool set_null/* = false*/);
 
     ///@}
 
     // ------------------------------------------------------------
     /*! @name Iterator access */
-    //@{
+    ///@{
 
     /** Provide const iterator access to container content  */
     const_iterator begin() const BMNOEXCEPT;
@@ -510,7 +500,7 @@ public:
 
     // ------------------------------------------------------------
     /*! @name Various traits                                     */
-    //@{
+    ///@{
     
     /** \brief set specified element to unassigned value (NULL)
         \param idx - element index
@@ -527,7 +517,7 @@ public:
 
     // ------------------------------------------------------------
     /*! @name Loading of sparse vector from C-style array       */
-    //@{
+    ///@{
     
     /*!
         \brief Import list of elements from a C-style array
@@ -620,10 +610,13 @@ public:
     ///@{
 
     /*! \brief resize to zero, free memory */
-    void clear() BMNOEXCEPT;
+    void clear_all(bool free_mem) BMNOEXCEPT;
+
+    /*! \brief resize to zero, free memory */
+    void clear() BMNOEXCEPT { clear_all(true); }
 
     /*!
-        \brief clear range (assign bit 0 for all plains)
+        \brief clear range (assign bit 0 for all planes)
         \param left  - interval start
         \param right - interval end (closed interval)
         \param set_null - set cleared values to unassigned (NULL)
@@ -652,6 +645,13 @@ public:
         \param sz - new size
     */
     void resize(size_type sz) { parent_type::resize(sz); }
+
+    /**
+        \brief recalculate size to exclude tail NULL elements
+        After this call size() will return the true size of the vector
+     */
+    void sync_size() BMNOEXCEPT;
+
     ///@}
         
     // ------------------------------------------------------------
@@ -663,7 +663,7 @@ public:
      
         \param sv        - sparse vector for comparison
         \param null_able - flag to consider NULL vector in comparison (default)
-                           or compare only value content plains
+                           or compare only value content planes
      
         \return true, if it is the same
     */
@@ -682,7 +682,7 @@ public:
         \param idx - vactor element index
         \param val - argument to compare with
      
-        \return 0 - equal, < 0 - vect[i] < str, >0 otherwise
+        \return 0 - equal, < 0 - vect[i] < val, >0 otherwise
     */
     int compare(size_type idx, const value_type val) const BMNOEXCEPT;
     
@@ -693,7 +693,7 @@ public:
     ///@{
 
     /*!
-        \brief run memory optimization for all vector plains
+        \brief run memory optimization for all vector planes
         \param temp_block - pre-allocated memory block to avoid unnecessary re-allocs
         \param opt_mode - requested compression depth
         \param stat - memory allocation statistics after optimization
@@ -705,7 +705,7 @@ public:
     /*!
        \brief Optimize sizes of GAP blocks
 
-       This method runs an analysis to find optimal GAP levels for all bit plains
+       This method runs an analysis to find optimal GAP levels for all bit planes
        of the vector.
     */
     void optimize_gap_size();
@@ -765,11 +765,23 @@ public:
     void copy_range(const sparse_vector<Val, BV>& sv,
                     size_type left, size_type right,
                     bm::null_support splice_null = bm::use_null);
-    
+
+
+    /**
+        Keep only specified interval in the sparse vector, clear all other
+        elements.
+
+        \param left  - interval start
+        \param right - interval end (closed interval)
+        \param splice_null - "use_null" copy range for NULL vector or not
+     */
+     void keep_range(size_type left, size_type right,
+                    bm::null_support splice_null = bm::use_null);
+
     /**
         @brief Apply value filter, defined by mask vector
      
-        All bit-plains are ANDed against the filter mask.
+        All bit-planes are ANDed against the filter mask.
     */
     void filter(const bvector_type& bv_mask);
 
@@ -781,7 +793,8 @@ public:
     ///@{
 
     
-    /*! \brief syncronize internal structures */
+    /*! \brief syncronize internal structures, build fast access index
+    */
     void sync(bool /*force*/) {}
     
     
@@ -820,7 +833,7 @@ public:
         \sa decode
         @internal
     */
-    size_type extract_plains(value_type* arr,
+    size_type extract_planes(value_type* arr,
                              size_type size,
                              size_type offset,
                              bool      zero_mem = true) const;
@@ -870,9 +883,13 @@ public:
     void set_allocator_pool(allocator_pool_type* pool_ptr) BMNOEXCEPT;
     
 protected:
-    enum octet_plains
+    enum octet_planes
     {
-        sv_octet_plains = sizeof(value_type)
+        sv_octet_planes = sizeof(value_type)
+    };
+    enum buf_size_e
+    {
+        n_buf_size = 1024 * 8
     };
 
 
@@ -899,6 +916,17 @@ protected:
     unsigned char* init_remap_buffer() BMNOEXCEPT { return 0; }
     void set_remap() BMNOEXCEPT { }
 
+    /// unused remap matrix type for compatibility with the sparse serializer
+    typedef
+    bm::heap_matrix<unsigned char,
+                    sizeof(value_type), /* ROWS */
+                    256,          /* COLS = number of chars in the ASCII set */
+                    typename bvector_type::allocator_type>
+                                                    remap_matrix_type;
+
+    const remap_matrix_type* get_remap_matrix() const { return 0; }
+    remap_matrix_type* get_remap_matrix() { return 0; }
+
     bool resolve_range(size_type from, size_type to,
                        size_type* idx_from, size_type* idx_to) const BMNOEXCEPT
     {
@@ -917,6 +945,7 @@ protected:
     template<class SVect> friend class sparse_vector_serializer;
     template<class SVect> friend class sparse_vector_deserializer;
 
+
 };
 
 
@@ -1013,12 +1042,12 @@ void sparse_vector<Val, BV>::import(const value_type* arr,
     
     if (offset < this->size_) // in case it touches existing elements
     {
-        // clear all plains in the range to provide corrrect import of 0 values
+        // clear all planes in the range to provide corrrect import of 0 values
         this->clear_range(offset, offset + arr_size - 1);
     }
     
     // transposition algorithm uses bitscan to find index bits and store it
-    // in temporary matrix (list for each bit plain), matrix here works
+    // in temporary matrix (list for each bit plane), matrix here works
     // when array gets to big - the list gets loaded into bit-vector using
     // bulk load algorithm, which is faster than single bit access
     //
@@ -1038,7 +1067,7 @@ void sparse_vector<Val, BV>::import(const value_type* arr,
             
             if (rl == transpose_window)
             {
-                bvector_type* bv = this->get_plain(p);
+                bvector_type* bv = this->get_plane(p);
                 const size_type* r = tm.row(p);
                 bv->set(r, rl, BM_SORTED);
                 row_len[p] = 0;
@@ -1054,7 +1083,7 @@ void sparse_vector<Val, BV>::import(const value_type* arr,
         unsigned rl = row_len[k];
         if (rl)
         {
-            bvector_type* bv = this->get_plain(k);
+            bvector_type* bv = this->get_plane(k);
             const size_type* r = tm.row(k);
             bv->set(r, rl, BM_SORTED);
         }
@@ -1074,6 +1103,18 @@ void sparse_vector<Val, BV>::import(const value_type* arr,
 
 //---------------------------------------------------------------------
 
+template<class Val, class BV>
+void sparse_vector<Val, BV>::sync_size() BMNOEXCEPT
+{
+    const bvector_type* bv_null = this->get_null_bvector();
+    if (!bv_null)
+        return;
+    bool found = bv_null->find_reverse(this->size_);
+    this->size_ += found;
+}
+
+//---------------------------------------------------------------------
+
 template<class Val, class BV>
 void sparse_vector<Val, BV>::import_back(const value_type* arr,
                                          size_type         arr_size,
@@ -1160,7 +1201,7 @@ sparse_vector<Val, BV>::gather(value_type*       arr,
             BM_ASSERT(0);
         } // switch
         
-        // single element hit, use plain random access
+        // single element hit, use plane random access
         if (r == i+1)
         {
             arr[i] = this->get(idx[i]);
@@ -1173,8 +1214,8 @@ sparse_vector<Val, BV>::gather(value_type*       arr,
         unsigned i0 = unsigned(nb >> bm::set_array_shift); // top block address
         unsigned j0 = unsigned(nb &  bm::set_array_mask);  // address in sub-block
         
-        unsigned eff_plains = this->effective_plains(); // TODO: get real effective plains for [i,j]
-        for (unsigned j = 0; j < eff_plains; ++j)
+        unsigned eff_planes = this->effective_planes(); // TODO: get real effective planes for [i,j]
+        for (unsigned j = 0; j < eff_planes; ++j)
         {
             const bm::word_t* blk = this->bmatr_.get_block(j, i0, j0);
             if (!blk)
@@ -1234,7 +1275,7 @@ sparse_vector<Val, BV>::gather(value_type*       arr,
                 continue;
             }
             bm::bit_block_gather_scatter(arr, blk, idx, r, i, j);
-        } // for (each plain)
+        } // for (each plane)
         
         i = r;
 
@@ -1327,7 +1368,7 @@ sparse_vector<Val, BV>::extract_range(value_type* arr,
 
 template<class Val, class BV>
 typename sparse_vector<Val, BV>::size_type
-sparse_vector<Val, BV>::extract_plains(value_type* arr,
+sparse_vector<Val, BV>::extract_planes(value_type* arr,
                                        size_type   size,
                                        size_type   offset,
                                        bool        zero_mem) const
@@ -1389,7 +1430,8 @@ sparse_vector<Val, BV>::extract(value_type* BMRESTRICT arr,
         {}
 
         void add_bits(size_type bv_offset,
-                      const unsigned char* bits, unsigned bits_size) BMNOEXCEPT
+                      const unsigned char* BMRESTRICT bits,
+                      unsigned bits_size) BMNOEXCEPT
         {
             // can be negative (-1) when bv base offset = 0 and sv = 1,2..
             size_type base = bv_offset - sv_off_; 
@@ -1455,14 +1497,14 @@ sparse_vector<Val, BV>::get(
     BM_ASSERT(i < size());
     
     value_type v = 0;
-    unsigned eff_plains = this->effective_plains();
-    for (unsigned j = 0; j < eff_plains; j+=4)
+    unsigned eff_planes = this->effective_planes();
+    for (unsigned j = 0; j < eff_planes; j+=4)
     {
         bool b = this->bmatr_.test_4rows(j);
         if (b)
         {
             value_type vm = (value_type)this->bmatr_.get_half_octet(i, j);
-            v |= vm << j;
+            v |= value_type(vm << j);
         }
     } // for j
     return v;
@@ -1489,7 +1531,7 @@ void sparse_vector<Val, BV>::clear(size_type idx, bool set_null)
     if (idx >= size())
         this->size_ = idx+1;
 
-    set_value(idx, (value_type)0);
+    set_value(idx, value_type(0));
     if (set_null)
     {
         bvector_type* bv_null = this->get_null_bvect();
@@ -1555,7 +1597,7 @@ void sparse_vector<Val, BV>::insert_value_no_null(size_type idx, value_type v)
     {
         if (v & mask)
         {
-            bvector_type* bv = this->get_plain(i);
+            bvector_type* bv = this->get_plane(i);
             bv->insert(idx, true);
         }
         else
@@ -1564,11 +1606,11 @@ void sparse_vector<Val, BV>::insert_value_no_null(size_type idx, value_type v)
             if (bv)
                 bv->insert(idx, false);
         }
-        mask <<= 1;
+        mask = value_type(mask << 1);
     } // for i
-    // insert 0 into all other existing plains
-    unsigned eff_plains = this->effective_plains();
-    for (; i < eff_plains; ++i)
+    // insert 0 into all other existing planes
+    unsigned eff_planes = this->effective_planes();
+    for (; i < eff_planes; ++i)
     {
         bvector_type* bv = this->bmatr_.get_row(i);
         if (bv)
@@ -1622,11 +1664,11 @@ void sparse_vector<Val, BV>::set_value_no_null(size_type idx, value_type v)
     unsigned i0 = unsigned(nb >> bm::set_array_shift); // top block address
     unsigned j0 = unsigned(nb &  bm::set_array_mask);  // address in sub-block
 
-    // clear the plains where needed
-    unsigned eff_plains = this->effective_plains();
+    // clear the planes where needed
+    unsigned eff_planes = this->effective_planes();
     unsigned bsr = v ? bm::bit_scan_reverse(v) : 0u;
         
-    for (unsigned i = bsr; i < eff_plains; ++i)
+    for (unsigned i = bsr; i < eff_planes; ++i)
     {
         const bm::word_t* blk = this->bmatr_.get_block(i, i0, j0);
         if (blk)
@@ -1644,7 +1686,7 @@ void sparse_vector<Val, BV>::set_value_no_null(size_type idx, value_type v)
         {
             if (v & mask)
             {
-                bvector_type* bv = this->get_plain(j);
+                bvector_type* bv = this->get_plane(j);
                 bv->set_bit_no_check(idx);
             }
             else
@@ -1656,7 +1698,7 @@ void sparse_vector<Val, BV>::set_value_no_null(size_type idx, value_type v)
                     bv->clear_bit_no_check(idx);
                 }
             }
-            mask <<=  1;
+            mask = value_type(mask << 1);
         } // for j
     }
 }
@@ -1679,16 +1721,16 @@ void sparse_vector<Val, BV>::inc(size_type idx)
 template<class Val, class BV>
 void sparse_vector<Val, BV>::inc_no_null(size_type idx)
 {
-    for (unsigned i = 0; i < parent_type::sv_value_plains; ++i)
+    for (unsigned i = 0; i < parent_type::sv_value_planes; ++i)
     {
-        bvector_type* bv = this->get_plain(i);
+        bvector_type* bv = this->get_plane(i);
         bool carry_over = bv->inc(idx);
         if (!carry_over)
             break;
     }
 }
 
-//---------------------------------------------------------------------
+//------------------------------------ ---------------------------------
 
 template<class Val, class BV>
 void sparse_vector<Val, BV>::inc_no_null(size_type idx, value_type v)
@@ -1700,9 +1742,9 @@ void sparse_vector<Val, BV>::inc_no_null(size_type idx, value_type v)
 //---------------------------------------------------------------------
 
 template<class Val, class BV>
-void sparse_vector<Val, BV>::clear() BMNOEXCEPT
+void sparse_vector<Val, BV>::clear_all(bool free_mem) BMNOEXCEPT
 {
-    parent_type::clear();
+    parent_type::clear_all(free_mem);
 }
 
 //---------------------------------------------------------------------
@@ -1768,8 +1810,8 @@ void sparse_vector<Val, BV>::optimize(
 template<class Val, class BV>
 void sparse_vector<Val, BV>::optimize_gap_size()
 {
-    unsigned stored_plains = this->stored_plains();
-    for (unsigned j = 0; j < stored_plains; ++j)
+    unsigned stored_planes = this->stored_planes();
+    for (unsigned j = 0; j < stored_planes; ++j)
     {
         bvector_type* bv = this->bmatr_.get_row(j);
         if (bv)
@@ -1791,20 +1833,20 @@ sparse_vector<Val, BV>::join(const sparse_vector<Val, BV>& sv)
         resize(arg_size);
     }
     bvector_type* bv_null = this->get_null_bvect();
-    unsigned plains;
+    unsigned planes;
     if (bv_null)
-        plains = this->stored_plains();
+        planes = this->stored_planes();
     else
-        plains = this->plains();
+        planes = this->planes();
     
-    for (unsigned j = 0; j < plains; ++j)
+    for (unsigned j = 0; j < planes; ++j)
     {
         const bvector_type* arg_bv = sv.bmatr_.row(j);
         if (arg_bv)
         {
             bvector_type* bv = this->bmatr_.get_row(j);
             if (!bv)
-                bv = this->get_plain(j);
+                bv = this->get_plane(j);
             *bv |= *arg_bv;
         }
     } // for j
@@ -1826,68 +1868,63 @@ sparse_vector<Val, BV>::merge(sparse_vector<Val, BV>& sv)
 {
     size_type arg_size = sv.size();
     if (this->size_ < arg_size)
-    {
         resize(arg_size);
-    }
+
     bvector_type* bv_null = this->get_null_bvect();
-    unsigned plains;
-    if (bv_null)
-        plains = this->stored_plains();
-    else
-        plains = this->plains();
-    
-    for (unsigned j = 0; j < plains; ++j)
-    {
-        bvector_type* arg_bv = sv.bmatr_.get_row(j);//sv.plains_[j];
-        if (arg_bv)
-        {
-            bvector_type* bv = this->bmatr_.get_row(j);//this->plains_[j];
-            if (!bv)
-                bv = this->get_plain(j);
-            bv->merge(*arg_bv);
-        }
-    } // for j
-    
+    unsigned planes = bv_null ? this->stored_planes() : this->planes();
+
+    this->merge_matr(sv.bmatr_, planes);
+
     // our vector is NULL-able but argument is not (assumed all values are real)
     if (bv_null && !sv.is_nullable())
-    {
         bv_null->set_range(0, arg_size-1);
-    }
-    
+
     return *this;
 }
 
 //---------------------------------------------------------------------
 
 template<class Val, class BV>
-void sparse_vector<Val, BV>::copy_range(const sparse_vector<Val, BV>& sv,
-                                        typename sparse_vector<Val, BV>::size_type left,
-                                        typename sparse_vector<Val, BV>::size_type right,
-                                        bm::null_support splice_null)
+void sparse_vector<Val, BV>::copy_range(
+                            const sparse_vector<Val, BV>& sv,
+                            typename sparse_vector<Val, BV>::size_type left,
+                            typename sparse_vector<Val, BV>::size_type right,
+                            bm::null_support splice_null)
 {
     if (left > right)
         bm::xor_swap(left, right);
-    //this->clear();
-    this->copy_range_plains(sv, left, right, splice_null);
+    this->copy_range_planes(sv, left, right, splice_null);
     this->resize(sv.size());
 }
 //---------------------------------------------------------------------
 
+template<class Val, class BV>
+void sparse_vector<Val, BV>::keep_range(size_type left, size_type right,
+            bm::null_support splice_null)
+{
+    if (right < left)
+        bm::xor_swap(left, right);
+    this->keep_range_no_check(left, right, splice_null);
+}
+
+
+//---------------------------------------------------------------------
+
 template<class Val, class BV>
 void sparse_vector<Val, BV>::filter(
-                    const typename sparse_vector<Val, BV>::bvector_type& bv_mask)
+                const typename sparse_vector<Val, BV>::bvector_type& bv_mask)
 {
     bvector_type* bv_null = this->get_null_bvect();
-    unsigned plains;
+    unsigned planes;
     if (bv_null)
     {
-        plains = this->stored_plains();
+        planes = this->stored_planes();
         bv_null->bit_and(bv_mask);
     }
     else
-        plains = this->plains();
+        planes = this->planes();
     
-    for (unsigned j = 0; j < plains; ++j)
+    for (unsigned j = 0; j < planes; ++j)
     {
         bvector_type* bv = this->bmatr_.get_row(j);
         if (bv)
@@ -2117,10 +2154,14 @@ template<class Val, class BV>
 void sparse_vector<Val, BV>::back_insert_iterator::add(
          typename sparse_vector<Val, BV>::back_insert_iterator::value_type v)
 {
-    size_type buf_idx = this->add_value_no_null(v);
+    typename sparse_vector<Val, BV>::size_type sz = sv_->size();
+    const value_type* data_ptr = (const value_type*)buffer_.data();
+    size_type buf_idx = size_type(buf_ptr_ - data_ptr);
+
+    this->add_value_no_null(v);
+
     if (bv_null_)
     {
-        typename sparse_vector<Val, BV>::size_type sz = sv_->size();
         bv_null_->set_bit_no_check(sz + buf_idx);
     }
 }
@@ -2128,8 +2169,7 @@ void sparse_vector<Val, BV>::back_insert_iterator::add(
 //---------------------------------------------------------------------
 
 template<class Val, class BV>
-typename sparse_vector<Val, BV>::size_type
-sparse_vector<Val, BV>::back_insert_iterator::add_value_no_null(
+void sparse_vector<Val, BV>::back_insert_iterator::add_value_no_null(
          typename sparse_vector<Val, BV>::back_insert_iterator::value_type v)
 {
     BM_ASSERT(sv_);
@@ -2139,18 +2179,16 @@ sparse_vector<Val, BV>::back_insert_iterator::add_value_no_null(
         buf_ptr_ = (value_type*)(buffer_.data());
         *buf_ptr_ = v;
         ++buf_ptr_;
-        return 0;
+        return;
     }
+    BM_ASSERT(buf_ptr_ && buffer_.data());
     if (buf_ptr_ - ((value_type*)buffer_.data()) >= n_buf_size)
     {
         this->flush();
         buf_ptr_ = (value_type*)(buffer_.data());
     }
-    
     *buf_ptr_ = v;
-    size_type buf_idx = size_type(buf_ptr_ - ((value_type*)buffer_.data()));
     ++buf_ptr_;
-    return buf_idx;
 }
 
 //---------------------------------------------------------------------
@@ -2196,8 +2234,14 @@ void sparse_vector<Val, BV>::back_insert_iterator::flush()
     if (this->empty())
         return;
     value_type* d = (value_type*)buffer_.data();
-    sv_->import_back(d, size_type(buf_ptr_ - d), false); //!set_not_null_);
-    buf_ptr_ = 0;
+    size_type arr_size = size_type(buf_ptr_ - d);
+    if (!arr_size)
+        return;
+
+    sv_->import_back(d, arr_size, false);
+    
+    buf_ptr_ = (value_type*) buffer_.data();
+    BM_ASSERT(buf_ptr_);
     block_idx_type nb = sv_->size() >> bm::set_block_shift;
     if (nb != prev_nb_)
     {
@@ -2212,7 +2256,6 @@ void sparse_vector<Val, BV>::back_insert_iterator::flush()
 
 } // namespace bm
 
-#include "bmundef.h"
 
 
 #endif
