@@ -58,10 +58,10 @@ template<class Val, class SV>
 class rsc_sparse_vector
 {
 public:
-    enum bit_plains
+    enum bit_planes
     {
-        sv_plains = (sizeof(Val) * 8 + 1),
-        sv_value_plains = (sizeof(Val) * 8)
+        sv_planes = (sizeof(Val) * 8 + 1),
+        sv_value_planes = (sizeof(Val) * 8)
     };
 
     typedef Val                                      value_type;
@@ -346,7 +346,7 @@ public:
     {
         if (this != &csv)
         {
-            clear();
+            clear_all(true);
             sv_.swap(csv.sv_);
             size_ = csv.size_; max_id_ = csv.max_id_; in_sync_ = csv.in_sync_;
             if (in_sync_)
@@ -371,7 +371,26 @@ public:
     /*! \brief return true if vector is empty
         \return true if empty
     */
-    bool empty() const { return sv_.empty(); }
+    bool empty() const BMNOEXCEPT { return sv_.empty(); }
+
+    /**
+        \brief recalculate size to exclude tail NULL elements
+        After this call size() will return the true size of the vector
+     */
+    void sync_size() BMNOEXCEPT;
+
+    /*
+       \brief Returns count of not NULL elements (population)
+              in the given range [left..right]
+       Uses rank-select index to accelerate the search (after sync())
+
+       \param left   - index of first bit start counting from
+       \param right  - index of last bit
+
+       @sa sync
+    */
+    size_type
+    count_range_notnull(size_type left, size_type right) const BMNOEXCEPT;
     
     ///@}
 
@@ -515,7 +534,7 @@ public:
     
     // ------------------------------------------------------------
     /*! @name Various traits                                     */
-    //@{
+    ///@{
     /**
         \brief if container supports NULL(unassigned) values (true)
     */
@@ -531,7 +550,7 @@ public:
     
     // ------------------------------------------------------------
     /*! @name Comparison */
-    //@{
+    ///@{
 
     /*!
         \brief check if another vector has the same content
@@ -542,11 +561,8 @@ public:
 
 
     // ------------------------------------------------------------
-    /*! @name Load-Export compressed vector with data */
-    
-    //@{
-
-    
+    /*! @name Load-Export compressed vector with data */    
+    ///@{
     /*!
         \brief Load compressed vector from a sparse vector (with NULLs)
         \param sv_src - source sparse vector
@@ -559,15 +575,19 @@ public:
     */
     void load_to(sparse_vector_type& sv) const;
     
-    //@}
+    ///@}
     
     // ------------------------------------------------------------
     /*! @name Iterator access */
-    //@{
+    ///@{
 
     /** Provide const iterator access to container content  */
-    const_iterator begin() const BMNOEXCEPT
-        { return const_iterator(this); }
+    const_iterator begin() const
+    {
+        if (!in_sync_)
+            throw_no_rsc_index(); // call sync() to build RSC fast access index
+        return const_iterator(this);
+    }
         
     /** Provide const iterator access to the end    */
     const_iterator end() const BMNOEXCEPT
@@ -587,7 +607,7 @@ public:
     ///@{
 
     /*!
-        \brief run memory optimization for all vector plains
+        \brief run memory optimization for all vector planes
         \param temp_block - pre-allocated memory block to avoid unnecessary re-allocs
         \param opt_mode - requested compression depth
         \param stat - memory allocation statistics after optimization
@@ -598,9 +618,15 @@ public:
         statistics* stat = 0);
     
     /*! \brief resize to zero, free memory
+        @param free_mem - free bit vector planes if true
     */
-    void clear() BMNOEXCEPT;
-    
+    void clear_all(bool free_mem) BMNOEXCEPT;
+
+    /*! \brief resize to zero, free memory
+        @param free_mem - free bit vector planes if true
+    */
+    void clear() BMNOEXCEPT { clear_all(true); }
+
     /*!
         @brief Calculates memory statistics.
 
@@ -615,6 +641,7 @@ public:
     void calc_stat(
            struct rsc_sparse_vector<Val, SV>::statistics* st) const BMNOEXCEPT;
 
+
     ///@}
 
     // ------------------------------------------------------------
@@ -646,9 +673,9 @@ public:
 
     // ------------------------------------------------------------
     /*! @name Fast access structues sync                         */
-    //@{
+    ///@{
     /*!
-        \brief Re-calculate prefix sum table used for rank search
+        \brief Re-calculate rank-select index for faster access to vector
         \param force - force recalculation even if it is already recalculated
     */
     void sync(bool force);
@@ -674,30 +701,30 @@ public:
     ///@{
 
     /*!
-        \brief get access to bit-plain, function checks and creates a plain
-        \return bit-vector for the bit plain
+        \brief get access to bit-plane, function checks and creates a plane
+        \return bit-vector for the bit plane
     */
-    bvector_type_const_ptr get_plain(unsigned i) const BMNOEXCEPT
-        { return sv_.get_plain(i); }
+    bvector_type_const_ptr get_plane(unsigned i) const BMNOEXCEPT
+        { return sv_.get_plane(i); }
 
-    bvector_type_ptr get_plain(unsigned i) BMNOEXCEPT
-        { return sv_.get_plain(i); }
+    bvector_type_ptr get_plane(unsigned i) BMNOEXCEPT
+        { return sv_.get_plane(i); }
     
     /*!
-        Number of effective bit-plains in the value type
+        Number of effective bit-planes in the value type
     */
-    unsigned effective_plains() const BMNOEXCEPT
-        { return sv_.effective_plains(); }
+    unsigned effective_planes() const BMNOEXCEPT
+        { return sv_.effective_planes(); }
     
     /*!
-        \brief get total number of bit-plains in the vector
+        \brief get total number of bit-planes in the vector
     */
-    static unsigned plains() BMNOEXCEPT
-        { return sparse_vector_type::plains(); }
+    static unsigned planes() BMNOEXCEPT
+        { return sparse_vector_type::planes(); }
 
-    /** Number of stored bit-plains (value plains + extra */
-    static unsigned stored_plains()
-        { return sparse_vector_type::stored_plains(); }
+    /** Number of stored bit-planes (value planes + extra */
+    static unsigned stored_planes()
+        { return sparse_vector_type::stored_planes(); }
 
     /*!
         \brief access dense vector
@@ -720,12 +747,19 @@ public:
     const bmatrix_type& get_bmatrix() const BMNOEXCEPT
         { return sv_.get_bmatrix(); }
 
+    /*! Get Rank-Select index pointer
+        @return NULL if sync() was not called to construct the index
+        @sa sync()
+    */
+    const rs_index_type* get_RS() const BMNOEXCEPT
+        { return in_sync_ ? bv_blocks_ptr_ : 0; }
+
     ///@}
     
 protected:
-    enum octet_plains
+    enum octet_planes
     {
-        sv_octet_plains = sizeof(value_type)
+        sv_octet_planes = sizeof(value_type)
     };
 
     /*!
@@ -741,7 +775,10 @@ protected:
     bool resolve_range(size_type from, size_type to, 
                        size_type* idx_from, size_type* idx_to) const BMNOEXCEPT;
     
-    void resize_internal(size_type sz) { sv_.resize_internal(sz); }
+    void resize_internal(size_type sz) 
+    { 
+        sv_.resize_internal(sz); 
+    }
     size_type size_internal() const BMNOEXCEPT { return sv_.size(); }
 
     bool is_remap() const BMNOEXCEPT { return false; }
@@ -749,6 +786,14 @@ protected:
     const unsigned char* get_remap_buffer() const BMNOEXCEPT { return 0; }
     unsigned char* init_remap_buffer() BMNOEXCEPT { return 0; }
     void set_remap() BMNOEXCEPT { }
+
+    /// unused remap matrix type for compatibility with the sparse serializer
+    typedef
+    bm::heap_matrix<unsigned char, 1, 1,
+                    typename bvector_type::allocator_type> remap_matrix_type;
+
+    const remap_matrix_type* get_remap_matrix() const { return 0; }
+    remap_matrix_type* get_remap_matrix() { return 0; }
     
     void push_back_no_check(size_type idx, value_type v);
 
@@ -760,11 +805,20 @@ private:
     /// Free rs-index
     void free_rs_index();
 
+    /**
+        \brief throw error that RSC index not constructed (call sync())
+        \internal
+        @sa sync
+    */
+    static
+    void throw_no_rsc_index();
+
 protected:
     template<class SVect> friend class sparse_vector_scanner;
     template<class SVect> friend class sparse_vector_serializer;
     template<class SVect> friend class sparse_vector_deserializer;
 
+
 private:
     sparse_vector_type            sv_;       ///< transpose-sparse vector for "dense" packing
     size_type                     size_;     ///< vector size (logical)
@@ -785,7 +839,7 @@ rsc_sparse_vector<Val, SV>::rsc_sparse_vector(bm::null_support null_able,
 : sv_(null_able, ap, bv_max_size, alloc), in_sync_(false)
 {
     BM_ASSERT(null_able == bm::use_null);
-    BM_ASSERT(int(sv_value_plains) == int(SV::sv_value_plains));
+    BM_ASSERT(int(sv_value_planes) == int(SV::sv_value_planes));
     size_ = max_id_ = 0;
     construct_rs_index();
 }
@@ -830,7 +884,7 @@ rsc_sparse_vector<Val, SV>::rsc_sparse_vector(
                           const rsc_sparse_vector<Val, SV>& csv)
 : sv_(csv.sv_), size_(csv.size_), max_id_(csv.max_id_), in_sync_(csv.in_sync_)
 {
-    BM_ASSERT(int(sv_value_plains) == int(SV::sv_value_plains));
+    BM_ASSERT(int(sv_value_planes) == int(SV::sv_value_planes));
     
     construct_rs_index();
     if (in_sync_)
@@ -1060,19 +1114,19 @@ void rsc_sparse_vector<Val, SV>::load_from(
     }
     else
     {
-        sv_.clear();
+        sv_.clear_all(true);
         *bv_null = *bv_null_src;
         
-        bm::rank_compressor<bvector_type> rank_compr; // re-used for plains
+        bm::rank_compressor<bvector_type> rank_compr; // re-used for planes
         
-        unsigned src_plains = sv_src.plains();
-        for (unsigned i = 0; i < src_plains; ++i)
+        unsigned src_planes = sv_src.planes();
+        for (unsigned i = 0; i < src_planes; ++i)
         {
-            const bvector_type* bv_src_plain = sv_src.get_plain(i);
-            if (bv_src_plain)
+            const bvector_type* bv_src_plane = sv_src.get_plane(i);
+            if (bv_src_plane)
             {
-                bvector_type* bv_plain = sv_.get_plain(i);
-                rank_compr.compress(*bv_plain, *bv_null, *bv_src_plain);
+                bvector_type* bv_plane = sv_.get_plane(i);
+                rank_compr.compress(*bv_plane, *bv_null, *bv_src_plane);
             }
         } // for
         size_type count = bv_null->count(); // set correct sizes
@@ -1087,7 +1141,7 @@ void rsc_sparse_vector<Val, SV>::load_from(
 template<class Val, class SV>
 void rsc_sparse_vector<Val, SV>::load_to(sparse_vector_type& sv) const
 {
-    sv.clear();
+    sv.clear_all(true);
     
     const bvector_type* bv_null_src = this->get_null_bvector();
     if (!bv_null_src)
@@ -1100,16 +1154,16 @@ void rsc_sparse_vector<Val, SV>::load_to(sparse_vector_type& sv) const
     BM_ASSERT(bv_null);
     *bv_null = *bv_null_src;
     
-    bm::rank_compressor<bvector_type> rank_compr; // re-used for plains
+    bm::rank_compressor<bvector_type> rank_compr; // re-used for planes
 
-    unsigned src_plains = sv_.plains();
-    for (unsigned i = 0; i < src_plains; ++i)
+    unsigned src_planes = sv_.planes();
+    for (unsigned i = 0; i < src_planes; ++i)
     {
-        const bvector_type* bv_src_plain = sv_.get_plain(i);
-        if (bv_src_plain)
+        const bvector_type* bv_src_plane = sv_.get_plane(i);
+        if (bv_src_plane)
         {
-            bvector_type* bv_plain = sv.get_plain(i);
-            rank_compr.decompress(*bv_plain, *bv_null, *bv_src_plain);
+            bvector_type* bv_plane = sv.get_plane(i);
+            rank_compr.decompress(*bv_plane, *bv_null, *bv_src_plane);
         }
     } // for
     sv.resize(this->size());
@@ -1125,19 +1179,28 @@ void rsc_sparse_vector<Val, SV>::sync(bool force)
     const bvector_type* bv_null = sv_.get_null_bvector();
     BM_ASSERT(bv_null);
     bv_null->build_rs_index(bv_blocks_ptr_); // compute popcount prefix list
-    
+ 
+    if (force)
+    {
+        sync_size();
+    }
+    in_sync_ = true;
+}
+
+//---------------------------------------------------------------------
+
+template<class Val, class SV>
+void rsc_sparse_vector<Val, SV>::sync_size() BMNOEXCEPT
+{
+    const bvector_type* bv_null = sv_.get_null_bvector();
+    BM_ASSERT(bv_null);
     // sync the max-id
     bool found = bv_null->find_reverse(max_id_);
     if (!found)
-    {
-        BM_ASSERT(!bv_null->any());
         max_id_ = size_ = 0;
-    }
     else
-    {
         size_ = max_id_ + 1;
-    }
-    in_sync_ = true;
+    sync(false);
 }
 
 //---------------------------------------------------------------------
@@ -1253,9 +1316,9 @@ void rsc_sparse_vector<Val, SV>::optimize(bm::word_t*  temp_block,
 //---------------------------------------------------------------------
 
 template<class Val, class SV>
-void rsc_sparse_vector<Val, SV>::clear() BMNOEXCEPT
+void rsc_sparse_vector<Val, SV>::clear_all(bool free_mem) BMNOEXCEPT
 {
-    sv_.clear();
+    sv_.clear_all(free_mem);
     in_sync_ = false;  max_id_ = size_ = 0;
 }
 
@@ -1304,6 +1367,18 @@ rsc_sparse_vector<Val, SV>::find_rank(size_type rank,
 
 //---------------------------------------------------------------------
 
+template<class Val, class SV>
+void rsc_sparse_vector<Val, SV>::throw_no_rsc_index()
+{
+#ifndef BM_NO_STL
+    throw std::domain_error("Rank-Select index not constructed, call sync() first");
+#else
+    BM_ASSERT_THROW(false, BM_ERR_RANGE);
+#endif
+}
+
+//---------------------------------------------------------------------
+
 
 template<class Val, class SV>
 typename rsc_sparse_vector<Val, SV>::size_type
@@ -1314,9 +1389,10 @@ rsc_sparse_vector<Val, SV>::decode(value_type* arr,
 {
     if (size == 0)
         return 0;
+    if (!in_sync_)
+        throw_no_rsc_index(); // call sync() to build RSC fast access index
         
     BM_ASSERT(arr);
-    BM_ASSERT(in_sync_);  // call sync() before decoding
     BM_ASSERT(bv_blocks_ptr_);
     
     if (idx_from >= this->size())
@@ -1332,14 +1408,16 @@ rsc_sparse_vector<Val, SV>::decode(value_type* arr,
 
     BM_ASSERT(rank == bv_null->count_range(0, idx_from) - bv_null->test(idx_from));
 
+    size_type i = 0;
+
     bvector_enumerator_type en_i = bv_null->get_enumerator(idx_from);
-    BM_ASSERT(en_i.valid());
+    if (!en_i.valid())
+        return i;
 
     if (zero_mem)
         ::memset(arr, 0, sizeof(value_type)*size);
 
     sparse_vector_const_iterator it = sv_.get_const_iterator(rank);
-    size_type i = 0;
     if (it.valid())
     {
         do
@@ -1375,7 +1453,7 @@ rsc_sparse_vector<Val, SV>::decode_buf(value_type*     arr,
 
     BM_ASSERT(arr && arr_buf_tmp);
     BM_ASSERT(arr != arr_buf_tmp);
-    BM_ASSERT(in_sync_);  // call sync() before decoding
+    BM_ASSERT(in_sync_);  // call sync() to build RSC fast access index
     BM_ASSERT(bv_blocks_ptr_);
 
     if ((bm::id_max - size) <= idx_from)
@@ -1463,7 +1541,7 @@ void rsc_sparse_vector<Val, SV>::copy_range(
     bool range_valid = csv.resolve_range(left, right, &sv_left, &sv_right);
     if (!range_valid)
     {
-        sv_.clear(); sv_.resize(size_);
+        sv_.clear_all(true); sv_.resize(size_);
         bvector_type* bv_null = sv_.get_null_bvect();
         bv_null->copy_range(*arg_bv_null, 0, right);
         return;
@@ -1485,6 +1563,25 @@ void rsc_sparse_vector<Val, SV>::merge_not_null(rsc_sparse_vector<Val, SV>& csv)
     sv_.merge(csv.sv_);
 }
 
+//---------------------------------------------------------------------
+
+template<class Val, class SV>
+typename rsc_sparse_vector<Val, SV>::size_type
+rsc_sparse_vector<Val, SV>::count_range_notnull(
+                                        size_type left,
+                                        size_type right) const BMNOEXCEPT
+{
+    if (left > right)
+        bm::xor_swap(left, right);
+
+    const bvector_type* bv_null = sv_.get_null_bvector();
+    size_type range = right - left;
+    if ((range < rs3_border0) || !in_sync_)
+    {
+        return bv_null->count_range_no_check(left, right);
+    }
+    return bv_null->count_range_no_check(left, right, *bv_blocks_ptr_);
+}
 
 //---------------------------------------------------------------------
 //
@@ -1699,7 +1796,6 @@ bool rsc_sparse_vector<Val, SV>::const_iterator::is_null() const BMNOEXCEPT
 
 } // namespace bm
 
-#include "bmundef.h"
 
 
 #endif
