@@ -576,6 +576,17 @@ bool sse4_is_all_one(const __m128i* BMRESTRICT block)
     return true;
 }
 
+/*!
+    @brief check if SSE wave is all oxFFFF...FFF
+    @ingroup SSE4
+*/
+BMFORCEINLINE
+bool sse42_test_all_one_wave(const void* ptr)
+{
+    return _mm_test_all_ones(_mm_loadu_si128((__m128i*)ptr));
+}
+
+
 /*!
     @brief check if wave of pointers is all NULL
     @ingroup SSE4
@@ -697,16 +708,25 @@ unsigned sse42_bit_block_calc_change(const __m128i* BMRESTRICT block,
     @ingroup SSE4
 */
 inline
-unsigned sse42_bit_block_calc_xor_change(const __m128i* BMRESTRICT block,
+void sse42_bit_block_calc_xor_change(const __m128i* BMRESTRICT block,
                                      const __m128i* BMRESTRICT xor_block,
-                                     unsigned size)
+                                     unsigned size,
+                                     unsigned* BMRESTRICT gc,
+                                     unsigned* BMRESTRICT bc)
 {
+#ifdef BM64_SSE4
+    bm::id64_t BM_ALIGN32 simd_buf[2] BM_ALIGN32ATTR;
+#else
+    ///bm::id64_t BM_ALIGN32 simd_buf[4] BM_ALIGN32ATTR;
+#endif
+
     const __m128i* block_end =
         ( __m128i*)((bm::word_t*)(block) + size);
     __m128i m1COshft, m2COshft;
 
     unsigned w0 = *((bm::word_t*)(block));
-    unsigned count = 1;
+    unsigned gap_count = 1;
+    unsigned bit_count = 0;
 
     unsigned co2, co1 = 0;
     for (;block < block_end; block += 2, xor_block += 2)
@@ -719,6 +739,30 @@ unsigned sse42_bit_block_calc_xor_change(const __m128i* BMRESTRICT block,
         m1A = _mm_xor_si128(m1A, m1B);
         m2A = _mm_xor_si128(m2A, m2B);
 
+        {
+#ifdef BM64_SSE4
+        _mm_store_si128 ((__m128i*)simd_buf, m1A);
+        bit_count += unsigned(_mm_popcnt_u64(simd_buf[0]) + _mm_popcnt_u64(simd_buf[1]));
+
+        _mm_store_si128 ((__m128i*)simd_buf, m2A);
+        bit_count += unsigned(_mm_popcnt_u64(simd_buf[0]) + _mm_popcnt_u64(simd_buf[1]));
+#else
+        bm::id_t m0 = _mm_extract_epi32(m1A, 0);
+        bm::id_t m1 = _mm_extract_epi32(m1A, 1);
+        bm::id_t m2 = _mm_extract_epi32(m1A, 2);
+        bm::id_t m3 = _mm_extract_epi32(m1A, 3);
+        bit_count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
+                          _mm_popcnt_u32(m2) + _mm_popcnt_u32(m3));
+
+        m0 = _mm_extract_epi32(m2A, 0);
+        m1 = _mm_extract_epi32(m2A, 1);
+        m2 = _mm_extract_epi32(m2A, 2);
+        m3 = _mm_extract_epi32(m2A, 3);
+        bit_count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
+                             _mm_popcnt_u32(m2) + _mm_popcnt_u32(m3));
+#endif
+        }
+
         __m128i m1CO = _mm_srli_epi32(m1A, 31);
         __m128i m2CO = _mm_srli_epi32(m2A, 31);
 
@@ -747,32 +791,33 @@ unsigned sse42_bit_block_calc_xor_change(const __m128i* BMRESTRICT block,
         m2A = _mm_xor_si128(m2A, m2As);
 
 #ifdef BM64_SSE4
-        bm::id64_t m0 = _mm_extract_epi64(m1A, 0);
-        bm::id64_t m1 = _mm_extract_epi64(m1A, 1);
-        count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+        _mm_store_si128 ((__m128i*)simd_buf, m1A);
+        gap_count += unsigned(_mm_popcnt_u64(simd_buf[0]) + _mm_popcnt_u64(simd_buf[1]));
 
-        m0 = _mm_extract_epi64(m2A, 0);
-        m1 = _mm_extract_epi64(m2A, 1);
-        count += unsigned(_mm_popcnt_u64(m0) + _mm_popcnt_u64(m1));
+        _mm_store_si128 ((__m128i*)simd_buf, m2A);
+        gap_count += unsigned(_mm_popcnt_u64(simd_buf[0]) + _mm_popcnt_u64(simd_buf[1]));
 #else
         bm::id_t m0 = _mm_extract_epi32(m1A, 0);
         bm::id_t m1 = _mm_extract_epi32(m1A, 1);
         bm::id_t m2 = _mm_extract_epi32(m1A, 2);
         bm::id_t m3 = _mm_extract_epi32(m1A, 3);
-        count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
+        gap_count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
                           _mm_popcnt_u32(m2) + _mm_popcnt_u32(m3));
 
         m0 = _mm_extract_epi32(m2A, 0);
         m1 = _mm_extract_epi32(m2A, 1);
         m2 = _mm_extract_epi32(m2A, 2);
         m3 = _mm_extract_epi32(m2A, 3);
-        count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
+        gap_count += unsigned(_mm_popcnt_u32(m0) + _mm_popcnt_u32(m1) +
                           _mm_popcnt_u32(m2) + _mm_popcnt_u32(m3));
 #endif
 
     }
-    count -= (w0 & 1u); // correct initial carry-in error
-    return count;
+    gap_count -= (w0 & 1u); // correct initial carry-in error
+    if (!gap_count)
+        ++gap_count; // must be >0
+    *gc = gap_count;
+    *bc = bit_count;
 }
 
 
@@ -973,12 +1018,14 @@ bool sse42_bit_find_first(const __m128i* BMRESTRICT block,
 #endif
 
 /*!
-     SSE4.2 check for one to two (variable len) 128 bit SSE lines for gap search results (8 elements)
+     SSE4.2 check for one to two (variable len) 128 bit SSE lines
+     for gap search results (8 elements)
      @ingroup SSE4
      \internal
 */
 inline
-unsigned sse4_gap_find(const bm::gap_word_t* BMRESTRICT pbuf, const bm::gap_word_t pos, const unsigned size)
+unsigned sse4_gap_find(const bm::gap_word_t* BMRESTRICT pbuf,
+                       const bm::gap_word_t pos, const unsigned size)
 {
     BM_ASSERT(size <= 16);
     BM_ASSERT(size);
@@ -1031,6 +1078,74 @@ unsigned sse4_gap_find(const bm::gap_word_t* BMRESTRICT pbuf, const bm::gap_word
     return size - bc;
 }
 
+/**
+    Hybrid binary search, starts as binary, then switches to linear scan
+
+   \param buf - GAP buffer pointer.
+   \param pos - index of the element.
+   \param is_set - output. GAP value (0 or 1).
+   \return GAP index.
+
+    @ingroup SSE4
+*/
+inline
+unsigned sse42_gap_bfind(const unsigned short* BMRESTRICT buf,
+                         unsigned pos, unsigned* BMRESTRICT is_set)
+{
+    unsigned start = 1;
+    unsigned end = 1 + ((*buf) >> 3);
+    unsigned dsize = end - start;
+
+    if (dsize < 17)
+    {
+        start = bm::sse4_gap_find(buf+1, (bm::gap_word_t)pos, dsize);
+        *is_set = ((*buf) & 1) ^ (start & 1);
+        BM_ASSERT(buf[start+1] >= pos);
+        BM_ASSERT(buf[start] < pos || (start==0));
+
+        return start+1;
+    }
+    unsigned arr_end = end;
+    while (start != end)
+    {
+        unsigned curr = (start + end) >> 1;
+        if (buf[curr] < pos)
+            start = curr + 1;
+        else
+            end = curr;
+
+        unsigned size = end - start;
+        if (size < 16)
+        {
+            size += (end != arr_end);
+            unsigned idx =
+                bm::sse4_gap_find(buf + start, (bm::gap_word_t)pos, size);
+            start += idx;
+
+            BM_ASSERT(buf[start] >= pos);
+            BM_ASSERT(buf[start - 1] < pos || (start == 1));
+            break;
+        }
+    }
+
+    *is_set = ((*buf) & 1) ^ ((start-1) & 1);
+    return start;
+}
+
+/**
+    Hybrid binary search, starts as binary, then switches to scan
+    @ingroup SSE4
+*/
+inline
+unsigned sse42_gap_test(const unsigned short* BMRESTRICT buf, unsigned pos)
+{
+    unsigned is_set;
+    bm::sse42_gap_bfind(buf, pos, &is_set);
+    return is_set;
+}
+
+
+
 /**
     Experimental (test) function to do SIMD vector search (lower bound)
     in sorted, growing array
@@ -1570,7 +1685,7 @@ bool sse42_shift_r1_and(__m128i* block,
 inline
 void sse42_bit_block_xor(bm::word_t*  target_block,
                    const bm::word_t*  block, const bm::word_t*  xor_block,
-                   bm::id64_t digest)
+                   bm::id64_t digest) BMNOEXCEPT
 {
     for (unsigned i = 0; i < bm::block_waves; ++i)
     {
@@ -1627,6 +1742,63 @@ void sse42_bit_block_xor(bm::word_t*  target_block,
     } // for i
 }
 
+/**
+    Build partial XOR product of 2 bit-blocks using digest mask
+
+    @param target_block - target ^= xor_block
+    @param xor_block - arg1
+    @param digest - mask for each block wave to XOR (if 1)
+
+    @ingroup SSE4
+    @internal
+*/
+inline
+void sse42_bit_block_xor_2way(bm::word_t* target_block,
+                              const bm::word_t*  xor_block,
+                              bm::id64_t digest) BMNOEXCEPT
+{
+    while (digest)
+    {
+        bm::id64_t t = bm::bmi_blsi_u64(digest); // d & -d;
+        unsigned wave = _mm_popcnt_u64(t - 1);
+        unsigned off = wave * bm::set_block_digest_wave_size;
+
+        const __m128i* sub_block = (const __m128i*) (xor_block + off);
+        __m128i* t_sub_block = (__m128i*)(target_block + off);
+
+        __m128i mA, mB, mC, mD;
+        mA = _mm_xor_si128(_mm_load_si128(sub_block),
+                              _mm_load_si128(t_sub_block));
+        mB = _mm_xor_si128(_mm_load_si128(sub_block+1),
+                              _mm_load_si128(t_sub_block+1));
+        mC = _mm_xor_si128(_mm_load_si128(sub_block+2),
+                              _mm_load_si128(t_sub_block+2));
+        mD = _mm_xor_si128(_mm_load_si128(sub_block+3),
+                              _mm_load_si128(t_sub_block+3));
+
+        _mm_store_si128(t_sub_block, mA);
+        _mm_store_si128(t_sub_block+1, mB);
+        _mm_store_si128(t_sub_block+2, mC);
+        _mm_store_si128(t_sub_block+3, mD);
+
+        mA = _mm_xor_si128(_mm_load_si128(sub_block+4),
+                              _mm_load_si128(t_sub_block+4));
+        mB = _mm_xor_si128(_mm_load_si128(sub_block+5),
+                              _mm_load_si128(t_sub_block+5));
+        mC = _mm_xor_si128(_mm_load_si128(sub_block+6),
+                              _mm_load_si128(t_sub_block+6));
+        mD = _mm_xor_si128(_mm_load_si128(sub_block+7),
+                              _mm_load_si128(t_sub_block+7));
+
+        _mm_store_si128(t_sub_block+4, mA);
+        _mm_store_si128(t_sub_block+5, mB);
+        _mm_store_si128(t_sub_block+6, mC);
+        _mm_store_si128(t_sub_block+7, mD);
+
+        digest = bm::bmi_bslr_u64(digest); // d &= d - 1;
+    } // while
+}
+
 
 
 #define VECT_XOR_ARR_2_MASK(dst, src, src_end, mask)\
@@ -1734,8 +1906,8 @@ void sse42_bit_block_xor(bm::word_t*  target_block,
 #define VECT_BLOCK_CHANGE(block, size) \
     sse42_bit_block_calc_change((__m128i*)block, size)
 
-#define VECT_BLOCK_XOR_CHANGE(block, xor_block, size) \
-    sse42_bit_block_calc_xor_change((__m128i*)block, (__m128i*)xor_block, size)
+#define VECT_BLOCK_XOR_CHANGE(block, xor_block, size, gc, bc) \
+    sse42_bit_block_calc_xor_change((__m128i*)block, (__m128i*)xor_block, size, gc, bc)
 
 #ifdef BM64_SSE4
 #define VECT_BLOCK_CHANGE_BC(block, gc, bc) \
@@ -1751,6 +1923,12 @@ void sse42_bit_block_xor(bm::word_t*  target_block,
 #define VECT_BIT_BLOCK_XOR(t, src, src_xor, d) \
     sse42_bit_block_xor(t, src, src_xor, d)
 
+#define VECT_BIT_BLOCK_XOR_2WAY(t, src_xor, d) \
+    sse42_bit_block_xor_2way(t, src_xor, d)
+
+
+#define VECT_GAP_BFIND(buf, pos, is_set) \
+    sse42_gap_bfind(buf, pos, is_set)
 
 #ifdef __GNUG__
 #pragma GCC diagnostic pop
