@@ -1,7 +1,7 @@
 #ifndef FORMATGUESS__HPP
 #define FORMATGUESS__HPP
 
-/*  $Id: format_guess.hpp 612523 2020-07-23 11:23:30Z ivanov $
+/*  $Id: format_guess.hpp 629211 2021-04-12 18:52:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -262,6 +262,9 @@ public:
     /// Get format hints
     CFormatHints& GetFormatHints(void) { return m_Hints; }
 
+    /// Check whether testing is enabled for given format
+    bool IsEnabled(EFormat format) const { return !m_Hints.IsDisabled(format); };
+
 protected:
     void Initialize();
 
