@@ -1,7 +1,7 @@
 #ifndef __COMPILE_TIME_BITS_HPP_INCLUDED__
 #define __COMPILE_TIME_BITS_HPP_INCLUDED__
 
-/*  $Id: compile_time_bits.hpp 597801 2019-11-27 15:23:57Z gotvyans $
+/*  $Id: compile_time_bits.hpp 620372 2020-11-20 16:51:49Z vakatov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -274,7 +274,7 @@ namespace std
         return in[i];
     }
     template<class T, size_t N>
-    constexpr size_t size(const ct::const_array<T, N>& in) noexcept
+    constexpr size_t size(const ct::const_array<T, N>& /*in*/) noexcept
     {
         return N;
     }
@@ -627,7 +627,7 @@ namespace compile_time_bits
         }
 
         template<size_t N, typename _Indices, std::size_t... I>
-        static constexpr auto construct_hashes(const init_type(&init)[N], const _Indices& indices, std::index_sequence<I...> is) noexcept
+        static constexpr auto construct_hashes(const init_type(&init)[N], const _Indices& indices, std::index_sequence<I...> /*is_unused*/) noexcept
             -> aligned_index<hash_type, N>
         {
             return { { select_hash<I>(init, indices) ...} };
