@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix.sh 603335 2020-03-10 17:11:15Z ivanov $
+# $Id: check_make_unix.sh 630684 2021-05-06 15:49:59Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -343,12 +343,17 @@ LSAN_OPTIONS="suppressions=\${script_dir}/common/check/lsan.supp:exitcode=0"
 export LSAN_OPTIONS
 TSAN_OPTIONS="suppressions=\${script_dir}/common/check/tsan.supp"
 export TSAN_OPTIONS
+# There also ASAN_OPTIONS, that we don't use right now
+# https://github.com/google/sanitizers/wiki/AddressSanitizerFlags
 
 # Disable BOOST tests to catch asynchronous system failures
 # (signals on *NIX platforms or structured exceptions on Windows)
 BOOST_TEST_CATCH_SYSTEM_ERRORS=no
 export BOOST_TEST_CATCH_SYSTEM_ERRORS
 
+BOOST_TEST_COLOR_OUTPUT=no
+export BOOST_TEST_COLOR_OUTPUT
+
 # Export some global vars
 top_srcdir="\$root_dir"
 export top_srcdir
