@@ -1,4 +1,4 @@
-/*  $Id: blast_options_builder.cpp 591152 2019-08-12 11:18:21Z fongah2 $
+/*  $Id: blast_options_builder.cpp 617780 2020-10-06 16:24:16Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -396,7 +396,16 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
 	    #endif
         }
         else if (CBlast4Field::Get(eBlastOpt_NegativeTaxidList).Match(p)) {
-    		m_NegativeTaxidList = v.GetInteger_list();
+#ifdef NCBI_STRICT_TAX_ID
+            const list<int>& int_list = v.GetInteger_list();
+            list<TTaxId> taxid_list;
+            ITERATE(list<int>, it, int_list) {
+                taxid_list.push_back(TAX_ID_FROM(int, *it));
+            }
+            m_NegativeTaxidList = taxid_list;
+#else
+            m_NegativeTaxidList = v.GetInteger_list();
+#endif
         } else {
             found = false;
         }
@@ -467,7 +476,16 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
         
     case 'T':
     	if (CBlast4Field::Get(eBlastOpt_TaxidList).Match(p)) {
-    		m_TaxidList = v.GetInteger_list();
+#ifdef NCBI_STRICT_TAX_ID
+            const list<int>& int_list = v.GetInteger_list();
+            list<TTaxId> taxid_list;
+            ITERATE(list<int>, it, int_list) {
+                taxid_list.push_back(TAX_ID_FROM(int, *it));
+            }
+            m_TaxidList = taxid_list;
+#else
+            m_TaxidList = v.GetInteger_list();
+#endif
     	} else {
     		found = false;
     	}
@@ -549,10 +567,11 @@ void CBlastOptionsBuilder::x_ApplyInteractions(CBlastOptionsHandle & boh)
 }
 
 /// Finder class for matching CBlast4_parameter
-struct SBlast4ParamFinder : public unary_function< CRef<CBlast4_parameter>, bool> {
+struct SBlast4ParamFinder
+{
     SBlast4ParamFinder(EBlastOptIdx opt_idx)
             : m_Target2Find(CBlast4Field::Get(opt_idx)) {}
-    result_type operator()(const argument_type& rhs) {
+    bool operator()(const CRef<CBlast4_parameter>& rhs) {
         return rhs.NotEmpty() ? m_Target2Find.Match(*rhs) : false;
     }
 private:
@@ -773,7 +792,7 @@ bool CBlastOptionsBuilder::HaveTaxidList()
 	return m_TaxidList.Have();
 }
 
-list<int> CBlastOptionsBuilder::GetTaxidList()
+list<TTaxId> CBlastOptionsBuilder::GetTaxidList()
 {
 	return m_TaxidList.Get();
 }
@@ -783,7 +802,7 @@ bool CBlastOptionsBuilder::HaveNegativeTaxidList()
 	return m_NegativeTaxidList.Have();
 }
 
-list<int> CBlastOptionsBuilder::GetNegativeTaxidList()
+list<TTaxId> CBlastOptionsBuilder::GetNegativeTaxidList()
 {
  return m_NegativeTaxidList.Get();
 }
