@@ -1,4 +1,4 @@
-/*  $Id: remote_blast.cpp 591152 2019-08-12 11:18:21Z fongah2 $ 
+/*  $Id: remote_blast.cpp 631493 2021-05-19 13:45:43Z ivanov $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -2037,13 +2037,13 @@ CRef<CBlastOptionsHandle> CRemoteBlast::GetSearchOptions()
         }
 
         if(bob.HaveTaxidList()){
-        	list<int> list = bob.GetTaxidList();
+        	list<TTaxId> list = bob.GetTaxidList();
         	m_TaxidList.clear();
         	m_TaxidList.insert(list.begin(), list.end());
         }
 
         if(bob.HaveNegativeTaxidList()){
-        	list<int> list = bob.GetNegativeTaxidList();
+        	list<TTaxId> list = bob.GetNegativeTaxidList();
         	m_NegativeTaxidList.clear();
         	m_NegativeTaxidList.insert(list.begin(), list.end());
         }
@@ -2270,7 +2270,7 @@ ExtractBlast4Request(CNcbiIstream& in)
 
         case CFormatGuess::eXml:
             {
-                auto_ptr<CObjectIStream> is(
+                unique_ptr<CObjectIStream> is(
                     CObjectIStream::Open(eSerial_Xml, in));
                 dynamic_cast<CObjectIStreamXml*>
                     (is.get())->SetEnforcedStdXml(true);
@@ -2308,7 +2308,7 @@ ExtractBlast4Request(CNcbiIstream& in)
 
     case CFormatGuess::eXml:
         {
-            auto_ptr<CObjectIStream> is(
+            unique_ptr<CObjectIStream> is(
                 CObjectIStream::Open(eSerial_Xml, in));
             dynamic_cast<CObjectIStreamXml*>
                 (is.get())->SetEnforcedStdXml(true);
@@ -2505,7 +2505,7 @@ CRemoteBlast::x_GetSearchResultsHTTP(void)
     int  l_total_bytes=0, n_read;
 	bool l_cached_ok = true;
 
-    auto_ptr<fstream> tmp_stream( CDirEntry::CreateTmpFile() );
+    unique_ptr<fstream> tmp_stream( CDirEntry::CreateTmpFile() );
     do{
 		n_read = CStreamUtils::Readsome(ios,incoming_buffer, read_max);
 		if( n_read > 0 ){
@@ -2543,7 +2543,7 @@ CRemoteBlast::x_GetSearchResultsHTTP(void)
     // read cached answer
 	swatch.Restart();
     try {
-		auto_ptr<CObjectIStream> 
+		unique_ptr<CObjectIStream> 
 	    in_stream( CObjectIStream::Open(eSerial_AsnBinary,  *tmp_stream) );
 		in_stream->Read(ObjectInfo(*one_reply), CObjectIStream::eNoFileHeader);
 
