@@ -1,4 +1,4 @@
-/*  $Id: search_strategy.cpp 591152 2019-08-12 11:18:21Z fongah2 $
+/*  $Id: search_strategy.cpp 631551 2021-05-19 13:51:56Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -445,9 +445,9 @@ void CExportStrategy::x_Process_SearchDb(CRef<CSearchDatabase> & db)
     if (!gi_list_limit.empty())
     {
 		#ifdef NCBI_STRICT_GI
-       		vector<int> int_gi_list;
+       		vector<Int8> int_gi_list;
        		ITERATE ( vector<TGi>, it, gi_list_limit ) {
-       			int_gi_list.push_back(GI_TO( int, *it));
+       			int_gi_list.push_back(GI_TO(Int8, *it));
        		}
        		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_GiList), int_gi_list);
 		#else
@@ -456,18 +456,26 @@ void CExportStrategy::x_Process_SearchDb(CRef<CSearchDatabase> & db)
     }
 
     CRef<CSeqDBGiList>& gilist = const_cast<CRef<CSeqDBGiList>& > (db->GetGiList());
-    if(gilist.NotEmpty()){
-    	vector<int> taxids((gilist->GetTaxIdsList().begin()), gilist->GetTaxIdsList().end());
-   		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_TaxidList), taxids);
+    if (gilist.NotEmpty()) {
+        vector<TTaxId> taxids((gilist->GetTaxIdsList().begin()), gilist->GetTaxIdsList().end());
+#ifdef NCBI_STRICT_TAX_ID
+        vector<int> int_taxids;
+        ITERATE(vector<TTaxId>, it, taxids) {
+            int_taxids.push_back(TAX_ID_TO(int, *it));
+        }
+        x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_TaxidList), int_taxids);
+#else
+        x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_TaxidList), taxids);
+#endif
     }
     // Set the negative GI list
     const CSearchDatabase::TGiList& neg_gi_list = db->GetNegativeGiListLimitation();
     if (!neg_gi_list.empty())
     {
 		#ifdef NCBI_STRICT_GI
-       		vector<int> int_gi_list;
+       		vector<Int8> int_gi_list;
        		ITERATE ( vector<TGi>, it, neg_gi_list) {
-       			int_gi_list.push_back(GI_TO( int, *it));
+       			int_gi_list.push_back(GI_TO(Int8, *it));
        		}
        		x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeGiList), int_gi_list);
 		#else
@@ -477,8 +485,16 @@ void CExportStrategy::x_Process_SearchDb(CRef<CSearchDatabase> & db)
 
     CRef<CSeqDBGiList>& neg_gilist = const_cast<CRef<CSeqDBGiList>& > (db->GetNegativeGiList());
     if(neg_gilist.NotEmpty()){
-       	vector<int> taxids((neg_gilist->GetTaxIdsList().begin()), neg_gilist->GetTaxIdsList().end());
-    	x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeTaxidList), taxids);
+       	vector<TTaxId> taxids((neg_gilist->GetTaxIdsList().begin()), neg_gilist->GetTaxIdsList().end());
+#ifdef NCBI_STRICT_TAX_ID
+        vector<int> int_taxids;
+        ITERATE(vector<TTaxId>, it, taxids) {
+            int_taxids.push_back(TAX_ID_TO(int, *it));
+        }
+        x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeTaxidList), int_taxids);
+#else
+        x_AddParameterToProgramOptions(CBlast4Field::Get(eBlastOpt_NegativeTaxidList), taxids);
+#endif
     }
 
     // Set the filtering algorithms
