@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.cpp 615351 2020-08-31 15:38:53Z fukanchi $
+/*  $Id: blast_app_util.cpp 632181 2021-05-27 13:23:25Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -182,6 +182,7 @@ InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
                   CRef<blast::CLocalDbAdapter>& db_adapter, 
                   CRef<objects::CScope>& scope)
 {
+	string dl = kEmptyStr;
     db_adapter.Reset();
 
     _ASSERT(db_args.NotEmpty());
@@ -224,10 +225,12 @@ InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
             // sequence data for formatting from this (local) source
             CRef<CSeqDB> seqdb = search_db->GetSeqDb();
             db_adapter.Reset(new CLocalDbAdapter(*search_db));
-            scope->AddDataLoader(RegisterOMDataLoader(seqdb));
+            dl = RegisterOMDataLoader(seqdb);
+            scope->AddDataLoader(dl);
         } catch (const CSeqDBException&) {
             // The BLAST database couldn't be found, report this for local
             // searches, but for remote searches go on.
+        	dl = kEmptyStr;
             if (is_remote_search ) {
                 db_adapter.Reset(new CLocalDbAdapter(*search_db));
             } else {
@@ -238,18 +241,15 @@ InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
 
     /// Set the BLASTDB data loader as the default data loader (if applicable)
     if (search_db.NotEmpty()) {
-        string dbloader_name =
-            s_FindBlastDbDataLoaderName(search_db->GetDatabaseName(),
-                                        search_db->IsProtein());
-        if ( !dbloader_name.empty() ) {
+        if ( dl != kEmptyStr) {
             // FIXME: will this work with multiple BLAST DBs?
-            scope->AddDataLoader(dbloader_name, 
-                             CBlastDatabaseArgs::kSubjectsDataLoaderPriority);
-            _TRACE("Setting " << dbloader_name << " priority to "
+            scope->AddDataLoader(dl,  CBlastDatabaseArgs::kSubjectsDataLoaderPriority);
+            _TRACE("Setting " << dl << " priority to "
                    << (int)CBlastDatabaseArgs::kSubjectsDataLoaderPriority
                    << " for subjects");
         }
     }
+    return;
 }
 
 string RegisterOMDataLoader(CRef<CSeqDB> db_handle)
@@ -482,8 +482,8 @@ s_ImportDatabase(const CBlast4_subject& subj,
         	}
         }
         if (opts_builder.HaveTaxidList()) {
-        	list<int>  list = opts_builder.GetTaxidList();
-           	set<int> taxids(list.begin(), list.end());
+        	list<TTaxId>  list = opts_builder.GetTaxidList();
+           	set<TTaxId> taxids(list.begin(), list.end());
         	gilist->AddTaxIds(taxids);
         }
         search_db->SetGiList(gilist);
@@ -497,8 +497,8 @@ s_ImportDatabase(const CBlast4_subject& subj,
        	    }
         }
         if (opts_builder.HaveNegativeTaxidList()) {
-        	list<int>  list = opts_builder.GetNegativeTaxidList();
-           	set<int> taxids(list.begin(), list.end());
+        	list<TTaxId>  list = opts_builder.GetNegativeTaxidList();
+           	set<TTaxId> taxids(list.begin(), list.end());
            	gilist->AddTaxIds(taxids);
         }
         search_db->SetNegativeGiList(gilist);
@@ -979,4 +979,76 @@ void LogRPSCmdOptions(blast::CBlastUsageReport & report, const CBlastAppArgs & a
 		report.AddParam(CBlastUsageReport::eOutputFmt, args.GetFormattingArgs()->GetFormattedOutputChoice());
 	}
 }
+
+int GetMTByQueriesBatchSize(EProgram p, int num_threads)
+{
+	    int batch_size = 0;
+
+		char * mt_query_batch_env = getenv("BLAST_MT_QUERY_BATCH_SIZE");
+		if (mt_query_batch_env) {
+			batch_size = NStr::StringToInt(mt_query_batch_env);
+		}
+		else {
+			int factor = 1;
+			if ( EProgramToEBlastProgramType(p) == eBlastTypeBlastn ) {
+				factor = 2;
+			}
+			batch_size = GetQueryBatchSize(p)/factor;
+		}
+		return batch_size;
+}
+
+void CheckMTByQueries_DBSize(CRef<CLocalDbAdapter> & db_adapter, const CBlastOptions & opt)
+{
+	CRef<CSearchDatabase> sdb = db_adapter->GetSearchDatabase();
+	const CRef<CSeqDBGiList>&  gi = sdb->GetGiList();
+	const CRef<CSeqDBGiList>& n_gi = sdb->GetNegativeGiList();
+
+	if (gi.NotEmpty() || n_gi.NotEmpty()) {
+		return;
+	}
+
+	EProgram prog = opt.GetProgram();
+	CRef<CSeqDB> seqdb = sdb->GetSeqDb();
+	Uint8 total_length = seqdb->GetTotalLength();
+	Uint8 length_limit = 0;
+
+	if (db_adapter->IsProtein()) {
+		static const Uint8 kMaxProtTotalLength = 2000000000;
+		length_limit = kMaxProtTotalLength;
+	}
+	else {
+		static const Uint8 kMaxNuclTotalLength = 14000000000;
+		length_limit = kMaxNuclTotalLength;
+	}
+
+	if(total_length > length_limit) {
+		string warn = "This database is probably too large to benefit from -mt_mode=1. " \
+					  "We suggest using -mt_mode=1 only if the database is less than";
+		if (db_adapter->IsProtein()) {
+			warn += " 2 billion residues ";
+		}
+		else {
+			warn += " 14 billion bases ";
+		}
+       	ERR_POST(Warning <<   warn + "or if the search is limited by an option such as -taxids, -taxidlist or -gilist.");
+	}
+	return;
+}
+
+void CheckMTByQueries_QuerySize(EProgram prog, int batch_size)
+{
+	string warning = "This set of queries is too small to fully benefit from the -mt_mode=1 option. "\
+			         "The total number of letters should be at least ";
+	warning += NStr::IntToString(batch_size);
+	EBlastProgramType p = EProgramToEBlastProgramType(prog);
+	if (Blast_QueryIsProtein(p)) {
+		warning += " residues per thread.";
+	}
+	else {
+		warning += " bases per thread.";
+	}
+    ERR_POST(Warning << warning);
+}
+
 END_NCBI_SCOPE
