@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.hpp 615351 2020-08-31 15:38:53Z fukanchi $
+/*  $Id: blast_app_util.hpp 632181 2021-05-27 13:23:25Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -319,6 +319,11 @@ void LogQueryInfo(blast::CBlastUsageReport & report, const blast::CBlastInput &
 void LogRPSBlastOptions(blast::CBlastUsageReport & report, const blast::CBlastOptions & opt);
 void LogRPSCmdOptions(blast::CBlastUsageReport & report, const blast::CBlastAppArgs & args);
 
+int GetMTByQueriesBatchSize(blast::EProgram p, int num_threads);
+
+void CheckMTByQueries_DBSize(CRef<blast::CLocalDbAdapter> & db_adapter, const blast::CBlastOptions & opt);
+void CheckMTByQueries_QuerySize(blast::EProgram prog, int batch_size);
+
 END_NCBI_SCOPE
 
 #endif /* APP__BLAST_APP_UTIL__HPP */
