@@ -1,4 +1,4 @@
-/*  $Id: blastn_app.cpp 615344 2020-08-31 15:37:55Z fukanchi $
+/*  $Id: blastn_app.cpp 632181 2021-05-27 13:23:25Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,7 +39,9 @@
 #include <algo/blast/blastinput/blastn_args.hpp>
 #include <algo/blast/api/objmgr_query_data.hpp>
 #include <algo/blast/format/blast_format.hpp>
+#include <util/profile/rtprofile.hpp>
 #include "blast_app_util.hpp"
+#include "blastn_node.hpp"
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 USING_NCBI_SCOPE;
@@ -70,6 +72,9 @@ private:
     /** @inheritDoc */
     virtual int Run();
 
+    int x_RunMTBySplitDB();
+    int x_RunMTBySplitQuery();
+
     /// This application's command line args
     CRef<CBlastnAppArgs> m_CmdLineArgs; 
     CBlastUsageReport m_UsageReport;
@@ -88,8 +93,26 @@ void CBlastnApp::Init()
 
 int CBlastnApp::Run(void)
 {
+	const CArgs& args = GetArgs();
+	CMTArgs mt_args(args);
+	if ((mt_args.GetMTMode() == CMTArgs::eSplitByQueries) &&
+		(mt_args.GetNumThreads() > 1)){
+		m_UsageReport.AddParam(CBlastUsageReport::eMTMode, CMTArgs::eSplitByQueries);
+		return x_RunMTBySplitQuery();
+	}
+	else {
+		return x_RunMTBySplitDB();
+	}
+}
+
+int CBlastnApp::x_RunMTBySplitDB()
+{
+    BLAST_PROF_START( APP.MAIN );
+    BLAST_PROF_START( APP.PRE );
+    BLAST_PROF_ADD2( PROGRAM, blastn ) ;
     int status = BLAST_EXIT_SUCCESS;
     CBlastAppDiagHandler bah;
+    int batch_num = 0;
 
     try {
 
@@ -185,6 +208,7 @@ int CBlastnApp::Run(void)
         int batch_size = m_CmdLineArgs->GetQueryBatchSize();
         if (batch_size) {
             input.SetBatchSize(batch_size);
+	    BLAST_PROF_ADD( BATCH_SIZE, (int)batch_size );
         } else {
             Int8 total_len = formatter.GetDbTotalLength();
             if (total_len > 0) {
@@ -192,9 +216,11 @@ int CBlastnApp::Run(void)
                 mixer.SetTargetHits(total_len / 3000);
             }
             input.SetBatchSize(mixer.GetBatchSize());
+	    BLAST_PROF_ADD( BATCH_SIZE, (int)mixer.GetBatchSize() );
         }
-        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
-
+	BLAST_PROF_STOP( APP.PRE );
+        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup() ) {
+	    BLAST_PROF_START( APP.LOOP.PRE );
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<IQueryFactory> queries(new CObjMgr_QueryFactory(*query_batch));
 
@@ -202,6 +228,7 @@ int CBlastnApp::Run(void)
 
             CRef<CSearchResultSet> results;
 
+	    BLAST_PROF_STOP( APP.LOOP.PRE );
             if (m_CmdLineArgs->ExecuteRemotely()) {
                 CRef<CRemoteBlast> rmt_blast = 
                     InitializeRemoteBlast(queries, db_args, opts_hndl,
@@ -209,13 +236,16 @@ int CBlastnApp::Run(void)
                           m_CmdLineArgs->GetClientId());
                 results = rmt_blast->GetResultSet();
             } else {
+	        BLAST_PROF_START( APP.LOOP.BLAST );
                 CLocalBlast lcl_blast(queries, opts_hndl, db_adapter);
                 lcl_blast.SetNumberOfThreads(m_CmdLineArgs->GetNumThreads());
+		        lcl_blast.SetBatchNumber( batch_num );
                 results = lcl_blast.Run();
                 if (!batch_size) 
                     input.SetBatchSize(mixer.GetBatchSize(lcl_blast.GetNumExtensions()));
+	        BLAST_PROF_STOP( APP.LOOP.BLAST );
             }
-
+	    BLAST_PROF_START( APP.LOOP.FMT );
             if (isArchiveFormat) {
                 formatter.WriteArchive(*queries, *opts_hndl, *results, 0, bah.GetMessages());
                 bah.ResetMessages();
@@ -226,8 +256,10 @@ int CBlastnApp::Run(void)
                     formatter.PrintOneResultSet(**result, query_batch);
                 }
             }
+	    BLAST_PROF_STOP( APP.LOOP.FMT );
+	    batch_num++;
         }
-
+        BLAST_PROF_START( APP.POST );
         formatter.PrintEpilog(opt);
 
         if (m_CmdLineArgs->ProduceDebugOutput()) {
@@ -236,6 +268,7 @@ int CBlastnApp::Run(void)
 
         LogQueryInfo(m_UsageReport, input);
         formatter.LogBlastSearchInfo(m_UsageReport);
+        BLAST_PROF_STOP( APP.POST );
     } CATCH_ALL(status)
 
     if(!bah.GetMessages().empty()) {
@@ -245,6 +278,80 @@ int CBlastnApp::Run(void)
 
 	m_UsageReport.AddParam(CBlastUsageReport::eNumThreads, (int) m_CmdLineArgs->GetNumThreads());
     m_UsageReport.AddParam(CBlastUsageReport::eExitStatus, status);
+    BLAST_PROF_STOP( APP.MAIN );
+    BLAST_PROF_ADD( THREADS , (int)m_CmdLineArgs->GetNumThreads() );
+    BLAST_PROF_ADD( BATCHES , (int)batch_num );
+    BLAST_PROF_ADD( EXIT_STATUS , (int)status );
+    BLAST_PROF_REPORT ;
+    return status;
+}
+
+int CBlastnApp::x_RunMTBySplitQuery()
+{
+    BLAST_PROF_START( APP.MAIN );
+    BLAST_PROF_START( APP.PRE );
+    BLAST_PROF_ADD2( PROGRAM, blastn ) ;
+    int status = BLAST_EXIT_SUCCESS;
+    CBlastAppDiagHandler bah;
+
+    // Allow the fasta reader to complain on invalid sequence input
+    SetDiagPostLevel(eDiag_Warning);
+    SetDiagPostPrefix("blastn");
+    SetDiagHandler(&bah, false);
+
+	try {
+    	const CArgs& args = GetArgs();
+    	CRef<CBlastOptionsHandle> opts_hndl;
+        if(RecoverSearchStrategy(args, m_CmdLineArgs)) {
+        	opts_hndl.Reset(&*m_CmdLineArgs->SetOptionsForSavedStrategy(args));
+        }
+        else {
+        	opts_hndl.Reset(&*m_CmdLineArgs->SetOptions(args));
+        }
+    	if(IsIStreamEmpty(m_CmdLineArgs->GetInputStream())){
+       		ERR_POST(Warning << "Query is Empty!");
+       		return BLAST_EXIT_SUCCESS;
+    	}
+    	CNcbiOstream & out_stream = m_CmdLineArgs->GetOutputStream();
+    	const int kMaxNumOfThreads = m_CmdLineArgs->GetNumThreads();
+		CBlastMasterNode master_node(out_stream, kMaxNumOfThreads);
+   		int chunk_num = 0;
+   	    int batch_size = GetMTByQueriesBatchSize(opts_hndl->GetOptions().GetProgram(), kMaxNumOfThreads);
+   		INFO_POST("Batch Size: " << batch_size);
+   		CBlastNodeInputReader input(m_CmdLineArgs->GetInputStream(), batch_size, 2000);
+		while (master_node.Processing()) {
+			if (!input.AtEOF()) {
+			 	if (!master_node.IsFull()) {
+					string qb;
+					int q_index = 0;
+					int num_q = input.GetQueryBatch(qb, q_index);
+					if (num_q > 0) {
+						CBlastNodeMailbox * mb(new CBlastNodeMailbox(chunk_num, master_node.GetBuzzer()));
+						CBlastnNode * t(new CBlastnNode(chunk_num, GetArguments(), args, bah, qb, q_index, num_q, mb));
+						master_node.RegisterNode(t, mb);
+						chunk_num ++;
+					}
+				}
+			}
+			else {
+				master_node.Shutdown();
+			}
+    	}
+
+		if(chunk_num < kMaxNumOfThreads){
+			CheckMTByQueries_QuerySize(opts_hndl->GetOptions().GetProgram(), batch_size);
+		}
+	} CATCH_ALL (status)
+
+    if(!bah.GetMessages().empty()) {
+    	const CArgs & a = GetArgs();
+    	PrintErrorArchive(a, bah.GetMessages());
+    }
+	    BLAST_PROF_STOP( APP.MAIN );
+    BLAST_PROF_ADD( THREADS , (int)m_CmdLineArgs->GetNumThreads() );
+    BLAST_PROF_ADD( EXIT_STATUS , (int)status );
+    BLAST_PROF_REPORT ;
+
     return status;
 }
 
