@@ -1,4 +1,4 @@
-/*  $Id: rpstblastn_app.cpp 615352 2020-08-31 15:39:03Z fukanchi $
+/*  $Id: rpstblastn_app.cpp 632182 2021-05-27 13:23:33Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,13 +97,15 @@ void CRPSTBlastnApp::Init()
 int CRPSTBlastnApp::Run(void)
 {
 	const CArgs& args = GetArgs();
-	if ((args[kArgMTMode].AsInteger() == 0)  || (args[kArgNumThreads].AsInteger() <= 1)){
-		return x_RunMTBySplitDB();
-	}
-	else {
+	CMTArgs mt_args(args);
+	if ((mt_args.GetMTMode() == CMTArgs::eSplitByQueries) &&
+			(mt_args.GetNumThreads() > 1)){
 		m_UsageReport.AddParam(CBlastUsageReport::eMTMode, args[kArgMTMode].AsInteger());
 		return x_RunMTBySplitQuery();
 	}
+	else {
+		return x_RunMTBySplitDB();
+	}
 }
 
 int CRPSTBlastnApp::x_RunMTBySplitDB(void)
@@ -246,7 +248,7 @@ int CRPSTBlastnApp::x_RunMTBySplitQuery(void)
 	if (mt_query_batch_env) {
 		batch_size = NStr::StringToInt(mt_query_batch_env);
 	}
-	cerr << "Batch Size: " << batch_size << endl;
+	INFO_POST("Batch Size: " << batch_size);
     // Allow the fasta reader to complain on invalid sequence input
     SetDiagPostLevel(eDiag_Warning);
     SetDiagPostPrefix("rpstblastn_mt");
@@ -254,7 +256,6 @@ int CRPSTBlastnApp::x_RunMTBySplitQuery(void)
 
 	try {
     	const CArgs& args = GetArgs();
-    	const int kMaxNumOfThreads = args[kArgNumThreads].AsInteger();
     	CRef<CBlastOptionsHandle> opts_hndl;
         if(RecoverSearchStrategy(args, m_CmdLineArgs)) {
         	opts_hndl.Reset(&*m_CmdLineArgs->SetOptionsForSavedStrategy(args));
@@ -267,6 +268,7 @@ int CRPSTBlastnApp::x_RunMTBySplitQuery(void)
        		return BLAST_EXIT_SUCCESS;
     	}
     	CNcbiOstream & out_stream = m_CmdLineArgs->GetOutputStream();
+    	const int kMaxNumOfThreads =  m_CmdLineArgs->GetNumThreads();
 		CBlastMasterNode master_node(out_stream, kMaxNumOfThreads);
    		int chunk_num = 0;
 
@@ -297,6 +299,10 @@ int CRPSTBlastnApp::x_RunMTBySplitQuery(void)
 
     	}
 
+
+		if(chunk_num < kMaxNumOfThreads){
+			CheckMTByQueries_QuerySize(opts_hndl->GetOptions().GetProgram(), batch_size);
+		}
 	} CATCH_ALL (status)
 
     if(!bah.GetMessages().empty()) {
