@@ -1,4 +1,4 @@
-/*  $Id: tblastn_app.cpp 616358 2020-09-15 12:19:53Z ivanov $
+/*  $Id: tblastn_app.cpp 632181 2021-05-27 13:23:25Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,7 @@
 #include <algo/blast/api/objmgr_query_data.hpp>
 #include <algo/blast/format/blast_format.hpp>
 #include "blast_app_util.hpp"
+#include "tblastn_node.hpp"
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 USING_NCBI_SCOPE;
@@ -69,6 +70,10 @@ private:
     virtual void Init();
     /** @inheritDoc */
     virtual int Run();
+
+    int x_RunMTBySplitDB();
+    int x_RunMTBySplitQuery();
+
     /// This application's command line args
     CRef<CTblastnAppArgs> m_CmdLineArgs;
     CBlastUsageReport m_UsageReport;
@@ -85,6 +90,20 @@ void CTblastnApp::Init()
 }
 
 int CTblastnApp::Run(void)
+{
+	const CArgs& args = GetArgs();
+	CMTArgs mt_args(args);
+	if ((mt_args.GetMTMode() == CMTArgs::eSplitByQueries) &&
+		(mt_args.GetNumThreads() > 1)){
+		m_UsageReport.AddParam(CBlastUsageReport::eMTMode, CMTArgs::eSplitByQueries);
+		return x_RunMTBySplitQuery();
+	}
+	else {
+		return x_RunMTBySplitDB();
+	}
+}
+
+int CTblastnApp::x_RunMTBySplitDB()
 {
     int status = BLAST_EXIT_SUCCESS;
     CBlastAppDiagHandler bah;
@@ -282,6 +301,67 @@ int CTblastnApp::Run(void)
     m_UsageReport.AddParam(CBlastUsageReport::eExitStatus, status);
     return status;
 }
+int CTblastnApp::x_RunMTBySplitQuery()
+{
+    int status = BLAST_EXIT_SUCCESS;
+    CBlastAppDiagHandler bah;
+
+    // Allow the fasta reader to complain on invalid sequence input
+    SetDiagPostLevel(eDiag_Warning);
+    SetDiagPostPrefix("tblastn");
+    SetDiagHandler(&bah, false);
+
+	try {
+    	const CArgs& args = GetArgs();
+    	CRef<CBlastOptionsHandle> opts_hndl;
+        if(RecoverSearchStrategy(args, m_CmdLineArgs)) {
+        	opts_hndl.Reset(&*m_CmdLineArgs->SetOptionsForSavedStrategy(args));
+        }
+        else {
+        	opts_hndl.Reset(&*m_CmdLineArgs->SetOptions(args));
+        }
+    	if(IsIStreamEmpty(m_CmdLineArgs->GetInputStream())){
+       		ERR_POST(Warning << "Query is Empty!");
+       		return BLAST_EXIT_SUCCESS;
+    	}
+    	CNcbiOstream & out_stream = m_CmdLineArgs->GetOutputStream();
+    	const int kMaxNumOfThreads = m_CmdLineArgs->GetNumThreads();
+		CBlastMasterNode master_node(out_stream, kMaxNumOfThreads);
+   		int chunk_num = 0;
+   	    int batch_size = GetMTByQueriesBatchSize(opts_hndl->GetOptions().GetProgram(), kMaxNumOfThreads);
+   		INFO_POST("Batch Size: " << batch_size);
+   		CBlastNodeInputReader input(m_CmdLineArgs->GetInputStream(), batch_size, 2000);
+		while (master_node.Processing()) {
+			if (!input.AtEOF()) {
+			 	if (!master_node.IsFull()) {
+					string qb;
+					int q_index = 0;
+					int num_q = input.GetQueryBatch(qb, q_index);
+					if (num_q > 0) {
+						CBlastNodeMailbox * mb(new CBlastNodeMailbox(chunk_num, master_node.GetBuzzer()));
+						CTblastnNode * t(new CTblastnNode(chunk_num, GetArguments(), args, bah, qb, q_index, num_q, mb));
+						master_node.RegisterNode(t, mb);
+						chunk_num ++;
+					}
+				}
+			}
+			else {
+				master_node.Shutdown();
+			}
+    	}
+
+		if(chunk_num < kMaxNumOfThreads){
+			CheckMTByQueries_QuerySize(opts_hndl->GetOptions().GetProgram(), batch_size);
+		}
+	} CATCH_ALL (status)
+
+    if(!bah.GetMessages().empty()) {
+    	const CArgs & a = GetArgs();
+    	PrintErrorArchive(a, bah.GetMessages());
+    }
+    return status;
+}
+
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 int NcbiSys_main(int argc, ncbi::TXChar* argv[])
