@@ -1,4 +1,4 @@
-/*  $Id: blastdb_convert.cpp 615364 2020-08-31 15:40:14Z fukanchi $
+/*  $Id: blastdb_convert.cpp 611918 2020-07-13 16:49:16Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -429,7 +429,7 @@ int CBlastdbConvertApp::Run(void)
              for (int oid = 0; oid < kNumOids; oid++) {
                   list<CRef<CSeq_id>> ids = vol->GetSeqIDs(oid);
                   lmdbdb->InsertEntries(ids, oid_total+oid);
-            	  set<int> tax_ids;
+            	  set<TTaxId> tax_ids;
             	  vol->GetAllTaxIDs(oid, tax_ids);
                   taxdb->InsertEntries(tax_ids, oid_total+oid);
              }
