@@ -1,4 +1,4 @@
-/*  $Id: blastdbcp.cpp 615363 2020-08-31 15:40:04Z fukanchi $
+/*  $Id: blastdbcp.cpp 617977 2020-10-08 18:29:47Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -287,7 +287,7 @@ public:
                     CBlast_def_line::TTaxIds leafs = bdl->GetLeafTaxIds();
                     const string id =
                             bdl->GetSeqid().front()->AsFastaString();
-                    set<int> ids = m_LeafTaxids[id];
+                    set<TTaxId> ids = m_LeafTaxids[id];
                     ids.insert(leafs.begin(), leafs.end());
                     m_LeafTaxids[id] = ids;
                 }
@@ -616,12 +616,12 @@ int BlastdbCopyApplication::Run(void)
 
     }}
     catch (CException& ex) {
-        LOG_POST( Error << ex );
+        ERR_POST( Error << ex );
         DeleteBlastDb(args[kArgOutput].AsString(), seq_type);
         retval = -1;
     }
     catch (...) {
-        LOG_POST( Error << "Unknown error in BlastdbCopyApplication::Run()" );
+        ERR_POST( Error << "Unknown error in BlastdbCopyApplication::Run()" );
         DeleteBlastDb(args[kArgOutput].AsString(), seq_type);
         retval = -2;
     }
