@@ -1,6 +1,6 @@
 #!/bin/sh
 #############################################################################
-# $Id: cmake-cfg-unix.sh 609379 2020-06-01 14:15:14Z ivanov $
+# $Id: cmake-cfg-unix.sh 620700 2020-11-27 14:08:44Z gouriano $
 #   Configure NCBI C++ toolkit using CMake build system.
 #   Author: Andrei Gourianov, gouriano@ncbi
 #############################################################################
@@ -8,18 +8,24 @@ initial_dir=`pwd`
 script_name=`basename $0`
 script_dir=`dirname $0`
 script_dir=`(cd "${script_dir}" ; pwd)`
+script_args="$@"
 tree_root=`pwd`
 extension="cmake_configure_ext.sh"
-NCBI_EXPERIMENTAL="ON"
 
 host_os=`uname`
 if test -z "${CMAKE_CMD}" -a $host_os = "Darwin"; then
   CMAKE_CMD=/Applications/CMake.app/Contents/bin/cmake
+  if test ! -x $CMAKE_CMD; then
+    CMAKE_CMD=/sw/bin/cmake
+    if test ! -x $CMAKE_CMD; then
+      CMAKE_CMD=""
+    fi
+  fi
 fi
 if [ -z "${CMAKE_CMD}" ]; then
   CMAKE_CMD=`which cmake 2>/dev/null`
   if test $? -ne 0; then
-    echo ERROR: CMake is not found
+    echo ERROR: CMake is not found 1>&2
     exit 1
   fi
 fi
@@ -34,19 +40,23 @@ SKIP_ANALYSIS="OFF"
 
 ############################################################################# 
 Check_function_exists() {
-  t=`type -t $1`
+  t=`type -t $1 2>/dev/null`
   test "$t" = "function"
 }
 
 Usage()
 {
-    cat <<EOF 1>&2
+    cat <<EOF
 USAGE:
-  $script_name [OPTIONS]...
+  $script_name [compiler] [OPTIONS]...
 SYNOPSIS:
   Configure NCBI C++ toolkit using CMake build system.
 OPTIONS:
   --help                     -- print Usage
+  compiler                   -- compiler name and version
+                                'name' is one of GCC, ICC, Clang, Xcode
+                                'version' depends on 'name'
+                    examples:   GCC 7.3.0,  ICC 19.0,  Clang 7.0.0,  Xcode
   --without-debug            -- build release versions of libs and apps
   --with-debug               -- build debug versions of libs and apps (default)
   --with-max-debug           -- enable extra runtime checks (esp. of STL usage)
@@ -95,7 +105,10 @@ EOF
 Error()
 {
   Usage
-  test -z "$1"  ||  echo ERROR: $1 1>&2
+  if [ -n "$1" ]; then
+    echo ----------------------------------------------------------------------
+    echo ERROR: $1 1>&2
+  fi
   cd "$initial_dir"
   exit 1
 }
@@ -108,17 +121,20 @@ Quote() {
 # parse arguments
 
 do_help="no"
+cxx_name=""
+cxx_version=""
 unknown=""
-while [ $# -ne 0 ]; do
-  case "$1" in 
+for arg in ${script_args}
+do
+  case "$arg" in 
     --help|-help|help|-h)
       do_help="yes"
     ;; 
     --rootdir=*)
-      tree_root=`(cd "${1#*=}" ; pwd)`
+      tree_root=`(cd "${arg#*=}" ; pwd)`
       ;; 
     --caller=*)
-      script_name=${1#*=}
+      script_name=${arg#*=}
       ;; 
     --with-static|--without-dll) 
       BUILD_SHARED_LIBS=OFF
@@ -155,51 +171,59 @@ while [ $# -ne 0 ]; do
       SKIP_ANALYSIS="ON"
       ;;
     --with-projects=*)
-      PROJECT_LIST=${1#*=}
+      PROJECT_LIST=${arg#*=}
       if [ -f "${tree_root}/$PROJECT_LIST" ]; then
         PROJECT_LIST="${tree_root}/$PROJECT_LIST"
       fi
       ;; 
     --with-tags=*)
-      PROJECT_TAGS=${1#*=}
+      PROJECT_TAGS=${arg#*=}
       if [ -f "${tree_root}/$PROJECT_TAGS" ]; then
         PROJECT_TAGS="${tree_root}/$PROJECT_TAGS"
       fi
       ;; 
     --with-targets=*)
-      PROJECT_TARGETS=${1#*=}
+      PROJECT_TARGETS=${arg#*=}
       if [ -f "${tree_root}/$PROJECT_TARGETS" ]; then
         PROJECT_TARGETS="${tree_root}/$PROJECT_TARGETS"
       fi
       ;; 
     --with-details=*)
-      PROJECT_DETAILS=${1#*=}
+      PROJECT_DETAILS=${arg#*=}
       ;; 
     --with-install=*)
-      INSTALL_PATH=${1#*=}
+      INSTALL_PATH=${arg#*=}
       ;; 
     --with-generator=*)
-      CMAKE_GENERATOR=${1#*=}
+      CMAKE_GENERATOR=${arg#*=}
       ;; 
     --with-components=*)
-      PROJECT_COMPONENTS=${1#*=}
+      PROJECT_COMPONENTS=${arg#*=}
       ;; 
     --with-features=*)
-      PROJECT_FEATURES="$PROJECT_FEATURES;${1#*=}"
+      PROJECT_FEATURES="$PROJECT_FEATURES;${arg#*=}"
       ;; 
     --with-build-root=*)
-      BUILD_ROOT=${1#*=}
+      BUILD_ROOT=${arg#*=}
       ;; 
     --with-prebuilt=*)
-      prebuilt_path=${1#*=}
+      prebuilt_path=${arg#*=}
       prebuilt_dir=`dirname $prebuilt_path`
       prebuilt_name=`basename $prebuilt_path`
       ;; 
+    [A-Z]*)
+      cxx_name=$arg
+      ;; 
+    [1-9]*)
+      cxx_version=$arg
+      ;; 
+    -D*)
+      CMAKE_ARGS="$CMAKE_ARGS $arg"
+      ;; 
     *) 
-      unknown="$unknown $1"
+      unknown="$unknown $arg"
       ;; 
   esac 
-  shift 
 done 
 
 if [ -f $tree_root/$extension ]; then
@@ -211,6 +235,20 @@ if [ $do_help = "yes" ]; then
   exit 0
 fi
 
+if [ -z "$CMAKECFGRECURSIONGUARD" -a -n "$cxx_name" ]; then
+  if [ "$cxx_name" = "Xcode" ]; then
+    exec $script_dir/cmake-cfg-xcode.sh  "$@"
+    exit $?
+  elif [ -x "$script_dir/cm${cxx_name}.sh" ]; then
+    export CMAKECFGRECURSIONGUARD="lock"
+    exec $script_dir/cm${cxx_name}.sh  "$@"
+    exit $?
+  else
+    echo ERROR: configuration script for compiler $cxx_name not found 1>&2
+    exit 1
+  fi
+fi
+
 if [ -n "$unknown" ]; then
   Check_function_exists configure_ext_ParseArgs
   if [ $? -eq 0 ]; then
@@ -226,8 +264,7 @@ if [ -n "$prebuilt_dir" ]; then
   if [ -f $prebuilt_dir/$prebuilt_name/cmake/buildinfo ]; then
     source $prebuilt_dir/$prebuilt_name/cmake/buildinfo
   else
-    echo ERROR:  Buildinfo not found in $prebuilt_dir/$prebuilt_name
-    exit 1
+    Error "Buildinfo not found in $prebuilt_dir/$prebuilt_name"
   fi
 fi
 
@@ -252,13 +289,19 @@ else
   if [ -z "$CC" ]; then
     CC=`which gcc 2>/dev/null`
     if test $? -ne 0; then
-      CC=""
+      CC=`which cc 2>/dev/null`
+      if test $? -ne 0; then
+        CC=""
+      fi
     fi
   fi
   if [ -z "$CXX" ]; then
     CXX=`which g++ 2>/dev/null`
     if test $? -ne 0; then
-      CXX=""
+      CXX=`which c++ 2>/dev/null`
+      if test $? -ne 0; then
+        CXX=""
+      fi
     fi
   fi
 fi
@@ -268,10 +311,14 @@ if [ -n "$CC" ]; then
   case "$ccname" in
     clang*)
       CC_NAME="Clang"
-      CC_VERSION=`$CC --version 2>/dev/null | awk 'NR==1{print $3}' | sed 's/[.]//g'`
+      if test $host_os = "Darwin"; then
+        CC_VERSION=`$CC --version 2>/dev/null | awk 'NR==1{print $4}' | sed 's/[.]//g'`
+      else
+        CC_VERSION=`$CC --version 2>/dev/null | awk 'NR==1{print $3}' | sed 's/[.]//g'`
+      fi
     ;;
     *)
-      if test $host_os = "Darwin"; then
+      if test $host_os = "Darwin" -o $host_os = "FreeBSD"; then
         CC_NAME=`$CC --version 2>/dev/null | awk 'NR==1{print $2}'`
         CC_VERSION=`$CC --version 2>/dev/null | awk 'NR==1{print $4}' | sed 's/[.]//g'`
         if [ $CC_NAME = "clang" ]; then
@@ -294,8 +341,6 @@ else
 fi
 ############################################################################# 
 
-CMAKE_ARGS="$CMAKE_ARGS -DNCBI_EXPERIMENTAL=$NCBI_EXPERIMENTAL"
-
 if [ -n "$CC" ]; then
   CMAKE_ARGS="$CMAKE_ARGS -DCMAKE_C_COMPILER=$(Quote "$CC")"
 fi
@@ -349,7 +394,10 @@ if false; then
   echo mkdir -p ${BUILD_ROOT}/build 
   echo Running "${CMAKE_CMD}" ${CMAKE_ARGS} "${tree_root}/src"
 else
-mkdir -p ${BUILD_ROOT}/build 
+mkdir -p ${BUILD_ROOT}/build
+if [ ! -d ${BUILD_ROOT}/build ]; then
+  Error "Failed to create directory: ${BUILD_ROOT}/build"
+fi
 cd ${BUILD_ROOT}/build 
 
 if [ -f "${tree_root}/CMakeLists.txt" ]; then
