@@ -1,6 +1,6 @@
 #!/bin/sh
 #############################################################################
-# $Id: cmake-cfg-xcode.sh 609379 2020-06-01 14:15:14Z ivanov $
+# $Id: cmake-cfg-xcode.sh 612738 2020-07-27 12:30:34Z gouriano $
 #   Configure NCBI C++ toolkit for XCode using CMake build system.
 #   Author: Andrei Gourianov, gouriano@ncbi
 #############################################################################
@@ -16,10 +16,13 @@ if [ -z "${CMAKE_CMD}" ]; then
   CMAKE_CMD=/Applications/CMake.app/Contents/bin/cmake
 fi
 if test ! -x $CMAKE_CMD; then
-  CMAKE_CMD=`which cmake 2>/dev/null`
-  if test $? -ne 0; then
-    echo ERROR: CMake is not found
-    exit 1
+  CMAKE_CMD=/sw/bin/cmake
+  if test ! -x $CMAKE_CMD; then
+    CMAKE_CMD=`which cmake 2>/dev/null`
+    if test $? -ne 0; then
+      echo ERROR: CMake is not found 1>&2
+      exit 1
+    fi
   fi
 fi
 
@@ -36,7 +39,7 @@ Check_function_exists() {
 
 Usage()
 {
-    cat <<EOF 1>&2
+    cat <<EOF
 USAGE:
   $script_name [OPTIONS]...
 SYNOPSIS:
@@ -84,7 +87,10 @@ EOF
 Error()
 {
   Usage
-  test -z "$1"  ||  echo ERROR: $1 1>&2
+  if [ -n "$1" ]; then
+    echo ----------------------------------------------------------------------
+    echo ERROR: $1 1>&2
+  fi
   cd "$initial_dir"
   exit 1
 }
@@ -156,6 +162,11 @@ while [ $# != 0 ]; do
     --without-analysis)
       SKIP_ANALYSIS="ON"
       ;;
+    [A-Z]*)
+      ;; 
+    -D*)
+      CMAKE_ARGS="$CMAKE_ARGS $1"
+      ;; 
     *) 
       unknown="$unknown $1"
       ;; 
@@ -187,22 +198,21 @@ if [ -n "$prebuilt_dir" ]; then
   if [ -f $prebuilt_dir/$prebuilt_name/cmake/buildinfo ]; then
     source $prebuilt_dir/$prebuilt_name/cmake/buildinfo
   else
-    echo ERROR:  Buildinfo not found in $prebuilt_dir/$prebuilt_name
-    exit 1
+    Error "Buildinfo not found in $prebuilt_dir/$prebuilt_name"
   fi
 fi
 
 ############################################################################# 
 XC=`which xcodebuild 2>/dev/null`
 if test $? -ne 0; then
-  echo ERROR: xcodebuild is not found
+  echo ERROR: xcodebuild is not found 1>&2
   exit 1
 fi
 CC_NAME=Xcode
 CC_VERSION=`xcodebuild -version | awk 'NR==1{print $2}'`
 ############################################################################# 
 
-CMAKE_ARGS="-DNCBI_EXPERIMENTAL=ON -G Xcode"
+CMAKE_ARGS="$CMAKE_ARGS -G Xcode"
 
 CMAKE_ARGS="$CMAKE_ARGS  -DNCBI_PTBCFG_PROJECT_COMPONENTS=$(Quote "${PROJECT_COMPONENTS}")"
 CMAKE_ARGS="$CMAKE_ARGS  -DNCBI_PTBCFG_PROJECT_FEATURES=$(Quote "${PROJECT_FEATURES}")"
@@ -226,6 +236,9 @@ fi
 cd ${tree_root}
 Check_function_exists configure_ext_PreCMake && configure_ext_PreCMake
 mkdir -p ${BUILD_ROOT}/build 
+if [ ! -d ${BUILD_ROOT}/build ]; then
+  Error "Failed to create directory: ${BUILD_ROOT}/build"
+fi
 cd ${BUILD_ROOT}/build 
 
 #echo Running "${CMAKE_CMD}" ${CMAKE_ARGS} "${tree_root}/src"
