@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_stream.cpp 594886 2019-10-10 16:37:09Z lavr $
+/* $Id: ncbi_conn_stream.cpp 624530 2021-02-01 13:22:09Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,15 +35,13 @@
 #include <ncbi_pch.hpp>
 #include "ncbi_ansi_ext.h"
 #include "ncbi_conn_streambuf.hpp"
+#include "ncbi_servicep.h"
 #include "ncbi_socketp.h"
 #include <connect/ncbi_conn_exception.hpp>
-#define NCBI_CONN_STREAM_EXPERIMENTAL_API  1  // Pick up MS-Win DLL linkage
 #include <connect/ncbi_conn_stream.hpp>
 #include <connect/ncbi_file_connector.h>
-#include <connect/ncbi_socket.hpp>
 #include <connect/ncbi_util.h>
 #include <stdlib.h>
-#include <string.h>
 
 
 BEGIN_NCBI_SCOPE
@@ -58,7 +56,9 @@ CConn_IOStream::CConn_IOStream(const TConnector& connector,
                                                   timeout, buf_size, flgs,
                                                   ptr, size))
 {
-    x_Init(m_CSb->GetCONN(), flgs);
+    init(Status() != eIO_Success
+         ? 0  // according to the standard (27.4.4.1.3), badbit is set here
+         : m_CSb);
 }
 
 
@@ -70,7 +70,9 @@ CConn_IOStream::CConn_IOStream(CONN conn, bool close,
                                                   timeout, buf_size, flgs,
                                                   ptr, size))
 {
-    x_Init(conn, flgs);
+    init(Status() != eIO_Success
+         ? 0  // according to the standard (27.4.4.1.3), badbit is set here
+         : m_CSb);
 }
 
 
@@ -112,14 +114,14 @@ EIO_Status CConn_IOStream::SetTimeout(EIO_Event       direction,
                                       const STimeout* timeout) const
 {
     CONN conn = GET_CONN(m_CSb);
-    return conn ? CONN_SetTimeout(conn, direction, timeout) : eIO_Closed;
+    return conn ? CONN_SetTimeout(conn, direction, timeout) : eIO_NotSupported;
 }
 
 
 const STimeout* CConn_IOStream::GetTimeout(EIO_Event direction) const
 {
     CONN conn = GET_CONN(m_CSb);
-    return conn ? CONN_GetTimeout(conn, direction) : 0;
+    return conn ? CONN_GetTimeout(conn, direction) : kDefaultTimeout;
 }
 
 
@@ -129,34 +131,46 @@ EIO_Status CConn_IOStream::Status(EIO_Event dir) const
 }
 
 
+EIO_Status CConn_IOStream::Fetch(const STimeout* timeout)
+{
+    EIO_Status status;
+    CONN conn = GET_CONN(m_CSb);
+    if (!conn) {
+        setstate(NcbiBadbit);
+        status = eIO_NotSupported;
+    } else
+        status = m_CSb->Fetch(timeout);
+    return status;
+}
+
+
 EIO_Status CConn_IOStream::Pushback(const CT_CHAR_TYPE* data, streamsize size)
 {
-    return m_CSb ? m_CSb->Pushback(data, size) : eIO_NotSupported;
+    EIO_Status status = m_CSb ? m_CSb->Pushback(data, size) : eIO_NotSupported;
+    if (status != eIO_Success)
+        clear(NcbiBadbit);
+    return status;
 }
 
+
+SOCK CConn_IOStream::GetSOCK(void)
+{
+    SOCK sock;
+    CONN conn = GET_CONN(m_CSb);
+    if (!conn  ||  CONN_GetSOCK(conn, &sock) != eIO_Success)
+        sock = 0;
+    return sock;
+}
+
+
 EIO_Status CConn_IOStream::Close(void)
 {
-    return m_CSb ? m_CSb->Close() : eIO_Closed;
-}
-
-
-void CConn_IOStream::x_Init(CONN conn, TConn_Flags flgs)
-{
-    if (conn) {
-        EIO_Status status;
-        if (!(flgs & fConn_DelayOpen)) {
-            SOCK s/*dummy*/;
-            // NB: CONN_Write(0 bytes) could have caused the same effect
-            (void) CONN_GetSOCK(conn, &s);  // Prompt CONN to actually open
-            status  = CONN_Status(conn, eIO_Open);
-        } else
-            status  = eIO_Success;
-        if (status == eIO_Success) {
-            init(m_CSb);
-            return;
-        }
-    }
-    init(0); // according to the standard (27.4.4.1.3), badbit is set here
+    if (!m_CSb)
+        return eIO_Closed;
+    EIO_Status status = m_CSb->Close();
+    if (status != eIO_Success  &&  status != eIO_Closed)
+        clear(NcbiBadbit);
+    return status;
 }
 
 
@@ -172,7 +186,7 @@ EIO_Status CConn_IOStream::SetCanceledCallback(const ICanceled* canceled)
 {
     CONN conn = GET_CONN(m_CSb);
     if (!conn)
-        return eIO_Closed;
+        return eIO_NotSupported;
 
     bool isset = m_Canceled.NotNull() ? 1 : 0;
 
@@ -266,107 +280,6 @@ CConn_SocketStream::CConn_SocketStream(SOCK            sock,
 }
 
 
-static CConn_IOStream::TConnector
-s_SocketConnectorBuilder(const SConnNetInfo* net_info,
-                         const STimeout*     timeout,
-                         const void*         data,
-                         size_t              size,
-                         TSOCK_Flags         flgs)
-{
-    EIO_Status status = eIO_Success;
-    bool       proxy = false;
-    SOCK       sock = 0;
-
-    _ASSERT(net_info);
-    if ((flgs & (fSOCK_LogOn | fSOCK_LogDefault)) == fSOCK_LogDefault
-        &&  net_info->debug_printout == eDebugPrintout_Data) {
-        flgs &= ~fSOCK_LogDefault;
-        flgs |=  fSOCK_LogOn;
-    }
-    if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port) {
-        status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
-        _ASSERT(!sock ^ !(status != eIO_Success));
-        if (status == eIO_Success
-            &&  (size  ||  (flgs & ~(fSOCK_LogOn | fSOCK_LogDefault)))) {
-            SSOCK_Init init;
-            memset(&init, 0, sizeof(init));
-            init.data = data;
-            init.size = size;
-            init.host = net_info->host;
-            init.cred = net_info->credentials;
-            SOCK s;
-            status = SOCK_CreateOnTopInternal(sock, 0, &s, &init, flgs);
-            _ASSERT(!s ^ !(status != eIO_Success));
-            SOCK_Destroy(sock);
-            sock = s;
-        }
-        proxy = true;
-    }
-    if (!sock  &&  (!proxy  ||  net_info->http_proxy_leak)) {
-        if (timeout == kDefaultTimeout)
-            timeout  = net_info->timeout;
-        if (!proxy  &&  net_info->debug_printout) {
-            SConnNetInfo* x_net_info = ConnNetInfo_Clone(net_info);
-            if (x_net_info) {
-                x_net_info->scheme = eURL_Unspec;
-                x_net_info->req_method = eReqMethod_Any;
-                x_net_info->firewall = 0;
-                x_net_info->stateless = 0;
-                x_net_info->lb_disable = 0;
-                x_net_info->user[0] = '\0';
-                x_net_info->pass[0] = '\0';
-                x_net_info->path[0] = '\0';
-                x_net_info->http_proxy_host[0] = '\0';
-                x_net_info->http_proxy_port    =   0;
-                x_net_info->http_proxy_user[0] = '\0';
-                x_net_info->http_proxy_pass[0] = '\0';
-                ConnNetInfo_SetUserHeader(x_net_info, 0);
-                if (x_net_info->http_referer) {
-                    free((void*) x_net_info->http_referer);
-                    x_net_info->http_referer = 0;
-                }
-                x_net_info->timeout = timeout;
-            }
-            ConnNetInfo_Log(x_net_info, eLOG_Note, CORE_GetLOG());
-            ConnNetInfo_Destroy(x_net_info);
-        }
-        SSOCK_Init init;
-        memset(&init, 0, sizeof(init));
-        init.data = data;
-        init.size = size;
-        init.host = net_info->host;
-        init.cred = net_info->credentials;
-        status = SOCK_CreateInternal(net_info->host, net_info->port, timeout,
-                                     &sock, &init, flgs);
-        _ASSERT(!sock ^ !(status != eIO_Success));
-    }
-    string hostport(net_info->host);
-    hostport += ':';
-    hostport += NStr::UIntToString(net_info->port);
-    CONNECTOR c = SOCK_CreateConnectorOnTopEx(sock, 1/*own*/,hostport.c_str());
-    if (!c) {
-        SOCK_Abort(sock);
-        SOCK_Close(sock);
-        status = eIO_Unknown;
-    }
-    return CConn_IOStream::TConnector(c, status);
-}
-
-
-CConn_SocketStream::CConn_SocketStream(const SConnNetInfo& net_info,
-                                       const void*         data,
-                                       size_t              size,
-                                       TSOCK_Flags         flgs,
-                                       const STimeout*     timeout,
-                                       size_t              buf_size)
-    : CConn_IOStream(s_SocketConnectorBuilder(&net_info, timeout,
-                                              data, size, flgs),
-                     timeout, buf_size)
-{
-    return;
-}
-
-
 static SOCK s_GrabSOCK(CSocket& socket)
 {
     SOCK sock = socket.GetSOCK();
@@ -406,6 +319,139 @@ struct Deleter<SConnNetInfo>
 };
 
 
+static CConn_IOStream::TConnector
+s_SocketConnectorBuilder(const SConnNetInfo* net_info,
+                         const STimeout*     timeout,
+                         const void*         data,
+                         size_t              size,
+                         TSOCK_Flags         flgs)
+{
+    EIO_Status status = eIO_Success;
+    bool       proxy = false;
+    SOCK       sock = 0;
+
+    _ASSERT(net_info);
+    if ((flgs & (fSOCK_LogOn | fSOCK_LogDefault)) == fSOCK_LogDefault
+        &&  net_info->debug_printout == eDebugPrintout_Data) {
+        flgs &= ~fSOCK_LogDefault;
+        flgs |=  fSOCK_LogOn;
+    }
+    if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port) {
+        status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
+        _ASSERT(!sock ^ !(status != eIO_Success));
+        if (status == eIO_Success
+            &&  (size  ||  (flgs & ~(fSOCK_LogOn | fSOCK_LogDefault)))) {
+            SSOCK_Init init;
+            memset(&init, 0, sizeof(init));
+            init.data = data;
+            init.size = size;
+            init.host = net_info->host;
+            init.cred = net_info->credentials;
+            SOCK s;
+            status = SOCK_CreateOnTopInternal(sock, 0, &s, &init, flgs);
+            _ASSERT(!s ^ !(status != eIO_Success));
+            SOCK_Destroy(sock);
+            sock = s;
+        }
+        proxy = true;
+    }
+    if (!sock  &&  (!proxy  ||  net_info->http_proxy_leak)) {
+        if (timeout == kDefaultTimeout)
+            timeout  = net_info->timeout;
+        if (!proxy  &&  net_info->debug_printout) {
+            AutoPtr<SConnNetInfo> x_net_info(ConnNetInfo_Clone(net_info));
+            if (x_net_info.get()) {
+                // manual cleanup of most fields req'd
+                x_net_info->scheme = eURL_Unspec;
+                x_net_info->req_method = eReqMethod_Any;
+                x_net_info->external = 0;
+                x_net_info->firewall = 0;
+                x_net_info->stateless = 0;
+                x_net_info->lb_disable = 0;
+                x_net_info->http_version = 0;
+                x_net_info->http_push_auth = 0;
+                x_net_info->http_proxy_leak = 0;
+                x_net_info->user[0] = '\0';
+                x_net_info->pass[0] = '\0';
+                x_net_info->path[0] = '\0';
+                x_net_info->http_proxy_host[0] = '\0';
+                x_net_info->http_proxy_port    =   0;
+                x_net_info->http_proxy_user[0] = '\0';
+                x_net_info->http_proxy_pass[0] = '\0';
+                ConnNetInfo_SetUserHeader(x_net_info.get(), 0);
+                if (x_net_info->http_referer) {
+                    free((void*) x_net_info->http_referer);
+                    x_net_info->http_referer = 0;
+                }
+                x_net_info->timeout = timeout;
+            }
+            ConnNetInfo_Log(x_net_info.get(), eLOG_Note, CORE_GetLOG());
+        }
+        SSOCK_Init init;
+        memset(&init, 0, sizeof(init));
+        init.data = data;
+        init.size = size;
+        init.host = net_info->host;
+        init.cred = net_info->credentials;
+        status = SOCK_CreateInternal(net_info->host, net_info->port, timeout,
+                                     &sock, &init, flgs);
+        _ASSERT(!sock ^ !(status != eIO_Success));
+    }
+    string hostport(net_info->host);
+    hostport += ':';
+    hostport += NStr::UIntToString(net_info->port);
+    CONNECTOR c = SOCK_CreateConnectorOnTopEx(sock, 1/*own*/,hostport.c_str());
+    if (!c) {
+        SOCK_Abort(sock);
+        SOCK_Close(sock);
+        status = eIO_Unknown;
+    }
+    return CConn_IOStream::TConnector(c, status);
+}
+
+
+CConn_SocketStream::CConn_SocketStream(const SConnNetInfo& net_info,
+                                       const void*         data,
+                                       size_t              size,
+                                       TSOCK_Flags         flgs,
+                                       const STimeout*     timeout,
+                                       size_t              buf_size)
+    : CConn_IOStream(s_SocketConnectorBuilder(&net_info, timeout,
+                                              data, size, flgs),
+                     timeout, buf_size)
+{
+    return;
+}
+
+
+// NB:  Must never be upcalled (directly or indirectly) from any stream ctor!
+EHTTP_HeaderParse SHTTP_StatusData::Parse(const char* header)
+{
+    int c, n;
+    m_Code = 0;
+    m_Text.clear();
+    m_Header = header;
+    if (sscanf(header, "%*s %u%n", &c, &n) < 1)
+        return eHTTP_HeaderError;
+    const char* str = m_Header.c_str() + n;
+    str += strspn(str, " \t");
+    const char* eol = strchr(str, '\n');
+    if (!eol)
+        eol = str + strlen(str);
+    while (eol > str) {
+        if (!isspace((unsigned char) eol[-1]))
+            break;
+        --eol;
+    }
+    m_Code = c;
+    m_Text.assign(str, (size_t)(eol - str));
+    return eHTTP_HeaderSuccess;
+}
+
+
+typedef AutoPtr< char, CDeleter<char> >  TTempCharPtr;
+
+
 static CConn_IOStream::TConnector
 s_HttpConnectorBuilder(const SConnNetInfo* net_info,
                        EReqMethod          method,
@@ -415,17 +461,21 @@ s_HttpConnectorBuilder(const SConnNetInfo* net_info,
                        const char*         path,
                        const char*         args,
                        const char*         user_header,
-                       void*               user_data,
-                       FHTTP_Adjust        adjust,
-                       FHTTP_Cleanup       cleanup,
-                       FHTTP_ParseHeader   parse_header,
+                       void*               x_data,
+                       FHTTP_Adjust        x_adjust,
+                       FHTTP_Cleanup       x_cleanup,
+                       FHTTP_ParseHeader   x_parse_header,
                        THTTP_Flags         flgs,
-                       const STimeout*     timeout)
+                       const STimeout*     timeout,
+                       void**              user_data_ptr,
+                       FHTTP_Cleanup*      user_cleanup_ptr,
+                       void*               user_data    = 0,
+                       FHTTP_Cleanup       user_cleanup = 0)
 {
     EReqMethod x_req_method;
-    AutoPtr<SConnNetInfo>
-        x_net_info(net_info
-                   ? ConnNetInfo_Clone(net_info) : ConnNetInfo_Create(0));
+    AutoPtr<SConnNetInfo> x_net_info(net_info
+                                     ? ConnNetInfo_Clone(net_info)
+                                     : ConnNetInfo_CreateInternal(0));
     if (!x_net_info.get()) {
         NCBI_THROW(CIO_Exception, eUnknown,
                    "CConn_HttpStream::CConn_HttpStream(): "
@@ -444,14 +494,14 @@ s_HttpConnectorBuilder(const SConnNetInfo* net_info,
     if (url  &&  !ConnNetInfo_ParseURL(x_net_info.get(), url)) {
         NCBI_THROW(CIO_Exception, eInvalidArg,
                    "CConn_HttpStream::CConn_HttpStream(): "
-                   " Bad URL");
+                   " Bad URL \"" + string(url) + '"');
     }
     if (host) {
         size_t len;
         if ((len = *host ? strlen(host) : 0) >= sizeof(x_net_info->host)) {
             NCBI_THROW(CIO_Exception, eInvalidArg,
                        "CConn_HttpStream::CConn_HttpStream(): "
-                       " Host too long");
+                       " Host too long \"" + string(host) + '"');
         }
         memcpy(x_net_info->host, host, ++len);
     }
@@ -460,27 +510,36 @@ s_HttpConnectorBuilder(const SConnNetInfo* net_info,
     if (path  &&  !ConnNetInfo_SetPath(x_net_info.get(), path)) {
         NCBI_THROW(CIO_Exception, eInvalidArg,
                    "CConn_HttpStream::CConn_HttpStream(): "
-                   " Path too long");
+                   " Path too long \"" + string(path) + '"');
     }
     if (args  &&  !ConnNetInfo_SetArgs(x_net_info.get(), args)) {
         NCBI_THROW(CIO_Exception, eInvalidArg,
                    "CConn_HttpStream::CConn_HttpStream(): "
-                   " Args too long");
+                   " Args too long \"" + string(args) + '"');
     }
     if (user_header  &&  *user_header
         &&  !ConnNetInfo_OverrideUserHeader(x_net_info.get(), user_header)) {
-        NCBI_THROW(CIO_Exception, eInvalidArg,
+        int x_dynamic = 0;
+        const char* x_message = NcbiMessagePlusError(&x_dynamic,
+                                                     "Cannot set user header",
+                                                     errno, 0);
+        TTempCharPtr msg_ptr(const_cast<char*> (x_message),
+                             x_dynamic ? eTakeOwnership : eNoOwnership);
+        NCBI_THROW(CIO_Exception, eUnknown,
                    "CConn_HttpStream::CConn_HttpStream(): "
-                   " Cannot set user header");
+                   " " + string(msg_ptr.get()));
     }
     if (timeout != kDefaultTimeout)
         x_net_info->timeout = timeout;
+    // NB: need these two inited now in case of early CONNECTOR->destroy() call
+    *user_data_ptr    = user_data;
+    *user_cleanup_ptr = user_cleanup;
     CONNECTOR c = HTTP_CreateConnectorEx(x_net_info.get(),
                                          flgs,
-                                         parse_header,
-                                         user_data,
-                                         adjust,
-                                         cleanup);
+                                         x_parse_header,
+                                         x_data,
+                                         x_adjust,
+                                         x_cleanup);
     return CConn_IOStream::TConnector(c);
 }
 
@@ -503,12 +562,14 @@ CConn_HttpStream::CConn_HttpStream(const string&   host,
                                             user_header.c_str(),
                                             this,
                                             x_Adjust,
-                                            0/*cleanup*/,
+                                            0/*x_Cleanup*/,
                                             x_ParseHeader,
                                             flgs,
-                                            timeout),
+                                            timeout,
+                                            &m_UserData,
+                                            &m_UserCleanup),
                      timeout, buf_size),
-      m_UserData(0), m_UserAdjust(0), m_UserCleanup(0), m_UserParseHeader(0)
+      m_UserAdjust(0), m_UserParseHeader(0)
 {
     return;
 }
@@ -528,12 +589,14 @@ CConn_HttpStream::CConn_HttpStream(const string&   url,
                                             0,
                                             this,
                                             x_Adjust,
-                                            0/*cleanup*/,
+                                            0/*x_Cleanup*/,
                                             x_ParseHeader,
                                             flgs,
-                                            timeout),
+                                            timeout,
+                                            &m_UserData,
+                                            &m_UserCleanup),
                      timeout, buf_size),
-      m_UserData(0), m_UserAdjust(0), m_UserCleanup(0), m_UserParseHeader(0)
+      m_UserAdjust(0), m_UserParseHeader(0)
 {
     return;
 }
@@ -555,12 +618,14 @@ CConn_HttpStream::CConn_HttpStream(const string&   url,
                                             user_header.c_str(),
                                             this,
                                             x_Adjust,
-                                            0/*cleanup*/,
+                                            0/*x_Cleanup*/,
                                             x_ParseHeader,
                                             flgs,
-                                            timeout),
+                                            timeout,
+                                            &m_UserData,
+                                            &m_UserCleanup),
                      timeout, buf_size),
-      m_UserData(0), m_UserAdjust(0), m_UserCleanup(0), m_UserParseHeader(0)
+      m_UserAdjust(0),  m_UserParseHeader(0)
 {
     return;
 }
@@ -589,10 +654,13 @@ CConn_HttpStream::CConn_HttpStream(const string&       url,
                                             cleanup ? x_Cleanup : 0,
                                             x_ParseHeader,
                                             flgs,
-                                            timeout),
+                                            timeout,
+                                            &m_UserData,
+                                            &m_UserCleanup,
+                                            user_data,
+                                            cleanup),
                      timeout, buf_size),
-      m_UserData(user_data),  m_UserAdjust(adjust),
-      m_UserCleanup(cleanup), m_UserParseHeader(parse_header)
+      m_UserAdjust(adjust), m_UserParseHeader(parse_header)
 {
     return;
 }
@@ -620,10 +688,13 @@ CConn_HttpStream::CConn_HttpStream(const SConnNetInfo* net_info,
                                             cleanup ? x_Cleanup : 0,
                                             x_ParseHeader,
                                             flgs,
-                                            timeout),
+                                            timeout,
+                                            &m_UserData,
+                                            &m_UserCleanup,
+                                            user_data,
+                                            cleanup),
                      timeout, buf_size),
-      m_UserData(user_data),  m_UserAdjust(adjust),
-      m_UserCleanup(cleanup), m_UserParseHeader(parse_header)
+      m_UserAdjust(adjust), m_UserParseHeader(parse_header)
 {
     return;
 }
@@ -631,42 +702,24 @@ CConn_HttpStream::CConn_HttpStream(const SConnNetInfo* net_info,
 
 CConn_HttpStream::~CConn_HttpStream()
 {
-    // Explicitly destroy so that the callbacks are not called out of context.
+    // Explicitly destroy so that the callbacks are not called out of context
     x_Destroy();
 }
 
 
-// NB: see CConn_ServiceStream::Fetch()
-EIO_Status CConn_HttpStream::Fetch(const STimeout* timeout)
+// WARNING: must not be called from CConn_HttpStream ctor (directly or not)!
+EHTTP_HeaderParse CConn_HttpStream::x_ParseHeader(const char* header,
+                                                  void*       data,
+                                                  int         code)
 {
-    CONN conn = GetCONN();
-    return !conn  ||  !flush()
-        ? eIO_Unknown : CONN_Wait(conn, eIO_Read, timeout);
-}
-
-
-// NB:  must never be upcalled (directly or indirectly) from any stream ctor
-//      for SHTTP_StatusData may yet be unbuilt (and the header field invalid)!
-static EHTTP_HeaderParse s_ParseHttpHeader(const char*       header,
-                                           SHTTP_StatusData& data)
-{
-    int c, n;
-    data.header = header;
-    if (sscanf(header, "%*s %u%n", &c, &n) < 1)
-        return eHTTP_HeaderError;
-    const char* str = data.header.c_str() + n;
-    str += strspn(str, " \t");
-    const char* eol = strchr(str, '\n');
-    if (!eol)
-        eol = str + strlen(str);
-    while (eol > str) {
-        if (!isspace((unsigned char) eol[-1]))
-            break;
-        --eol;
-    }
-    data.code = c;
-    data.text.assign(str, (size_t)(eol - str));
-    return eHTTP_HeaderSuccess;
+    CConn_HttpStream* http = reinterpret_cast<CConn_HttpStream*>(data);
+    EHTTP_HeaderParse rv = http->m_StatusData.Parse(header);
+    if (rv != eHTTP_HeaderSuccess)
+        return rv;
+    _ASSERT(!code  ||  code == http->m_StatusData.m_Code);
+    return http->m_UserParseHeader
+        ? http->m_UserParseHeader(header, http->m_UserData, code)
+        : eHTTP_HeaderSuccess;
 }
 
 
@@ -678,6 +731,7 @@ int CConn_HttpStream::x_Adjust(SConnNetInfo* net_info,
     bool modified;
     CConn_HttpStream* http = reinterpret_cast<CConn_HttpStream*>(data);
     if (count == (unsigned int)(-1)  &&  !http->m_URL.empty()) {
+        http->m_StatusData.Clear();
         if (!ConnNetInfo_ParseURL(net_info, http->m_URL.c_str()))
             return 0/*failure*/;
         http->m_URL.erase();
@@ -702,63 +756,58 @@ void CConn_HttpStream::x_Cleanup(void* data)
 }
 
 
-EHTTP_HeaderParse CConn_HttpStream::x_ParseHeader(const char* header,
-                                                  void*       data,
-                                                  int         code)
-{
-    CConn_HttpStream* http = reinterpret_cast<CConn_HttpStream*>(data);
-    EHTTP_HeaderParse rv = s_ParseHttpHeader(header, http->m_StatusData);
-    if (rv != eHTTP_HeaderSuccess)
-        return rv;
-    _ASSERT(!code  ||  code == http->m_StatusData.code);
-    return http->m_UserParseHeader
-        ? http->m_UserParseHeader(header, http->m_UserData, code)
-        : eHTTP_HeaderSuccess;
-}
-
-
 static CConn_IOStream::TConnector
-s_ServiceConnectorBuilder(const char*                           service,
-                          TSERV_Type                            types,
-                          const SConnNetInfo*                   net_info,
-                          const char*                           user_header,
-                          const SSERVICE_Extra*                 extra,
-                          CConn_ServiceStream::SSERVICE_CBData* cbdata,
-                          FSERVICE_Reset                        reset,
-                          FHTTP_Adjust                          adjust,
-                          FSERVICE_Cleanup                      cleanup,
-                          FHTTP_ParseHeader                     parse_header,
-                          FSERVICE_GetNextInfo                  get_next_info,
-                          const STimeout*                       timeout)
+s_ServiceConnectorBuilder(const char*                          service,
+                          TSERV_Type                           types,
+                          const SConnNetInfo*                  net_info,
+                          const char*                          user_header,
+                          const SSERVICE_Extra*                extra,
+                          CConn_ServiceStream::SSERVICE_CBData*cbdata,
+                          FSERVICE_Reset                       x_reset,
+                          FHTTP_Adjust                         x_adjust,
+                          FSERVICE_Cleanup                     x_cleanup,
+                          FHTTP_ParseHeader                    x_parse_header,
+                          FSERVICE_GetNextInfo                 x_get_next_info,
+                          const STimeout*                      timeout)
 {
-    AutoPtr<SConnNetInfo>
-        x_net_info(net_info ?
-                   ConnNetInfo_Clone(net_info) : ConnNetInfo_Create(service));
+    AutoPtr<SConnNetInfo> x_net_info(net_info
+                                     ? ConnNetInfo_Clone(net_info)
+                                     : ConnNetInfo_Create(service));
     if (!x_net_info.get()) {
         NCBI_THROW(CIO_Exception, eUnknown,
                    "CConn_ServiceStream::CConn_ServiceStream(): "
                    " Out of memory");
     }
-    if (user_header  &&  *user_header)
-        ConnNetInfo_OverrideUserHeader(x_net_info.get(), user_header);
+    if (user_header  &&  *user_header
+        &&  !ConnNetInfo_OverrideUserHeader(x_net_info.get(), user_header)) {
+        int x_dynamic = 0;
+        const char* x_message = NcbiMessagePlusError(&x_dynamic,
+                                                     "Cannot set user header",
+                                                     errno, 0);
+        TTempCharPtr msg_ptr(const_cast<char*> (x_message),
+                             x_dynamic ? eTakeOwnership : eNoOwnership);
+        NCBI_THROW(CIO_Exception, eUnknown,
+                   "CConn_ServiceStream::CConn_ServiceStream(): "
+                   " " + string(msg_ptr.get()));
+    }
     if (timeout != kDefaultTimeout)
         x_net_info->timeout = timeout;
     if (extra)
         memcpy(&cbdata->extra, extra, sizeof(cbdata->extra));
     else
         memset(&cbdata->extra, 0,     sizeof(cbdata->extra));
-    _ASSERT(!reset          ||  (extra  &&  extra->reset));
-    _ASSERT(!adjust         ||  (extra  &&  extra->adjust));
-    _ASSERT(!cleanup        ||  (extra  &&  extra->cleanup));
-    _ASSERT(!get_next_info  ||  (extra  &&  extra->get_next_info));
+    _ASSERT(!x_reset          ||  (extra  &&  extra->reset));
+    _ASSERT(!x_adjust         ||  (extra  &&  extra->adjust));
+    _ASSERT(!x_cleanup        ||  (extra  &&  extra->cleanup));
+    _ASSERT(!x_get_next_info  ||  (extra  &&  extra->get_next_info));
     SSERVICE_Extra x_extra;
     memset(&x_extra, 0, sizeof(x_extra));
     x_extra.data          = cbdata;
-    x_extra.reset         = reset;
-    x_extra.adjust        = adjust;
-    x_extra.cleanup       = cleanup;
-    x_extra.parse_header  = parse_header;
-    x_extra.get_next_info = get_next_info;
+    x_extra.reset         = x_reset;
+    x_extra.adjust        = x_adjust;
+    x_extra.cleanup       = x_cleanup;
+    x_extra.parse_header  = x_parse_header;
+    x_extra.get_next_info = x_get_next_info;
     x_extra.flags         = extra ? extra->flags : 0;
     CONNECTOR c = SERVICE_CreateConnectorEx(service,
                                             types,
@@ -805,7 +854,7 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
                                          size_t                buf_size)
     : CConn_IOStream(s_ServiceConnectorBuilder(service.c_str(),
                                                types,
-                                               0, //net_info
+                                               0, // net_info
                                                user_header.c_str(),
                                                extra,
                                                &m_CBD,
@@ -828,34 +877,22 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
 
 CConn_ServiceStream::~CConn_ServiceStream()
 {
-    // Explicitly destroy so that the callbacks are not called out of context.
+    // Explicitly destroy so that the callbacks are not called out of context
     x_Destroy();
 }
 
 
-// NB: see CConn_HttpStream::Fetch()
-EIO_Status CConn_ServiceStream::Fetch(const STimeout* timeout)
+EHTTP_HeaderParse CConn_ServiceStream::x_ParseHeader(const char* header,
+                                                     void* data, int code)
 {
-    CONN conn = GetCONN();
-    return !conn  ||  !flush()
-        ? eIO_Unknown : CONN_Wait(conn, eIO_Read, timeout);
-}
-
-
-SOCK CConn_ServiceStream::GetSOCK(void)
-{
-    SOCK sock;
-    CONN conn = GetCONN();
-    if (!conn  ||  CONN_GetSOCK(conn, &sock) != eIO_Success)
-        sock = 0;
-    return sock;
-}
-
-
-void CConn_ServiceStream::x_Reset(void* data)
-{
-    SSERVICE_CBData* cbd = static_cast<SSERVICE_CBData*>(data);
-    cbd->extra.reset(cbd->extra.data);
+    SSERVICE_CBData* cbd = reinterpret_cast<SSERVICE_CBData*>(data);
+    EHTTP_HeaderParse rv = cbd->status.Parse(header);
+    if (rv != eHTTP_HeaderSuccess)
+        return rv;
+    _ASSERT(!code  ||  code == cbd->status.m_Code);
+    return cbd->extra.parse_header
+        ? cbd->extra.parse_header(header, cbd->extra.data, code)
+        : eHTTP_HeaderSuccess;
 }
 
 
@@ -863,35 +900,30 @@ int/*bool*/ CConn_ServiceStream::x_Adjust(SConnNetInfo* net_info,
                                           void*         data,
                                           unsigned int  count)
 {
-    SSERVICE_CBData* cbd = static_cast<SSERVICE_CBData*>(data);
+    SSERVICE_CBData* cbd = reinterpret_cast<SSERVICE_CBData*>(data);
+    if (count != (unsigned int)(-1))
+        cbd->status.Clear();  // never call from CConn_ServiceStream ctor!
     return cbd->extra.adjust(net_info, cbd->extra.data, count);
 }
 
 
+void CConn_ServiceStream::x_Reset(void* data)
+{
+    SSERVICE_CBData* cbd = reinterpret_cast<SSERVICE_CBData*>(data);
+    cbd->extra.reset(cbd->extra.data);
+}
+
+
 void CConn_ServiceStream::x_Cleanup(void* data)
 {
-    SSERVICE_CBData* cbd = static_cast<SSERVICE_CBData*>(data);
+    SSERVICE_CBData* cbd = reinterpret_cast<SSERVICE_CBData*>(data);
     cbd->extra.cleanup(cbd->extra.data);
 }
 
 
-EHTTP_HeaderParse CConn_ServiceStream::x_ParseHeader(const char* header,
-                                                     void* data, int code)
-{
-    SSERVICE_CBData* cbd = static_cast<SSERVICE_CBData*>(data);
-    EHTTP_HeaderParse rv = s_ParseHttpHeader(header, cbd->status);
-    if (rv != eHTTP_HeaderSuccess)
-        return rv;
-    _ASSERT(!code  ||  code == cbd->status.code);
-    return cbd->extra.parse_header
-        ? cbd->extra.parse_header(header, cbd->extra.data, code)
-        : eHTTP_HeaderSuccess;
-}
-
-
 const SSERV_Info* CConn_ServiceStream::x_GetNextInfo(void* data,SERV_ITER iter)
 {
-    SSERVICE_CBData* cbd = static_cast<SSERVICE_CBData*>(data);
+    SSERVICE_CBData* cbd = reinterpret_cast<SSERVICE_CBData*>(data);
     return cbd->extra.get_next_info(cbd->extra.data, iter);
 }
 
@@ -1027,9 +1059,9 @@ EIO_Status CConn_PipeStream::Close(void)
         return Status(eIO_Write);
     // NB:  This can lead to wrong order for a close callback, if any fired by
     // CConn_IOStream::Close():  the callback will be late and actually coming
-    // _after_ the pipe gets closed here.  There's no way to avoid this...
+    // _after_ the pipe gets closed here.  There's no easy way to avoid this...
     EIO_Status status = m_Pipe->Close(&m_ExitCode);
-    CConn_IOStream::Close();
+    (void) CConn_IOStream::Close();
     return status;
 }
 
@@ -1119,7 +1151,7 @@ EIO_Status CConn_FtpStream::Drain(const STimeout* timeout)
 {
     const STimeout* r_timeout = kInfiniteTimeout/*0*/;
     const STimeout* w_timeout = kInfiniteTimeout/*0*/;
-    static char block[16384]; /*NB:shared sink*/
+    static char sink[16384]; /*NB:shared sink*/
     CONN conn = GetCONN();
     if (conn) {
         size_t n;
@@ -1128,19 +1160,19 @@ EIO_Status CConn_FtpStream::Drain(const STimeout* timeout)
         _VERIFY(SetTimeout(eIO_Read,  timeout) == eIO_Success);
         _VERIFY(SetTimeout(eIO_Write, timeout) == eIO_Success);
         // Cause any upload-in-progress to abort
-        CONN_Read (conn, block, sizeof(block), &n, eIO_ReadPlain);
+        CONN_Read(conn, sink, sizeof(sink), &n, eIO_ReadPlain);
         // Cause any command-in-progress to abort
         CONN_Write(conn, "NOOP\n", 5, &n, eIO_WritePersist);
     }
     clear();
-    while (read(block, sizeof(block)))
+    while (read(sink, sizeof(sink)))
         ;
     if (!conn)
         return eIO_Closed;
     EIO_Status status;
     do {
         size_t n;
-        status = CONN_Read(conn, block, sizeof(block), &n, eIO_ReadPersist);
+        status = CONN_Read(conn, sink, sizeof(sink), &n, eIO_ReadPersist);
     } while (status == eIO_Success);
     _VERIFY(CONN_SetTimeout(conn, eIO_Read,  r_timeout) == eIO_Success);
     _VERIFY(CONN_SetTimeout(conn, eIO_Write, w_timeout) == eIO_Success);
@@ -1198,7 +1230,7 @@ void CConn_FTPDownloadStream::x_InitDownload(const string& file, Uint8 offset)
         status  = Status(eIO_Write);
     }
     if (status != eIO_Success)
-        setstate(NcbiBadbit);
+        clear(NcbiBadbit);
 }
 
 
@@ -1236,11 +1268,15 @@ void CConn_FTPUploadStream::x_InitUpload(const string& file, Uint8 offset)
     EIO_Status status;
     if (offset) {
         write("REST ", 5) << NStr::UInt8ToString(offset) << NcbiFlush;
-        status = Status(eIO_Write);
+        status  = Status(eIO_Write);
     } else
-        status = eIO_Success;
-    if (good()  &&  status == eIO_Success)
+        status  = eIO_Success;
+    if (good()  &&  status == eIO_Success) {
         write("STOR ", 5) << file << NcbiFlush;
+        status  = Status(eIO_Write);
+    }
+    if (status != eIO_Success)
+        clear(NcbiBadbit);
 }
 
 
@@ -1275,9 +1311,7 @@ static bool x_IsIdentifier(const string& str)
 }
 
 
-extern
-NCBI_XCONNECT_EXPORT  // FIXME: To remove once the API is fully official
-CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
+extern CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
 {
     {
         class CInPlaceConnIniter : protected CConnIniter
@@ -1287,8 +1321,11 @@ CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
     bool svc = x_IsIdentifier(url);
 
     AutoPtr<SConnNetInfo> net_info
-        (ConnNetInfo_Create(svc ? url.c_str() : 0));
-
+        (ConnNetInfo_CreateInternal
+         (svc
+          ? TTempCharPtr(SERV_ServiceName(url.c_str())).get()
+          : NStr::StartsWith(url, "ftp://", NStr::eNocase)
+          ? "_FTP" : 0));
     if (svc)
         return new CConn_ServiceStream(url, fSERV_Any, net_info.get());
 
@@ -1313,19 +1350,25 @@ CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
                                         kDefaultTimeout, buf_size);
         case eURL_File:
             if (*net_info->host  ||  net_info->port)
-                break; /*not supported*/
+                break; // not supported
             if (net_info->debug_printout) {
+                // manual cleanup of most fields req'd
                 net_info->req_method = eReqMethod_Any;
+                net_info->external = 0;
                 net_info->firewall = 0;
                 net_info->stateless = 0;
                 net_info->lb_disable = 0;
+                net_info->http_version = 0;
+                net_info->http_push_auth = 0;
                 net_info->http_proxy_leak = 0;
+                net_info->user[0] = '\0';
+                net_info->pass[0] = '\0';
                 net_info->http_proxy_host[0] = '\0';
                 net_info->http_proxy_port    =   0;
                 net_info->http_proxy_user[0] = '\0';
                 net_info->http_proxy_pass[0] = '\0';
                 net_info->max_try = 0;
-                net_info->timeout = kInfiniteTimeout;
+                net_info->timeout = kInfiniteTimeout/*0*/;
                 ConnNetInfo_SetUserHeader(net_info.get(), 0);
                 if (net_info->http_referer) {
                     free((void*) net_info->http_referer);
@@ -1350,19 +1393,4 @@ CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
 }
 
 
-const char* CIO_Exception::GetErrCodeString(void) const
-{
-    switch (GetErrCode()) {
-    case eTimeout:       return "eIO_Timeout";
-    case eInterrupt:     return "eIO_Interrupt";
-    case eInvalidArg:    return "eIO_InvalidArg";
-    case eNotSupported:  return "eIO_NotSupported";
-    case eUnknown:       return "eIO_Unknown";
-    case eClosed:        return "eIO_Closed";
-    default:             break;
-    }
-    return CException::GetErrCodeString();
-}
-
-
 END_NCBI_SCOPE
