@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_streambuf.cpp 587031 2019-05-28 18:07:45Z lavr $
+/* $Id: ncbi_conn_streambuf.cpp 624532 2021-02-01 13:23:19Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,25 +44,63 @@
 BEGIN_NCBI_SCOPE
 
 
-string CConn_Streambuf::x_Message(const char* msg)
+/*ARGSUSED*/
+static inline bool x_IsThrowable(EIO_Status status)
+{
+    _ASSERT(status != eIO_Success);
+#if (defined(NCBI_COMPILER_GCC)  &&  NCBI_COMPILER_VERSION < 700)  \
+    ||  defined(NCBI_COMPILER_LLVM_CLANG)
+    // For C++ STLs that have a bug that sentry ctor does not include try/catch
+    // so exceptions leak instead of setting badbit as the standard requires.
+    return false;
+#else
+    return status != eIO_Timeout ? true : false;
+#endif
+}
+
+
+static inline bool x_CheckConn(CONN conn)
+{
+    if (conn)
+        return true;
+#if !defined(NCBI_COMPILER_GCC)  ||  NCBI_COMPILER_GCC >= 700
+    NCBI_IO_CHECK(eIO_Closed);
+#endif // !NCBI_COMPILER_GCC || NCBI_COMPILER_VERSION>=700
+    return false;
+}
+
+
+string CConn_Streambuf::x_Message(const char*     method,
+                                  const char*     message,
+                                  EIO_Status      status,
+                                  const STimeout* timeout)
 {
     const char* type = m_Conn ? CONN_GetType    (m_Conn) : 0;
     char*       text = m_Conn ? CONN_Description(m_Conn) : 0;
-    string result("CConn_Streambuf::");
-    result += msg;
-    if (type  ||  text) {
-        result += " (";
-        result += type ? type : "UNDEF";
-        if (text) {
-            _ASSERT(*text);
+    string result("[CConn_Streambuf::");
+    result += method;
+    result += '(';
+    if (type) {
+        result += type;
+        if (text)
             result += "; ";
-            result += text;
-            free(text);
-        }
-        result += ')';
     }
+    if (text) {
+        _ASSERT(*text);
+        result += text;
+        free(text);
+    }
+    result += ")]  ";
+    result += message;
     result += ": ";
-    result += IO_StatusStr(m_Status);
+    if (status == eIO_Success)
+        status  = m_Status;
+    result += IO_StatusStr(status);
+    if (status == eIO_Timeout  &&  timeout  &&  timeout != kDefaultTimeout) {
+        char tmo[40];
+        ::sprintf(tmo, "[%u.%06us]", timeout->sec, timeout->usec);
+        result += tmo;
+    }
     return result;
 }
 
@@ -82,7 +120,7 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR                   connector,
     if (!connector) {
         if (m_Status == eIO_Success)
             m_Status  = eIO_InvalidArg;
-        ERR_POST_X(2, x_Message("CConn_Streambuf():  NULL connector"));
+        ERR_POST_X(2, x_Message("CConn_Streambuf", "NULL connector"));
         return;
     }
     if (!(flgs & (CConn_IOStream::fConn_Untie |
@@ -93,9 +131,9 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR                   connector,
         (m_Status = CONN_CreateEx(connector, fCONN_Supplement
                                   | (m_Tie ? 0 : flgs & fCONN_Untie), &m_Conn))
         != eIO_Success) {
-        ERR_POST_X(3, x_Message("CConn_Streambuf():  CONN_Create() failed"));
+        ERR_POST_X(3, x_Message("CConn_Streambuf", "CONN_Create() failed"));
         _ASSERT(!m_Conn  &&  !connector->meta  &&  !connector->next);
-        if (connector->destroy)
+        if (connector->destroy) // NB: cleanup callback (if any) must be valid!
             connector->destroy(connector);
         return;
     }
@@ -118,7 +156,7 @@ CConn_Streambuf::CConn_Streambuf(CONN                        conn,
 {
     if (!m_Conn) {
         m_Status = eIO_InvalidArg;
-        ERR_POST_X(1, x_Message("CConn_Streambuf():  NULL connection"));
+        ERR_POST_X(1, x_Message("CConn_Streambuf", "NULL connection"));
         return;
     }
     if (!(flgs & (CConn_IOStream::fConn_Untie |
@@ -157,31 +195,41 @@ void CConn_Streambuf::x_Init(const STimeout* timeout, size_t buf_size,
     if (buf_size) {
         m_WriteBuf = new
             CT_CHAR_TYPE[buf_size
-                         << !(flgs & (CConn_IOStream::fConn_ReadUnbuffered |
-                                      CConn_IOStream::fConn_WriteUnbuffered))];
+                         << ((flgs & (CConn_IOStream::fConn_ReadUnbuffered |
+                                      CConn_IOStream::fConn_WriteUnbuffered))
+                             ? 0 : 1)];
         if (!(flgs & CConn_IOStream::fConn_ReadUnbuffered))
-            m_BufSize = buf_size;
-        if (flgs & CConn_IOStream::fConn_WriteUnbuffered)
-            buf_size = 0;
+            m_BufSize =  buf_size;
+        if (  flgs & CConn_IOStream::fConn_WriteUnbuffered)
+            buf_size  =  0;
         if (!(flgs & CConn_IOStream::fConn_ReadUnbuffered))
-            m_ReadBuf = m_WriteBuf + buf_size;
-    } /* else see ctor */
-
-    if (buf_size)
+            m_ReadBuf =  m_WriteBuf + buf_size;
         setp(m_WriteBuf, m_WriteBuf + buf_size);
-    /* else setp(0, 0) */
+    }/* else
+        setp(0, 0) */
+
     if (ptr) {
         m_Initial = true;
         setg(ptr,        ptr,       ptr + size);   // Initial get area
     } else
         setg(m_ReadBuf,  m_ReadBuf, m_ReadBuf);    // Empty get area
-    _ASSERT(m_ReadBuf  &&  m_BufSize);
+    _ASSERT(m_ReadBuf  &&  m_BufSize);             // NB: See ctor
 
     SCONN_Callback cb;
     cb.func = x_OnClose; /* NCBI_FAKE_WARNING: WorkShop */
     cb.data = this;
     CONN_SetCallback(m_Conn, eCONN_OnClose, &cb, &m_Cb);
     m_CbValid = true;
+
+    if (!(flgs & CConn_IOStream::fConn_DelayOpen)) {
+        SOCK s/*dummy*/;
+        // NB: CONN_Write(0 bytes) could have caused the same effect
+        (void) CONN_GetSOCK(m_Conn, &s);  // Prompt CONN to actually open
+        if ((m_Status = CONN_Status(m_Conn, eIO_Open)) != eIO_Success) {
+            ERR_POST_X(17, x_Message("CConn_Streambuf", "Failed to open",
+                                     m_Status, timeout));
+        }
+    }
 }
 
 
@@ -212,8 +260,8 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
         EIO_Status x_status = x_Pushback();
         if (x_status != eIO_Success  &&  x_status != eIO_NotSupported) {
             status = m_Status = x_status;
-            ERR_POST_X(13, x_Message("Close(): "
-                                     " CONN_Pushback() failed"));
+            ERR_POST_X(13, x_Message("Close",
+                                     "CONN_Pushback() failed"));
         }
     }
     setg(0, 0, 0);
@@ -224,12 +272,22 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
         if (x_status != eIO_Success) {
             status = m_Status = x_status;
             if (CONN_Status(m_Conn, eIO_Open) == eIO_Success) {
-                _TRACE(x_Message("Close(): "
-                                 " Cannot finalize implicitly"
+                _TRACE(x_Message("Close",
+                                 "Cannot finalize implicitly"
                                  ", data loss may result"));
             }
-        } else if (sync() != 0)
-            status = m_Status != eIO_Success ? m_Status : eIO_Unknown;
+        } else {
+            bool synced = false;
+            try {
+                if (sync() == 0)
+                    synced = true;
+            }
+            catch (...) {
+                _ASSERT(!synced);
+            }
+            if (!synced)
+                _VERIFY((status = m_Status) != eIO_Success);
+        }
     }
     setp(0, 0);
 
@@ -245,8 +303,8 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
                 CONN_SetCallback(c, eCONN_OnClose, &cb, 0);
         }
         if (m_Close  &&  (m_Status = CONN_Close(c)) != eIO_Success) {
-            _TRACE(x_Message("Close(): "
-                             " CONN_Close() failed"));
+            _TRACE(x_Message("Close",
+                             "CONN_Close() failed"));
             if (status == eIO_Success)
                 status  = m_Status;
         }
@@ -264,30 +322,48 @@ EIO_Status CConn_Streambuf::x_OnClose(CONN           _DEBUG_ARG(conn),
                                       void*          data)
 {
     CConn_Streambuf* sb = static_cast<CConn_Streambuf*>(data);
-
     _ASSERT(type == eCONN_OnClose  &&  sb  &&  conn);
     _ASSERT(!sb->m_Conn  ||  sb->m_Conn == conn);
     return sb->x_Close(false);
 }
 
 
-EIO_Status CConn_Streambuf::Pushback(const CT_CHAR_TYPE* data, streamsize size)
+CNcbiStreambuf* CConn_Streambuf::setbuf(CT_CHAR_TYPE* buf, streamsize buf_size)
 {
-    if (!m_Conn)
-        return eIO_Closed;
-
-    if ((!m_Initial  &&  (m_Status = x_Pushback()) != eIO_Success)
-        ||  (m_Status = CONN_Pushback(m_Conn, data, size)) != eIO_Success) {
-        ERR_POST_X(14, x_Message("Pushback(): "
-                                 " CONN_Pushback() failed"));
+    if (buf  ||  buf_size) {
+        NCBI_THROW(CConnException, eConn,
+                   "CConn_Streambuf::setbuf() only allowed with (0, 0)");
     }
-    return m_Status;
+
+    if (m_Conn) {
+        EIO_Status status;
+        if (!m_Initial  &&  (status = x_Pushback()) != eIO_Success) {
+            ERR_POST_X(11, Critical << x_Message("setbuf",
+                                                 "Read data pending",
+                                                 status));
+        }
+        if (x_Sync() != 0) {
+            ERR_POST_X(12, Critical << x_Message("setbuf",
+                                                 "Write data pending"));
+        }
+    }
+    setp(0, 0);
+
+    delete[] m_WriteBuf;
+    m_WriteBuf = 0;
+
+    m_ReadBuf  = &x_Buf;
+    m_BufSize  = 1;
+
+    if (!m_Conn  ||  !m_Initial)
+        setg(m_ReadBuf, m_ReadBuf, m_ReadBuf);
+    return this;
 }
 
 
 CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
 {
-    if (!m_Conn)
+    if (!x_CheckConn(m_Conn))
         return CT_EOF;
 
     size_t n_written;
@@ -315,8 +391,10 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
         } while (n_towrite  &&  m_Status == eIO_Success);
         if (n_towrite) {
             _ASSERT(m_Status != eIO_Success);
-            ERR_POST_X(4, x_Message("overflow(): "
-                                    " CONN_Write() failed"));
+            ERR_POST_X(4, x_Message("overflow",
+                                    "CONN_Write() failed"));
+            if (x_IsThrowable(m_Status))
+                NCBI_IO_CHECK(m_Status);
             return CT_EOF;
         }
     } else if (!CT_EQ_INT_TYPE(c, CT_EOF)) {
@@ -326,8 +404,10 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
         _ASSERT(n_written <= 1);
         if (!n_written) {
             _ASSERT(m_Status != eIO_Success);
-            ERR_POST_X(5, x_Message("overflow(): "
-                                    " CONN_Write(1) failed"));
+            ERR_POST_X(5, x_Message("overflow",
+                                    "CONN_Write(1) failed"));
+            if (x_IsThrowable(m_Status))
+                NCBI_IO_CHECK(m_Status);
             return CT_EOF;
         }
         x_PPos += (CT_OFF_TYPE) 1;
@@ -336,8 +416,10 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
 
     _ASSERT(CT_EQ_INT_TYPE(c, CT_EOF));
     if ((m_Status = CONN_Flush(m_Conn)) != eIO_Success) {
-        ERR_POST_X(9, x_Message("overflow(): "
-                                " CONN_Flush() failed"));
+        ERR_POST_X(9, x_Message("overflow",
+                                "CONN_Flush() failed"));
+        if (x_IsThrowable(m_Status))
+            NCBI_IO_CHECK(m_Status);
         return CT_EOF;
     }
     return CT_NOT_EOF(CT_EOF);
@@ -346,7 +428,7 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
 
 streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
 {
-    if (!m_Conn  ||  m < 0)
+    if (!x_CheckConn(m_Conn)  ||  m < 0)
         return 0;
 
     _ASSERT((Uint8) m < numeric_limits<size_t>::max());
@@ -358,7 +440,7 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
     do {
         if (pbase()) {
             if (n  &&  pbase() + n < epptr()) {
-                // Would entirely fit into the buffer not causing an overflow
+                // would entirely fit into the buffer not causing an overflow
                 x_written = (size_t)(epptr() - pptr());
                 if (x_written > n)
                     x_written = n;
@@ -380,8 +462,8 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
                 _ASSERT(x_written <= x_towrite);
                 if (!x_written) {
                     _ASSERT(m_Status != eIO_Success);
-                    ERR_POST_X(6, x_Message("xsputn(): "
-                                            " CONN_Write() failed"));
+                    ERR_POST_X(6, x_Message("xsputn",
+                                            "CONN_Write() failed"));
                     break;
                 }
                 memmove(pbase(), pbase() + x_written, x_towrite - x_written);
@@ -396,8 +478,8 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
         _ASSERT(x_written <= n);
         if (!x_written  &&  n) {
             _ASSERT(m_Status != eIO_Success);
-            ERR_POST_X(7, x_Message("xsputn(): "
-                                    " CONN_Write(direct) failed"));
+            ERR_POST_X(7, x_Message("xsputn",
+                                    "CONN_Write(direct) failed"));
             break;
         }
         x_PPos    += (CT_OFF_TYPE) x_written;
@@ -421,6 +503,8 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
         }
     }
 
+    if (!n_written  &&  x_IsThrowable(m_Status))
+        NCBI_IO_CHECK(m_Status);
     return (streamsize) n_written;
 }
 
@@ -429,7 +513,7 @@ CT_INT_TYPE CConn_Streambuf::underflow(void)
 {
     _ASSERT(gptr() >= egptr());
 
-    if (!m_Conn)
+    if (!x_CheckConn(m_Conn))
         return CT_EOF;
 
     // flush output buffer, if tied up to it
@@ -450,8 +534,10 @@ CT_INT_TYPE CConn_Streambuf::underflow(void)
     if (!n_read) {
         _ASSERT(m_Status != eIO_Success);
         if (m_Status != eIO_Closed) {
-            ERR_POST_X(8, x_Message("underflow(): "
-                                    " CONN_Read() failed"));
+            ERR_POST_X(8, x_Message("underflow",
+                                    "CONN_Read() failed"));
+            if (x_IsThrowable(m_Status))
+                NCBI_IO_CHECK(m_Status);
         }
         return CT_EOF;
     }
@@ -508,8 +594,8 @@ streamsize CConn_Streambuf::x_Read(CT_CHAR_TYPE* buf, streamsize m)
         if (!x_read) {
             _ASSERT(!x_toread  ||  m_Status != eIO_Success);
             if (m_Status != eIO_Success  &&  m_Status != eIO_Closed) {
-                ERR_POST_X(10, x_Message("xsgetn(): "
-                                         " CONN_Read() failed"));
+                ERR_POST_X(10, x_Message("xsgetn",
+                                         "CONN_Read() failed"));
             }
             break;
         }
@@ -537,13 +623,17 @@ streamsize CConn_Streambuf::x_Read(CT_CHAR_TYPE* buf, streamsize m)
         n       -= x_read;
     } while (n);
 
+    if (!n_read  &&  m_Status != eIO_Closed  &&  x_IsThrowable(m_Status))
+        NCBI_IO_CHECK(m_Status);
     return (streamsize) n_read;
 }
 
 
 streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 {
-    return m_Conn ? x_Read(buf, m) : 0;
+    _ASSERT(egptr() >= gptr());
+
+    return x_CheckConn(m_Conn) ? x_Read(buf, m) : 0;
 }
 
 
@@ -553,7 +643,7 @@ streamsize CConn_Streambuf::showmanyc(void)
 
     _ASSERT(gptr() >= egptr());
 
-    if (!m_Conn)
+    if (!x_CheckConn(m_Conn))
         return -1L;
 
     // flush output buffer, if tied up to it
@@ -565,6 +655,7 @@ streamsize CConn_Streambuf::showmanyc(void)
     if (timeout == kDefaultTimeout) {
         // HACK * HACK * HACK
         tmo = ((SMetaConnector*) m_Conn)->default_timeout;
+        _ASSERT(tmo != kDefaultTimeout);
     } else
         tmo = timeout;
 
@@ -588,6 +679,8 @@ streamsize CConn_Streambuf::showmanyc(void)
             _ASSERT(0);
             /*FALLTHRU*/
         default:
+            if (x_IsThrowable(m_Status))
+                NCBI_IO_CHECK(m_Status);
             break;
         }
         return       0;  // no data available immediately
@@ -609,37 +702,6 @@ int CConn_Streambuf::sync(void)
 }
 
 
-CNcbiStreambuf* CConn_Streambuf::setbuf(CT_CHAR_TYPE* buf, streamsize buf_size)
-{
-    if (buf  ||  buf_size) {
-        NCBI_THROW(CConnException, eConn,
-                   "CConn_Streambuf::setbuf() only allowed with (0, 0)");
-    }
-
-    if (m_Conn) {
-        if (!m_Initial  &&  x_Pushback() != eIO_Success) {
-            ERR_POST_X(11, Critical << x_Message("setbuf(): "
-                                                 " Read data pending"));
-        }
-        if (x_Sync() != 0) {
-            ERR_POST_X(12, Critical << x_Message("setbuf(): "
-                                                 " Write data pending"));
-        }
-    }
-    setp(0, 0);
-
-    delete[] m_WriteBuf;
-    m_WriteBuf = 0;
-
-    m_ReadBuf  = &x_Buf;
-    m_BufSize  = 1;
-
-    if (!m_Conn  ||  !m_Initial)
-        setg(m_ReadBuf, m_ReadBuf, m_ReadBuf);
-    return this;
-}
-
-
 CT_POS_TYPE CConn_Streambuf::seekoff(CT_OFF_TYPE        off,
                                      IOS_BASE::seekdir  whence,
                                      IOS_BASE::openmode which)
@@ -664,6 +726,75 @@ CT_POS_TYPE CConn_Streambuf::seekoff(CT_OFF_TYPE        off,
 }
 
 
+EIO_Status CConn_Streambuf::Pushback(const CT_CHAR_TYPE* data, streamsize size)
+{
+    if (!m_Conn)
+        return eIO_Closed;
+
+    if ((!m_Initial  &&  (m_Status = x_Pushback()) != eIO_Success)
+        ||  (m_Status = CONN_Pushback(m_Conn, data, size)) != eIO_Success) {
+        ERR_POST_X(14, x_Message("Pushback",
+                                 "CONN_Pushback() failed"));
+    }
+    return m_Status;
+}
+
+
+EIO_Status CConn_Streambuf::Fetch(const STimeout* timeout)
+{
+    if (!m_Conn)
+        return eIO_Closed;
+
+    if (timeout == kDefaultTimeout) {
+        // HACK * HACK * HACK
+        timeout = ((SMetaConnector*) m_Conn)->default_timeout;
+        _ASSERT(timeout != kDefaultTimeout);
+        if (!timeout)
+            timeout = &g_NcbiDefConnTimeout;
+    }
+
+    // try to flush buffer first
+    if (pbase() < pptr()) {
+        const STimeout* x_tmo = CONN_GetTimeout(m_Conn, eIO_Write);
+        _VERIFY(CONN_SetTimeout(m_Conn, eIO_Write, timeout) == eIO_Success);
+        bool synced = false;
+        try {
+            if (sync() == 0)
+                synced = true;
+        }
+        catch (CIO_Exception& _DEBUG_ARG(ex)) {
+            _ASSERT(!synced  &&  EIO_Status(ex.GetErrCode()) == m_Status);
+        }
+        catch (...) {
+            _VERIFY(CONN_SetTimeout(m_Conn, eIO_Write, x_tmo) == eIO_Success);
+            throw;
+        }
+        _VERIFY(CONN_SetTimeout(m_Conn, eIO_Write, x_tmo) == eIO_Success);
+
+        if (!synced) {
+            _ASSERT(m_Status != eIO_Success);
+            ERR_POST_X(15, x_Message("Fetch",
+                                     "Failed to flush",
+                                     m_Status, timeout));
+        }
+    }
+
+    // check if input is already pending
+    if (gptr() < egptr())
+        return eIO_Success;
+
+    // now wait for some input
+    EIO_Status status = CONN_Wait(m_Conn, eIO_Read, timeout);
+    if (status != eIO_Success) {
+        ERR_POST_X(16, (status == eIO_Timeout ? Warning : Error)
+                   << x_Message("Fetch",
+                                "CONN_Wait() failed",
+                                status, timeout));
+    }
+    return status;
+}
+
+
 const char* CConnException::GetErrCodeString(void) const
 {
     switch (GetErrCode()) {
@@ -674,4 +805,19 @@ const char* CConnException::GetErrCodeString(void) const
 }
 
 
+const char* CIO_Exception::GetErrCodeString(void) const
+{
+    switch (GetErrCode()) {
+    case eTimeout:       return "eIO_Timeout";
+    case eInterrupt:     return "eIO_Interrupt";
+    case eInvalidArg:    return "eIO_InvalidArg";
+    case eNotSupported:  return "eIO_NotSupported";
+    case eUnknown:       return "eIO_Unknown";
+    case eClosed:        return "eIO_Closed";
+    default:             break;
+    }
+    return CConnException::GetErrCodeString();
+}
+
+
 END_NCBI_SCOPE
