@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_test.cpp 598883 2019-12-16 17:09:52Z lavr $
+/* $Id: ncbi_conn_test.cpp 606555 2020-04-23 14:38:32Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,13 +31,13 @@
  */
 
 #include <ncbi_pch.hpp>
+#include "ncbi_comm.h"
+#include "ncbi_priv.h"
+#include "ncbi_servicep.h"
 #include <corelib/ncbiutil.hpp>
 #include <corelib/stream_utils.hpp>
 #include <connect/ncbi_conn_test.hpp>
 #include <connect/ncbi_socket.hpp>
-#include "ncbi_comm.h"
-#include "ncbi_priv.h"
-#include "ncbi_servicep.h"
 #include <iterator>
 #include <stdlib.h>
 
@@ -64,14 +64,7 @@ static const char kFWSign[] =
     "NCBI Firewall Daemon:  Invalid ticket.  Connection closed.";
 
 
-const STimeout CConnTest::kTimeout = {
-    (unsigned int)  DEF_CONN_TIMEOUT,
-    (unsigned int)((DEF_CONN_TIMEOUT - (unsigned int) DEF_CONN_TIMEOUT)
-                   * (double) kMicroSecondsPerSecond)
-};
-
-
-inline bool operator > (const STimeout* t1, const STimeout& t2)
+inline static bool operator > (const STimeout* t1, const STimeout& t2)
 {
     if (!t1)
         return true;
@@ -81,6 +74,12 @@ inline bool operator > (const STimeout* t1, const STimeout& t2)
 }
 
 
+inline static bool x_Large(const STimeout* t)
+{
+    return t > g_NcbiDefConnTimeout;
+}
+
+
 CConnTest::CConnTest(const STimeout* timeout,
                      CNcbiOstream* output, SIZE_TYPE width)
     : m_Output(output), m_Width(width),
@@ -93,7 +92,7 @@ CConnTest::CConnTest(const STimeout* timeout,
 void CConnTest::SetTimeout(const STimeout* timeout)
 {
     if (timeout) {
-        x_Timeout = timeout == kDefaultTimeout ? kTimeout : *timeout;
+        x_Timeout = timeout == kDefaultTimeout ? g_NcbiDefConnTimeout : *timeout;
         m_Timeout = &x_Timeout;
     } else
         m_Timeout = kInfiniteTimeout/*0*/;
@@ -595,7 +594,7 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
                 temp += "]CONN_SERVICE_NAME=\"";
                 temp += str;
                 temp += "\" from your configuration\n";
-            } else if (status != eIO_Timeout  ||  m_Timeout > kTimeout)
+            } else if (status != eIO_Timeout  ||  x_Large(m_Timeout))
                 temp += "; please contact " + HELP_EMAIL + '\n';
         }
         if (status != eIO_Timeout) {
@@ -626,8 +625,10 @@ EIO_Status CConnTest::x_GetFirewallConfiguration(const SConnNetInfo* net_info)
     static const char kFWDUrl[] = "/IEB/ToolBox/NETWORK/fwd_check.cgi";
 
     char fwdurl[128];
-    if (!ConnNetInfo_GetValue(0, "FWD_URL", fwdurl, sizeof(fwdurl), kFWDUrl))
+    if (!ConnNetInfo_GetValueInternal(0, "FWD_URL",
+                                      fwdurl, sizeof(fwdurl), kFWDUrl)) {
         return eIO_InvalidArg;
+    }
     SAuxData* auxdata = new SAuxData(m_Canceled, 0);
     CConn_HttpStream fwdcgi(fwdurl, net_info, kEmptyStr/*ushdr*/, s_GoodHeader,
                             auxdata, s_Adjust, s_Cleanup, 0/*flg*/, m_Timeout);
@@ -779,25 +780,28 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             empty = m_Fwd.empty()  &&  !m_FwdFB.empty();
             temp = "OK: ";
             if (!empty) {
-                stable_sort(m_Fwd.begin(),   m_Fwd.end());
                 temp += NStr::UInt8ToString(m_Fwd.size());
+                stable_sort(m_Fwd.begin(),   m_Fwd.end());
             }
-            size_t down = 0;
-            if (!m_FwdFB.empty()) {
+            size_t up = m_FwdFB.size();
+            if (up) {
                 stable_sort(m_FwdFB.begin(), m_FwdFB.end());
                 if (!empty)
                     temp += " + ";
-                temp += NStr::UInt8ToString(m_FwdFB.size());
+                temp += NStr::UInt8ToString(up);
+                size_t down = 0;
                 ITERATE(vector<CConnTest::CFWConnPoint>, cp, m_FwdFB) {
                     if (cp->status != eIO_Success)
                         ++down;
                 }
-                if (down)
+                if (down) {
                     temp += " - " + NStr::UInt8ToString(down);
+                    up -= down;
+                }
             }
-            down -= m_Fwd.size() + m_FwdFB.size();
-            if (down) {
-                temp += -down == 1 ? " port" : " ports";
+            up += m_Fwd.size();
+            if (up) {
+                temp += up == 1 ? " port" : " ports";
                 empty = false;
             } else
                 empty = true;
@@ -809,7 +813,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
         }
     } else if (status == eIO_Timeout) {
         temp = x_TimeoutMsg();
-        if (m_Timeout > kTimeout)
+        if (x_Large(m_Timeout))
             temp += "You may want to contact " + HELP_EMAIL;
     } else
         temp = "Please contact " + HELP_EMAIL;
@@ -864,13 +868,6 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
 }
 
 
-static inline SOCK x_GetSOCK(const CConn_IOStream* s)
-{
-    SOCK sock;
-    return CONN_GetSOCK(s->GetCONN(), &sock) == eIO_Success ? sock : 0;
-}
-
-
 EIO_Status CConnTest::CheckFWConnections(string* reason)
 {
     static const STimeout kZeroTmo = { 0, 0 };
@@ -983,18 +980,13 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
             }
             if (status != eIO_Success)
                 break;
-            vector< AutoPtr<CSocket> >  sock;
-            vector<CSocketAPI::SPoll>   poll;
+            vector<CSocketAPI::SPoll>  poll;
             ITERATE(vector<CFWCheck>, ck, fwck) {
                 CConn_IOStream* fw = ck->first.get();
                 if (!fw  ||  ck->second->status != eIO_Success)
                     continue;
-                AutoPtr<CSocket> s(new CSocket);
-                s->Reset(x_GetSOCK(fw), eNoOwnership, eCopyTimeoutsFromSOCK);
-                sock.push_back(s);
-                poll.push_back(CSocketAPI::SPoll(sock.back().get(), eIO_Read));
+                poll.push_back(CSocketAPI::SPoll(&fw->GetSocket(), eIO_Read));
             }
-            _ASSERT(poll.size() == sock.size());
             if (!poll.size())
                 break;
             status = CSocketAPI::Poll(poll, net_info->timeout);
@@ -1282,7 +1274,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                     temp += "The most likely reason for the failure is that"
                         " your firewall is still blocking ports as reported"
                         " above\n";
-                } else if (status != eIO_Timeout  ||  m_Timeout > kTimeout)
+                } else if (status != eIO_Timeout  ||  x_Large(m_Timeout))
                     temp += "Please contact " + HELP_EMAIL + '\n';
                 SERV_Close(iter);
             }
