@@ -1,4 +1,4 @@
-/* $Id: ncbi_connection.c 586755 2019-05-22 14:20:42Z lavr $
+/* $Id: ncbi_connection.c 606402 2020-04-21 14:50:28Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,6 +61,7 @@
                      : 0);                                                 \
       char stbuf[80];                                                      \
       if ((EIO_Status) status == eIO_Timeout  &&  timeout) {               \
+          assert(timeout != kDefaultTimeout);                              \
           sprintf(stbuf, "%s[%u.%06u]", ststr,                             \
                   (unsigned int)(timeout->sec + timeout->usec / 1000000),  \
                   (unsigned int)               (timeout->usec % 1000000)); \
@@ -109,7 +110,7 @@
       if (conn->magic != CONNECTION_MAGIC) {                            \
           static const STimeout* timeout = 0/*dummy*/;                  \
           CONN_LOG_EX(subcode, func_name, eLOG_Critical,                \
-                      "Corrupted connection handle", 0);                \
+                      "Corrupt connection handle", 0);                  \
           assert(0);                                                    \
       }                                                                 \
   } while (0)
@@ -171,7 +172,7 @@ typedef struct SConnectionTag {
     TNCBI_BigCount  r_pos;       /* read and ...                             */
     TNCBI_BigCount  w_pos;       /*          ... write positions             */
 
-    SCONN_Callback  cb[CONN_N_CALLBACKS];
+    SCONN_Callback  cb[CONN_N_CALLBACKS+1]; /* +1 includes dummy no-callback */
 
     unsigned int    magic;       /* magic cookie for integrity checks        */
 } SConnection;
@@ -218,8 +219,6 @@ static EIO_Status x_Callback(CONN conn, ECONN_Callback type, unsigned int flag)
     assert(!flag  ||  (type == eCONN_OnTimeout
                        &&  (flag | eIO_ReadWrite) == eIO_ReadWrite));
     assert(conn  &&  idx < CONN_N_CALLBACKS);
-    if (conn->state == eCONN_Unusable)
-        return eIO_Closed;
     if (!(func = conn->cb[idx].func))
         return type == eCONN_OnTimeout ? eIO_Timeout : eIO_Success;
     data = conn->cb[idx].data;
@@ -266,7 +265,8 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
     EIO_Status status;
     CONNECTOR  x_conn;
 
-    assert(conn->meta.list  ||  conn->state == eCONN_Unusable);
+    assert(!close  ||  !connector);
+    assert(!conn->meta.list == !(conn->state != eCONN_Unusable));
 
     /* flush connection first, if open */
     status = conn->meta.list  &&  conn->state == eCONN_Open
@@ -280,7 +280,7 @@ static EIO_Status x_ReInit(CONN conn, CONNECTOR connector, int/*bool*/ close)
                 break;
             status = eIO_NotSupported;
             CONN_LOG(4, ReInit, eLOG_Critical, "Partial re-init not allowed");
-            conn->state = eCONN_Bad;
+            conn->state = eCONN_Corrupt;
             return status;
         }
     }
@@ -355,10 +355,9 @@ static EIO_Status s_Open(CONN conn)
     switch (conn->state) {
     case eCONN_Unusable:
         return eIO_InvalidArg;
-    case eCONN_Bad:
-        return eIO_Closed;
     case eCONN_Cancel:
         return eIO_Interrupt;
+    case eCONN_Bad:
     case eCONN_Corrupt:
         return eIO_Unknown;
     default:
@@ -396,7 +395,7 @@ static EIO_Status s_Open(CONN conn)
         if (conn->state == eCONN_Closed)
             conn->state  = eCONN_Bad;
     }
-    return status;
+    return status == eIO_Closed ? eIO_Unknown : status;
 }
 
 
@@ -555,6 +554,7 @@ extern EIO_Status CONN_SetTimeout
     default:
         sprintf(errbuf, "Unknown event #%u", (unsigned int) event);
         CONN_LOG_EX(9, SetTimeout, eLOG_Error, errbuf, eIO_InvalidArg);
+        assert(0);
         return eIO_InvalidArg;
     }
 
@@ -610,8 +610,8 @@ extern EIO_Status CONN_Wait
 
     CONN_NOT_NULL(13, Wait);
 
-    if ((event != eIO_Read  &&  event != eIO_Write)
-        ||  timeout == kDefaultTimeout) {
+    if (event != eIO_Read  &&  event != eIO_Write) {
+        assert(0);
         return eIO_InvalidArg;
     }
 
@@ -625,6 +625,10 @@ extern EIO_Status CONN_Wait
         return eIO_Success;
 
     /* call current connector's "WAIT" method */
+    if (timeout == kDefaultTimeout) {
+        timeout  = conn->meta.default_timeout;
+        assert(timeout != kDefaultTimeout);
+    }
     status = conn->meta.wait
         ? conn->meta.wait(conn->meta.c_wait, event, timeout)
         : eIO_NotSupported;
@@ -636,7 +640,7 @@ extern EIO_Status CONN_Wait
         switch (status) {
         case eIO_Timeout:
             if (!timeout)
-                level = eLOG_Warning;
+                level = eLOG_Error; /*impossible*/
             else if (timeout->sec | timeout->usec)
                 level = eLOG_Trace;
             else
@@ -775,6 +779,7 @@ extern EIO_Status CONN_Write
     case eIO_WritePersist:
         return s_CONN_WritePersist(conn, buf, size, n_written);
     default:
+        assert(0);
         return eIO_NotSupported;
     }
 
@@ -991,6 +996,7 @@ extern EIO_Status CONN_Read
     case eIO_ReadPersist:
         return s_CONN_ReadPersist(conn, buf, size, n_read);
     default:
+        assert(0);
         return eIO_NotSupported;
     }
 
@@ -1150,6 +1156,7 @@ extern EIO_Status CONN_SetCallback
         char errbuf[80];
         sprintf(errbuf, "Unknown callback type #%u", (unsigned int) type);
         CONN_LOG_EX(29, SetCallback, eLOG_Error, errbuf, eIO_InvalidArg);
+        assert(0);
         return eIO_InvalidArg;
     }
 
@@ -1182,14 +1189,17 @@ extern EIO_Status CONN_GetSOCK(CONN conn, SOCK* sock)
     assert((conn->state & eCONN_Open)  &&  conn->meta.list);
 
     x_conn = conn->meta.list;
-    if (x_conn  &&  x_conn->meta  &&  x_conn->meta->get_type
-        &&  x_conn->meta->get_type(x_conn->meta->c_get_type)
-        == g_kNcbiSockNameAbbr) {
-        /* HACK * HACK * HACK */
-        SOCK* x_sock = (SOCK*) x_conn->handle;
-        if (x_sock) {
-            *sock = *x_sock;
-            return eIO_Success;
+    if (x_conn  &&  x_conn->meta  &&  x_conn->meta->get_type) {
+        const char* type = x_conn->meta->get_type(x_conn->meta->c_get_type);
+        if (type == g_kNcbiSockNameAbbr
+            ||  ((type = strrchr(type, '/')) != 0
+                 &&  strcmp(++type, g_kNcbiSockNameAbbr) == 0)) {
+            /* HACK * HACK * HACK */
+            SOCK* x_sock = (SOCK*) x_conn->handle;
+            if (x_sock) {
+                *sock = *x_sock;
+                return eIO_Success;
+            }
         }
     }
     return eIO_Closed;
