@@ -1,4 +1,4 @@
-/* $Id: ncbi_ftp_connector.c 593816 2019-09-25 01:34:38Z lavr $
+/* $Id: ncbi_ftp_connector.c 610782 2020-06-23 20:07:07Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,8 +33,8 @@
  *
  *   Minimum FTP implementation: RFC 1123 (4.1.2.13)
  *
- *   See <connect/ncbi_connector.h> for the detailed specification of
- *   the connector's methods and structures.
+ *   See <connect/ncbi_connector.h> for the detailed specification of the
+ *   connector's methods and structures.
  *
  *   Note:  We do not implement transfers of files whose names include CR or LF
  *          character(s):  for those to work, all FTP commands will have to be
@@ -48,6 +48,7 @@
 
 #include "ncbi_ansi_ext.h"
 #include "ncbi_priv.h"
+#include "ncbi_servicep.h"
 #include <connect/ncbi_ftp_connector.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -66,16 +67,17 @@
  ***********************************************************************/
 
 enum EFTP_Feature {                /* NB: values must occupy 12 bits at most */
-    fFtpFeature_NOOP =  0x001,     /* all implementations MUST support       */
-    fFtpFeature_SYST =  0x002,
-    fFtpFeature_SITE =  0x004,
-    fFtpFeature_FEAT =  0x008,
-    fFtpFeature_MDTM =  0x010,
-    fFtpFeature_SIZE =  0x020,
-    fFtpFeature_REST =  0x040,
-    fFtpFeature_MLSx =  0x080,
-    fFtpFeature_EPRT =  0x100,
-    fFtpFeature_EPSV = 0x1000,
+    fFtpFeature_NOOP =  0x001,     /* all implementations MUST support RFC959*/
+    fFtpFeature_SYST =  0x002,     /* RFC 959                                */
+    fFtpFeature_SITE =  0x004,     /* RFC 959, 1123                          */
+    fFtpFeature_FEAT =  0x008,     /* RFC 3659                               */
+    fFtpFeature_MDTM =  0x010,     /* RFC 3659                               */
+    fFtpFeature_SIZE =  0x020,     /* RFC 3659                               */
+    fFtpFeature_REST =  0x040,     /* RFC 3659, NB: FEAT                     */
+    fFtpFeature_MLSx =  0x080,     /* RFC 3659                               */
+    fFtpFeature_EPRT =  0x100,     /* RFC 2428                               */
+    fFtpFeature_MFF  =  0x200,     /* draft-somers-ftp-mfxx-04, NB: FEAT     */
+    fFtpFeature_EPSV = 0x1000,     /* RFC 2428                               */
     fFtpFeature_APSV = 0x3000      /* EPSV ALL -- a la "APSV" from RFC 1579  */
 };
 typedef unsigned short TFTP_Features; /* bitwise OR of EFtpFeature */
@@ -107,6 +109,7 @@ typedef struct {
 } SFTPConnector;
 
 
+
 static const STimeout kFailsafeTimeout = { 10, 0 };
 static const STimeout kZeroTimeout     = {  0, 0 };
 static const char     kDigits[] = "0123456789";
@@ -464,7 +467,7 @@ static EIO_Status x_FTPHelpCB(SFTPConnector* xxx, int code,
             else
                 xxx->feat &= (TFTP_Features)(~fFtpFeature_EPRT);
         }
-        if ((s = x_4Word(line, "EPSV")) != 0) {  /* RFC 2428 (cf 1579) */
+        if ((s = x_4Word(line, "EPSV")) != 0) {  /* RFC 2428 (cf RFC 1579) */
             if (s[4 + strspn(s + 4, " \t")] != '*')
                 xxx->feat |=                  fFtpFeature_EPSV;
             else
@@ -498,7 +501,9 @@ static EIO_Status x_FTPFeatCB(SFTPConnector* xxx, int code,
     if (!lineno)
         return code == 211 ? eIO_Success : eIO_NotSupported;
     if (code  &&  strlen(line) >= 4
-        &&  (!line[4]  ||  isspace((unsigned char) line[4]))) {
+        &&  (!line[4]
+             ||  isspace((unsigned char) line[3])
+             ||  isspace((unsigned char) line[4]))) {
         assert(code == 211);
         if      (strncasecmp(line, "MDTM", 4) == 0)
             xxx->feat |= fFtpFeature_MDTM;
@@ -512,6 +517,10 @@ static EIO_Status x_FTPFeatCB(SFTPConnector* xxx, int code,
             xxx->feat |= fFtpFeature_EPRT;
         else if (strncasecmp(line, "EPSV", 4) == 0)
             xxx->feat |= fFtpFeature_EPSV;
+        else if (strncasecmp(line, "MFMT", 4) == 0)
+            xxx->feat |= fFtpFeature_MFF;   /* fFtpFeature_MFF not yet supp */
+        else if (strncasecmp(line, "MFF ", 4) == 0)
+            xxx->feat |= fFtpFeature_MFF;   /* NB: modify facts must follow */
     }
     return eIO_Success;
 }
@@ -720,7 +729,7 @@ static EIO_Status x_FTPDir(SFTPConnector* xxx,
     if (toupper((unsigned char)(*cmd)) == 'R') { /* [X]RMD */
         if (code != 250)
             return eIO_Unknown;
-    }else if (toupper((unsigned char)(*cmd)) == 'C') { /* [X]CWD, CDUP/XCUP */
+    } else if (toupper((unsigned char)(*cmd)) == 'C') { /* [X]CWD, CDUP/XCUP */
         if (code != 200  &&  code != 250)
             return eIO_Unknown;
         /* fixup codes w/accordance to RFC959 */
@@ -854,7 +863,6 @@ static EIO_Status x_FTPEpsv(SFTPConnector*  xxx,
         return n == 200 ? eIO_Success : eIO_NotSupported;
     if (n != 229)
         return xxx->feat & fFtpFeature_APSV ? eIO_Unknown : eIO_NotSupported;
-    buf[sizeof(buf) - 1] = '\0';
     if (!(s = strchr(buf, '('))  ||  !(d = *++s)  ||  *++s != d  ||  *++s != d
         ||  sscanf(++s, "%u%c%n", &p, buf, &n) < 2  ||  p > 0xFFFF
         ||  *buf != d  ||  s[n] != ')') {
@@ -882,7 +890,6 @@ static EIO_Status x_FTPPasv(SFTPConnector*  xxx,
     status = s_FTPReply(xxx, &code, buf, sizeof(buf) - 1, 0);
     if (status != eIO_Success  ||  code != 227)
         return eIO_Unknown;
-    buf[sizeof(buf) - 1] = '\0';
     for (;;) {
         char* c;
         size_t len;
@@ -1649,7 +1656,7 @@ static EIO_Status s_FTPPollCntl(SFTPConnector* xxx, const STimeout* timeout)
         if (timeout != &kZeroTimeout) {
             SOCK_SetTimeout(xxx->cntl, eIO_Read,
                             timeout ? timeout : &kFailsafeTimeout);
-            timeout  = &kZeroTimeout;
+            timeout  = &kZeroTimeout;  /* set timeout external / only once */
         }
         status = s_FTPReply(xxx, &code, buf, sizeof(buf) - 1, 0);
         if (status == eIO_Success) {
@@ -2073,9 +2080,7 @@ static EIO_Status s_VT_Flush
             : xxx->r_status != eIO_Success ? xxx->r_status
             : xxx->w_status != eIO_Success ? xxx->w_status : eIO_Closed;
     }
-    if (!BUF_Size(xxx->wbuf))
-        return eIO_Success;
-    return s_FTPExecute(xxx, timeout);
+    return BUF_Size(xxx->wbuf) ? s_FTPExecute(xxx, timeout) : eIO_Success;
 }
 
 
@@ -2279,7 +2284,9 @@ extern CONNECTOR s_CreateConnector(const SConnNetInfo*  info,
         free(ccc);
         return 0;
     }
-    xinfo = info ? ConnNetInfo_Clone(info) : ConnNetInfo_Create("_FTP");
+    xinfo = (info
+             ? ConnNetInfo_Clone(info)
+             : ConnNetInfo_CreateInternal("_FTP"));
     if (!(xxx->info = xinfo)) {
         free(ccc);
         free(xxx);
