@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_session.cpp 578389 2019-01-17 18:08:54Z lavr $
+/* $Id: ncbi_http_session.cpp 612999 2020-07-30 19:22:55Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,13 +32,14 @@
  */
 
 #include <ncbi_pch.hpp>
+#include "ncbi_ansi_ext.h"
+#include "ncbi_servicep.h"
 #include <corelib/ncbifile.hpp>
 #include <corelib/request_ctx.hpp>
 #include <corelib/ncbimtx.hpp>
 #include <corelib/ncbistr.hpp>
 #include <connect/ncbi_http_session.hpp>
 #include <stdlib.h>
-#include "ncbi_ansi_ext.h"
 
 
 BEGIN_NCBI_SCOPE
@@ -145,10 +146,10 @@ void CHttpHeaders::ClearAll(void)
 }
 
 
-void CHttpHeaders::ParseHttpHeader(const CTempString& headers)
+void s_ParseHttpHeader(const CTempString& from, CHttpHeaders::THeaders& to)
 {
     list<CTempString> lines;
-    NStr::Split(headers, HTTP_EOL, lines,
+    NStr::Split(from, HTTP_EOL, lines,
                 NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
 
     string name, value;
@@ -162,11 +163,18 @@ void CHttpHeaders::ParseHttpHeader(const CTempString& headers)
         name  = line->substr(0, delim);
         value = line->substr(delim + 1);
         NStr::TruncateSpacesInPlace(value, NStr::eTrunc_Both);
-        m_Headers[name].push_back(value);
+        to[name].push_back(value);
     }
 }
 
 
+// Have to keep this method as it may be used by some clients
+void CHttpHeaders::ParseHttpHeader(const CTempString& headers)
+{
+    s_ParseHttpHeader(headers, m_Headers);
+}
+
+
 string CHttpHeaders::GetHttpHeader(void) const
 {
     string ret;
@@ -455,13 +463,13 @@ bool CHttpFormData::IsEmpty(void) const
 //
 
 
-CHttpResponse::CHttpResponse(CHttpSession&   session,
+CHttpResponse::CHttpResponse(CHttpSession_Base&   session,
                              const CUrl&     url,
-                             CHttpStreamRef& stream)
+                             shared_ptr<iostream> stream)
     : m_Session(&session),
       m_Url(url),
       m_Location(url),
-      m_Stream(&stream),
+      m_Stream(move(stream)),
       m_Headers(new CHttpHeaders),
       m_StatusCode(0)
 {
@@ -470,27 +478,27 @@ CHttpResponse::CHttpResponse(CHttpSession&   session,
 
 CNcbiIstream& CHttpResponse::ContentStream(void) const
 {
-    _ASSERT(m_Stream  &&  m_Stream->IsInitialized());
+    _ASSERT(m_Stream);
     if ( !CanGetContentStream() ) {
         NCBI_THROW(CHttpSessionException, eBadStream,
             string("Content stream is not available for status '") +
             NStr::NumericToString(m_StatusCode) + " " +
             m_StatusText + "'");
     }
-    return m_Stream->GetConnStream();
+    return *m_Stream;
 }
 
 
 CNcbiIstream& CHttpResponse::ErrorStream(void) const
 {
-    _ASSERT(m_Stream  &&  m_Stream->IsInitialized());
+    _ASSERT(m_Stream);
     if ( CanGetContentStream() ) {
         NCBI_THROW(CHttpSessionException, eBadStream,
             string("Error stream is not available for status '") +
             NStr::NumericToString(m_StatusCode) + " " +
             m_StatusText + "'");
     }
-    return m_Stream->GetConnStream();
+    return *m_Stream;
 }
 
 
@@ -500,25 +508,13 @@ bool CHttpResponse::CanGetContentStream(void) const
 }
 
 
-void CHttpResponse::x_ParseHeader(const char* header)
+void CHttpResponse::x_Update(CHttpHeaders::THeaders headers, int status_code, string status_text)
 {
-    // Prevent collecting multiple headers on redirects.
-    m_Headers->ClearAll();
-    m_Headers->ParseHttpHeader(header);
-    
-    m_Session->x_SetCookies(m_Headers->GetAllValues(CHttpHeaders::eSetCookie),
-        &m_Location);
-
-    // Parse status code/text.
-    const char* eol = strstr(header, HTTP_EOL);
-    string status = eol ? string(header, eol - header) : header;
-    if ( NStr::StartsWith(status, "HTTP/") ) {
-        int text_pos = 0;
-        sscanf(status.c_str(), "%*s %d %n", &m_StatusCode, &text_pos);
-        if (text_pos > 0) {
-            m_StatusText = status.substr(text_pos);
-        }
-    }
+    m_Headers->m_Headers.swap(headers);
+    m_StatusCode = status_code;
+    m_StatusText = move(status_text);
+    const auto& cookies = m_Headers->GetAllValues(CHttpHeaders::eSetCookie);
+    m_Session->x_SetCookies(cookies, &m_Location);
 }
 
 
@@ -532,7 +528,8 @@ unsigned short SGetHttpDefaultRetries::operator()(void) const
 #define _STR(s)  #s
 #define  STR(s)  _STR(s)
     char buf[16];
-    ConnNetInfo_GetValue(0, REG_CONN_MAX_TRY, buf, sizeof(buf), STR(DEF_CONN_MAX_TRY));
+    ConnNetInfo_GetValueInternal(0, REG_CONN_MAX_TRY,
+                                 buf, sizeof(buf), STR(DEF_CONN_MAX_TRY));
 #undef   STR
 #undef  _STR
     int maxtry = atoi(buf);
@@ -547,7 +544,7 @@ unsigned short x_RetriesToMaxtry(unsigned short retries)
 }
 
 
-CHttpRequest::CHttpRequest(CHttpSession& session,
+CHttpRequest::CHttpRequest(CHttpSession_Base& session,
                            const CUrl&   url,
                            EReqMethod    method)
     : m_Session(&session),
@@ -687,27 +684,32 @@ CHttpResponse CHttpRequest::Execute(void)
     SRetryProcessing retry_processing(m_RetryProcessing, m_Deadline, m_Url,
             m_Method, m_Headers, m_FormData);
     CRef<CHttpResponse> ret;
+    auto protocol = m_Session->GetProtocol();
 
     do {
         // Connection not open yet.
         // Only POST and PUT support sending form data.
         bool have_data = m_FormData  &&  !m_FormData.Empty();
         if ( !m_Response ) {
-            x_InitConnection(have_data);
+            if (m_Stream) {
+                NCBI_THROW(CHttpSessionException, eBadRequest,
+                    "An attempt to execute HTTP request already being executed");
+            }
+
+            m_Session->x_StartRequest(protocol, *this, have_data);
         }
         _ASSERT(m_Response);
-        _ASSERT(m_Stream  &&  m_Stream->IsInitialized());
-        CConn_IOStream& out = m_Stream->GetConnStream();
+        _ASSERT(m_Stream);
         if ( have_data ) {
-            m_FormData->WriteFormData(out);
+            m_FormData->WriteFormData(*m_Stream);
         }
         // Send data to the server and close output stream.
-        out.peek();
-        m_Stream.Reset();
+        m_Stream->peek();
+        m_Stream.reset();
         ret = m_Response;
         m_Response.Reset();
     }
-    while (retry_processing(ret->Headers()));
+    while (m_Session->x_Downgrade(*ret, protocol) || retry_processing(ret->Headers()));
 
     return *ret;
 }
@@ -720,11 +722,16 @@ CNcbiOstream& CHttpRequest::ContentStream(void)
             "Request method does not allow writing to the output stream");
     }
     if ( !m_Stream ) {
-        x_InitConnection(false);
+        if (m_Response) {
+            NCBI_THROW(CHttpSessionException, eBadRequest,
+                "An attempt to execute HTTP request already being executed");
+        }
+
+        m_Session->x_StartRequest(m_Session->GetProtocol(), *this, false);
     }
     _ASSERT(m_Response);
-    _ASSERT(m_Stream  &&  m_Stream->IsInitialized());
-    return m_Stream->GetConnStream();
+    _ASSERT(m_Stream);
+    return *m_Stream;
 }
 
 
@@ -745,12 +752,26 @@ CHttpFormData& CHttpRequest::FormData(void)
 }
 
 
+void CHttpRequest::x_AdjustHeaders(bool use_form_data)
+{
+    x_AddCookieHeader(m_Url, true);
+    if (use_form_data) {
+        m_Headers->SetValue(CHttpHeaders::eContentType,
+            m_FormData->GetContentTypeStr());
+    }
+}
+
+
+void CHttpRequest::x_UpdateResponse(CHttpHeaders::THeaders headers, int status_code, string status_text)
+{
+    if (m_Response) {
+        m_Response->x_Update(move(headers), status_code, move(status_text));
+    }
+}
+
+
 void CHttpRequest::x_InitConnection(bool use_form_data)
 {
-    if (m_Response  ||  m_Stream) {
-        NCBI_THROW(CHttpSessionException, eBadRequest,
-            "An attempt to execute HTTP request already being executed");
-    }
     SConnNetInfo* net_info = ConnNetInfo_Create(
         m_Url.IsService() ? m_Url.GetService().c_str() : 0);
     if (m_Session->GetProtocol() == CHttpSession::eHTTP_11) {
@@ -769,16 +790,10 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
 
     // Save headers set automatically (e.g. from CONN_HTTP_USER_HEADER).
     if (net_info->http_user_header) {
-        CHttpHeaders usr_hdr;
-        usr_hdr.ParseHttpHeader(net_info->http_user_header);
-        m_Headers->Merge(usr_hdr);
+        s_ParseHttpHeader(net_info->http_user_header, m_Headers->m_Headers);
     }
 
-    x_AddCookieHeader(m_Url, true);
-    if (use_form_data) {
-        m_Headers->SetValue(CHttpHeaders::eContentType,
-            m_FormData->GetContentTypeStr());
-    }
+    x_AdjustHeaders(use_form_data);
     string headers = m_Headers->GetHttpHeader();
 
     if ( !m_Timeout.IsDefault() ) {
@@ -789,12 +804,11 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
         net_info->max_try = x_RetriesToMaxtry(m_Retries);
     }
 
-    m_Stream.Reset(new TStreamRef);
-    m_Response.Reset(new CHttpResponse(*m_Session, m_Url, *m_Stream));
+    m_Response.Reset(new CHttpResponse(*m_Session, m_Url));
     if ( !m_Url.IsService() ) {
         // Connect using HTTP.
         m_IsService = false;
-        m_Stream->SetConnStream(new CConn_HttpStream(
+        m_Stream.reset(new CConn_HttpStream(
             m_Url.ComposeUrl(CUrlArgs::eAmp_Char),
             net_info,
             headers.c_str(),
@@ -815,16 +829,25 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
         x_extra.parse_header = sx_ParseHeader;
         x_extra.flags = m_Session->GetHttpFlags() | fHTTP_AdjustOnRedirect;
         ConnNetInfo_SetUserHeader(net_info, headers.c_str());
-        m_Stream->SetConnStream(new CConn_ServiceStream(
+        m_Stream.reset(new CConn_ServiceStream(
             m_Url.GetService(), // Ignore other fields now, set them in sx_Adjust
             fSERV_Http,
             net_info,
             &x_extra));
     }
+    m_Response->m_Stream = m_Stream;
     ConnNetInfo_Destroy(net_info);
 }
 
 
+void CHttpRequest::x_InitConnection2(shared_ptr<iostream> stream, bool is_service)
+{
+    m_Stream = move(stream);
+    m_IsService = is_service;
+    m_Response.Reset(new CHttpResponse(*m_Session, m_Url, m_Stream));
+}
+
+
 bool CHttpRequest::x_CanSendData(void) const
 {
     return m_Method == eReqMethod_Post  ||
@@ -856,7 +879,24 @@ EHTTP_HeaderParse CHttpRequest::sx_ParseHeader(const char* http_header,
     // Response can be NULL, e.g. if an exception was thrown while
     // initializing the request.
     if ( resp ) {
-        resp->x_ParseHeader(http_header);
+        // Prevent collecting multiple headers on redirects.
+        CHttpHeaders::THeaders headers;
+        s_ParseHttpHeader(http_header, headers);
+        
+        // Parse status code/text.
+        const char* eol = strstr(http_header, HTTP_EOL);
+        string status = eol ? string(http_header, eol - http_header) : http_header;
+        int status_code = 0;
+        string status_text;
+        if ( NStr::StartsWith(status, "HTTP/") ) {
+            int text_pos = 0;
+            sscanf(status.c_str(), "%*s %d %n", &status_code, &text_pos);
+            if (text_pos > 0) {
+                status_text = status.substr(text_pos);
+            }
+        }
+
+        resp->x_Update(headers, status_code, status_text);
     }
     // Always read response body - normal content or error.
     return eHTTP_HeaderContinue;
@@ -955,24 +995,24 @@ CHttpRequest& CHttpRequest::SetRetryProcessing(ESwitch on_off)
 
 
 ///////////////////////////////////////////////////////
-//  CHttpSession::
+//  CHttpSession_Base::
 //
 
 
-CHttpSession::CHttpSession(void)
+CHttpSession_Base::CHttpSession_Base(void)
     : m_Protocol(eHTTP_10),
       m_HttpFlags(0)
 {
 }
 
 
-CHttpRequest CHttpSession::NewRequest(const CUrl& url, ERequestMethod method)
+CHttpRequest CHttpSession_Base::NewRequest(const CUrl& url, ERequestMethod method)
 {
     return CHttpRequest(*this, url, EReqMethod(method));
 }
 
 
-CHttpResponse CHttpSession::Get(const CUrl&     url,
+CHttpResponse CHttpSession_Base::Get(const CUrl& url,
                                 const CTimeout& timeout,
                                 THttpRetries    retries)
 {
@@ -983,7 +1023,7 @@ CHttpResponse CHttpSession::Get(const CUrl&     url,
 }
 
 
-CHttpResponse CHttpSession::Post(const CUrl&     url,
+CHttpResponse CHttpSession_Base::Post(const CUrl& url,
                                  CTempString     data,
                                  CTempString     content_type,
                                  const CTimeout& timeout,
@@ -1003,7 +1043,7 @@ CHttpResponse CHttpSession::Post(const CUrl&     url,
 }
 
 
-CHttpResponse CHttpSession::Put(const CUrl&     url,
+CHttpResponse CHttpSession_Base::Put(const CUrl& url,
                                 CTempString     data,
                                 CTempString     content_type,
                                 const CTimeout& timeout,
@@ -1027,7 +1067,7 @@ CHttpResponse CHttpSession::Put(const CUrl&     url,
 DEFINE_STATIC_FAST_MUTEX(s_SessionMutex);
 
 
-void CHttpSession::x_SetCookies(const CHttpHeaders::THeaderValues& cookies,
+void CHttpSession_Base::x_SetCookies(const CHttpHeaders::THeaderValues& cookies,
                                 const CUrl*                        url)
 {
     CFastMutexGuard lock(s_SessionMutex);
@@ -1037,7 +1077,7 @@ void CHttpSession::x_SetCookies(const CHttpHeaders::THeaderValues& cookies,
 }
 
 
-string CHttpSession::x_GetCookies(const CUrl& url) const
+string CHttpSession_Base::x_GetCookies(const CUrl& url) const
 {
     string cookies;
     CFastMutexGuard lock(s_SessionMutex);
