@@ -1,4 +1,4 @@
-/* $Id: ncbi_mbedtls.c 599049 2019-12-18 18:59:30Z lavr $
+/* $Id: ncbi_mbedtls.c 629075 2021-04-09 14:32:17Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 #include "ncbi_ansi_ext.h"
 #include "ncbi_connssl.h"
 #include "ncbi_priv.h"
-#include <connect/ncbi_connutil.h>
+#include "ncbi_servicep.h"
 #include <connect/ncbi_mbedtls.h>
 #include <stdlib.h>
 
@@ -179,7 +179,9 @@ static EIO_Status x_RetryStatus(SOCK sock, EIO_Event direction)
     if (direction == eIO_Open) {
         EIO_Status r_status = SOCK_Status(sock, eIO_Read);
         EIO_Status w_status = SOCK_Status(sock, eIO_Write);
-        status = r_status > w_status ? r_status : w_status;
+        status = r_status != eIO_Closed  &&  w_status != eIO_Closed
+            ? r_status > w_status ? r_status : w_status
+            : eIO_Closed;
     } else
         status = SOCK_Status(sock, direction);
     return status == eIO_Success ? eIO_Timeout : status;
@@ -211,7 +213,9 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
         status = eIO_Timeout;
         break;
     case MBEDTLS_ERR_THREADING_FEATURE_UNAVAILABLE:
+    case MBEDTLS_ERR_SSL_NO_USABLE_CIPHERSUITE:
     case MBEDTLS_ERR_SSL_FEATURE_UNAVAILABLE:
+    case MBEDTLS_ERR_SSL_UNKNOWN_CIPHER:
     case MBEDTLS_ERR_SSL_INTERNAL_ERROR:
         status = eIO_NotSupported;
         break;
@@ -219,24 +223,25 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
     case MBEDTLS_ERR_SSL_BAD_INPUT_DATA:
         status = eIO_InvalidArg;
         break;
-    case MBEDTLS_ERR_NET_CONN_RESET:
     case MBEDTLS_ERR_SSL_CONN_EOF:
     case MBEDTLS_ERR_SSL_PEER_CLOSE_NOTIFY:
-    case MBEDTLS_ERR_SSL_FATAL_ALERT_MESSAGE:
         status = eIO_Closed;
         break;
+    case MBEDTLS_ERR_NET_CONN_RESET:
+    case MBEDTLS_ERR_SSL_FATAL_ALERT_MESSAGE:
+        status = eIO_Unknown;
+        break;
     case MBEDTLS_ERR_NET_RECV_FAILED:
         if (sock->r_status != eIO_Success  &&
-            sock->r_status != eIO_Unknown) {
+            sock->r_status != eIO_Closed) {
             status = (EIO_Status) sock->r_status;
         } else
             status = eIO_Unknown;
         break;
     case MBEDTLS_ERR_NET_SEND_FAILED:
-        if (sock->w_status != eIO_Success  &&
-            sock->w_status != eIO_Unknown) {
+        if (sock->w_status != eIO_Success)
             status = (EIO_Status) sock->w_status;
-        } else
+        else
             status = eIO_Unknown;
         break;
     case MBEDTLS_ERR_SSL_NON_FATAL:
@@ -246,9 +251,9 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
         break;
     }
 
+    assert(status != eIO_Success);
     CORE_TRACEF(("MBEDTLS error %d -> CONNECT MBEDTLS status %s",
                  error, IO_StatusStr(status)));
-
     return status;
 }
 
@@ -290,7 +295,7 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
         CORE_TRACEF(("CONNECT MBEDTLS status %s -> %s %d",
                      IO_StatusStr(status), error ? "error" : "errno", x_err));
         if (!error)
-            errno = x_err; /* restore errno that may be clobbered by log */
+            errno = x_err; /* restore errno that might have been clobbered */
     }}
 
     if (error)
@@ -319,6 +324,7 @@ static void* s_MbedTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
                ? MBEDTLS_SSL_IS_CLIENT
                : MBEDTLS_SSL_IS_SERVER);
     mbedtls_ssl_context* session;
+    void** xcred;
     int err;
 
     if (end == MBEDTLS_SSL_IS_SERVER) {
@@ -338,8 +344,9 @@ static void* s_MbedTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
             *error = 0;
             return 0;
         }
-        CORE_LOG(eLOG_Critical, "MBEDTLS credentials not implemented");
-    }
+        xcred = ctx->cred->data;
+    } else
+        xcred = 0;
 
     if (!(session = (mbedtls_ssl_context*) malloc(sizeof(*session)))) {
         *error = errno;
@@ -347,9 +354,14 @@ static void* s_MbedTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
     }
     mbedtls_ssl_init(session);
 
-    if ((err = mbedtls_ssl_setup(session, &s_MbedTlsConf))        != 0  ||
+    if ((err = mbedtls_ssl_setup(session, &s_MbedTlsConf))        != 0   ||
         (ctx->host  &&  *ctx->host
-         &&  (err = mbedtls_ssl_set_hostname(session, ctx->host)) != 0)) {
+         &&  (err = mbedtls_ssl_set_hostname(session, ctx->host)) != 0)  ||
+        (xcred
+         &&  (err = mbedtls_ssl_set_hs_own_cert
+              (session,
+               (mbedtls_x509_crt*)   xcred[0],
+               (mbedtls_pk_context*) xcred[1])) != 0)) {
         mbedtls_ssl_free(session);
         free(session);
         *error = err;
@@ -670,9 +682,12 @@ static EIO_Status s_MbedTlsInit(FSSLPull pull, FSSLPush push)
                                 MBEDTLS_SSL_PRESET_DEFAULT);
     mbedtls_ssl_conf_authmode(&s_MbedTlsConf, MBEDTLS_SSL_VERIFY_NONE);
 
-    val = ConnNetInfo_GetValue(0, "MBEDTLS_LOGLEVEL", buf, sizeof(buf), 0);
-    if (!val  ||  !*val)
-        val = ConnNetInfo_GetValue(0, "TLS_LOGLEVEL", buf, sizeof(buf), 0);
+    val = ConnNetInfo_GetValueInternal(0, "MBEDTLS_LOGLEVEL",
+                                       buf, sizeof(buf), 0);
+    if (!val  ||  !*val) {
+        val = ConnNetInfo_GetValueInternal(0, "TLS_LOGLEVEL",
+                                           buf, sizeof(buf), 0);
+    }
     CORE_LOCK_READ;
     if (val  &&  *val) {
         ELOG_Level level;
@@ -775,12 +790,24 @@ extern SOCKSSL NcbiSetupMbedTls(void)
 }
 
 
-extern NCBI_CRED NcbiCredMbedTls(void* xcred)
+extern NCBI_CRED NcbiCredMbedTls(void* xcert, void* xpkey)
 {
-    struct SNcbiCred* cred = (NCBI_CRED) calloc(xcred ? 2 : 1, sizeof(*cred));
-    if (cred  &&  xcred) {
+    struct SNcbiCred* cred;
+    size_t size = sizeof(*cred);
+    if (xcert  &&  xpkey) {
+        size <<= 1;
+        size  += 2 * sizeof(void*);
+    }
+    cred = (NCBI_CRED) malloc(size);
+    if (cred) {
+        memset(cred, 0, size);
         cred->type = eNcbiCred_MbedTls;
-        cred->data = xcred;
+        if (xcert  &&  xpkey) {
+            void** data = (void**)((char*) cred + 2 * sizeof(*cred));
+            data[0]     = xcert;
+            data[1]     = xpkey;
+            cred->data  = data;
+        }
     }
     return cred;
 }
