@@ -1,4 +1,4 @@
-/* $Id: ncbi_namerd.c 603614 2020-03-13 13:35:52Z ivanov $
+/* $Id: ncbi_namerd.c 612632 2020-07-24 17:49:15Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -412,7 +412,7 @@ static TReqMethod s_GetReqMethod()
 {
     char val[20];
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_API_REQ_KEY, val, sizeof(val)-1,
         REG_NAMERD_API_REQ_DEF))
     {
@@ -434,7 +434,7 @@ static EBURLScheme s_GetScheme()
 {
     char val[12];
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_API_SCHEME_KEY, val, sizeof(val)-1,
         REG_NAMERD_API_SCHEME_DEF))
     {
@@ -466,7 +466,7 @@ static int s_GetHttpProxy(char* host, size_t host_size, unsigned short* port_p)
             break;
     }
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_PROXY_HOST_KEY, host, host_size - 1,
         REG_NAMERD_PROXY_HOST_DEF)  ||  ! *host)
     {
@@ -475,7 +475,7 @@ static int s_GetHttpProxy(char* host, size_t host_size, unsigned short* port_p)
         return 0;
     }
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
             REG_NAMERD_PROXY_PORT_KEY, port_str, sizeof(port_str) - 1,
             REG_NAMERD_PROXY_PORT_DEF)  ||
          ! *port_str  ||
@@ -661,7 +661,7 @@ static TNCBI_Time s_ParseExpires(time_t tt_now, const char* expires)
         int     now_hour = tm_now.tm_hour;
         int     now_min  = tm_now.tm_min;
         int     now_sec  = tm_now.tm_sec;
-        char    now_str[32]; /* need 21 for 'yyyy-mm-ddThh:mm:ssZ' */
+        char    now_str[40]; /* need 21 for 'yyyy-mm-ddThh:mm:ssZ' */
         double  td_diff, td_hour, td_min, td_sec;
         const char* td_sign;
 
@@ -1249,7 +1249,7 @@ static void s_UpdateDtabFromRegistry(char** dtab_p, int* success_p,
         return;
     }
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_DTAB_KEY, val, MAX_QRY_STR_LEN,
         REG_NAMERD_DTAB_DEF))
     {
@@ -1587,7 +1587,7 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
     /* Make sure a net_info exists. */
     SConnNetInfo* new_net_info = NULL;
     if ( ! net_info) {
-        new_net_info = ConnNetInfo_Create(iter->name);
+        new_net_info = ConnNetInfo_CreateInternal(iter->name);
         if ( ! new_net_info) {
             CORE_LOG_X(eNSub_Alloc, eLOG_Critical, "Couldn't create net_info.");
             s_Close(iter);
@@ -1639,19 +1639,19 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
         return NULL;
     }
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_API_HOST_KEY, data->net_info->host,
         sizeof(data->net_info->host), REG_NAMERD_API_HOST_DEF))
     {
         data->net_info->host[0] = NIL;
     }
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_API_PATH_KEY, data->net_info->path,
         sizeof(data->net_info->path), REG_NAMERD_API_PATH_DEF))
     {
         data->net_info->path[0] = NIL;
     }
-    if (ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if (ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_API_ENV_KEY, namerd_env, sizeof(namerd_env),
         REG_NAMERD_API_ENV_DEF))
     {
@@ -1662,7 +1662,7 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
             strcat(data->net_info->path, namerd_env);
         }
     }
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+    if ( ! ConnNetInfo_GetValueInternal(REG_NAMERD_SECTION,
         REG_NAMERD_API_ARGS_KEY, namerd_args, sizeof(namerd_args),
         REG_NAMERD_API_ARGS_DEF)
          ||  strlen(namerd_args) + strlen(iter->name) >= sizeof(namerd_args))
