@@ -1,4 +1,4 @@
-/* $Id: ncbi_sendmail.c 575956 2018-12-06 20:25:49Z lavr $
+/* $Id: ncbi_sendmail.c 606555 2020-04-23 14:38:32Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
 
 #include "ncbi_ansi_ext.h"
 #include "ncbi_priv.h"
-#include <connect/ncbi_connutil.h>
+#include "ncbi_servicep.h"
 #include <connect/ncbi_sendmail.h>
 #include <ctype.h>
 #include <errno.h>
@@ -280,15 +280,16 @@ static void x_Sendmail_InitEnv(void)
     if (s_MxPort)
         return;
 
-    if (!ConnNetInfo_GetValue(0, "MX_TIMEOUT", buf, sizeof(buf), 0)  ||  !*buf
-        ||  (tmo = NCBI_simple_atof(buf, &e)) < 0.000001  ||  errno  ||  !*e) {
+    if (!ConnNetInfo_GetValueInternal(0, "MX_TIMEOUT", buf, sizeof(buf), 0)
+        ||  !*buf  ||  (tmo = NCBI_simple_atof(buf, &e)) < 0.000001
+        ||  errno  ||  !*e) {
         tmo = 120.0/*2 min*/;
     }
-    if (!ConnNetInfo_GetValue(0, "MX_PORT", buf, sizeof(buf), 0)
+    if (!ConnNetInfo_GetValueInternal(0, "MX_PORT", buf, sizeof(buf), 0)
         ||  !(port = (unsigned int) atoi(buf))  ||  port > 65535) {
         port = CONN_PORT_SMTP;
     }
-    if (!ConnNetInfo_GetValue(0, "MX_HOST", buf, sizeof(buf), 0)
+    if (!ConnNetInfo_GetValueInternal(0, "MX_HOST", buf, sizeof(buf), 0)
         ||  !*buf) {
 #if defined(NCBI_OS_UNIX)  &&  !defined(NCBI_OS_CYGWIN)
         strcpy(buf, "localhost");
@@ -297,12 +298,10 @@ static void x_Sendmail_InitEnv(void)
 #endif /*NCBI_OS_UNIX && !NCBI_OS_CYGWIN*/
     }
 
-    CORE_LOCK_WRITE;
     s_MxTimeout.sec  = (unsigned int)  tmo;
     s_MxTimeout.usec = (unsigned int)((tmo - s_MxTimeout.sec) * 1000000.0);
     strcpy(s_MxHost, buf);
     s_MxPort = (unsigned short) port;
-    CORE_UNLOCK;
 }
 
 
