@@ -1,4 +1,4 @@
-/*  $Id: grid_worker.cpp 608072 2020-05-11 17:58:23Z ivanov $
+/*  $Id: grid_worker.cpp 620109 2020-11-16 19:42:44Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -572,11 +572,15 @@ int SGridWorkerNodeImpl::Run(
     NStr::Split(m_SynRegistry->Get("server", "master_nodes", kEmptyStr), " ;,", vhosts);
 
     ITERATE(vector<string>, it, vhosts) {
-        if (auto address = CNetServer::SAddress::Parse(NStr::TruncateSpaces(*it))) {
+        if (auto address = SSocketAddress::Parse(NStr::TruncateSpaces(*it))) {
             m_Masters.insert(move(address));
         }
     }
 
+    if (vhosts.size() && m_Masters.empty()) {
+        LOG_POST_X(41, Warning << "All hosts from master_nodes were ignored");
+    }
+
     vhosts.clear();
 
     NStr::Split(m_SynRegistry->Get("server", "admin_hosts", kEmptyStr), " ;,", vhosts);
@@ -587,6 +591,10 @@ int SGridWorkerNodeImpl::Run(
             m_AdminHosts.insert(ha);
     }
 
+    if (vhosts.size() && m_AdminHosts.empty()) {
+        LOG_POST_X(42, Warning << "All hosts from admin_hosts were ignored");
+    }
+
     m_CommitJobInterval = m_SynRegistry->Get("server", "commit_job_interval", COMMIT_JOB_INTERVAL_DEFAULT);
     if (m_CommitJobInterval == 0)
         m_CommitJobInterval = 1;
@@ -954,7 +962,7 @@ bool CGridWorkerNode::IsHostInAdminHostsList(const string& host) const
 
 bool SGridWorkerNodeImpl::x_AreMastersBusy() const
 {
-    ITERATE(set<CNetServer::SAddress>, it, m_Masters) {
+    ITERATE(set<SSocketAddress>, it, m_Masters) {
         STimeout tmo = {0, 500};
         CSocket socket(it->host, it->port, &tmo, eOff);
         if (socket.GetStatus(eIO_Open) != eIO_Success)
