@@ -1,4 +1,4 @@
-/*  $Id: netcache_api.cpp 608072 2020-05-11 17:58:23Z ivanov $
+/*  $Id: netcache_api.cpp 607687 2020-05-06 16:16:59Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -65,7 +65,7 @@ BEGIN_NCBI_SCOPE
 
 struct SFallbackServer
 {
-    using TAddress = CNetServer::SAddress;
+    using TAddress = SSocketAddress;
 
     static const TAddress* Get()
     {
@@ -77,7 +77,7 @@ private:
     static TAddress Init();
 };
 
-CNetServer::SAddress SFallbackServer::Init()
+SSocketAddress SFallbackServer::Init()
 {
     try {
         return TAddress::Parse(TCGI_NetCacheFallbackServer::GetDefault());
@@ -536,7 +536,7 @@ CNetServerConnection SNetCacheAPIImpl::InitiateWriteCmd(
         try {
             exec_result = m_Service.FindServerAndExec(cmd, false);
         } catch (CNetSrvConnException& e) {
-            const CNetServer::SAddress* backup = SFallbackServer::Get();
+            const SSocketAddress* backup = SFallbackServer::Get();
 
             if (backup == NULL) {
                 LOG_POST(Info << "Fallback server address is not configured.");
