@@ -1,4 +1,4 @@
-/*  $Id: ncbi_param.cpp 608309 2020-05-14 12:35:05Z ivanov $
+/*  $Id: ncbi_param.cpp 606111 2020-04-20 13:34:35Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -398,7 +398,7 @@ string NCBI_XNCBI_EXPORT g_GetConfigString(const char* section,
                                            const char* variable,
                                            const char* env_var_name,
                                            const char* default_value,
-                                           int* src)
+                                           CParamBase::EParamSource* src)
 {
     // Check the environment first - if the name is customized CNcbiRegistry
     // will not find it and can use INI file value instead.
