@@ -1,4 +1,4 @@
-/* $Id: ncbi_system.cpp 613789 2020-08-12 18:02:48Z ivanov $
+/* $Id: ncbi_system.cpp 628412 2021-03-31 13:48:10Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1123,9 +1123,7 @@ bool GetCurrentProcessTimes(double* user_time, double* system_time)
 extern int GetProcessFDCount(int* soft_limit, int* hard_limit)
 {
 #ifdef NCBI_OS_UNIX
-    int            fd_count = 0;
-    struct dirent* dp;
-
+    int     fd_count = 0;
     rlim_t  cur_limit = -1;
     rlim_t  max_limit = -1;
 
@@ -1140,7 +1138,7 @@ extern int GetProcessFDCount(int* soft_limit, int* hard_limit)
 
     DIR* dir = opendir("/proc/self/fd/");
     if (dir) {
-        while ((dp = readdir(dir)) != NULL) {
+        while (readdir(dir) != NULL) {
             ++fd_count;
         }
         closedir(dir);
@@ -1197,12 +1195,11 @@ extern int GetProcessFDCount(int* soft_limit, int* hard_limit)
 extern int GetProcessThreadCount(void)
 {
 #ifdef NCBI_OS_LINUX
-    int            thread_count = 0;
-    struct dirent* dp;
+    int thread_count = 0;
 
     DIR* dir = opendir("/proc/self/task/");
     if (dir) {
-        while ((dp = readdir(dir)) != NULL)
+        while (readdir(dir) != NULL)
             ++thread_count;
         closedir(dir);
         thread_count -= 2;  // '.' and '..'
@@ -1502,6 +1499,13 @@ bool VerifyCpuCompatibility(string* message)
     // Compiles with SSE 4.2 support -- verify that CPU allow it
 
     #if defined(NCBI_SSE)  &&  NCBI_SSE >= 42
+        
+        if (getenv("NCBI_RUN_UNDER_VALGRIND")) {
+            // Skip if runs under Valgrind memory checker, 
+            // seems cpuid produce wrong results there, probably if running on VM
+            return true;
+        }
+
         unsigned eax, ebx, ecx, edx;
         #ifdef NCBI_OS_MSWIN
             int cpuid[4];
@@ -1538,6 +1542,7 @@ CCpuFeatures::InstructionSet::InstructionSet(void)
       f81_ECX_  { 0 },
       f81_EDX_  { 0 }
 {
+#if defined(__i386__) || defined(__x86_64__) || defined(_M_IX86) || defined(_M_X64)
     int nIds   = 0;
     int nExIds = 0;
 
@@ -1622,6 +1627,7 @@ CCpuFeatures::InstructionSet::InstructionSet(void)
         memcpy(brand + 32, m_ExtData[4].data(), sizeof(registers));
         m_BrandStr = brand;
     }
+#endif
 };
 
 
