@@ -1,4 +1,4 @@
-/*  $Id: request_ctx.cpp 593248 2019-09-16 12:18:31Z grichenk $
+/*  $Id: request_ctx.cpp 632469 2021-06-02 11:12:10Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,6 +51,9 @@ static const char* kPassThrough_ClientIp = "ncbi_client_ip";
 static const char* kPassThrough_Dtab = "ncbi_dtab";
 
 
+CAtomicCounter CRequestContext::sm_VersionCounter;
+
+
 CRequestContext::CRequestContext(TContextFlags flags)
     : m_RequestID(0),
       m_AppState(eDiagAppState_NotSet),
@@ -64,7 +67,8 @@ CRequestContext::CRequestContext(TContextFlags flags)
       m_AutoIncOnPost(false),
       m_Flags(flags),
       m_OwnerTID(-1),
-      m_IsReadOnly(false)
+      m_IsReadOnly(false),
+      m_Version(sm_VersionCounter.Add(1))
 {
     x_LoadEnvContextProperties();
 }
@@ -212,11 +216,13 @@ void CRequestContext::SetClientIP(const string& client)
     string ip = NStr::TruncateSpaces(client);
     if ( !NStr::IsIPAddress(ip) ) {
         m_ClientIP = kBadIP;
+        x_Modify();
         ERR_POST_X(25, "Bad client IP value: " << ip);
         return;
     }
 
     m_ClientIP = ip;
+    x_Modify();
 }
 
 
@@ -345,31 +351,33 @@ void CRequestContext::x_SetHitID(const CSharedHitId& hit_id)
         ERR_POST_X(28, Warning << "Changing hit ID after one has been logged. "
             "New hit id is: " << hit);
     }
-    if (m_HitID.GetHitId() == hit) return;
 
-    static CSafeStatic<TOnBadHitId> action;
-    if ( !IsValidHitID(hit) ) {
-        switch ( action->Get() ) {
-        case eOnBadPHID_Ignore:
-            return;
-        case eOnBadPHID_AllowAndReport:
-            ERR_POST_X(27, Warning << "Bad hit ID format: " << hit);
-            break;
-        case eOnBadPHID_IgnoreAndReport:
-            ERR_POST_X(27, Warning << "Bad hit ID format: " << hit);
-            return;
-        case eOnBadPHID_Throw:
-            NCBI_THROW(CRequestContextException, eBadHit,
-                "Bad hit ID format: " + hit);
-            break;
-        case eOnBadPHID_Allow:
-            break;
+    if (m_HitID.GetHitId() != hit) {
+        static CSafeStatic<TOnBadHitId> action;
+        if (!IsValidHitID(hit)) {
+            switch (action->Get()) {
+            case eOnBadPHID_Ignore:
+                return;
+            case eOnBadPHID_AllowAndReport:
+                ERR_POST_X(27, Warning << "Bad hit ID format: " << hit);
+                break;
+            case eOnBadPHID_IgnoreAndReport:
+                ERR_POST_X(27, Warning << "Bad hit ID format: " << hit);
+                return;
+            case eOnBadPHID_Throw:
+                NCBI_THROW(CRequestContextException, eBadHit,
+                    "Bad hit ID format: " + hit);
+                break;
+            case eOnBadPHID_Allow:
+                break;
+            }
         }
     }
     x_SetProp(eProp_HitID);
 
     m_SubHitIDCache.clear();
     m_HitID = hit_id;
+    x_Modify();
     m_HitIDLoggedFlag = 0;
     x_LogHitID();
 }
@@ -397,8 +405,14 @@ void CRequestContext::x_UpdateSubHitID(bool increment, CTempString prefix)
     // duplicate phids in different threads.
     string hit_id = GetHitID();
 
-    unsigned int sub_hit_id = increment ?
-        m_HitID.GetNextSubHitId() : m_HitID.GetCurrentSubHitId();
+    unsigned int sub_hit_id;
+    if (increment) {
+        sub_hit_id = m_HitID.GetNextSubHitId();
+        x_Modify();
+    }
+    else {
+        sub_hit_id = m_HitID.GetCurrentSubHitId();
+    }
 
     // Cache the string so that C code can use it.
     string subhit = prefix + NStr::NumericToString(sub_hit_id);
@@ -434,6 +448,7 @@ void CRequestContext::SetSessionID(const string& session)
     }
     x_SetProp(eProp_SessionID);
     m_SessionID.SetString(session);
+    x_Modify();
 }
 
 
@@ -551,6 +566,7 @@ CRef<CRequestContext> CRequestContext::Clone(void) const
     ret->m_AutoIncOnPost = m_AutoIncOnPost;
     ret->m_Flags = m_Flags;
     ret->m_IsReadOnly = m_IsReadOnly;
+    // NOTE: Do not clone m_Version - the new context gets its own unique version.
     return ret;
 }
 
@@ -860,6 +876,7 @@ const char* CRequestContextException::GetErrCodeString(void) const
 CRequestContextGuard_Base::CRequestContextGuard_Base(CRequestContext* context, TFlags flags)
     : m_Flags(flags)
 {
+    m_OriginatesFromThrow = uncaught_exception();
     CDiagContext& ctx = GetDiagContext();
     if ( context ) {
         m_SavedContext.Reset(&ctx.GetRequestContext());
@@ -882,7 +899,7 @@ CRequestContextGuard_Base::~CRequestContextGuard_Base(void)
     // If released, do not perform any actions.
     if ( !m_RequestContext ) return;
 
-    if ( uncaught_exception() ) {
+    if ( uncaught_exception()  &&  !m_OriginatesFromThrow ) {
         SetStatus(m_ErrorStatus);
     }
     else if ( !m_RequestContext->IsSetRequestStatus() ) {
