@@ -1,4 +1,4 @@
-/*  $Id: test_boost.cpp 601610 2020-02-11 15:01:11Z ucko $
+/*  $Id: test_boost.cpp 631063 2021-05-12 17:33:12Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -143,6 +143,12 @@ namespace but = boost::unit_test;
 
 BEGIN_NCBI_SCOPE
 
+#ifdef SYSTEM_MUTEX_INITIALIZER
+SSystemFastMutex g_NcbiTestMutex = STATIC_FAST_MUTEX_INITIALIZER;
+#else
+CAutoInitializeStaticFastMutex g_NcbiTestMutex;
+#endif
+
 #if BOOST_VERSION >= 105900
 inline
 static bool s_IsEnabled(const but::test_unit& tu) {
@@ -2276,7 +2282,9 @@ main(int argc, char* argv[])
         }
 #endif
         framework::run();
+#if BOOST_VERSION < 106700
         results_reporter::make_report();
+#endif
         made_report = true;
 
         if (
