@@ -1,4 +1,4 @@
-/*  $Id: version.cpp 612086 2020-07-15 11:49:39Z ivanov $
+/*  $Id: version.cpp 619781 2020-11-10 18:40:38Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -729,6 +729,13 @@ string CVersionAPI::Print(const string& appname, TPrintFlags flags) const
         os << " Build-Signature: " << ' ' << NCBI_SIGNATURE << endl;
     }
 #endif
+    if (flags & fGI64bit) {
+#ifdef NCBI_INT8_GI
+        os << " GI-64bit:  TRUE" << endl;
+#else
+        os << " GI-64bit:  FALSE" << endl;
+#endif
+    }
 
     if (flags & fBuildInfo) {
         os << m_BuildInfo.Print();
