@@ -1,4 +1,4 @@
-/*  $Id: data_source.cpp 601304 2020-02-05 16:14:26Z grichenk $
+/*  $Id: data_source.cpp 610968 2020-06-26 12:55:17Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1309,9 +1309,9 @@ string CDataSource::GetLabel(const CSeq_id_Handle& idh)
 }
 
 
-int CDataSource::GetTaxId(const CSeq_id_Handle& idh)
+TTaxId CDataSource::GetTaxId(const CSeq_id_Handle& idh)
 {
-    int ret = -1;
+    TTaxId ret = INVALID_TAX_ID;
     SSeqMatch_DS match = x_GetSeqMatch(idh);
     if ( match ) {
         ret = match.m_Bioseq->GetTaxId();
