@@ -1,4 +1,4 @@
-/*  $Id: scope_info.cpp 602761 2020-03-02 18:13:10Z vasilche $
+/*  $Id: scope_info.cpp 630071 2021-04-26 17:10:15Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -633,6 +633,7 @@ void CDataSource_ScopeInfo::ResetDS(void)
     }
     m_TSE_InfoMap.clear();
     m_TSE_BySeqId.clear();
+    m_ReplacedTSEs.clear();
     {{
         TTSE_LockSetMutex::TWriteLockGuard guard2(m_TSE_LockSetMutex);
         m_TSE_LockSet.clear();
@@ -1026,7 +1027,12 @@ CTSE_Lock CTSE_ScopeInfo::SUnloadedInfo::LockTSE(void)
 {
     _ASSERT(m_Source);
     _ASSERT(m_BlobId);
-    return m_Source->GetDataLoader()->GetBlobById(m_BlobId);
+    CTSE_Lock lock = m_Source->GetDataLoader()->GetBlobById(m_BlobId);
+    if ( !lock ) {
+        NCBI_THROW_FMT(CLoaderException, eConnectionFailed,
+                       "Data loader GetBlobById("<<m_BlobId.ToString()<<") returned null");
+    }
+    return lock;
 }
 
 
@@ -1135,7 +1141,13 @@ void CTSE_ScopeInfo::x_UserLockTSE(void)
             // scenario - in several threads:
             //   1. get new TSE handle
             //   2. call RemoveFromHistory()
-            GetDSInfo().AcquireTSEUserLock(*this);
+            try {
+                GetDSInfo().AcquireTSEUserLock(*this);
+            }
+            catch ( ... ) {
+                x_UserUnlockTSE();
+                throw;
+            }
         }
     }
 }
@@ -1170,7 +1182,14 @@ void CTSE_ScopeUserLocker::Lock(CTSE_ScopeInfo* tse) const
 {
     CObjectCounterLocker::Lock(tse);
     tse->x_InternalLockTSE();
-    tse->x_UserLockTSE();
+    try {
+        tse->x_UserLockTSE();
+    }
+    catch ( ... ) {
+        tse->x_InternalUnlockTSE();
+        CObjectCounterLocker::Unlock(tse);
+        throw;
+    }
 }
 
 
