@@ -1,4 +1,4 @@
-/*  $Id: snp_annot_info.cpp 513580 2016-09-13 11:58:16Z ivanov $
+/*  $Id: snp_annot_info.cpp 607110 2020-04-30 12:40:14Z grichenk $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -1071,7 +1071,7 @@ void CSeq_annot_SNP_Info::SetGi(TGi gi)
 void CSeq_annot_SNP_Info::OffsetGi(TIntId gi_offset)
 {
     if ( m_Seq_id->IsGi() ) {
-        m_Seq_id->SetGi(m_Seq_id->GetGi() + gi_offset);
+        m_Seq_id->SetGi(m_Seq_id->GetGi() + GI_FROM(TIntId, gi_offset));
     }
 }
 
