@@ -1,4 +1,4 @@
-/*  $Id: taxFormat.cpp 582698 2019-03-19 16:16:44Z zaretska $
+/*  $Id: taxFormat.cpp 613840 2020-08-13 14:11:10Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -207,10 +207,10 @@ public:
 
     ITreeIterator::EAction Execute(const ITaxon1Node* tax_node)
     {
-        int taxid = tax_node->GetTaxId();
+        TTaxId taxid = tax_node->GetTaxId();
                
         //if m_Curr != NULL there was Level End - we are at branch processing
-        int currTaxid = (m_Curr) ? m_Curr->taxid : 0;
+        TTaxId currTaxid = m_Curr ? m_Curr->taxid : ZERO_TAX_ID;
         bool useTaxid = false;                       
         if(taxid == currTaxid) {//branch processing            
             m_Curr->numHits += m_Curr->seqInfoList.size();                                                            
@@ -297,7 +297,7 @@ public:
 
     ITreeIterator::EAction LevelBegin(const ITaxon1Node* tax_node)
     {
-        int taxid = tax_node->GetTaxId();        
+        TTaxId taxid = tax_node->GetTaxId();        
         if((*m_TreeTaxInfoMap).count(taxid) != 0 ) {//sequence is in alignment            
             m_Depth++;
             m_Lineage.push_back(taxid); //Push current taxid info into stack
@@ -309,7 +309,7 @@ public:
 
     ITreeIterator::EAction Execute(const ITaxon1Node* tax_node)
     {
-        int taxid = tax_node->GetTaxId();        
+        TTaxId taxid = tax_node->GetTaxId();        
         if((*m_TreeTaxInfoMap).count(taxid) != 0 ) {//sequence is in alignment            
             (*m_TreeTaxInfoMap)[taxid].depth = m_Depth;
             for(size_t i = 0; i < m_Lineage.size(); i++) {
@@ -322,7 +322,7 @@ public:
 
     ITreeIterator::EAction LevelEnd(const ITaxon1Node* tax_node)
     {
-        int taxid = tax_node->GetTaxId();        
+        TTaxId taxid = tax_node->GetTaxId();        
         if((*m_TreeTaxInfoMap).count(taxid) != 0 ) {//sequence is in alignment            
             m_Depth--;
             m_Lineage.pop_back();
@@ -344,7 +344,7 @@ public:
 
     CTaxFormat::TSeqTaxInfoMap       *m_TreeTaxInfoMap;    
     int                              m_Depth;
-    vector <int>                     m_Lineage;
+    vector <TTaxId>                  m_Lineage;
     bool                             m_Debug;
 };
 
@@ -357,7 +357,7 @@ void CTaxFormat::DisplayOrgReport(CNcbiOstream& out)
     string taxIdToSeqsMap;
 
     for(size_t i = 0; i < m_BlastResTaxInfo->orderedTaxids.size(); i++) {
-        int taxid = m_BlastResTaxInfo->orderedTaxids[i];
+        TTaxId taxid = m_BlastResTaxInfo->orderedTaxids[i];
         STaxInfo seqsForTaxID = m_BlastResTaxInfo->seqTaxInfoMap[taxid];
         
         string orgHeader = (m_ConnectToTaxServer) ? m_TaxFormatTemplates->orgReportOrganismHeader : 
@@ -398,7 +398,7 @@ void CTaxFormat::DisplayOrgReport(CNcbiOstream& out)
 
         //string taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->taxIdToSeqsMap,"giList",seqsForTaxID.giList);
         string taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->taxIdToSeqsMap,"giList",seqsForTaxID.accList);
-        taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(taxIdToSeqsRow,"taxid",taxid);
+        taxIdToSeqsRow = CAlignFormatUtil::MapTemplate(taxIdToSeqsRow,"taxid", TAX_ID_TO(TIntId, taxid));
         taxIdToSeqsMap += taxIdToSeqsRow;
     }   
     orgReportData = CAlignFormatUtil::MapTemplate(m_TaxFormatTemplates->orgReportTable,"table_rows",orgReportData);
@@ -430,7 +430,7 @@ void CTaxFormat::DisplayLineageReport(CNcbiOstream& out)
             STaxInfo alnSeqTaxInfo = *iter;
 //            taxInfo.lineage.push_back(iter->taxid);            
             for(size_t i = 0; i< alnSeqTaxInfo.lineage.size();i++) {
-                int taxid = alnSeqTaxInfo.lineage[i];                
+                TTaxId taxid = alnSeqTaxInfo.lineage[i];
                 if(header) {                                        
                     STaxInfo taxInfo = GetTaxTreeInfo(taxid);                   
 
@@ -460,12 +460,12 @@ void CTaxFormat::DisplayLineageReport(CNcbiOstream& out)
 void CTaxFormat::DisplayTaxonomyReport(CNcbiOstream& out) 
 {
     x_InitTaxReport();    
-    vector <int> taxTreeTaxids = GetTaxTreeTaxIDs();        
+    vector <TTaxId> taxTreeTaxids = GetTaxTreeTaxIDs();        
     string taxReportRows;
     taxReportRows = m_TaxFormatTemplates->taxonomyReportTableHeader;//!!!Do the same for organism report
 
     for(size_t i = 0; i < taxTreeTaxids.size(); i++) {
-        int taxid = taxTreeTaxids[i];
+        TTaxId taxid = taxTreeTaxids[i];
         STaxInfo taxInfo = GetTaxTreeInfo(taxid);     
         string taxReportTableRow;             
         if(isTaxidInAlign(taxid)) {                                                     
@@ -543,8 +543,8 @@ void CTaxFormat::x_LoadTaxTree(void)
 {
     x_InitTaxClient();                
     if(!m_TaxTreeLoaded) {
-        vector <int> taxidsToRoot;    
-        vector <int> alignTaxids = GetAlignTaxIDs();
+        vector <TTaxId> taxidsToRoot;    
+        vector <TTaxId> alignTaxids = GetAlignTaxIDs();
 
         bool tax_load_ok = false;
         if(m_TaxClient->IsAlive()) {          
@@ -553,7 +553,7 @@ void CTaxFormat::x_LoadTaxTree(void)
 
             //Adding alignTaxids
             for(size_t i = 0; i < alignTaxids.size(); i++) {
-                int tax_id = alignTaxids[i];                                    
+                TTaxId tax_id = alignTaxids[i];                                    
 
                 if(!m_TaxClient->IsAlive()) break;
                 const ITaxon1Node* tax_node = NULL;
@@ -561,7 +561,7 @@ void CTaxFormat::x_LoadTaxTree(void)
                 if(!tax_load_ok) break;                
 
                 if(tax_node && tax_node->GetTaxId() != tax_id) {
-                    int newTaxid = tax_node->GetTaxId();
+                    TTaxId newTaxid = tax_node->GetTaxId();
                     if(m_Debug) {
                         cerr << "*******TAXID MISMATCH: changing " << tax_id << " to " << tax_node->GetTaxId() << "-" << endl;
                     }
@@ -578,8 +578,8 @@ void CTaxFormat::x_LoadTaxTree(void)
             }
         }
         if(m_TaxClient->IsAlive() && tax_load_ok) {          
-            ITERATE(vector<int>, it, taxidsToRoot) {
-                int tax_id = *it;                 
+            ITERATE(vector<TTaxId>, it, taxidsToRoot) {
+                TTaxId tax_id = *it;                 
 
                 if(!m_TaxClient->IsAlive()) break;
                 tax_load_ok |= m_TaxClient->LoadNode(tax_id);
@@ -603,12 +603,12 @@ void CTaxFormat::x_LoadTaxTree(void)
 }
 
 
-void CTaxFormat::x_PrintTaxInfo(vector <int> taxids, string title)
+void CTaxFormat::x_PrintTaxInfo(vector <TTaxId> taxids, string title)
 {
     if(m_Debug) {
         cerr << "******" << title << "**********" << endl;
         for(size_t i = 0; i < taxids.size(); i++) {
-            int taxid = taxids[i];                            
+            TTaxId taxid = taxids[i];                            
             STaxInfo taxInfo = GetTaxTreeInfo(taxid);                         
             string lineage;
             for(size_t j = 0; j < taxInfo.lineage.size(); j++) {
@@ -636,7 +636,7 @@ void CTaxFormat::x_InitOrgTaxMetaData(void)
         CDownwardTreeFiller dwnwFiller(&m_TaxTreeinfo->seqTaxInfoMap);
         dwnwFiller.SetDebugMode(m_Debug);
         m_TreeIterator->TraverseDownward(dwnwFiller);
-        vector <int> taxTreeTaxids = GetTaxTreeTaxIDs();
+        vector <TTaxId> taxTreeTaxids = GetTaxTreeTaxIDs();
 
         x_PrintTaxInfo(taxTreeTaxids,"Taxonomy tree");                     
     }
@@ -648,12 +648,12 @@ void CTaxFormat::x_InitBlastNameTaxInfo(STaxInfo &taxInfo)
     if(m_TaxClient && m_TaxClient->IsAlive()) {
         m_TaxClient->GetBlastName(taxInfo.taxid,taxInfo.blastName);
         list< CRef< CTaxon1_name > > nameList;
-        taxInfo.blNameTaxid = m_TaxClient->SearchTaxIdByName(taxInfo.blastName,CTaxon1::eSearch_Exact,&nameList);        
+        taxInfo.blNameTaxid = TAX_ID_TO(int, m_TaxClient->SearchTaxIdByName(taxInfo.blastName,CTaxon1::eSearch_Exact,&nameList));
         if(taxInfo.blNameTaxid == -1) //more than one name in the list
         ITERATE(list < CRef< CTaxon1_name > >, iter, nameList) {                           
             ncbi::TIntId blastNameCde = m_TaxClient->GetNameClassId("blast name");
             if((*iter)->CanGetTaxid() && (*iter)->IsSetCde() && (*iter)->GetCde() == blastNameCde) {
-                taxInfo.blNameTaxid = (*iter)->GetTaxid();
+                taxInfo.blNameTaxid = TAX_ID_TO(int, (*iter)->GetTaxid());
             }
         }
     }
@@ -661,7 +661,7 @@ void CTaxFormat::x_InitBlastNameTaxInfo(STaxInfo &taxInfo)
 
 static string s_TaxidLinageToString(CTaxFormat::STaxInfo const& info)
 {
-    vector <int> lineage = info.lineage;            
+    vector <TTaxId> lineage = info.lineage;            
     string strTaxidLineage;
     for(size_t j = 0; j < lineage.size(); j++) {
         if(!strTaxidLineage.empty()) strTaxidLineage += " ";
@@ -680,12 +680,12 @@ static bool s_SortByLinageToBestHit(CTaxFormat::STaxInfo const& info1,
 }
 
 
-static vector <int> s_InitAlignHitLineage(vector <int> bestHitLinage, struct CTaxFormat::STaxInfo &taxTreeInfo)
+static vector <TTaxId> s_InitAlignHitLineage(vector <TTaxId> bestHitLinage, struct CTaxFormat::STaxInfo &taxTreeInfo)
 {
-    vector <int> alignHitLineage;
+    vector <TTaxId> alignHitLineage;
     for(size_t i = 0; i < bestHitLinage.size(); i++) {
         if(taxTreeInfo.lineage.size() - 1 >= i) {
-          int taxid = bestHitLinage[i];
+          TTaxId taxid = bestHitLinage[i];
           if(taxTreeInfo.lineage[i] == taxid) {
               alignHitLineage.push_back(taxid);
           }          
@@ -701,11 +701,11 @@ void CTaxFormat::x_PrintLineage(void)
     if(m_Debug) {
         cerr << "*********Lineage*********" << endl;
         ITERATE(list <STaxInfo>, iter, m_AlnLineageTaxInfo) {             
-            int taxid = iter->taxid;    
+            TTaxId taxid = iter->taxid;    
             string name = iter->scientificName;            
             cerr << "taxid" << taxid << " " << name << ": "; 
             for(size_t i = 0; i< iter->lineage.size();i++) {                
-                int lnTaxid = iter->lineage[i];            
+                TTaxId lnTaxid = iter->lineage[i];
                 cerr << " " << lnTaxid << " " << GetTaxTreeInfo(lnTaxid).scientificName + "," ;
             }
             cerr << endl;
@@ -715,14 +715,14 @@ void CTaxFormat::x_PrintLineage(void)
 
 void CTaxFormat::x_InitLineageMetaData(void)
 {
-        int betsHitTaxid = m_BlastResTaxInfo->orderedTaxids[0];
+        TTaxId betsHitTaxid = m_BlastResTaxInfo->orderedTaxids[0];
         m_BestHitLineage = GetTaxTreeInfo(betsHitTaxid).lineage; 
-        vector <int> alignTaxids = GetAlignTaxIDs();
+        vector <TTaxId> alignTaxids = GetAlignTaxIDs();
         
         list <STaxInfo> alnTaxInfo;
         for(size_t i = 0; i < alignTaxids.size(); i++) {                    
             STaxInfo taxInfo = GetTaxTreeInfo(alignTaxids[i]);
-            vector <int> alignHitLineage = s_InitAlignHitLineage(m_BestHitLineage,taxInfo);
+            vector <TTaxId> alignHitLineage = s_InitAlignHitLineage(m_BestHitLineage,taxInfo);
             taxInfo.lineage = alignHitLineage;
             x_InitBlastNameTaxInfo(taxInfo);                                    
             m_AlnLineageTaxInfo.push_back(taxInfo);            
@@ -730,7 +730,7 @@ void CTaxFormat::x_InitLineageMetaData(void)
         m_AlnLineageTaxInfo.sort(s_SortByLinageToBestHit);                
         ITERATE(list <STaxInfo>, iter, m_AlnLineageTaxInfo) {            
             for(size_t i = 0; i< iter->lineage.size();i++) {                
-                int lnTaxid = iter->lineage[i];  
+                TTaxId lnTaxid = iter->lineage[i];
                 STaxInfo &taxInfo = GetTaxTreeInfo(lnTaxid);
                 x_InitBlastNameTaxInfo(taxInfo);                                                    
             }            
@@ -744,7 +744,7 @@ void CUpwardTreeFiller::x_InitTaxInfo(const ITaxon1Node* tax_node)
     CTaxFormat::STaxInfo *info = new CTaxFormat::STaxInfo;        
 
     
-    int taxid = tax_node->GetTaxId();    
+    TTaxId taxid = tax_node->GetTaxId();    
     if(m_SeqAlignTaxInfoMap.count(taxid) != 0 ) {//sequence is in alignment
         info->seqInfoList = m_SeqAlignTaxInfoMap[taxid].seqInfoList;        
     }   
@@ -757,7 +757,7 @@ void CUpwardTreeFiller::x_InitTaxInfo(const ITaxon1Node* tax_node)
 
 void CUpwardTreeFiller::x_InitTreeTaxInfo(void)
 {
-    int taxid = m_Curr->taxid;
+    TTaxId taxid = m_Curr->taxid;
     if(m_TreeTaxInfo->seqTaxInfoMap.count(taxid) == 0) {              
         CTaxFormat::STaxInfo seqsForTaxID;
         seqsForTaxID.taxid = m_Curr->taxid;
@@ -775,7 +775,7 @@ void CUpwardTreeFiller::x_InitTreeTaxInfo(void)
 }
 
 
-bool CTaxFormat::isTaxidInAlign(int taxid)
+bool CTaxFormat::isTaxidInAlign(TTaxId taxid)
 {
     bool ret = false;
     if(m_BlastResTaxInfo->seqTaxInfoMap.count(taxid) != 0 && m_BlastResTaxInfo->seqTaxInfoMap[taxid].seqInfoList.size() > 0) {
@@ -807,7 +807,7 @@ CTaxFormat::SSeqInfo *CTaxFormat::x_FillTaxDispParams(const CBioseq_Handle& bsp_
 
     seqInfo->displGi = seqInfo->gi;
     seqInfo->dispSeqID = seqInfo->label;
-    seqInfo->taxid = 0;	
+    seqInfo->taxid = ZERO_TAX_ID;	
 	seqInfo->title = CDeflineGenerator().GenerateDefline(bsp_handle);			    
     if(m_DisplayOption == eText) x_InitTextFormatInfo(seqInfo); 
     return seqInfo;
@@ -841,7 +841,7 @@ CTaxFormat::SSeqInfo *CTaxFormat::x_FillTaxDispParams(const CRef< CBlast_def_lin
         string total_bit_score_buf, raw_score_buf;
         CAlignFormatUtil::GetScoreString(evalue, bits, 0, 0, seqInfo->evalue, seqInfo->bit_score, total_bit_score_buf,raw_score_buf);
 
-	    int taxid = 0;
+	    TTaxId taxid = ZERO_TAX_ID;
 		
 		if(bdl->IsSetTaxid() &&  bdl->CanGetTaxid()){
 		    taxid = bdl->GetTaxid();
@@ -865,7 +865,7 @@ void CTaxFormat::x_InitTaxInfoMap(void)
 {
     int score, sum_n, num_ident;
     double bits, evalue;
-    list<string> use_this_seqid;
+    
     
     m_BlastResTaxInfo = new SBlastResTaxInfo;
     CConstRef<CSeq_id> previousId,subid;    
@@ -874,7 +874,8 @@ void CTaxFormat::x_InitTaxInfoMap(void)
         subid = &((*iter)->GetSeq_id(1));
 
         if(!previousId.Empty() && subid->Match(*previousId)) continue;        
-
+        
+        list<string> use_this_seqid;
         CAlignFormatUtil::GetAlnScores(**iter, score, bits, evalue,
                                        sum_n, num_ident, use_this_seqid);        
         try {
@@ -892,7 +893,7 @@ void CTaxFormat::x_InitTaxInfoMap(void)
                     SSeqInfo *seqInfo = x_FillTaxDispParams(*iter,handle,bits,evalue,use_this_seqid);
                     if(seqInfo) {
                         if(dispGI == ZERO_GI && dispSeqID.empty()){
-                            int taxid;                                
+                            TTaxId taxid;
                             CAlignFormatUtil::GetDisplayIds(handle,*seqInfo->seqID,use_this_seqid,&dispGI,&taxid,&dispSeqID);                            
                             if(dispGI == ZERO_GI && !dispSeqID.empty()) {
                                 dispSeqID = seqInfo->label; //accesion without version
@@ -922,10 +923,10 @@ void CTaxFormat::x_InitTaxInfoMap(void)
 
 void CTaxFormat::x_InitBlastDBTaxInfo(CTaxFormat::SSeqInfo *seqInfo)    
 {
-    int taxid = seqInfo->taxid;
+    TTaxId taxid = seqInfo->taxid;
     if(m_BlastResTaxInfo->seqTaxInfoMap.count(taxid) == 0){		                            
         SSeqDBTaxInfo taxInfo;
-        CSeqDB::GetTaxInfo(taxid, taxInfo);                
+        CSeqDB::GetTaxInfo(taxid, taxInfo);
 
         STaxInfo seqsForTaxID;
         seqsForTaxID.taxid = taxid;
@@ -954,7 +955,7 @@ void CTaxFormat::x_InitBlastDBTaxInfo(CTaxFormat::SSeqInfo *seqInfo)
 string CTaxFormat::x_MapSeqTemplate(string seqTemplate, STaxInfo &taxInfo)
 {
     SSeqInfo *seqInfo = taxInfo.seqInfoList[0];
-    int taxid = taxInfo.taxid;
+    TTaxId taxid = taxInfo.taxid;
     
     string reportTableRow = CAlignFormatUtil::MapTemplate(seqTemplate,"acc",m_BlastResTaxInfo->seqTaxInfoMap[taxid].accList);        
     reportTableRow = CAlignFormatUtil::MapTemplate(reportTableRow,"descr",seqInfo->title);    
@@ -1006,7 +1007,7 @@ string CTaxFormat::x_MapTaxInfoTemplate(string tableRowTemplate,STaxInfo &taxInf
     }
     
     reportTableRow = CAlignFormatUtil::MapTemplate(reportTableRow,"bl_taxid",taxInfo.blNameTaxid);
-    reportTableRow = CAlignFormatUtil::MapTemplate(reportTableRow,"taxid",taxInfo.taxid);
+    reportTableRow = CAlignFormatUtil::MapTemplate(reportTableRow,"taxid",TAX_ID_TO(int, taxInfo.taxid));
     reportTableRow = CAlignFormatUtil::MapTemplate(reportTableRow,"taxBrowserURL",m_TaxBrowserURL);    
     reportTableRow = CAlignFormatUtil::MapTemplate(reportTableRow,"rid",m_Rid);
     int numHits = taxInfo.seqInfoList.size();
