@@ -1,4 +1,4 @@
-/*  $Id: seqdb.cpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: seqdb.cpp 611131 2020-06-29 18:42:01Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -874,10 +874,7 @@ void CSeqDB::AccessionToOids(const string & acc, vector<int> & oids) const
 
     if (oids.empty()) {
         try {
-            TGi gi = GI_FROM(
-                    TIntId,
-                    NStr::StringToNumeric<TIntId>(acc, NStr::fConvErr_NoThrow)
-            );
+            TGi gi = NStr::StringToNumeric<TGi>(acc, NStr::fConvErr_NoThrow);
             int oid(-1);
 
             if (gi > ZERO_GI  &&  m_Impl->GiToOidwFilterCheck(gi, oid)) {
