@@ -1,4 +1,4 @@
-/*  $Id: seqdbgilistset.cpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: seqdbgilistset.cpp 618513 2020-10-21 16:07:12Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -163,7 +163,7 @@ void s_ProcessSeqIDsForV5 (vector<string> & idlist)
             }
             tmplist.push_back(seqid.GetSeqIdString(true));
             } catch (CException & e) {
-                    LOG_POST(e.GetMsg());
+                    ERR_POST(e.GetMsg());
                     NCBI_THROW(CSeqDBException, eArgErr, "Invalid seq id: " + idlist[i]);
 
             }
@@ -444,7 +444,7 @@ CSeqDBGiListSet::GetNodeIdList(const CSeqDB_Path & filename,
     // data sources and estimate the time needed for different
     // techniques.  This has not been done.
 
-    bool mixed_ids = m_UserList.Empty() || (!! m_UserList->GetNumSis());
+    bool mixed_ids = m_UserList.Empty() || (!! m_UserList->GetNumSis())|| (m_UserList->GetNumTaxIds());
 
     if (! mixed_ids) {
         if ((m_UserList->GetNumTis() && gilist->GetNumGis()) ||
