@@ -1,4 +1,4 @@
-/*  $Id: seqdbimpl.cpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: seqdbimpl.cpp 631567 2021-05-19 13:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,8 @@
 #include <sstream>
 #include <algorithm>
 #include <serial/enumvalues.hpp>
+#include <serial/objistr.hpp>
+#include <serial/objistrasnb.hpp>
 #include <algo/blast/core/blast_seqsrc.h>
 
 BEGIN_NCBI_SCOPE
@@ -146,6 +148,8 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
 
     SetIterationRange(oid_begin, oid_end);
 
+    reusable_inpstr =  new CObjectIStreamAsnBinary ; 
+
     CHECK_MARKER();
 }
 
@@ -170,6 +174,7 @@ CSeqDBImpl::CSeqDBImpl(bool use_atlas_lock)
 {
     INIT_CLASS_MARK();
 
+    reusable_inpstr =  new CObjectIStreamAsnBinary ; 
     CHECK_MARKER();
 }
 
@@ -197,7 +202,13 @@ void CSeqDBImpl::SetIterationRange(int oid_begin, int oid_end)
 CSeqDBImpl::~CSeqDBImpl()
 {
     CHECK_MARKER();
-
+    if( reusable_inpstr ) {
+	delete reusable_inpstr;
+	reusable_inpstr = NULL;
+    }
+    else {
+        cerr << "\n(=)\n";
+    }
     SetNumberOfThreads(0);
 
     CSeqDBLockHold locked(m_Atlas);
@@ -245,6 +256,7 @@ void CSeqDBImpl::x_GetOidList(CSeqDBLockHold & locked)
         }
         //DebugDumpText(cerr, "CSeqDBImpl after m_OIDList initialization", 10);
         //ft->Print();
+        m_Atlas.Unlock(locked);
     }
 }
 
@@ -325,7 +337,7 @@ CSeqDBImpl::GetNextOIDChunk(int         & begin_chunk, // out
     // fill the cache for all sequence in mmaped slice
     if (m_NumThreads) {
         SSeqResBuffer * buffer = m_CachedSeqs[cacheID];
-        x_FillSeqBuffer(buffer, begin_chunk, locked);
+        x_FillSeqBuffer(buffer, begin_chunk);
         end_chunk = begin_chunk + buffer->results.size();
     } else {
         end_chunk = begin_chunk + oid_size;
@@ -391,25 +403,21 @@ void CSeqDBImpl::ResetInternalChunkBookmark()
 int CSeqDBImpl::GetSeqLength(int oid) const
 {
     CHECK_MARKER();
-    CSeqDBLockHold locked(m_Atlas);
-    //m_Atlas.MentionOid(oid, m_NumOIDs, locked);
 
-    return x_GetSeqLength(oid, locked);
+    return x_GetSeqLength(oid);
 }
 
-int CSeqDBImpl::x_GetSeqLength(int oid, CSeqDBLockHold & locked) const
+int CSeqDBImpl::x_GetSeqLength(int oid) const
 {
-    m_Atlas.Lock(locked);
-
     int vol_oid = 0;
 
     if ('p' == m_SeqType) {
         if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
-            return vol->GetSeqLengthProt(vol_oid, locked);
+            return vol->GetSeqLengthProt(vol_oid);
         }
     } else {
         if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
-            return vol->GetSeqLengthExact(vol_oid, locked);
+            return vol->GetSeqLengthExact(vol_oid);
         }
     }
 
@@ -419,18 +427,16 @@ int CSeqDBImpl::x_GetSeqLength(int oid, CSeqDBLockHold & locked) const
 int CSeqDBImpl::GetSeqLengthApprox(int oid) const
 {
     CHECK_MARKER();
-    CSeqDBLockHold locked(m_Atlas);
-    //m_Atlas.MentionOid(oid, m_NumOIDs, locked);
 
     int vol_oid = 0;
 
     if ('p' == m_SeqType) {
         if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
-            return vol->GetSeqLengthProt(vol_oid, locked);
+            return vol->GetSeqLengthProt(vol_oid);
         }
     } else {
         if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
-            return vol->GetSeqLengthApprox(vol_oid, locked);
+            return vol->GetSeqLengthApprox(vol_oid);
         }
     }
 
@@ -644,17 +650,11 @@ void CSeqDBImpl::RetSequence(const char ** buffer) const
 
 void CSeqDBImpl::RetAmbigSeq(const char ** buffer) const
 {
-    // This returns an allocated object.
-
-    CSeqDBLockHold locked(m_Atlas);
-    m_Atlas.Lock(locked);
-
-    m_Atlas.RetRegion(*buffer);//Keep this
+    CSeqDBAtlas::RetRegion(*buffer);//Keep this
     *buffer = 0;
 }
 
-void CSeqDBImpl::x_RetSeqBuffer(SSeqResBuffer * buffer,
-                                CSeqDBLockHold & locked ) const
+void CSeqDBImpl::x_RetSeqBuffer(SSeqResBuffer * buffer) const
 {
     // client must return sequence before getting a new one
     if (buffer->checked_out > 0) {
@@ -662,13 +662,6 @@ void CSeqDBImpl::x_RetSeqBuffer(SSeqResBuffer * buffer,
     }
 
     buffer->checked_out = 0;
-
-    m_Atlas.Lock(locked);
-/*
-    for(Uint4 index = 0; index < buffer->results.size(); ++index) {
-        m_Atlas.RetRegion(buffer->results[index].address);
-    }
-*/
     buffer->results.clear();
 }
 
@@ -683,24 +676,17 @@ int CSeqDBImpl::x_GetSeqBuffer(SSeqResBuffer * buffer, int oid,
         return buffer->results[index].length;
     }
 
-    // Not in cache, fill the cache
-    CSeqDBLockHold locked(m_Atlas);
-    m_Atlas.Lock(locked);
-    x_FillSeqBuffer(buffer, oid, locked);
+    x_FillSeqBuffer(buffer, oid);
     (buffer->checked_out)++;
     *seq = buffer->results[0].address;
     return buffer->results[0].length;
 }
 
 void CSeqDBImpl::x_FillSeqBuffer(SSeqResBuffer  *buffer,
-                                 int             oid,
-                                 CSeqDBLockHold &locked) const
+                                 int             oid) const
 {
-    // Must lock the atlas
-    m_Atlas.Lock(locked);
-
     // clear the buffer first
-    x_RetSeqBuffer(buffer, locked);
+    x_RetSeqBuffer(buffer);
 
     buffer->oid_start = oid;
     Int4 vol_oid = 0;
@@ -711,17 +697,16 @@ void CSeqDBImpl::x_FillSeqBuffer(SSeqResBuffer  *buffer,
         const char * seq;
         Int8 tot_length = m_Atlas.GetSliceSize() / (4*m_NumThreads) + 1;
 
-        res.length = vol->GetSequence(vol_oid++, &seq, locked);
+        res.length = vol->GetSequence(vol_oid++, &seq);
         if (res.length < 0) return;
         // must return at least one sequence
         do {
             tot_length -= res.length;
             res.address = seq;
             buffer->results.push_back(res);
-            res.length = vol->GetSequence(vol_oid++, &seq, locked);
+            res.length = vol->GetSequence(vol_oid++, &seq);
         } while (res.length >= 0 && tot_length >= res.length && vol_oid < m_RestrictEnd);
 
-        //if (res.length >= 0)  m_Atlas.RetRegion(seq);
         return;
     }
 
@@ -731,9 +716,7 @@ void CSeqDBImpl::x_FillSeqBuffer(SSeqResBuffer  *buffer,
 int CSeqDBImpl::GetSequence(int oid, const char ** buffer) const
 {
     CHECK_MARKER();
-
     CSeqDBLockHold locked(m_Atlas);
-
     if (m_NumThreads) {
         int cacheID = x_GetCacheID(locked);
         return x_GetSeqBuffer(m_CachedSeqs[cacheID], oid, buffer);
@@ -741,11 +724,8 @@ int CSeqDBImpl::GetSequence(int oid, const char ** buffer) const
 
     int vol_oid = 0;
 
-    m_Atlas.Lock(locked);
-    //m_Atlas.MentionOid(oid, m_NumOIDs, locked);
-
     if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
-        return vol->GetSequence(vol_oid, buffer, locked);
+        return vol->GetSequence(vol_oid, buffer);
     }
 
     NCBI_THROW(CSeqDBException, eArgErr, CSeqDB::kOidNotFound);
@@ -777,10 +757,6 @@ int CSeqDBImpl::GetAmbigSeq(int               oid,
                             CSeqDB::TSequenceRanges * masks) const
 {
     CHECK_MARKER();
-    CSeqDBLockHold locked(m_Atlas);
-
-    m_Atlas.Lock(locked);
-    //m_Atlas.MentionOid(oid, m_NumOIDs, locked);
 
     int vol_oid = 0;
     if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
@@ -789,8 +765,7 @@ int CSeqDBImpl::GetAmbigSeq(int               oid,
                                 nucl_code,
                                 alloc_type,
                                 region,
-                                masks,
-                                locked);
+                                masks);
     }
 
     NCBI_THROW(CSeqDBException, eArgErr, CSeqDB::kOidNotFound);
@@ -804,11 +779,6 @@ int CSeqDBImpl::GetAmbigPartialSeq(int                oid,
                                    CSeqDB::TSequenceRanges  * masks) const
 {
 	   CHECK_MARKER();
-	    CSeqDBLockHold locked(m_Atlas);
-
-	    m_Atlas.Lock(locked);
-	    //m_Atlas.MentionOid(oid, m_NumOIDs, locked);
-
 	    int vol_oid = 0;
 	    if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
 	        return vol->GetAmbigPartialSeq(vol_oid,
@@ -816,8 +786,7 @@ int CSeqDBImpl::GetAmbigPartialSeq(int                oid,
 	                                       nucl_code,
 	                                       alloc_type,
 	                                       partial_ranges,
-	                                       masks,
-	                                       locked);
+	                                       masks);
 	    }
 
 	    NCBI_THROW(CSeqDBException, eArgErr, CSeqDB::kOidNotFound);
@@ -837,7 +806,10 @@ list< CRef<CSeq_id> > CSeqDBImpl::GetSeqIDs(int oid)
     }
 
     if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
-        return vol->GetSeqIDs(vol_oid);
+	if( ! reusable_inpstr   ) {
+           reusable_inpstr =  new CObjectIStreamAsnBinary ; 
+	}
+        return vol->GetSeqIDs(vol_oid, reusable_inpstr);
     }
 
     NCBI_THROW(CSeqDBException, eArgErr, CSeqDB::kOidNotFound);
@@ -1145,10 +1117,9 @@ void CSeqDBImpl::FlushSeqMemory()
 bool CSeqDBImpl::PigToOid(int pig, int & oid) const
 {
     CHECK_MARKER();
-    CSeqDBLockHold locked(m_Atlas);
 
     for(int i = 0; i < m_VolSet.GetNumVols(); i++) {
-        if (m_VolSet.GetVol(i)->PigToOid(pig, oid, locked)) {
+        if (m_VolSet.GetVol(i)->PigToOid(pig, oid)) {
             oid += m_VolSet.GetVolOIDStart(i);
             return true;
         }
@@ -1584,8 +1555,6 @@ void CSeqDBImpl::x_ScanTotals(bool             approx,
 
     const CSeqDBVol * volp = 0;
 
-    m_Atlas.Lock(locked);
-
     for(int oid = 0; x_CheckOrFindOID(oid, locked); oid++) {
         ++ oid_count;
 
@@ -1598,12 +1567,12 @@ void CSeqDBImpl::x_ScanTotals(bool             approx,
         if (totlen || maxlen || minlen) {
             int len;
             if ('p' == m_SeqType) {
-                len = volp->GetSeqLengthProt(vol_oid, locked);
+                len = volp->GetSeqLengthProt(vol_oid);
             } else {
                 if (approx) {
-                    len = volp->GetSeqLengthApprox(vol_oid, locked);
+                    len = volp->GetSeqLengthApprox(vol_oid);
                 } else {
-                    len = volp->GetSeqLengthExact(vol_oid, locked);
+                    len = volp->GetSeqLengthExact(vol_oid);
                 }
             }
             max_count = max(len, max_count);
@@ -1686,18 +1655,13 @@ void CSeqDBImpl::GetRawSeqAndAmbig(int           oid,
 {
     CHECK_MARKER();
 
-    CSeqDBLockHold locked(m_Atlas);
-    m_Atlas.Lock(locked);
-    //m_Atlas.MentionOid(oid, m_NumOIDs, locked);
-
     int vol_oid = 0;
 
     if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
         vol->GetRawSeqAndAmbig(vol_oid,
                                buffer,
                                seq_length,
-                               ambig_length,
-                               locked);
+                               ambig_length);
 
         return;
     }
@@ -1822,15 +1786,13 @@ void CSeqDBImpl::GetStringBounds(string * low_id,
                                  string * high_id,
                                  int    * count)
 {
-    CSeqDBLockHold locked(m_Atlas);
-
     bool found = false;
 
     for(int i = 0; i < m_VolSet.GetNumVols(); i++) {
         string vlow, vhigh;
         int vcount(0);
 
-        m_VolSet.GetVol(i)->GetStringBounds(vlow, vhigh, vcount, locked);
+        m_VolSet.GetVol(i)->GetStringBounds(vlow, vhigh, vcount);
 
         if (vcount) {
             s_AccumulateMinMaxCount(vlow,
@@ -1856,17 +1818,13 @@ void CSeqDBImpl::SetOffsetRanges(int                oid,
                                  bool               cache_data)
 {
     CHECK_MARKER();
-    CSeqDBLockHold locked(m_Atlas);
     int vol_oid = 0;
 
-    m_Atlas.Lock(locked);
-
     if (const CSeqDBVol * vol = m_VolSet.FindVol(oid, vol_oid)) {
         vol->SetOffsetRanges(vol_oid,
                              offset_ranges,
                              append_ranges,
-                             cache_data,
-                             locked);
+                             cache_data);
     } else {
         NCBI_THROW(CSeqDBException, eArgErr, CSeqDB::kOidNotFound);
     }
@@ -1875,12 +1833,10 @@ void CSeqDBImpl::SetOffsetRanges(int                oid,
 void CSeqDBImpl::FlushOffsetRangeCache()
 {
     CHECK_MARKER();
-    CSeqDBLockHold locked(m_Atlas);
-    m_Atlas.Lock(locked);
 
     for(int vol_idx = 0; vol_idx < m_VolSet.GetNumVols(); vol_idx++) {
         CSeqDBVol* volp = m_VolSet.GetVolNonConst(vol_idx);
-        volp->FlushOffsetRangeCache(locked);
+        volp->FlushOffsetRangeCache();
     }
 }
 
@@ -2616,7 +2572,7 @@ void CSeqDBImpl::SetNumberOfThreads(int num_threads, bool force_mt)
 
         for (int thread = num_threads; thread < m_NumThreads; ++thread) {
             SSeqResBuffer * buffer = m_CachedSeqs.back();
-            x_RetSeqBuffer(buffer, locked);
+            x_RetSeqBuffer(buffer);
             m_CachedSeqs.pop_back();
             delete buffer;
         }
