@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBIMPL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBIMPL_HPP
 
-/*  $Id: seqdbimpl.hpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: seqdbimpl.hpp 631537 2021-05-19 13:50:49Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1249,7 +1249,7 @@ private:
     /// @param oid The ordinal id of the sequence. [in]
     /// @param locked The lock holder object for this thread. [in]
     /// @return The length of the sequence in bases.
-    int x_GetSeqLength(int oid, CSeqDBLockHold & locked) const;
+    int x_GetSeqLength(int oid) const;
 
     /// Get local cache ID for current thread.
     ///
@@ -1397,16 +1397,18 @@ private:
     mutable vector<SSeqResBuffer *> m_CachedSeqs;
 
     /// Fill up the buffer
-    void x_FillSeqBuffer(SSeqResBuffer * buffer, int oid, CSeqDBLockHold &locked) const;
+    void x_FillSeqBuffer(SSeqResBuffer * buffer, int oid) const;
 
     /// Get sequence from buffer
     int x_GetSeqBuffer(SSeqResBuffer * buffer, int oid, const char ** seq) const;
 
     /// Return sequence to buffer
-    void x_RetSeqBuffer(SSeqResBuffer * buffer, CSeqDBLockHold & locked) const;
+    void x_RetSeqBuffer(SSeqResBuffer * buffer) const;
 
     /// Initialize Id Set
     void x_InitIdSet();
+
+    CObjectIStreamAsnBinary  *reusable_inpstr; 
 };
 
 END_NCBI_SCOPE
