@@ -1,4 +1,4 @@
-/*  $Id: seqdbvol.cpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: seqdbvol.cpp 631553 2021-05-19 13:52:19Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -66,7 +66,7 @@ TGi CSeqDBGiIndex::GetSeqGI(TOid             oid,
 
     TIndx offset = oid * m_Size + 32;    
     data = m_Lease.GetFileDataPtr(offset);
-    return GI_FROM(Int4, SeqDB_GetStdOrd((Int4 *) data));
+    return GI_FROM(Uint4, SeqDB_GetStdOrd((Uint4 *) data));
 }
 
 CSeqDBVol::CSeqDBVol(CSeqDBAtlas        & atlas,
@@ -87,10 +87,6 @@ CSeqDBVol::CSeqDBVol(CSeqDBAtlas        & atlas,
       m_HaveColumns  (false),
       m_SeqFileOpened(false),
       m_HdrFileOpened(false),
-      m_PigFileOpened(false),
-      m_GiFileOpened (false),
-      m_StrFileOpened(false),
-      m_TiFileOpened (false),
       m_HashFileOpened(false),
       m_OidFileOpened(false)
 {
@@ -116,8 +112,7 @@ CSeqDBVol::OpenSeqFile(CSeqDBLockHold & locked) const{
 
 void
 CSeqDBVol::x_OpenSeqFile(void) const {
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
+    CFastMutexGuard mtx_gurad(m_MtxSeq);
     if (!m_SeqFileOpened && m_Idx->GetNumOIDs() != 0) {
         m_Seq.Reset(new CSeqDBSeqFile(m_Atlas, m_VolName, (m_IsAA?'p':'n')));
     }
@@ -126,8 +121,7 @@ CSeqDBVol::x_OpenSeqFile(void) const {
 
 void
 CSeqDBVol::x_OpenHdrFile(void) const {
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
+    CFastMutexGuard mtx_gurad(m_MtxHdr);
     if (!m_HdrFileOpened && m_Idx->GetNumOIDs() != 0) {
         m_Hdr.Reset(new CSeqDBHdrFile(m_Atlas, m_VolName, (m_IsAA?'p':'n')));
     }
@@ -136,70 +130,102 @@ CSeqDBVol::x_OpenHdrFile(void) const {
 
 void
 CSeqDBVol::x_OpenPigFile(void) const{
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
-    if (!m_PigFileOpened &&
-         CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'p') &&
-         m_Idx->GetNumOIDs() != 0) {
-        m_IsamPig =
-            new CSeqDBIsam(m_Atlas,
-                           m_VolName,
-                           (m_IsAA?'p':'n'),
-                           'p',
-                           ePigId);
+    CFastMutexGuard mtx_gurad(m_MtxPig);
+    if (m_IsamPig.NotEmpty()) {
+    	m_IsamPig->AddReference();
+    }
+    else if (CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'p') &&
+             m_Idx->GetNumOIDs() != 0) {
+        m_IsamPig = new CSeqDBIsam(m_Atlas, m_VolName, (m_IsAA?'p':'n'), 'p', ePigId);
+    }
+}
+
+void
+CSeqDBVol::x_UnleasePigFile(void) const{
+    CFastMutexGuard mtx_gurad(m_MtxPig);
+    if (m_IsamPig.NotEmpty()) {
+    	if (m_IsamPig->ReferencedOnlyOnce()) {
+    		m_IsamPig.Reset();
+    	}
+    	else {
+    		m_IsamPig->RemoveReference();
+    	}
     }
-    m_PigFileOpened = true;
 }
 
 void
 CSeqDBVol::x_OpenGiFile(void) const{
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
-    if (!m_GiFileOpened &&
-         CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'n') &&
-         m_Idx->GetNumOIDs() != 0) {
-        m_IsamGi =
-            new CSeqDBIsam(m_Atlas,
-                           m_VolName,
-                           (m_IsAA?'p':'n'),
-                           'n',
-                           eGiId);
+    CFastMutexGuard mtx_gurad(m_MtxGi);
+    if (m_IsamGi.NotEmpty()) {
+    	m_IsamGi->AddReference();
+    }
+    else if (CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'n') &&
+			   m_Idx->GetNumOIDs() != 0) {
+        m_IsamGi = new CSeqDBIsam(m_Atlas, m_VolName, (m_IsAA?'p':'n'), 'n', eGiId);
+    }
+}
+
+void
+CSeqDBVol::x_UnleaseGiFile(void) const{
+    CFastMutexGuard mtx_gurad(m_MtxGi);
+    if (m_IsamGi.NotEmpty()) {
+    	if (m_IsamGi->ReferencedOnlyOnce()) {
+    		m_IsamGi.Reset();
+    	}
+    	else {
+    		m_IsamGi->RemoveReference();
+    	}
     }
-    m_GiFileOpened = true;
 }
 
 void
 CSeqDBVol::x_OpenStrFile(void) const{
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
-    if (!m_StrFileOpened &&
-         CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 's') &&
-         m_Idx->GetNumOIDs() != 0) {
-        m_IsamStr =
-            new CSeqDBIsam(m_Atlas,
-                           m_VolName,
-                           (m_IsAA?'p':'n'),
-                           's',
-                           eStringId);
+    CFastMutexGuard mtx_gurad(m_MtxStr);
+    if (m_IsamStr.NotEmpty()) {
+    	m_IsamStr->AddReference();
+    }
+    else if(CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 's') &&
+            m_Idx->GetNumOIDs() != 0) {
+        m_IsamStr = new CSeqDBIsam(m_Atlas, m_VolName, (m_IsAA?'p':'n'), 's', eStringId);
+    }
+}
+
+void
+CSeqDBVol::x_UnleaseStrFile(void) const{
+    CFastMutexGuard mtx_gurad(m_MtxStr);
+    if (m_IsamStr.NotEmpty()) {
+    	if (m_IsamStr->ReferencedOnlyOnce()) {
+    		m_IsamStr.Reset();
+    	}
+    	else {
+    		m_IsamStr->RemoveReference();
+    	}
     }
-    m_StrFileOpened = true;
 }
 
 void
 CSeqDBVol::x_OpenTiFile(void) const{
-    static CFastMutex mtx;
-    CFastMutexGuard mtx_gurad(mtx);
-    if (!m_TiFileOpened &&
-         CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 't') &&
-         m_Idx->GetNumOIDs() != 0) {
-        m_IsamTi =
-            new CSeqDBIsam(m_Atlas,
-                           m_VolName,
-                           (m_IsAA?'p':'n'),
-                           't',
-                           eTiId);
+    CFastMutexGuard mtx_gurad(m_MtxTi);
+    if (m_IsamTi.NotEmpty()) {
+    	m_IsamTi->AddReference();
+    }
+    else if (CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 't') &&
+             m_Idx->GetNumOIDs() != 0) {
+        m_IsamTi = new CSeqDBIsam(m_Atlas, m_VolName, (m_IsAA?'p':'n'), 't', eTiId);
+    }
+}
+
+void
+CSeqDBVol::x_UnleaseTiFile(void) const{
+    CFastMutexGuard mtx_gurad(m_MtxTi);
+    if (m_IsamTi.NotEmpty()) {
+    	if (m_IsamTi->ReferencedOnlyOnce()) {
+    		m_IsamTi.Reset();
+    	}
+    	else {
+    		m_IsamTi->RemoveReference();
+    	}
     }
-    m_TiFileOpened = true;
 }
 
 void
@@ -249,7 +275,7 @@ string CSeqDBVol::GetLMDBFileName() const
 	return m_Idx->GetLMDBFileName();
 }
 
-int CSeqDBVol::GetSeqLengthProt(int oid, CSeqDBLockHold & locked) const
+int CSeqDBVol::GetSeqLengthProt(int oid) const
 {
     TIndx start_offset = 0;
     TIndx end_offset   = 0;
@@ -265,12 +291,11 @@ int CSeqDBVol::GetSeqLengthProt(int oid, CSeqDBLockHold & locked) const
 
 // Assumes locked.
 
-int CSeqDBVol::GetSeqLengthExact(int oid, CSeqDBLockHold & locked) const
+int CSeqDBVol::GetSeqLengthExact(int oid) const
 {
     TIndx start_offset = 0;
     TIndx end_offset   = 0;
 
-    //m_Atlas.Lock(locked);
     if (!m_SeqFileOpened) x_OpenSeqFile();
     m_Idx->GetSeqStartEnd(oid, start_offset, end_offset);
 
@@ -289,7 +314,7 @@ int CSeqDBVol::GetSeqLengthExact(int oid, CSeqDBLockHold & locked) const
     return (whole_bytes * 4) + remainder;
 }
 
-int CSeqDBVol::GetSeqLengthApprox(int oid, CSeqDBLockHold & locked) const
+int CSeqDBVol::GetSeqLengthApprox(int oid) const
 {
     TIndx start_offset = 0;
     TIndx end_offset   = 0;
@@ -1023,12 +1048,12 @@ CSeqDBVol::x_GetTaxDefline(int                    oid,
 
     // 2. if there is a preferred gi, bump it to the top.
 
-    if (preferred_gi || preferred_seqid) {
+    if (preferred_gi != ZERO_GI || preferred_seqid) {
         CRef<CBlast_def_line_set> new_bdls(new CBlast_def_line_set);
 
         CRef<const CSeq_id> seqid;
-        if (preferred_gi) {
-            seqid.Reset(new CSeq_id(CSeq_id::e_Gi, GI_TO(TIntId, preferred_gi)));
+        if (preferred_gi != ZERO_GI) {
+            seqid.Reset(new CSeq_id(CSeq_id::e_Gi, preferred_gi));
         } else {
             seqid.Reset(preferred_seqid);
         }
@@ -1285,7 +1310,7 @@ CSeqDBVol::GetBioseq(int                    oid,
         CRef<const CSeq_id > seqid;
         CRef<const CSeq_id > seqid_lcl;
         if (target_gi != ZERO_GI) {
-            seqid.Reset(new CSeq_id(CSeq_id::e_Gi, GI_TO(TIntId, target_gi)));
+            seqid.Reset(new CSeq_id(CSeq_id::e_Gi, target_gi));
             seqid_lcl.Reset(new CSeq_id(CSeq_id::e_Local, NStr::NumericToString(target_gi)));
         }
         else {
@@ -1333,7 +1358,7 @@ CSeqDBVol::GetBioseq(int                    oid,
 
     if (seqdata) {
         const char * seq_buffer = 0;
-        int length = x_GetSequence(oid, & seq_buffer, false, false);
+        int length = x_GetSequence(oid, & seq_buffer);
 
         if (length < 1) {
             return null_result;
@@ -1427,8 +1452,7 @@ CSeqDBVol::GetBioseq(int                    oid,
 }
 
 char * CSeqDBVol::x_AllocType(size_t           length,
-                              ESeqDBAllocType  alloc_type,
-                              CSeqDBLockHold & locked) const
+                              ESeqDBAllocType  alloc_type) const
 {
     // Allocation using the atlas is not intended for the end user.
     // 16 bytes are added as insurance against potential off-by-one or
@@ -1449,7 +1473,7 @@ char * CSeqDBVol::x_AllocType(size_t           length,
 
     case eAtlas:
     default:
-        retval = m_Atlas.Alloc(length + 16, locked, false);
+        retval = CSeqDBAtlas::Alloc(length + 16, false);
     }
 
     return retval;
@@ -1460,12 +1484,11 @@ int CSeqDBVol::GetAmbigSeq(int                oid,
                            int                nucl_code,
                            ESeqDBAllocType    alloc_type,
                            SSeqDBSlice      * region,
-                           CSeqDB::TSequenceRanges  * masks,
-                           CSeqDBLockHold   & locked) const
+                           CSeqDB::TSequenceRanges  * masks) const
 {
     char * buf1 = 0;
     int baselen =
-        x_GetAmbigSeq(oid, & buf1, nucl_code, alloc_type, region, masks, locked);
+        x_GetAmbigSeq(oid, & buf1, nucl_code, alloc_type, region, masks);
 
     *buffer = buf1;
     return baselen;
@@ -1499,8 +1522,7 @@ int CSeqDBVol::GetAmbigPartialSeq(int                oid,
                                   int                nucl_code,
                                   ESeqDBAllocType    alloc_type,
                                   CSeqDB::TSequenceRanges  * partial_ranges,
-                                  CSeqDB::TSequenceRanges  * masks,
-                                  CSeqDBLockHold  & locked) const
+                                  CSeqDB::TSequenceRanges  * masks) const
 {
 
 	if ((partial_ranges == NULL) || (partial_ranges->size() == 0)) {
@@ -1508,7 +1530,7 @@ int CSeqDBVol::GetAmbigPartialSeq(int                oid,
 	}
 
     const char * tmp(0);
-    int base_length = x_GetSequence(oid, &tmp, false, false);
+    int base_length = x_GetSequence(oid, &tmp);
 	if (base_length < 1) {
 	    NCBI_THROW(CSeqDBException, eFileErr, "Error: could not get sequence or range.");
 	}
@@ -1520,7 +1542,7 @@ int CSeqDBVol::GetAmbigPartialSeq(int                oid,
     }
 
     bool sentinel = (nucl_code == kSeqDBNuclBlastNA8);
-    *buffer = x_AllocType(base_length + (sentinel ? 2 : 0), alloc_type, locked);
+    *buffer = x_AllocType(base_length + (sentinel ? 2 : 0), alloc_type);
     char *seq = *buffer + (sentinel ? 1 : 0);
 
     vector<Int4> ambchars;
@@ -1567,18 +1589,14 @@ int CSeqDBVol::x_GetAmbigSeq(int                oid,
                              int                nucl_code,
                              ESeqDBAllocType    alloc_type,
                              SSeqDBSlice      * region,
-                             CSeqDB::TSequenceRanges *masks,
-                             CSeqDBLockHold   & locked) const
+                             CSeqDB::TSequenceRanges *masks) const
 {
     // Note the use of the third argument of x_GetSequence() to manage
     // the lifetime of the acquired region.  Specifying false for that
     // argument ties the lifetime to the CSeqDBSeqFile's memory lease.
 
     const char * tmp(0);
-    int base_length = x_GetSequence(oid,
-                                    &tmp,
-                                    false,                                   
-                                    false);
+    int base_length = x_GetSequence(oid, &tmp);
 
     if (region && region->end > base_length )
         NCBI_THROW(CSeqDBException, eFileErr, "Error: region beyond sequence range.");
@@ -1592,14 +1610,14 @@ int CSeqDBVol::x_GetAmbigSeq(int                oid,
     if (m_Idx->GetSeqType() == 'p') {
 
         tmp += range.begin;
-        *buffer = x_AllocType(base_length, alloc_type, locked);
+        *buffer = x_AllocType(base_length, alloc_type);
         memcpy(*buffer, tmp, base_length);
         s_SeqDBMaskSequence(*buffer - range.begin, masks, (char)21, range);
 
     } else {
 
         bool sentinel = (nucl_code == kSeqDBNuclBlastNA8);
-        *buffer = x_AllocType(base_length + (sentinel ? 2 : 0), alloc_type, locked);
+        *buffer = x_AllocType(base_length + (sentinel ? 2 : 0), alloc_type);
         char *seq = *buffer - range.begin + (sentinel ? 1 : 0);
 
         // Get ambiguity characters.
@@ -1729,10 +1747,7 @@ void SeqDB_UnpackAmbiguities(const CTempString & sequence,
 
 
 int CSeqDBVol::x_GetSequence(int              oid,
-                             const char    ** buffer,
-                             bool             keep,                             
-                             bool             can_release,
-                             bool             in_lease) const
+                             const char    ** buffer) const
 {
     TIndx start_offset = 0;
     TIndx end_offset   = 0;
@@ -1823,6 +1838,28 @@ list< CRef<CSeq_id> > CSeqDBVol::GetSeqIDs(int  oid) const
     return seqids;
 }
 
+list< CRef<CSeq_id> > CSeqDBVol::GetSeqIDs(int  oid, CObjectIStreamAsnBinary *inpstr ) const
+{
+    list< CRef< CSeq_id > > seqids;
+
+    CRef<CBlast_def_line_set> defline_set =
+        x_GetFilteredHeader(oid, NULL, inpstr);
+
+    if ((! defline_set.Empty()) && defline_set->CanGet()) {
+        ITERATE(list< CRef<CBlast_def_line> >, defline, defline_set->Get()) {
+            if (! (*defline)->CanGetSeqid()) {
+                continue;
+            }
+
+            ITERATE(list< CRef<CSeq_id> >, seqid, (*defline)->GetSeqid()) {
+                seqids.push_back(*seqid);
+            }
+        }
+    }
+
+    return seqids;
+}
+
 TGi CSeqDBVol::GetSeqGI(int              oid,
                         CSeqDBLockHold & locked) const
 {
@@ -2003,6 +2040,117 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
     return BDLS;
 }
 
+CRef<CBlast_def_line_set>
+CSeqDBVol::x_GetFilteredHeader(int                    oid,
+                               bool                 * changed,
+			       CObjectIStreamAsnBinary  *inpstr ) const
+{
+    typedef list< CRef<CBlast_def_line> > TBDLL;
+    typedef TBDLL::iterator               TBDLLIter;
+
+    //m_Atlas.Lock(locked);
+
+    const bool useCache = (CThread::GetSelf() == 0 ? true : false);
+    TDeflineCacheItem & cached = m_DeflineCache.Lookup(oid);
+
+    if (useCache && cached.first.NotEmpty()) {
+        if (changed) {
+            *changed = cached.second;
+        }
+
+        return cached.first;
+    }
+
+    bool asn_changed = false;
+
+    CRef<CBlast_def_line_set> BDLS =
+        x_GetHdrAsn1(oid, true, & asn_changed, inpstr);
+
+    bool id_filter = x_HaveIdFilter();
+
+    if (id_filter || m_MemBit) {
+        // Create the memberships mask (should this be fixed to allow
+        // membership bits greater than 32?)
+
+        TBDLL & dl = BDLS->Set();
+
+        for(TBDLLIter iter = dl.begin(); iter != dl.end(); ) {
+            const CBlast_def_line & defline = **iter;
+
+            bool have_memb = true;
+
+            if (m_MemBit) {
+                have_memb =
+                    defline.CanGetMemberships() &&
+                    defline.IsSetMemberships() &&
+                    (! defline.GetMemberships().empty());
+
+                if (have_memb) {
+                    int bits = defline.GetMemberships().front();
+                    int memb_mask = 0x1 << (m_MemBit-1);
+
+                    if ((bits & memb_mask) == 0) {
+                        have_memb = false;
+                    }
+                }
+            }
+
+            // Here we must pass both the user-gi and volume-gi test,
+            // for each defline, but not necessarily for each Seq-id.
+            if (have_memb) {
+            	if (id_filter && defline.CanGetSeqid()) {
+            		have_memb = false;
+            		bool have_user = false, have_volume = false;
+            		ITERATE(list< CRef<CSeq_id> >, seqid, defline.GetSeqid()) {
+            			x_FilterHasId(**seqid, have_user, have_volume);
+            			if (have_user && have_volume) break;
+            		}
+            		have_memb = have_user && have_volume;
+            	}
+
+            	if(have_memb && (!m_UserGiList.Empty()) && (m_UserGiList->GetNumTaxIds() > 0)) {
+               		have_memb = s_IncludeDefline_Taxid(defline, m_UserGiList->GetTaxIdsList());
+            	}
+
+            	if (!have_memb && !m_VolumeGiLists.empty()) {
+               		NON_CONST_ITERATE(TGiLists, vtaxid, m_VolumeGiLists) {
+               			if( (*vtaxid)->GetNumTaxIds() > 0) {
+               				have_memb = s_IncludeDefline_Taxid(defline, (*vtaxid)->GetTaxIdsList());
+               				if(have_memb){
+               					break;
+               				}
+               			}
+               		}
+            	}
+
+            	if(have_memb && (!m_NegativeList.Empty()) && (m_NegativeList->GetNumTaxIds() > 0)) {
+               		have_memb = s_IncludeDefline_NegativeTaxid(defline, m_NegativeList->GetTaxIdsList());
+            	}
+            }
+
+            if (! have_memb) {
+                TBDLLIter eraseme = iter++;
+                dl.erase(eraseme);
+                asn_changed = true;
+            } else {
+                iter++;
+            }
+        }
+    }
+
+    if (useCache)
+    {
+    	if (asn_changed) {
+       	 cached.first = BDLS;
+       	 cached.second = asn_changed;
+    	} else {
+       	 cached.first = BDLS;
+       	 cached.second = asn_changed;
+    	}
+    }
+
+    return BDLS;
+}
 CRef<CBlast_def_line_set>
 CSeqDBVol::x_GetHdrAsn1(int              oid,
                         bool             adjust_oids,
@@ -2051,6 +2199,59 @@ CSeqDBVol::x_GetHdrAsn1(int              oid,
     return bdls;
 }
 
+CRef<CBlast_def_line_set>
+CSeqDBVol::x_GetHdrAsn1(int              oid,
+                        bool             adjust_oids,
+                        bool           * changed,
+			CObjectIStreamAsnBinary *inpstr ) const
+{
+    CRef<CBlast_def_line_set> bdls;
+
+    CTempString raw_data = x_GetHdrAsn1Binary(oid);
+
+    if (! raw_data.size()) {
+        return bdls;
+    }
+
+    /*
+     * FixNonPrint(how);
+     * ResetThisState();
+     * OpenFromBuffer(buffer, size);
+     */
+    inpstr->Close(); // insted of ResetThisState
+    inpstr->OpenFromBuffer( raw_data.data(), raw_data.size());
+
+    bdls.Reset(new objects::CBlast_def_line_set);
+
+    (*inpstr)  >> *bdls; // NEW METHOD
+
+    if (adjust_oids && bdls.NotEmpty() && m_VolStart) {
+        NON_CONST_ITERATE(list< CRef<CBlast_def_line> >, dl, bdls->Set()) {
+            if (! (**dl).CanGetSeqid()) {
+                continue;
+            }
+
+            NON_CONST_ITERATE(list< CRef<CSeq_id> >, id, (*dl)->SetSeqid()) {
+                CSeq_id & seqid = **id;
+
+                if (seqid.Which() == CSeq_id::e_General) {
+                    CDbtag & dbt = seqid.SetGeneral();
+
+                    if (dbt.GetDb() == "BL_ORD_ID") {
+                        int vol_oid = dbt.GetTag().GetId();
+                        dbt.SetTag().SetId(m_VolStart + vol_oid);
+                        if (changed) {
+                            *changed = true;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    return bdls;
+}
+
 CTempString
 CSeqDBVol::x_GetHdrAsn1Binary(int oid) const
 {
@@ -2162,14 +2363,15 @@ int CSeqDBVol::GetMinLength() const
     return m_Idx->GetMinLength();
 }
 
-bool CSeqDBVol::PigToOid(int pig, int & oid, CSeqDBLockHold & locked) const
+bool CSeqDBVol::PigToOid(int pig, int & oid) const
 {
-    if (!m_PigFileOpened) x_OpenPigFile();
-    if (m_IsamPig.Empty()) {
-        return false;
+	bool rv = false;
+    x_OpenPigFile();
+    if (m_IsamPig.NotEmpty()) {
+    	rv = m_IsamPig->PigToOid(pig, oid);
+    	x_UnleasePigFile();
     }
-
-    return m_IsamPig->PigToOid(pig, oid);
+    return rv;
 }
 
 bool CSeqDBVol::GetPig(int oid, int & pig, CSeqDBLockHold & locked) const
@@ -2212,7 +2414,7 @@ bool CSeqDBVol::TiToOid(Int8                   ti,
     // this point (in the call stack) uses int; code above this level,
     // up to the user level, uses Int8.
 
-    if (!m_TiFileOpened) x_OpenTiFile();
+    x_OpenTiFile();
     if (m_IsamTi.Empty()) {
         // If the "nti/ntd" files become ubiquitous, this could be
         // removed.  For now, I will look up trace IDs in the string
@@ -2227,30 +2429,36 @@ bool CSeqDBVol::TiToOid(Int8                   ti,
         if (oids.size()) {
             oid = oids[0];
         }
-
         return ! oids.empty();
     }
-
-    return m_IsamTi->IdToOid(ti, oid);
+    else {
+    	bool rv = false;
+    	rv = m_IsamTi->IdToOid(ti, oid);
+    	x_UnleaseTiFile();
+    	return rv;
+    }
 }
 
 bool CSeqDBVol::GiToOid(TGi gi, int & oid, CSeqDBLockHold & locked) const
 {
-    if (!m_GiFileOpened) x_OpenGiFile();
+	bool rv = false;
+     x_OpenGiFile();
     if (m_IsamGi.Empty()) {
         return false;
     }
-
-    return m_IsamGi->IdToOid(GI_TO(Int8, gi), oid);
+    rv = m_IsamGi->IdToOid(GI_TO(Int8, gi), oid);
+    x_UnleaseGiFile();
+    return rv;
 }
 
 void CSeqDBVol::IdsToOids(CSeqDBGiList   & ids,
                           CSeqDBLockHold & locked) const
 {
     if (ids.GetNumGis()) {
-        if (!m_GiFileOpened) x_OpenGiFile();
+         x_OpenGiFile();
         if (m_IsamGi.NotEmpty()) {
             m_IsamGi->IdsToOids(m_VolStart, m_VolEnd, ids);            
+            x_UnleaseGiFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
@@ -2259,9 +2467,10 @@ void CSeqDBVol::IdsToOids(CSeqDBGiList   & ids,
     }
 
     if (ids.GetNumTis()) {
-        if (!m_TiFileOpened) x_OpenTiFile();
+        x_OpenTiFile();
         if (m_IsamTi.NotEmpty()) {
             m_IsamTi->IdsToOids(m_VolStart, m_VolEnd, ids);
+            x_UnleaseTiFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
@@ -2270,9 +2479,10 @@ void CSeqDBVol::IdsToOids(CSeqDBGiList   & ids,
     }
 
     if (ids.GetNumPigs()) {
-        if (!m_PigFileOpened) x_OpenPigFile();
+        x_OpenPigFile();
         if (m_IsamPig.NotEmpty()) {
             m_IsamPig->IdsToOids(m_VolStart, m_VolEnd, ids);
+            x_UnleasePigFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
@@ -2281,16 +2491,16 @@ void CSeqDBVol::IdsToOids(CSeqDBGiList   & ids,
     }
 
     if (ids.GetNumSis() && (GetLMDBFileName() == kEmptyStr)) {
-        if (!m_StrFileOpened) x_OpenStrFile();
+        x_OpenStrFile();
         if (m_IsamStr.NotEmpty()) {
             m_IsamStr->IdsToOids(m_VolStart, m_VolEnd, ids);
+            x_UnleaseStrFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
                        "SI list specified but no ISAM file found for SI in " + m_VolName);
         }
     }
-    x_UnLeaseIsam();
 }
 
 void CSeqDBVol::IdsToOids(CSeqDBNegativeList & ids,
@@ -2299,9 +2509,10 @@ void CSeqDBVol::IdsToOids(CSeqDBNegativeList & ids,
     // Numeric translation is done in batch mode.
 
     if (ids.GetNumGis()) {
-        if (!m_GiFileOpened) x_OpenGiFile();
+         x_OpenGiFile();
         if (m_IsamGi.NotEmpty()) {
             m_IsamGi->IdsToOids(m_VolStart, m_VolEnd, ids);
+            x_UnleaseGiFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
@@ -2310,9 +2521,10 @@ void CSeqDBVol::IdsToOids(CSeqDBNegativeList & ids,
     }
 
     if (ids.GetNumTis()) {
-        if (!m_TiFileOpened) x_OpenTiFile();
+        x_OpenTiFile();
         if (m_IsamTi.NotEmpty()) {
             m_IsamTi->IdsToOids(m_VolStart, m_VolEnd, ids);
+            x_UnleaseTiFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
@@ -2321,16 +2533,16 @@ void CSeqDBVol::IdsToOids(CSeqDBNegativeList & ids,
     }
 
     if (ids.GetNumSis()) {
-        if (!m_StrFileOpened) x_OpenStrFile();
+        x_OpenStrFile();
         if (m_IsamStr.NotEmpty()) {
             m_IsamStr->IdsToOids(m_VolStart, m_VolEnd, ids);
+            x_UnleaseStrFile();
         } else {
             NCBI_THROW(CSeqDBException,
                        eArgErr,
                        "SI list specified but no ISAM file found for SI in " + m_VolName);
         }
     }
-    x_UnLeaseIsam();
 }
 
 bool CSeqDBVol::GetGi(int                    oid,
@@ -2339,8 +2551,7 @@ bool CSeqDBVol::GetGi(int                    oid,
 {
     gi = INVALID_GI;
 
-    if (!m_GiFileOpened) x_OpenGiFile();
-    if (m_IsamGi.Empty()) {
+    if (!CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'n')) {
         return false;
     }
 
@@ -2390,57 +2601,64 @@ void CSeqDBVol::x_StringToOids(const string          & acc,
 
     switch(id_type) {
     case eStringId:
-        if (!m_StrFileOpened) x_OpenStrFile();
+        x_OpenStrFile();
         if (! m_IsamStr.Empty()) {
             // Not simplified
             vcheck = true;
             m_IsamStr->StringToOids(str_id, oids, simpler, vcheck);
+            x_UnleaseStrFile();
         }
         break;
 
     case ePigId:
         // Converted to PIG type.
-        if (!m_PigFileOpened) x_OpenPigFile();
+        x_OpenPigFile();
         if (! m_IsamPig.Empty()) {
             int oid(-1);
 
             if (m_IsamPig->PigToOid((int) ident, oid)) {
                 oids.push_back(oid);
             }
+            x_UnleasePigFile();
         }
         break;
 
     case eGiId:
         // Converted to GI type.
-        if (!m_GiFileOpened) x_OpenGiFile();
+        x_OpenGiFile();
         if (! m_IsamGi.Empty()) {
             int oid(-1);
 
             if (m_IsamGi->IdToOid(ident, oid)) {
                 oids.push_back(oid);
             }
+            x_UnleaseGiFile();
         }
         break;
 
     case eTiId:
         // Converted to TI type.
-        if (!m_TiFileOpened) x_OpenTiFile();
-        if (!m_StrFileOpened) x_OpenStrFile();
+         x_OpenTiFile();
         if (! m_IsamTi.Empty()) {
             int oid(-1);
 
             if (m_IsamTi->IdToOid(ident, oid)) {
                 oids.push_back(oid);
             }
-        } else if (m_IsamStr) {
-            // Not every database with TIs has a TI index, so fall
-            // back to a string comparison if the first attempt fails.
-            //
-            // 1. TI's don't have versions.
-            // 2. Specify "adjusted" as true, because lookup of
-            //    "gb|.." and similar tricks are not needed for TIs.
-
-            m_IsamStr->StringToOids(acc, oids, true, vcheck);
+            x_UnleaseTiFile();
+        }
+        else {
+        	x_OpenStrFile();
+        	if (m_IsamStr.NotEmpty()) {
+            	// Not every database with TIs has a TI index, so fall
+            	// back to a string comparison if the first attempt fails.
+            	//
+            	// 1. TI's don't have versions.
+            	// 2. Specify "adjusted" as true, because lookup of
+            	//    "gb|.." and similar tricks are not needed for TIs.
+        		m_IsamStr->StringToOids(acc, oids, true, vcheck);
+        		x_UnleaseStrFile();
+        	}
         }
         break;
 
@@ -2465,7 +2683,6 @@ void CSeqDBVol::x_StringToOids(const string          & acc,
     if (vcheck) {
         x_CheckVersions(acc, oids);
     }
-    x_UnLeaseIsam();    
 }
 
 void CSeqDBVol::x_CheckVersions(const string         & acc,
@@ -2552,28 +2769,6 @@ void CSeqDBVol::SeqidToOids(CSeq_id              & seqid,
 
 }
 
-void CSeqDBVol::x_UnLeaseIsam(void) const
-{
-    if(m_Atlas.GetOpenedFilseCount() > CSeqDBAtlas::e_MaxFileDescritors) {
-        if (m_IsamPig.NotEmpty()) {
-            m_PigFileOpened = false;
-            m_IsamPig->UnLease();
-        }
-        if (m_IsamGi.NotEmpty()) {
-            m_GiFileOpened = false;
-            m_IsamGi->UnLease();
-        }
-        if (m_IsamStr.NotEmpty()) {
-            m_StrFileOpened = false;
-            m_IsamStr->UnLease();
-        }    
-        if (m_IsamTi.NotEmpty()) {
-            m_TiFileOpened = false;
-            m_IsamTi->UnLease();
-        }
-    }
-}
-
 void CSeqDBVol::UnLease()
 {
     m_Idx->UnLease();
@@ -2697,7 +2892,7 @@ CSeqDBVol::GetSeqData(int              oid,
         const char * buffer(0);
         TSeqPos      length(0);
 
-        length = x_GetSequence(oid, & buffer, false, false);
+        length = x_GetSequence(oid, & buffer);
 
         if ((begin >= end) || (end > length)) {
             NCBI_THROW(CSeqDBException,
@@ -2723,8 +2918,7 @@ CSeqDBVol::GetSeqData(int              oid,
                                nucl_code,
                                eNew,
                                & slice,
-                               NULL,
-                               locked);
+                               NULL);
 
         // validity of begin, end, and length has already been checked by
         // overloaded x_GetSequence()
@@ -2756,8 +2950,7 @@ void
 CSeqDBVol::GetRawSeqAndAmbig(int              oid,
                              const char    ** buffer,
                              int            * seq_length,
-                             int            * amb_length,
-                             CSeqDBLockHold & locked) const
+                             int            * amb_length ) const
 {
     if (seq_length)
         *seq_length = 0;
@@ -2849,7 +3042,7 @@ void CSeqDBVol::GetGiBounds(TGi            & low_id,
                             CSeqDBLockHold & locked) const
 {
     //m_Atlas.Lock(locked);
-    if (!m_GiFileOpened) x_OpenGiFile();
+    x_OpenGiFile();
     low_id = ZERO_GI;
     high_id = ZERO_GI;
     count = 0;
@@ -2864,6 +3057,7 @@ void CSeqDBVol::GetGiBounds(TGi            & low_id,
 
         s_SeqDBFitsInFour(L);
         s_SeqDBFitsInFour(H);
+        x_UnleaseGiFile();
     }
 }
 
@@ -2873,7 +3067,7 @@ void CSeqDBVol::GetPigBounds(int            & low_id,
                              CSeqDBLockHold & locked) const
 {
     //m_Atlas.Lock(locked);
-    if (!m_PigFileOpened) x_OpenPigFile();
+    x_OpenPigFile();
     low_id = high_id = count = 0;
 
     if (m_IsamPig.NotEmpty()) {
@@ -2886,33 +3080,32 @@ void CSeqDBVol::GetPigBounds(int            & low_id,
 
         s_SeqDBFitsInFour(L);
         s_SeqDBFitsInFour(H);
+        x_UnleasePigFile();
     }
 }
 
 void CSeqDBVol::GetStringBounds(string         & low_id,
                                 string         & high_id,
-                                int            & count,
-                                CSeqDBLockHold & locked) const
+                                int            & count) const
 {
-    //m_Atlas.Lock(locked);
-    if (!m_StrFileOpened) x_OpenStrFile();
+    x_OpenStrFile();
     count = 0;
     low_id.erase();
     high_id.erase();
 
     if (m_IsamStr.NotEmpty()) {
         m_IsamStr->GetIdBounds(low_id, high_id, count);
+        x_UnleaseStrFile();
     }
 }
 
 void CSeqDBVol::SetOffsetRanges(int                oid,
                                 const TRangeList & offset_ranges,
                                 bool               append_ranges,
-                                bool               cache_data,
-                                CSeqDBLockHold   & locked) const
+                                bool               cache_data) const
 {
-    //m_Atlas.Lock(locked);
 
+	 CFastMutexGuard mtx_gurad(m_MtxCachedRange);
     if (offset_ranges.empty() && (! cache_data) && (! append_ranges)) {
         // Specifying no-cache plus an empty offset range list, means
         // that we are clearing out this sequence.  In this case, just
@@ -2937,7 +3130,7 @@ void CSeqDBVol::SetOffsetRanges(int                oid,
         }
 
         if (R.Empty()) {
-            R.Reset(new CSeqDBRangeList(m_Atlas));
+            R.Reset(new CSeqDBRangeList());
         }
     }
 
@@ -2958,9 +3151,9 @@ void CSeqDBVol::SetOffsetRanges(int                oid,
     R->SetRanges(offset_ranges, append_ranges, cache_data);
 }
 
-void CSeqDBVol::FlushOffsetRangeCache(CSeqDBLockHold& locked)
+void CSeqDBVol::FlushOffsetRangeCache()
 {
-    //m_Atlas.Lock(locked);
+	CFastMutexGuard mtx_gurad(m_MtxCachedRange);
     m_RangeCache.clear();
 }
 
@@ -3100,10 +3293,10 @@ void CSeqDBVol::x_OpenAllColumns(CSeqDBLockHold & locked)
     for(size_t i = 0; i < alpha.size(); i++) {
         ei[1] = ed[1] = ed2[1] = alpha[i];
 
-        if (CSeqDBColumn::ColumnExists(m_VolName, ei, m_Atlas, locked)) {
+        if (CSeqDBColumn::ColumnExists(m_VolName, ei, m_Atlas)) {
 
-            bool big   = CSeqDBColumn::ColumnExists(m_VolName, ed, m_Atlas, locked);
-            bool small = CSeqDBColumn::ColumnExists(m_VolName, ed2, m_Atlas, locked);
+            bool big   = CSeqDBColumn::ColumnExists(m_VolName, ed, m_Atlas);
+            bool small = CSeqDBColumn::ColumnExists(m_VolName, ed2, m_Atlas);
 
             if ( ! (big || small)) continue;
 
