@@ -1,4 +1,4 @@
-/*  $Id: writedb_impl.cpp 616350 2020-09-15 12:19:05Z ivanov $
+/*  $Id: writedb_impl.cpp 618513 2020-10-21 16:07:12Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,7 +97,7 @@ CWriteDB_Impl::~CWriteDB_Impl()
     try {
     	Close();
     } catch (const CWriteDBException& e) {
-	 LOG_POST(Error << "BLAST Database creation error: " << e.GetMsg());
+        ERR_POST(Error << "BLAST Database creation error: " << e.GetMsg());
     }
 
 }
