@@ -1,4 +1,4 @@
-/*  $Id: bdbloader.cpp 612733 2020-07-27 11:38:27Z ivanov $
+/*  $Id: bdbloader.cpp 631543 2021-05-19 13:51:16Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -148,16 +148,32 @@ CBlastDbDataLoader::SBlastDbParam::SBlastDbParam(CRef<CSeqDB> db_handle,
 }
 
 static const string kPrefix = "BLASTDB_";
+static const string kPrefixThread = kPrefix + "THREAD";
 string CBlastDbDataLoader::GetLoaderNameFromArgs(const SBlastDbParam& param)
 {
-    return kPrefix + param.m_DbName + DbTypeToStr(param.m_DbType);
+	int t_id = CThread::GetSelf();
+	if (t_id != 0) {
+		string thread_id = kPrefixThread + NStr::IntToString(t_id) + "_";
+		return thread_id + param.m_DbName + DbTypeToStr(param.m_DbType);
+	}
+	else {
+		return kPrefix + param.m_DbName + DbTypeToStr(param.m_DbType);
+	}
 }
 
 string CBlastDbDataLoader::GetLoaderNameFromArgs(CConstRef<CSeqDB> db_handle)
 {
     _ASSERT(db_handle.NotEmpty());
-    return kPrefix + db_handle->GetDBNameList() + 
-        DbTypeToStr(SeqTypeToDbType(db_handle->GetSequenceType()));
+	int t_id = CThread::GetSelf();
+	if (t_id != 0) {
+		string thread_id = kPrefixThread + NStr::IntToString(t_id) + "_";
+		return thread_id + db_handle->GetDBNameList() +
+				DbTypeToStr(SeqTypeToDbType(db_handle->GetSequenceType()));
+	}
+	else {
+		return kPrefix + db_handle->GetDBNameList() +
+				DbTypeToStr(SeqTypeToDbType(db_handle->GetSequenceType()));
+	}
 }
 
 
@@ -196,7 +212,6 @@ typedef pair<int, CSeq_id_Handle> TBlastDbId;
 /// strings.
 template<>
 struct PConvertToString<TBlastDbId>
-    : public unary_function<TBlastDbId, string>
 {
     /// Convert TBlastDbId (blob IDs) to human readable strings.
     /// @param v The value to convert. [in]
@@ -274,7 +289,7 @@ void CBlastDbDataLoader::x_LoadData(const CSeq_id_Handle& idh,
     lock.SetLoaded();
 }
 
-int CBlastDbDataLoader::GetTaxId(const CSeq_id_Handle& idh)
+TTaxId CBlastDbDataLoader::GetTaxId(const CSeq_id_Handle& idh)
 {
     return m_BlastDb->GetTaxId(idh);
 }
