@@ -1,4 +1,4 @@
-/*  $Id: gbnative.cpp 595664 2019-10-24 17:12:18Z vasilche $
+/*  $Id: gbnative.cpp 620489 2020-11-23 18:09:39Z sadyrovr $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 #include <objtools/data_loaders/genbank/gbloader_params.h>
 #include <objtools/data_loaders/genbank/impl/dispatcher.hpp>
 #include <objtools/data_loaders/genbank/impl/request_result.hpp>
+#include <objtools/data_loaders/genbank/impl/wgsmaster.hpp>
 
 #include <objtools/data_loaders/genbank/reader_interface.hpp>
 #include <objtools/data_loaders/genbank/writer_interface.hpp>
@@ -714,18 +715,18 @@ string CGBDataLoader_Native::GetLabel(const CSeq_id_Handle& idh)
 }
 
 
-int CGBDataLoader_Native::GetTaxId(const CSeq_id_Handle& idh)
+TTaxId CGBDataLoader_Native::GetTaxId(const CSeq_id_Handle& idh)
 {
     if ( CReadDispatcher::CannotProcess(idh) ) {
-        return -1;
+        return INVALID_TAX_ID;
     }
     CGBReaderRequestResult result(this, idh);
     CLoadLockTaxId lock(result, idh);
     if ( !lock.IsLoadedTaxId() ) {
         m_Dispatcher->LoadSeq_idTaxId(result, idh);
     }
-    int taxid = lock.IsLoadedTaxId()? lock.GetTaxId(): -1;
-    if ( taxid == -1 ) {
+    TTaxId taxid = lock.IsLoadedTaxId()? lock.GetTaxId(): INVALID_TAX_ID;
+    if ( taxid == INVALID_TAX_ID ) {
         return CDataLoader::GetTaxId(idh);
     }
     return taxid;
@@ -800,7 +801,7 @@ void CGBDataLoader_Native::GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds&
         // we have to fall back to full sequence retrieval in such cases
         bool retry = false;
         for ( size_t i = 0; i < ids.size(); ++i ) {
-            if ( loaded[i] && ret[i] == -1 ) {
+            if ( loaded[i] && ret[i] == INVALID_TAX_ID ) {
                 loaded[i] = false;
                 retry = true;
             }
@@ -1495,7 +1496,7 @@ void CGBDataLoader_Native::GetChunk(TChunk chunk)
 {
     CReader::TChunkId id = chunk->GetChunkId();
     if ( id == kMasterWGS_ChunkId ) {
-        CProcessor::LoadWGSMaster(this, chunk);
+        CWGSMasterSupport::LoadWGSMaster(this, chunk);
     }
     else {
         CGBReaderRequestResult result(this, CSeq_id_Handle());
@@ -1513,7 +1514,7 @@ void CGBDataLoader_Native::GetChunks(const TChunkSet& chunks)
     ITERATE(TChunkSet, it, chunks) {
         CReader::TChunkId id = (*it)->GetChunkId();
         if ( id == kMasterWGS_ChunkId ) {
-            CProcessor::LoadWGSMaster(this, *it);
+            CWGSMasterSupport::LoadWGSMaster(this, *it);
         }
         else {
             chunk_ids[(*it)->GetBlobId()].push_back(id);
