@@ -1,4 +1,4 @@
-/*  $Id: reader_id1.cpp 604647 2020-03-31 14:53:53Z ivanov $
+/*  $Id: reader_id1.cpp 631410 2021-05-18 11:56:35Z ivanov $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -421,13 +421,17 @@ bool CId1Reader::LoadSeq_idBlob_ids(CReaderRequestResult& result,
                 TBlobIds blob_ids;
                 CRef<CBlob_id> blob_id(new CBlob_id);
                 ESat sat = iter->second.first;
-                Int8 sat_key = num;
+                Int4 sat_key = Int4(num);
+                Int4 sub_sat = iter->second.second;
                 if ( sat == eSat_ANNOT_CDD || sat == eSat_ANNOT ) {
-                    sat_key = CProcessor::ConvertGiFromOM(sat_key);
+                    num = GI_TO(Int8, CProcessor::ConvertGiFromOM(GI_FROM(Int8, num)));
+                    // pack gi into sat_key and sub_sat
+                    sat_key = Int4(num);
+                    sub_sat |= Int4((num>>32)<<16);
                 }
                 blob_id->SetSat(sat);
                 blob_id->SetSatKey(sat_key);
-                blob_id->SetSubSat(iter->second.second);
+                blob_id->SetSubSat(sub_sat);
                 blob_ids.push_back(CBlob_Info(blob_id, fBlobHasAllLocal));
                 ids.SetLoadedBlob_ids(CFixedBlob_ids(eTakeOwnership, blob_ids));
                 return true;
@@ -441,7 +445,7 @@ bool CId1Reader::LoadSeq_idBlob_ids(CReaderRequestResult& result,
     }
     TSequenceGi data = gi_lock.GetGi();
     TGi gi = gi_lock.GetGi(data);
-    if ( !gi ) {
+    if ( gi == ZERO_GI ) {
         // no gi -> no Seq-ids
         SetAndSaveNoSeq_idBlob_ids(result, seq_id, sel, gi_lock);
         return true;
@@ -534,16 +538,7 @@ bool CId1Reader::LoadGiBlob_ids(CReaderRequestResult& result,
             blob_ids.push_back(CBlob_Info(blob_id, fBlobHasAllLocal));
         }}
         if ( info.IsSetExtfeatmask() ) {
-            int ext_feat = info.GetExtfeatmask();
-            while ( ext_feat ) {
-                int bit = ext_feat & ~(ext_feat-1);
-                ext_feat -= bit;
-                CRef<CBlob_id> blob_id(new CBlob_id);
-                blob_id->SetSat(GetAnnotSat(bit));
-                blob_id->SetSatKey(CProcessor::ConvertGiFromOM(gi));
-                blob_id->SetSubSat(bit);
-                blob_ids.push_back(CBlob_Info(blob_id, fBlobHasExtAnnot));
-            }
+            CreateExtAnnotBlob_ids(blob_ids, GI_TO(TIntId, CProcessor::ConvertGiFromOM(gi)), info.GetExtfeatmask());
         }
     }
     else {
@@ -697,17 +692,16 @@ void CId1Reader::x_SetBlobRequest(CID1server_request& request,
 void CId1Reader::x_SetParams(CID1server_maxcomplex& params,
                              const CBlob_id& blob_id)
 {
-    int bits = (~blob_id.GetSubSat() & 0xffff) << 4;
-    params.SetMaxplex(eEntry_complexities_entry | bits);
-    params.SetGi(ZERO_GI);
-    params.SetEnt(blob_id.GetSatKey());
-    int sat = blob_id.GetSat();
-    if ( IsAnnotSat(sat) ) {
-        params.SetMaxplex(eEntry_complexities_entry); // TODO: TEMP: remove
-        params.SetSat("ANNOT:"+NStr::IntToString(blob_id.GetSubSat()));
+    if ( IsAnnotSat(blob_id.GetSat()) ) {
+        params.SetMaxplex(eEntry_complexities_entry);
+        params.SetSat("ANNOT:"+NStr::NumericToString(GetExtAnnotSubSat(blob_id)));
+        params.SetGi(GI_FROM(TIntId, GetExtAnnotGi(blob_id)));
     }
     else {
-        params.SetSat(NStr::IntToString(sat));
+        params.SetMaxplex(eEntry_complexities_entry | 0xffff0); // exclude ext annots
+        params.SetSat(NStr::NumericToString(blob_id.GetSat()));
+        params.SetEnt(blob_id.GetSatKey());
+        params.SetGi(ZERO_GI);
     }
 }
 
