@@ -1,4 +1,4 @@
-/*  $Id: processors.cpp 604647 2020-03-31 14:53:53Z ivanov $
+/*  $Id: processors.cpp 631409 2021-05-18 11:56:25Z ivanov $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -38,6 +38,7 @@
 #include <objtools/data_loaders/genbank/impl/dispatcher.hpp>
 #include <objtools/data_loaders/genbank/reader.hpp>
 #include <objtools/data_loaders/genbank/impl/statistics.hpp>
+#include <objtools/data_loaders/genbank/impl/wgsmaster.hpp>
 
 #include <objtools/data_loaders/genbank/reader_snp.hpp>
 #include <objtools/error_codes.hpp>
@@ -431,8 +432,9 @@ bool CProcessor::OffsetId(CSeq_id& id, TIntId gi_offset)
         return false;
     }
     if ( id.IsGi() ) {
-        if ( TGi gi = id.GetGi() ) {
-            id.SetGi(gi + gi_offset);
+        TGi gi = id.GetGi();
+        if ( gi != ZERO_GI ) {
+            id.SetGi(gi + GI_FROM(TIntId, gi_offset));
             return true;
         }
     }
@@ -477,8 +479,9 @@ bool CProcessor::OffsetId(CSeq_id_Handle& id, TIntId gi_offset)
         return false;
     }
     if ( id.IsGi() ) {
-        if ( TGi gi = id.GetGi() ) {
-            id = CSeq_id_Handle::GetGiHandle(gi + gi_offset);
+        TGi gi = id.GetGi();
+        if ( gi != ZERO_GI ) {
+            id = CSeq_id_Handle::GetGiHandle(gi + GI_FROM(TIntId, gi_offset));
             return true;
         }
     }
@@ -605,392 +608,9 @@ bool s_CanBeWGSBlob(const CBlob_id& blob_id)
 }
 
 
-static
-bool s_GoodLetters(CTempString s) {
-    ITERATE ( CTempString, it, s ) {
-        if ( !isalpha(*it & 0xff) ) {
-            return false;
-        }
-    }
-    return true;
-}
-
-
-static
-bool s_GoodDigits(CTempString s) {
-    bool have_non_zero = false;
-    ITERATE ( CTempString, it, s ) {
-        if ( *it != '0' ) {
-            have_non_zero = true;
-            if ( !isdigit(*it & 0xff) ) {
-                return false;
-            }
-        }
-    }
-    return have_non_zero;
-}
-
-
-static
-CSeq_id_Handle s_GetWGSMasterSeq_id(const CSeq_id_Handle& idh)
-{
-    CSeq_id_Handle master_idh;
-
-    switch ( idh.Which() ) { // shortcut to exclude all non Textseq-id types
-    case CSeq_id::e_not_set:
-    case CSeq_id::e_Local:
-    case CSeq_id::e_Gi:
-    case CSeq_id::e_Gibbsq:
-    case CSeq_id::e_Gibbmt:
-    case CSeq_id::e_Giim:
-    case CSeq_id::e_Patent:
-    case CSeq_id::e_General:
-    case CSeq_id::e_Pdb:
-        return master_idh;
-    default:
-        break;
-    }
-
-    CConstRef<CSeq_id> id = idh.GetSeqId();
-    const CTextseq_id* text_id = id->GetTextseq_Id();
-    if ( !text_id || !text_id->IsSetAccession() ) {
-        return master_idh;
-    }
-
-    CTempString acc = text_id->GetAccession();
-
-    CSeq_id::EAccessionInfo type = CSeq_id::IdentifyAccession(acc);
-    bool is_cage_ddbj = false;
-    switch ( type & CSeq_id::eAcc_division_mask ) {
-        // accepted accession types
-    case CSeq_id::eAcc_mga: // 2019/02/08 : For now, it's just CAGE DDBJ
-        is_cage_ddbj = true;
-    case CSeq_id::eAcc_wgs:
-    case CSeq_id::eAcc_wgs_intermed:
-    case CSeq_id::eAcc_tsa:
-        break;
-    default:
-        return master_idh;
-    }
-
-    SIZE_TYPE digits_pos = acc.find_first_of("0123456789");
-    bool have_nz = NStr::StartsWith(acc, "NZ_");
-    SIZE_TYPE letters_pos = (have_nz ? 3 : 0);
-
-    // First check the prefix and suffix lengths.
-    // WGS/TSA/TLS prefixes have 4 or 6 letters; CAGE DDBJ prefixes have 5 letters
-    // WGS/TSA/TLS suffixes have 6-8 or 7-9 digits; CAGE DDBJ suffixes have 7 digits 
-    SIZE_TYPE min_digits = 0;
-    SIZE_TYPE max_digits = 0;
-
-    if (is_cage_ddbj) {
-        if (digits_pos != 5)
-            return master_idh;
-        min_digits = 7;
-        max_digits = 7;
-    } else {
-        if (digits_pos != letters_pos+4 && digits_pos != letters_pos+6)
-            return master_idh;
-        min_digits = ((digits_pos == letters_pos+4) ? 6 : 7);
-        max_digits = min_digits + 2;
-    }
-
-    SIZE_TYPE digits_count = acc.size() - digits_pos;
-    if (digits_count < min_digits || digits_count > max_digits)
-        return master_idh;
-
-    // Check that prefix and suffix actually consist of letters and digits respectively.
-    if ( !s_GoodLetters(acc.substr(letters_pos, digits_pos-letters_pos)) ) {
-        return master_idh;
-    }
-    if ( !s_GoodDigits(acc.substr(digits_pos)) ) {
-        return master_idh;
-    }
-
-    // Exclude master accessions
-    // Non-CAGE-DDBJ master accessions may also contain a 2-digit version 
-    int version = 0;
-    Uint8 row_id = 0;
-    if (is_cage_ddbj) {
-        version = 1;
-        row_id = NStr::StringToNumeric<Uint8>(acc.substr(digits_pos));
-    } else {
-        version = NStr::StringToNumeric<int>(acc.substr(digits_pos, 2));
-        row_id = NStr::StringToNumeric<Uint8>(acc.substr(digits_pos+2));
-    }
-    if ( !version || !row_id ) {
-        return master_idh;
-    }
-
-    CSeq_id master_id;
-    master_id.Assign(*id);
-    CTextseq_id* master_text_id =
-        const_cast<CTextseq_id*>(master_id.GetTextseq_Id());
-    string master_acc = acc.substr(0, digits_pos);
-    master_acc.resize(acc.size(), '0');
-    master_text_id->Reset();
-    master_text_id->SetAccession(master_acc);
-    master_text_id->SetVersion(version);
-    master_idh = CSeq_id_Handle::GetHandle(master_id);
-    return master_idh;
-}
-
-
-static const int kForceDescrMask = ((1<<CSeqdesc::e_Pub) |
-                                    (1<<CSeqdesc::e_Comment) |
-                                    (1<<CSeqdesc::e_User));
-
-static const int kOptionalDescrMask = ((1<<CSeqdesc::e_Source) |
-                                       (1<<CSeqdesc::e_Molinfo) |
-                                       (1<<CSeqdesc::e_Create_date) |
-                                       (1<<CSeqdesc::e_Update_date) |
-                                       (1<<CSeqdesc::e_Genbank) |
-                                       (1<<CSeqdesc::e_Embl));
-
-static const int kGoodDescrMask = kForceDescrMask | kOptionalDescrMask;
-
-
-static
-bool s_IsGoodDescr(const CSeqdesc& desc, int mask, const TUserObjectTypesSet& uo_types)
-{
-    if ( desc.Which() == CSeqdesc::e_User ) {
-        const CObject_id& type = desc.GetUser().GetType();
-        if ( type.Which() == CObject_id::e_Str ) {
-            string name = type.GetStr();
-            // Only a few user object types are eligible to be taken from master
-            if ( name == "DBLink" ||
-                 name == "GenomeProjectsDB" ||
-                 name == "StructuredComment" ||
-                 name == "FeatureFetchPolicy" ) {
-                // For StructuredComment, extract the comment prefix and add to the name
-                if (name == "StructuredComment") {
-                    // This loop should normally stop on the first iteration...
-                    ITERATE (CUser_object::TData, it, desc.GetUser().GetData()) {
-                        if ((*it)->GetLabel().IsStr() &&
-                            (*it)->GetLabel().GetStr() == "StructuredCommentPrefix") {
-                            string data = ((*it)->GetData().IsStr() ?
-                                           (string) (*it)->GetData().GetStr() :
-                                           NStr::IntToString((*it)->GetData().GetInt()));
-                            name += "|" + data;
-                            break;
-                        }
-                    }
-                }
-                // Check if this user object type should be skipped because it already exists
-                if (uo_types.count(name) == 0)
-                    return true;
-            }
-        }
-    }
-    else if ( (1 << desc.Which()) & mask ) {
-        return true;
-    }
-    return false;
-}
-
-
-static
-void s_AddMasterDescr(CBioseq_Info& seq, const CSeq_descr& src)
-{
-    int existing_mask = 0;
-    CSeq_descr::Tdata& dst = seq.x_SetDescr().Set();
-    ITERATE ( CSeq_descr::Tdata, it, dst ) {
-        const CSeqdesc& desc = **it;
-        existing_mask |= 1 << desc.Which();
-    }
-    ITERATE ( CSeq_descr::Tdata, it, src.Get() ) {
-        int mask = 1 << (*it)->Which();
-        if ( mask & kOptionalDescrMask ) {
-            if ( mask & existing_mask ) {
-                continue;
-            }
-        }
-        else if ( !(mask & kForceDescrMask) ) {
-            continue;
-        }
-        dst.push_back(*it);
-    }
-}
-
-
-static
-CRef<CSeq_descr> s_GetWGSMasterDescr(CDataLoader* loader,
-                                     const CSeq_id_Handle& master_idh,
-                                     int mask, TUserObjectTypesSet& uo_types)
-{
-    CRef<CSeq_descr> ret;
-    CDataLoader::TTSE_LockSet locks =
-        loader->GetRecordsNoBlobState(master_idh, CDataLoader::eBioseqCore);
-    ITERATE ( CDataLoader::TTSE_LockSet, it, locks ) {
-        CConstRef<CBioseq_Info> bs_info =
-            (*it)->FindMatchingBioseq(master_idh);
-        if ( !bs_info ) {
-            continue;
-        }
-        if ( bs_info->IsSetDescr() ) {
-            const CSeq_descr::Tdata& descr = bs_info->GetDescr().Get();
-            ITERATE ( CSeq_descr::Tdata, it, descr ) {
-                if ( s_IsGoodDescr(**it, mask, uo_types) ) {
-                    if ( !ret ) {
-                        ret = new CSeq_descr;
-                    }
-                    ret->Set().push_back(*it);
-                }
-            }
-        }
-        break;
-    }
-    return ret;
-}
-
-
-class CWGSBioseqUpdater_Base : public CBioseqUpdater
-{
-public:
-    CWGSBioseqUpdater_Base(const CSeq_id_Handle& master_idh)
-        : m_MasterId(master_idh)
-        {
-        }
-
-    bool HasMasterId(const CBioseq_Info& seq) const {
-        if ( m_MasterId ) {
-            const CBioseq_Info::TId& ids = seq.GetId();
-            ITERATE ( CBioseq_Info::TId, it, ids ) {
-                if ( s_GetWGSMasterSeq_id(*it) == m_MasterId ) {
-                    return true;
-                }
-            }
-        }
-        return false;
-    }
-    
-private:
-    CSeq_id_Handle m_MasterId;
-};
-
-
-class CWGSBioseqUpdaterChunk : public CWGSBioseqUpdater_Base
-{
-public:
-    CWGSBioseqUpdaterChunk(const CSeq_id_Handle& master_idh)
-        : CWGSBioseqUpdater_Base(master_idh)
-        {
-        }
-
-    virtual void Update(CBioseq_Info& seq) {
-        if ( HasMasterId(seq) ) {
-            // register master descr chunk
-            seq.x_AddDescrChunkId(kGoodDescrMask, kMasterWGS_ChunkId);
-        }
-    }
-};
-
-
-class CWGSBioseqUpdaterDescr : public CWGSBioseqUpdater_Base
-{
-public:
-    CWGSBioseqUpdaterDescr(const CSeq_id_Handle& master_idh,
-                           CRef<CSeq_descr> descr)
-        : CWGSBioseqUpdater_Base(master_idh),
-          m_Descr(descr)
-        {
-        }
-
-    virtual void Update(CBioseq_Info& seq) {
-        if ( m_Descr &&
-             seq.x_NeedUpdate(seq.fNeedUpdate_descr) &&
-             HasMasterId(seq) ) {
-            s_AddMasterDescr(seq, *m_Descr);
-        }
-    }
-
-private:
-    CRef<CSeq_descr> m_Descr;
-};
-
-
-class CWGSMasterChunkInfo : public CTSE_Chunk_Info
-{
-public:
-    CWGSMasterChunkInfo(const CSeq_id_Handle& master_idh,
-                        int mask, TUserObjectTypesSet& uo_types
-)
-        : CTSE_Chunk_Info(kMasterWGS_ChunkId),
-          m_MasterId(master_idh),
-          m_DescrMask(mask),
-          m_UserObjectTypes(move(uo_types))
-        {
-        }
-
-    CSeq_id_Handle m_MasterId;
-    int m_DescrMask;
-    TUserObjectTypesSet m_UserObjectTypes;
-};
-
-
 END_LOCAL_NAMESPACE;
 
 
-const bool kAddMasterDescrToTSE = true;
-
-
-void CProcessor::LoadWGSMaster(CDataLoader* loader,
-                               CRef<CTSE_Chunk_Info> chunk)
-{
-    CWGSMasterChunkInfo& chunk_info =
-        dynamic_cast<CWGSMasterChunkInfo&>(*chunk);
-    CSeq_id_Handle id = chunk_info.m_MasterId;
-    int mask = chunk_info.m_DescrMask;
-    CRef<CSeq_descr> descr =
-        s_GetWGSMasterDescr(loader, id, mask, chunk_info.m_UserObjectTypes);
-    if ( descr ) {
-        if ( kAddMasterDescrToTSE ) {
-            chunk->x_LoadDescr(CTSE_Chunk_Info::TPlace(), *descr);
-        }
-        else {
-            CRef<CBioseqUpdater> upd(new CWGSBioseqUpdaterDescr(id, descr));
-            const_cast<CTSE_Split_Info&>(chunk->GetSplitInfo()).x_SetBioseqUpdater(upd);
-        }
-    }
-    chunk->SetLoaded();
-}
-
-
-void CProcessor::AddWGSMaster(CLoadLockSetter& blob)
-{
-    CTSE_Info::TSeqIds ids;
-    CTSE_LoadLock& lock = blob.GetTSE_LoadLock();
-    lock->GetBioseqsIds(ids);
-    ITERATE ( CTSE_Info::TSeqIds, it, ids ) {
-        if ( CSeq_id_Handle id = s_GetWGSMasterSeq_id(*it) ) {
-            int mask = kGoodDescrMask;
-            TUserObjectTypesSet existing_uo_types;
-            if ( kAddMasterDescrToTSE ) {
-                // exclude existing descr types except forced ones (User, Pub, Comment)
-                mask &= ~lock->x_GetBaseInfo().x_GetExistingDescrMask() | kForceDescrMask;
-                lock->x_GetBaseInfo().x_AddExistingUserObjectTypes(existing_uo_types);
-                if ( lock->IsSet() ) {
-                    if ( auto first_entry = lock->GetSet().GetFirstEntry() ) {
-                        mask &= ~first_entry->x_GetBaseInfo().x_GetExistingDescrMask() | kForceDescrMask;
-                        first_entry->x_GetBaseInfo().x_AddExistingUserObjectTypes(existing_uo_types);
-                    }
-                }
-            }
-            CRef<CTSE_Chunk_Info> chunk(new CWGSMasterChunkInfo(id, mask, existing_uo_types));
-            lock->GetSplitInfo().AddChunk(*chunk);
-            if ( kAddMasterDescrToTSE ) {
-                chunk->x_AddDescInfo(mask, 0);
-            }
-            else {
-                CRef<CBioseqUpdater> upd(new CWGSBioseqUpdaterChunk(id));
-                lock->SetBioseqUpdater(upd);
-            }
-            break;
-        }
-    }
-}
-
-
 namespace {
     inline
     CRef<CWriter::CBlobStream> OpenStream(CWriter* writer,
@@ -1399,7 +1019,7 @@ void CProcessor_SE::ProcessObjStream(CReaderRequestResult& result,
         if ( chunk_id == kMain_ChunkId &&
              s_CanBeWGSBlob(blob_id) &&
              result.GetAddWGSMasterDescr() ) {
-            AddWGSMaster(setter);
+            CWGSMasterSupport::AddWGSMaster(setter.GetTSE_LoadLock());
         }
         setter.SetLoaded();
 
@@ -1932,7 +1552,7 @@ void CProcessor_ID2::ProcessData(CReaderRequestResult& result,
             setter.SetSeq_entry(*entry);
             if ( s_CanBeWGSBlob(blob_id) &&
                  result.GetAddWGSMasterDescr() ) {
-                AddWGSMaster(setter);
+                CWGSMasterSupport::AddWGSMaster(setter.GetTSE_LoadLock());
             }
             setter.SetLoaded();
         }
@@ -2007,7 +1627,7 @@ void CProcessor_ID2::ProcessData(CReaderRequestResult& result,
             CSplitParser::Attach(*lock, *split_info);
             if ( s_CanBeWGSBlob(blob_id) &&
                  result.GetAddWGSMasterDescr() ) {
-                AddWGSMaster(setter);
+                CWGSMasterSupport::AddWGSMaster(setter.GetTSE_LoadLock());
             }
             setter.SetLoaded();
         }
@@ -2600,11 +2220,11 @@ void CProcessor_ExtAnnot::Process(CReaderRequestResult& result,
         setter.GetTSE_LoadLock()->SetName(name);
     }
 
-    TGi gi = ConvertGiToOM(blob_id.GetSatKey());
+    TGi gi = ConvertGiToOM(GI_FROM(Uint4, Uint4(blob_id.GetSatKey())));
     CSeq_id_Handle gih = CSeq_id_Handle::GetGiHandle(gi);
     CSeq_id seq_id;
     seq_id.SetGeneral().SetDb(db_name);
-    seq_id.SetGeneral().SetTag().SetId8(gi);
+    seq_id.SetGeneral().SetTag().SetId8(GI_TO(Int8, gi));
     CSeq_id_Handle seh = CSeq_id_Handle::GetHandle(seq_id);
     
     CRef<CTSE_Chunk_Info> chunk(new CTSE_Chunk_Info(kDelayedMain_ChunkId));
