@@ -1,4 +1,4 @@
-/*  $Id: feattable_edit.cpp 612522 2020-07-23 11:23:26Z ivanov $
+/*  $Id: feattable_edit.cpp 632623 2021-06-03 17:38:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -26,7 +26,7 @@
 * Author: Frank Ludwig, NCBI
 *
 * File Description:
-*   Convenience wrapper around some of the other feature processing finctions
+*   Convenience wrapper around some of the other feature processing functions
 *   in the xobjedit library.
 */
 
@@ -117,12 +117,37 @@ CFeatTableEdit::CFeatTableEdit(
     unsigned int locusTagNumber,
     unsigned int startingFeatId,
     IObjtoolsListener* pMessageListener) :
-    //  -------------------------------------------------------------------------
+//  -------------------------------------------------------------------------
     mAnnot(annot),
+    mSequenceSize(0),
     mpMessageListener(pMessageListener),
-    mNextFeatId(startingFeatId),
+    mLocusTagPrefix(locusTagPrefix),
     mLocusTagNumber(locusTagNumber),
-    mLocusTagPrefix(locusTagPrefix)
+    mNextFeatId(startingFeatId)
+{
+    mpScope.Reset(new CScope(*CObjectManager::GetInstance()));
+    mpScope->AddDefaults();
+    mHandle = mpScope->AddSeq_annot(mAnnot);
+    mEditHandle = mpScope->GetEditHandle(mHandle);
+    mTree = feature::CFeatTree(mHandle);
+};
+
+
+//  -------------------------------------------------------------------------
+CFeatTableEdit::CFeatTableEdit(
+    CSeq_annot& annot,
+    unsigned int sequenceSize,
+    const string& locusTagPrefix,
+    unsigned int locusTagNumber,
+    unsigned int startingFeatId,
+    IObjtoolsListener* pMessageListener) :
+//  -------------------------------------------------------------------------
+    mAnnot(annot),
+    mSequenceSize(sequenceSize),
+    mpMessageListener(pMessageListener),
+    mLocusTagPrefix(locusTagPrefix),
+    mLocusTagNumber(locusTagNumber),
+    mNextFeatId(startingFeatId)
 {
     mpScope.Reset(new CScope(*CObjectManager::GetInstance()));
     mpScope->AddDefaults();
@@ -584,7 +609,7 @@ void CFeatTableEdit::xAddTranscriptAndProteinIdsToMrna(const string& cds_transcr
         return;
     }
 
-    bool use_local_ids = false; // 
+    bool use_local_ids = false;
 
     string transcript_id = mrna.GetNamedQual("transcript_id");
     if (NStr::IsBlank(transcript_id)) {
@@ -1481,10 +1506,10 @@ string CFeatTableEdit::xNextLocusTag()
     return nextTag;
 }
 
-//	----------------------------------------------------------------------------
+//  ----------------------------------------------------------------------------
 string CFeatTableEdit::xNextProteinId(
     const CMappedFeat& mf)
-    //	----------------------------------------------------------------------------
+//  ----------------------------------------------------------------------------
 {
     const string dbPrefix("gnl|");
 
@@ -1521,10 +1546,10 @@ string CFeatTableEdit::xNextProteinId(
     return proteinId;
 }
 
-//	----------------------------------------------------------------------------
+//  ----------------------------------------------------------------------------
 string CFeatTableEdit::xNextTranscriptId(
     const CMappedFeat& cds)
-    //	----------------------------------------------------------------------------
+//  ----------------------------------------------------------------------------
 {
     const string dbPrefix("gnl|");
 
@@ -1562,10 +1587,63 @@ string CFeatTableEdit::xNextTranscriptId(
     return transcriptId;
 }
 
+//  ----------------------------------------------------------------------------
+CRef<CSeq_loc> CFeatTableEdit::xGetGeneLocation(
+    const CSeq_loc& baseLoc)
+//  ----------------------------------------------------------------------------
+{
+    CRef<CSeq_loc> pEnvelope(new CSeq_loc);
+
+    auto baseStart = baseLoc.GetStart(eExtreme_Positional);
+    auto baseStop = baseLoc.GetStop(eExtreme_Positional);
+    auto& baseId = *baseLoc.GetId();
+
+    if (mSequenceSize == 0  ||  baseStart <= baseStop) {
+        pEnvelope->SetInt();
+        pEnvelope->SetId(baseId);
+        pEnvelope->SetInt().SetFrom(baseStart);
+        pEnvelope->SetInt().SetTo(baseStop);
+        pEnvelope->SetInt().SetStrand(baseLoc.GetStrand());
+    }
+    else {
+        if (baseLoc.GetStrand() != eNa_strand_minus) {
+            CRef<CSeq_interval> pTop(new CSeq_interval);
+            pTop->SetId().Assign(baseId);
+            pTop->SetFrom(baseStart);
+            pTop->SetTo(mSequenceSize);
+            pTop->SetStrand(baseLoc.GetStrand());
+            pEnvelope->SetPacked_int().AddInterval(*pTop);
+            CRef<CSeq_interval> pBottom(new CSeq_interval);
+            pBottom->SetId().Assign(baseId);
+            pBottom->SetFrom(0);
+            pBottom->SetTo(baseStop);
+            pBottom->SetStrand(baseLoc.GetStrand());
+            pEnvelope->SetPacked_int().AddInterval(*pBottom);
+            pEnvelope->ChangeToMix();
+        }
+        else {
+            CRef<CSeq_interval> pBottom(new CSeq_interval);
+            pBottom->SetId().Assign(baseId);
+            pBottom->SetFrom(0);
+            pBottom->SetTo(baseStop);
+            pBottom->SetStrand(baseLoc.GetStrand());
+            pEnvelope->SetPacked_int().AddInterval(*pBottom);
+            CRef<CSeq_interval> pTop(new CSeq_interval);
+            pTop->SetId().Assign(baseId);
+            pTop->SetFrom(baseStart);
+            pTop->SetTo(mSequenceSize);
+            pTop->SetStrand(baseLoc.GetStrand());
+            pEnvelope->SetPacked_int().AddInterval(*pTop);
+            pEnvelope->ChangeToMix();
+        }
+    }
+    return pEnvelope;
+}
+
 //  ----------------------------------------------------------------------------
 CRef<CSeq_feat> CFeatTableEdit::xMakeGeneForFeature(
     const CMappedFeat& rna)
-    //  ----------------------------------------------------------------------------
+//  ----------------------------------------------------------------------------
 {
     CRef<CSeq_feat> pGene;
     //const CSeq_loc& loc = rna.GetOriginalFeature().GetLocation();
@@ -1579,13 +1657,7 @@ CRef<CSeq_feat> CFeatTableEdit::xMakeGeneForFeature(
         return pGene;
     }
     pGene.Reset(new CSeq_feat);
-    pGene->SetLocation().SetInt();
-    pGene->SetLocation().SetId(*rna.GetLocation().GetId());
-    pGene->SetLocation().SetInt().SetFrom(rna.GetLocation().GetStart(
-        eExtreme_Positional));
-    pGene->SetLocation().SetInt().SetTo(rna.GetLocation().GetStop(
-        eExtreme_Positional));
-    pGene->SetLocation().SetInt().SetStrand(rna.GetLocation().GetStrand());
+    pGene->SetLocation(*xGetGeneLocation(rna.GetLocation()));
     pGene->SetData().SetGene();
     return pGene;
 }
@@ -1830,7 +1902,7 @@ static void s_AppendProtRefInfo(CProt_ref& current_ref, const CProt_ref& other_r
     }
 
     if (other_ref.IsSetDb()) {
-        for (const auto pDBtag : other_ref.GetDb()) {
+        for (const auto& pDBtag : other_ref.GetDb()) {
             current_ref.SetDb().push_back(pDBtag);
         }
     }
@@ -1874,7 +1946,7 @@ static void s_SetProtRef(const CSeq_feat& cds,
                 }
             }
         }
-    } // prot_ref.IsSetName() 
+    } // prot_ref.IsSetName()
 }
 
 static bool AssignLocalIdIfEmpty(ncbi::objects::CSeq_feat& feature, unsigned& id)
@@ -2084,6 +2156,104 @@ void CFeatTableEdit::xGenerate_mRNA_Product(CSeq_feat& cd_feature)
 #endif
 }
 
+//  ============================================================================
+string
+sGetFeatMapKey(const CObject_id& objectId)
+//  ============================================================================
+{
+    if (objectId.IsStr()) {
+        return objectId.GetStr();
+    }
+    return string("id:") + NStr::NumericToString(objectId.GetId());
+}
+
+//  ============================================================================
+void
+CFeatTableEdit::MergeFeatures(
+    const CSeq_annot::TData::TFtable& otherFeatures)
+//  ============================================================================
+{
+    FeatMap featMap;
+
+    auto& thisFeatures = mAnnot.SetData().SetFtable();
+    for (auto pThisFeat: thisFeatures) {
+    //(assumption: only local IDs are at issue)
+        if (!pThisFeat->IsSetId()  ||  !pThisFeat->GetId().IsLocal()) {
+            continue;
+        }
+        const auto& this_oid = pThisFeat->GetId().GetLocal();
+        featMap[sGetFeatMapKey(this_oid)] = pThisFeat;
+    }
+    for (auto pOtherFeat: otherFeatures) {
+        if (!pOtherFeat->IsSetId()  ||  !pOtherFeat->GetId().IsLocal()) {
+            thisFeatures.push_back(pOtherFeat);
+            continue;
+        }
+        const auto& other_oid = pOtherFeat->GetId().GetLocal();
+        if (featMap.find(sGetFeatMapKey(other_oid)) == featMap.end()) {
+            thisFeatures.push_back(pOtherFeat);
+            continue;
+        }
+        xRenameFeatureId(other_oid, featMap);
+        thisFeatures.push_back(pOtherFeat);
+    }
+}
+
+//  ==============================================================================
+bool OjectIdsAreEqual (
+//  ==============================================================================
+    const CObject_id& lhs,
+    const CObject_id& rhs)
+{
+    if (lhs.Which() != rhs.Which()) {
+        return false;
+    }
+    if (lhs.IsStr()) {
+        return (lhs.GetStr() == rhs.GetStr());
+    }
+    return (lhs.GetId() == rhs.GetId());
+};
+
+//  =============================================================================
+void
+CFeatTableEdit::xRenameFeatureId(
+    const CObject_id& oldFeatId,
+    FeatMap& featMap)
+//  =============================================================================
+{
+    string oldFeatIdAsStr;
+    if (oldFeatId.IsStr()) {
+        oldFeatIdAsStr = oldFeatId.GetStr();
+    }
+    else {
+        oldFeatIdAsStr = NStr::NumericToString(oldFeatId.GetId());
+    }
+
+    string newFeatIdAsStr = oldFeatIdAsStr + "x";
+    CRef<CObject_id> pNewFeatId(new CObject_id);
+    pNewFeatId->SetStr(oldFeatIdAsStr + "x");
+    while (featMap.find(sGetFeatMapKey(*pNewFeatId)) != featMap.end()) {
+        pNewFeatId->SetStr() += "x";
+    }
+
+    auto pBaseFeat = featMap[sGetFeatMapKey(oldFeatId)];
+    pBaseFeat->SetId().SetLocal(*pNewFeatId);
+    featMap.erase(sGetFeatMapKey(oldFeatId));
+    featMap[sGetFeatMapKey(*pNewFeatId)] = pBaseFeat;
+
+    for (auto it = featMap.begin(); it != featMap.end(); ++it) {
+        auto pFeat = it->second;
+        for (auto& pXref: pFeat->SetXref()) {
+            if (!pXref->IsSetId()  ||  !pXref->GetId().IsLocal()) {
+                continue;
+            }
+            if (OjectIdsAreEqual(pXref->GetId().GetLocal(), oldFeatId)) {
+                pXref->SetId().SetLocal(*pNewFeatId);
+            }
+        }
+    }
+}
+
 END_SCOPE(edit)
 END_SCOPE(objects)
 END_NCBI_SCOPE
