@@ -1,4 +1,4 @@
-/*  $Id: comment_item.cpp 588215 2019-06-19 14:08:16Z dondosha $
+/*  $Id: comment_item.cpp 629259 2021-04-13 13:28:40Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -348,7 +348,7 @@ string CCommentItem::GetStringForBankIt(const CUser_object& uo, bool dump_mode)
     }
 
     CNcbiOstrstream text;
-    string pfx = "";
+    string pfx;
     if ( uvc != 0 ) {
         text << pfx << "Vector Explanation: " << *uvc;
         pfx = "~";
@@ -1203,7 +1203,7 @@ string CCommentItem::GetStringForModelEvidance(const CBioseqContext& ctx, const
         }
         text << " (";
         int count = 0;
-        string prefix = "";
+        string prefix;
         FOR_EACH_STRING_IN_LIST (str, me.assembly) {
 #ifdef NEW_HTML_FMT
             string tr_name;
@@ -1608,11 +1608,6 @@ string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_s
                << data_str
                << "\">" << data_str << "</a>";
         return CNcbiOstrstreamToString(result);
-    } else if ( NStr::Equal (label_str, "Source Identifier") && NStr::Equal (source, "JCVI") ) {
-        result << "<a href=\"http://tigrfams.jcvi.org/cgi-bin/HmmReportPage.cgi?acc="
-               << data_str
-               << "\">" << data_str << "</a>";
-        return CNcbiOstrstreamToString(result);
     } else if ( NStr::Equal (label_str, "Evidence Accession") && NStr::Equal (source, "NCBI SPARCLE") ) {
         string fst;
         string snd;
