@@ -1,4 +1,4 @@
-/*  $Id: defline_item.cpp 613774 2020-08-12 16:32:22Z ivanov $
+/*  $Id: defline_item.cpp 621839 2020-12-17 13:42:42Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -85,7 +85,7 @@ void CDeflineItem::x_GatherInfo(CBioseqContext& ctx)
     CConstRef<CBioseq> bioseq = ctx.GetHandle().GetBioseqCore();
     CScope& scope = ctx.GetScope();
     sequence::CDeflineGenerator::TUserFlags flags = 0;
-    if ( ctx.Config().UseAutoDef() ) {
+    if ( ! ctx.Config().DoNotUseAutoDef() ) {
         flags |= sequence::CDeflineGenerator::fUseAutoDef;
     }
     if ( ctx.Config().IgnoreExistingTitle() ) {
