@@ -1,4 +1,4 @@
-/*  $Id: psg_client.cpp 612393 2020-07-21 13:51:24Z ivanov $
+/*  $Id: psg_client.cpp 628921 2021-04-07 18:46:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,10 +41,15 @@
 #include <type_traits>
 
 #include <corelib/ncbitime.hpp>
+#include <corelib/ncbi_base64.h>
 #include <connect/ncbi_socket.hpp>
 #include <connect/ncbi_service.h>
 #include <connect/ncbi_connutil.h>
 
+#include <serial/serial.hpp>
+#include <serial/objistrasnb.hpp>
+#undef ThrowError // unfortunately
+
 #include "psg_client_impl.hpp"
 
 BEGIN_NCBI_SCOPE
@@ -163,8 +168,96 @@ void SPSG_BlobReader::CheckForNewChunks()
 
 
 static_assert(is_nothrow_move_constructible<CPSG_BioId>::value, "CPSG_BioId move constructor must be noexcept");
+
+string CPSG_BioId::Repr() const
+{
+    return m_Type == TType::e_not_set ? m_Id : m_Id + '~' + to_string(m_Type);
+}
+
+CPSG_BioId s_GetBioId(const CJsonNode& data)
+{
+    auto type = static_cast<CPSG_BioId::TType>(data.GetInteger("seq_id_type"));
+    auto accession = data.GetString("accession");
+    auto name_node = data.GetByKeyOrNull("name");
+    auto name = name_node && name_node.IsString() ? name_node.AsString() : string();
+    auto version = static_cast<int>(data.GetInteger("version"));
+    return { objects::CSeq_id(type, accession, name, version).AsFastaString(), type };
+};
+
+ostream& operator<<(ostream& os, const CPSG_BioId& bio_id)
+{
+    if (bio_id.GetType()) os << "seq_id_type=" << bio_id.GetType() << '&';
+    return os << "seq_id=" << bio_id.GetId();
+}
+
+
 static_assert(is_nothrow_move_constructible<CPSG_BlobId>::value, "CPSG_BlobId move constructor must be noexcept");
 
+string CPSG_BlobId::Repr() const
+{
+    return m_LastModified.IsNull() ? m_Id : m_Id + '~' + to_string(m_LastModified.GetValue());
+}
+
+unique_ptr<CPSG_DataId> s_GetDataId(const SPSG_Args& args)
+{
+    try {
+        const auto& blob_id = args.GetValue("blob_id");
+
+        if (blob_id.empty()) {
+            auto id2_chunk = NStr::StringToNumeric<Uint8>(args.GetValue("id2_chunk"));
+            return unique_ptr<CPSG_DataId>(new CPSG_ChunkId(id2_chunk, args.GetValue("id2_info")));
+        }
+
+        CPSG_BlobId::TLastModified last_modified;
+        const auto& last_modified_str = args.GetValue("last_modified");
+
+        if (!last_modified_str.empty()) {
+            last_modified = NStr::StringToNumeric<Int8>(last_modified_str);
+        }
+
+        return unique_ptr<CPSG_DataId>(new CPSG_BlobId(blob_id, move(last_modified)));
+    }
+    catch (...) {
+        NCBI_THROW_FMT(CPSG_Exception, eServerError,
+                "Both blob_id[+last_modified] and id2_chunk+id2_info pairs are missing/corrupted in server response: " <<
+                args.GetQueryString(CUrlArgs::eAmp_Char));
+    }
+}
+
+CPSG_BlobId s_GetBlobId(const CJsonNode& data)
+{
+    CPSG_BlobId::TLastModified last_modified;
+
+    if (data.HasKey("last_modified")) {
+        last_modified = data.GetInteger("last_modified");
+    }
+
+    if (data.HasKey("blob_id")) {
+        return { data.GetString("blob_id"), last_modified };
+    }
+
+    auto sat = static_cast<int>(data.GetInteger("sat"));
+    auto sat_key = static_cast<int>(data.GetInteger("sat_key"));
+    return { sat, sat_key, last_modified };
+}
+
+ostream& operator<<(ostream& os, const CPSG_BlobId& blob_id)
+{
+    if (!blob_id.GetLastModified().IsNull()) os << "last_modified=" << blob_id.GetLastModified().GetValue() << '&';
+    return os << "blob_id=" << blob_id.GetId();
+}
+
+
+string CPSG_ChunkId::Repr() const
+{
+    return to_string(m_Id2Chunk) + '~' + m_Id2Info;
+}
+
+ostream& operator<<(ostream& os, const CPSG_ChunkId& chunk_id)
+{
+    return os << "id2_chunk=" << chunk_id.GetId2Chunk() << "&id2_info=" << chunk_id.GetId2Info();
+}
+
 
 template <class TReplyItem>
 TReplyItem* CPSG_Reply::SImpl::CreateImpl(TReplyItem* item, const vector<SPSG_Chunk>& chunks)
@@ -172,11 +265,73 @@ TReplyItem* CPSG_Reply::SImpl::CreateImpl(TReplyItem* item, const vector<SPSG_Ch
     if (chunks.empty()) return item;
 
     unique_ptr<TReplyItem> rv(item);
-    rv->m_Data = CJsonNode::ParseJSON(chunks.front());
+    rv->m_Data = CJsonNode::ParseJSON(chunks.front(), CJsonNode::fStandardJson);
 
     return rv.release();
 }
 
+struct SItemTypeAndReason : pair<CPSG_ReplyItem::EType, CPSG_SkippedBlob::EReason>
+{
+    static SItemTypeAndReason Get(const SPSG_Args& args);
+
+private:
+    using TBase = pair<CPSG_ReplyItem::EType, CPSG_SkippedBlob::EReason>;
+    using TBase::TBase;
+    SItemTypeAndReason(CPSG_ReplyItem::EType type) : TBase(type, CPSG_SkippedBlob::eUnknown) {}
+};
+
+SItemTypeAndReason SItemTypeAndReason::Get(const SPSG_Args& args)
+{
+    const auto item_type = args.GetValue("item_type");
+
+    if (item_type == "blob") {
+        const auto reason = args.GetValue("reason");
+
+        if (reason.empty()) {
+            return CPSG_ReplyItem::eBlobData;
+
+        } else if (reason == "excluded") {
+            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eExcluded };
+
+        } else if (reason == "inprogress") {
+            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eInProgress };
+
+        } else if (reason == "sent") {
+            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eSent };
+
+        } else {
+            return { CPSG_ReplyItem::eSkippedBlob, CPSG_SkippedBlob::eUnknown };
+        }
+
+    } else if (item_type == "bioseq_info") {
+        return CPSG_ReplyItem::eBioseqInfo;
+
+    } else if (item_type == "blob_prop") {
+        return CPSG_ReplyItem::eBlobInfo;
+
+    } else if (item_type == "bioseq_na") {
+        return CPSG_ReplyItem::eNamedAnnotInfo;
+
+    } else if (item_type == "public_comment") {
+        return CPSG_ReplyItem::ePublicComment;
+
+    } else if (item_type == "processor") {
+        return CPSG_ReplyItem::eProcessor;
+
+    } else {
+        if (TPSG_FailOnUnknownItems::GetDefault()) {
+            NCBI_THROW_FMT(CPSG_Exception, eServerError, "Received unknown item type: " << item_type);
+        }
+
+        static atomic_bool reported(false);
+
+        if (!reported.exchange(true)) {
+            ERR_POST("Received unknown item type: " << item_type);
+        }
+
+        return CPSG_ReplyItem::eEndOfReply;
+    }
+}
 
 shared_ptr<CPSG_ReplyItem> CPSG_Reply::SImpl::Create(SPSG_Reply::SItem::TTS* item_ts)
 {
@@ -196,47 +351,48 @@ shared_ptr<CPSG_ReplyItem> CPSG_Reply::SImpl::Create(SPSG_Reply::SItem::TTS* ite
 
     auto& chunks = item_locked->chunks;
     auto& args = item_locked->args;
-    auto item_type = args.GetValue("item_type");
+    auto processor_id = args.GetValue("processor_id");
 
-    if (item_type == "blob") {
-        auto blob_id = args.GetValue("blob_id");
-        auto reason = args.GetValue("reason");
+    const auto& state = item_locked->state.GetState();
+    const auto itar = SItemTypeAndReason::Get(args);
 
-        if (reason.empty()) {
-            unique_ptr<CPSG_BlobData> blob_data(new CPSG_BlobData(blob_id));
-            blob_data->m_Stream.reset(new SPSG_RStream(item_ts));
-            rv.reset(blob_data.release());
-        } else {
-            auto r = CPSG_SkippedBlob::eUnknown;
+    if ((state == SPSG_Reply::SState::eNotFound) || (state == SPSG_Reply::SState::eError)) {
+        rv.reset(new CPSG_ReplyItem(itar.first));
 
-            if (reason == "excluded") {
-                r = CPSG_SkippedBlob::eExcluded;
-            } else if (reason == "inprogress") {
-                r = CPSG_SkippedBlob::eInProgress;
-            } else if (reason == "sent") {
-                r = CPSG_SkippedBlob::eSent;
-            }
+    } else if (itar.first == CPSG_ReplyItem::eBlobData) {
+        unique_ptr<CPSG_BlobData> blob_data(new CPSG_BlobData(s_GetDataId(args)));
+        blob_data->m_Stream.reset(new SPSG_RStream(item_ts));
+        rv.reset(blob_data.release());
 
-            rv.reset(new CPSG_SkippedBlob(blob_id, r));
-        }
+    } else if (itar.first == CPSG_ReplyItem::eSkippedBlob) {
+        auto data_id = s_GetDataId(args);
+        auto blob_id = move(dynamic_cast<CPSG_BlobId&>(*data_id));
+        rv.reset(new CPSG_SkippedBlob(move(blob_id), itar.second));
 
-    } else if (item_type == "bioseq_info") {
+    } else if (itar.first == CPSG_ReplyItem::eBioseqInfo) {
         rv.reset(CreateImpl(new CPSG_BioseqInfo, chunks));
 
-    } else if (item_type == "blob_prop") {
-        auto blob_id = args.GetValue("blob_id");
-        rv.reset(CreateImpl(new CPSG_BlobInfo(blob_id), chunks));
+    } else if (itar.first == CPSG_ReplyItem::eBlobInfo) {
+        rv.reset(CreateImpl(new CPSG_BlobInfo(s_GetDataId(args)), chunks));
 
-    } else if (item_type == "bioseq_na") {
+    } else if (itar.first == CPSG_ReplyItem::eNamedAnnotInfo) {
         auto name = args.GetValue("na");
         rv.reset(CreateImpl(new CPSG_NamedAnnotInfo(name), chunks));
 
+    } else if (itar.first == CPSG_ReplyItem::ePublicComment) {
+        auto text = chunks.empty() ? string() : chunks.front();
+        rv.reset(new CPSG_PublicComment(s_GetDataId(args), text));
+
+    } else if (itar.first == CPSG_ReplyItem::eProcessor) {
+        rv.reset(new CPSG_ReplyItem(CPSG_ReplyItem::eProcessor));
+
     } else {
-        NCBI_THROW_FMT(CPSG_Exception, eServerError, "Received unknown item type: " << item_type);
+        return rv;
     }
 
     rv->m_Impl.reset(impl.release());
     rv->m_Reply = user_reply_locked;
+    rv->m_ProcessorId = move(processor_id);
     return rv;
 }
 
@@ -297,17 +453,10 @@ const char* s_GetTSE(CPSG_Request_Biodata::EIncludeData include_data)
 
 string CPSG_Queue::SImpl::x_GetAbsPathRef(shared_ptr<const CPSG_Request> user_request)
 {
-    auto& ioc = m_Service.ioc;
     ostringstream os;
     user_request->x_GetAbsPathRef(os);
+    os << m_Service.ioc.GetUrlArgs();
 
-    switch (ioc.params.use_cache) {
-        case EPSG_UseCache::eDefault:                         break;
-        case EPSG_UseCache::eNo:      os << "&use_cache=no";  break;
-        case EPSG_UseCache::eYes:     os << "&use_cache=yes"; break;
-    }
-
-    os << ioc.GetClientId();
     if (const auto hops = user_request->m_Hops) os << "&hops=" << hops;
     return os.str();
 }
@@ -324,24 +473,21 @@ const char* s_GetAccSubstitution(EPSG_AccSubstitution acc_substitution)
 }
 
 
-struct CPSG_Request::x_GetBioIdParams
+const char* s_GetAutoBlobSkipping(ESwitch value)
 {
-    const string& id;
-    const CPSG_BioId::TType& type;
-
-    x_GetBioIdParams(const CPSG_BioId& bio_id) : id(bio_id.m_Id), type(bio_id.m_Type) {}
-
-    friend ostream& operator<<(ostream& os, const x_GetBioIdParams& bio_id)
-    {
-        if (bio_id.type) os << "seq_id_type=" << bio_id.type << '&';
-        return os << "seq_id=" << bio_id.id;
+    switch (value) {
+        case eDefault: break;
+        case eOn:      return "&auto_blob_skipping=yes";
+        case eOff:     return "&auto_blob_skipping=no";
     }
-};
+
+    return "";
+}
 
 
 void CPSG_Request_Biodata::x_GetAbsPathRef(ostream& os) const
 {
-    os << "/ID/get?" << x_GetBioIdParams(m_BioId);
+    os << "/ID/get?" << m_BioId;
 
     if (const auto tse = s_GetTSE(m_IncludeData)) os << "&tse=" << tse;
 
@@ -350,23 +496,24 @@ void CPSG_Request_Biodata::x_GetAbsPathRef(ostream& os) const
 
         char delimiter = '=';
         for (const auto& blob_id : m_ExcludeTSEs) {
-            os << delimiter << blob_id.Get();
+            os << delimiter << blob_id.GetId();
             delimiter = ',';
         }
     }
 
     os << s_GetAccSubstitution(m_AccSubstitution);
+    os << s_GetAutoBlobSkipping(m_AutoBlobSkipping);
 }
 
 void CPSG_Request_Resolve::x_GetAbsPathRef(ostream& os) const
 {
-    os << "/ID/resolve?" << x_GetBioIdParams(m_BioId) << "&fmt=json&psg_protocol=yes";
+    os << "/ID/resolve?" << m_BioId << "&fmt=json";
 
     auto value = "yes";
     auto include_info = m_IncludeInfo;
     const auto max_bit = (numeric_limits<unsigned>::max() >> 1) + 1;
 
-    if (include_info & max_bit) {
+    if (include_info & CPSG_Request_Resolve::TIncludeInfo(max_bit)) {
         os << "&all_info=yes";
         value = "no";
         include_info = ~include_info;
@@ -390,16 +537,14 @@ void CPSG_Request_Resolve::x_GetAbsPathRef(ostream& os) const
 
 void CPSG_Request_Blob::x_GetAbsPathRef(ostream& os) const
 {
-    os << "/ID/getblob?blob_id=" << m_BlobId.Get();
-
-    if (!m_LastModified.empty()) os << "&last_modified=" << m_LastModified;
+    os << "/ID/getblob?" << m_BlobId;
 
     if (const auto tse = s_GetTSE(m_IncludeData)) os << "&tse=" << tse;
 }
 
 void CPSG_Request_NamedAnnotInfo::x_GetAbsPathRef(ostream& os) const
 {
-    os << "/ID/get_na?" << x_GetBioIdParams(m_BioId) << "&names=";
+    os << "/ID/get_na?" << m_BioId << "&names=";
 
     for (const auto& name : m_AnnotNames) {
         os << name << ",";
@@ -407,14 +552,15 @@ void CPSG_Request_NamedAnnotInfo::x_GetAbsPathRef(ostream& os) const
 
     // Remove last comma (there must be some output after seekp to succeed)
     os.seekp(-1, ios_base::cur);
-    os << "&fmt=json&psg_protocol=yes";
+
+    if (const auto tse = s_GetTSE(m_IncludeData)) os << "&tse=" << tse;
+
     os << s_GetAccSubstitution(m_AccSubstitution);
 }
 
-void CPSG_Request_TSE_Chunk::x_GetAbsPathRef(ostream& os) const
+void CPSG_Request_Chunk::x_GetAbsPathRef(ostream& os) const
 {
-    os << "/ID/get_tse_chunk?tse_id=" << m_TSE_BlobId.Get() <<
-        "&chunk=" << m_ChunkNo << "&split_version=" << m_SplitVersion;
+    os << "/ID/get_tse_chunk?" << m_ChunkId;
 }
 
 bool CPSG_Queue::SImpl::SendRequest(shared_ptr<const CPSG_Request> user_request, const CDeadline& deadline)
@@ -485,14 +631,14 @@ CPSG_ReplyItem::CPSG_ReplyItem(EType type) :
 }
 
 
-CPSG_BlobData::CPSG_BlobData(CPSG_BlobId id) :
+CPSG_BlobData::CPSG_BlobData(unique_ptr<CPSG_DataId> id) :
     CPSG_ReplyItem(eBlobData),
     m_Id(move(id))
 {
 }
 
 
-CPSG_BlobInfo::CPSG_BlobInfo(CPSG_BlobId id) :
+CPSG_BlobInfo::CPSG_BlobInfo(unique_ptr<CPSG_DataId> id) :
     CPSG_ReplyItem(eBlobInfo),
     m_Id(move(id))
 {
@@ -518,11 +664,6 @@ string CPSG_BlobInfo::GetFormat() const
     return "asn.1";
 }
 
-Uint8 CPSG_BlobInfo::GetVersion() const
-{
-    return static_cast<Uint8>(m_Data.GetInteger("last_modified"));
-}
-
 Uint8 CPSG_BlobInfo::GetStorageSize() const
 {
     return static_cast<Uint8>(m_Data.GetInteger("size"));
@@ -583,84 +724,14 @@ string CPSG_BlobInfo::GetUsername() const
     return m_Data.GetString("username");
 }
 
-struct SId2Info
+string CPSG_BlobInfo::GetId2Info() const
 {
-    enum : size_t { eSat, eInfo, eNChunks, eSplitVer, eMinSize = eNChunks + 1 };
-
-    vector<CTempString> values;
-    int sat = 0;
-
-    SId2Info(const CJsonNode& data, const CPSG_BlobId& id);
-
-    explicit operator bool() const { return !values.empty() && (sat != 0); }
-
-private:
-    string m_Value;
-};
-
-SId2Info::SId2Info(const CJsonNode& data, const CPSG_BlobId& id)
-{
-    if (!data.HasKey("id2_info")) return;
-
-    m_Value = data.GetString("id2_info");
-
-    if (m_Value.empty()) return;
-
-    NStr::Split(m_Value, ".", values);
-
-    if (values.size() < eMinSize ) {
-        NCBI_THROW_FMT(CPSG_Exception, eServerError, "Wrong id2_info format: " << m_Value <<
-                " for blob '" << id.Get() << '\'');
-    }
-
-    auto sat_str = values[eSat];
-
-    if (!sat_str.empty()) {
-        sat = NStr::StringToInt(sat_str);
-    }
+    return m_Data.GetString("id2_info");
 }
 
-CPSG_BlobId CPSG_BlobInfo::GetSplitInfoBlobId() const
+Uint8 CPSG_BlobInfo::GetNChunks() const
 {
-    SId2Info id2_info(m_Data, m_Id);
-
-    if (!id2_info) return kEmptyStr;
-
-    auto sat_key = id2_info.values[SId2Info::eInfo];
-    return CPSG_BlobId(id2_info.sat, NStr::StringToInt(sat_key));
-}
-
-CPSG_BlobId CPSG_BlobInfo::GetChunkBlobId(unsigned split_chunk_no) const
-{
-    if (split_chunk_no == 0) return kEmptyStr;
-
-    int index = static_cast<int>(split_chunk_no);
-
-    SId2Info id2_info(m_Data, m_Id);
-
-    if (!id2_info) return kEmptyStr;
-
-    auto info = NStr::StringToInt(id2_info.values[SId2Info::eInfo]);
-
-    if (info <= 0) return kEmptyStr;
-
-    auto nchunks = NStr::StringToInt(id2_info.values[SId2Info::eNChunks]);
-
-    if (nchunks <= 0) return kEmptyStr;
-    if (nchunks < index) return kEmptyStr;
-
-    return CPSG_BlobId(id2_info.sat, info + index - nchunks - 1);
-}
-
-CPSG_BlobInfo::TSplitVersion CPSG_BlobInfo::GetSplitVersion() const
-{
-    SId2Info id2_info(m_Data, m_Id);
-
-    if (!id2_info) return 0;
-    if (id2_info.values.size() <= SId2Info::eSplitVer) return 0;
-
-    auto split_ver = id2_info.values[SId2Info::eSplitVer];
-    return split_ver.empty() ? 0 : NStr::StringToInt(split_ver);
+    return static_cast<Uint8>(m_Data.GetInteger("n_chunks"));
 }
 
 
@@ -679,12 +750,7 @@ CPSG_BioseqInfo::CPSG_BioseqInfo()
 
 CPSG_BioId CPSG_BioseqInfo::GetCanonicalId() const
 {
-    auto type = static_cast<CPSG_BioId::TType>(m_Data.GetInteger("seq_id_type"));
-    auto accession = m_Data.GetString("accession");
-    auto name_node = m_Data.GetByKeyOrNull("name");
-    auto name = name_node && name_node.IsString() ? name_node.AsString() : string();
-    auto version = static_cast<int>(m_Data.GetInteger("version"));
-    return objects::CSeq_id(type, accession, name, version).AsFastaString();
+    return s_GetBioId(m_Data);
 };
 
 vector<CPSG_BioId> CPSG_BioseqInfo::GetOtherIds() const
@@ -698,10 +764,10 @@ vector<CPSG_BioId> CPSG_BioseqInfo::GetOtherIds() const
         error = !seq_id.IsArray() || (seq_id.GetSize() != 2);
 
         if (!error) {
-            auto tag = objects::CSeq_id::eFasta_AsTypeAndContent;
             auto type = static_cast<CPSG_BioId::TType>(seq_id.GetAt(0).AsInteger());
             auto content = seq_id.GetAt(1).AsString();
-            rv.emplace_back(objects::CSeq_id(tag, type, content).AsFastaString());
+            rv.emplace_back(string(objects::CSeq_id::WhichFastaTag(type))
+                            + '|' + content, type);
         }
     }
 
@@ -741,14 +807,12 @@ CPSG_BioseqInfo::TState CPSG_BioseqInfo::GetState() const
 
 CPSG_BlobId CPSG_BioseqInfo::GetBlobId() const
 {
-    auto sat = static_cast<int>(m_Data.GetInteger("sat"));
-    auto sat_key = static_cast<int>(m_Data.GetInteger("sat_key"));
-    return CPSG_BlobId(sat, sat_key);
+    return s_GetBlobId(m_Data);
 }
 
 TTaxId CPSG_BioseqInfo::GetTaxId() const
 {
-    return static_cast<TTaxId>(m_Data.GetInteger("tax_id"));
+    return TAX_ID_FROM(Int8, m_Data.GetInteger("tax_id"));
 }
 
 int CPSG_BioseqInfo::GetHash() const
@@ -777,7 +841,8 @@ CPSG_Request_Resolve::TIncludeInfo CPSG_BioseqInfo::IncludedInfo() const
     if (m_Data.HasKey("length"))                                          rv |= CPSG_Request_Resolve::fLength;
     if (m_Data.HasKey("seq_state"))                                       rv |= CPSG_Request_Resolve::fChainState;
     if (m_Data.HasKey("state"))                                           rv |= CPSG_Request_Resolve::fState;
-    if (m_Data.HasKey("sat") && m_Data.HasKey("sat_key"))                 rv |= CPSG_Request_Resolve::fBlobId;
+    if (m_Data.HasKey("blob_id") ||
+        (m_Data.HasKey("sat") && m_Data.HasKey("sat_key")))               rv |= CPSG_Request_Resolve::fBlobId;
     if (m_Data.HasKey("tax_id"))                                          rv |= CPSG_Request_Resolve::fTaxId;
     if (m_Data.HasKey("hash"))                                            rv |= CPSG_Request_Resolve::fHash;
     if (m_Data.HasKey("date_changed"))                                    rv |= CPSG_Request_Resolve::fDateChanged;
@@ -793,23 +858,51 @@ CPSG_NamedAnnotInfo::CPSG_NamedAnnotInfo(string name) :
 {
 }
 
+
+string CPSG_NamedAnnotInfo::GetId2AnnotInfo() const
+{
+    auto node = m_Data.GetByKeyOrNull("seq_annot_info");
+    return node && node.IsString() ? node.AsString() : string();
+}
+
+
+CPSG_NamedAnnotInfo::TId2AnnotInfoList CPSG_NamedAnnotInfo::GetId2AnnotInfoList() const
+{
+    TId2AnnotInfoList ret;
+    auto info_str = GetId2AnnotInfo();
+    if (!info_str.empty()) {
+        auto in_string = NStr::Base64Decode(info_str);
+        istringstream in_stream(in_string);
+        CObjectIStreamAsnBinary in(in_stream);
+        while ( in.HaveMoreData() ) {
+            CRef<TId2AnnotInfo> info(new TId2AnnotInfo);
+            in >> *info;
+            ret.push_back(info);
+        }
+    }
+    return ret;
+}
+
+
+CPSG_BioId CPSG_NamedAnnotInfo::GetAnnotatedId() const
+{
+    return s_GetBioId(m_Data);
+}
+
 CRange<TSeqPos> CPSG_NamedAnnotInfo::GetRange() const
 {
-    auto start = static_cast<TSeqPos>(m_Data.GetInteger("start"));
-    auto stop = static_cast<TSeqPos>(m_Data.GetInteger("stop"));
-    return { start, stop };
+    if (auto start = static_cast<TSeqPos>(m_Data.GetInteger("start"))) {
+        if (auto stop = static_cast<TSeqPos>(m_Data.GetInteger("stop"))) {
+            return { start, stop };
+        }
+    }
+
+    return CRange<TSeqPos>::GetWhole();
 }
 
 CPSG_BlobId CPSG_NamedAnnotInfo::GetBlobId() const
 {
-    auto sat = static_cast<int>(m_Data.GetInteger("sat"));
-    auto sat_key = static_cast<int>(m_Data.GetInteger("sat_key"));
-    return { sat, sat_key };
-}
-
-Uint8 CPSG_NamedAnnotInfo::GetVersion() const
-{
-    return static_cast<Uint8>(m_Data.GetInteger("last_modified"));
+    return s_GetBlobId(m_Data);
 }
 
 template <class TResult>
@@ -832,7 +925,7 @@ template <class TResult>
 TResult SAnnotInfoProcessor<TResult>::operator()(const CPSG_ReplyItem* item, const CJsonNode& data) const
 {
     auto annot_info_str(NStr::Unescape(data.GetString("annot_info")));
-    auto annot_info(CJsonNode::ParseJSON(annot_info_str));
+    auto annot_info(CJsonNode::ParseJSON(annot_info_str, CJsonNode::fStandardJson));
 
     if (!annot_info.IsObject()) ThrowError(item, annot_info);
 
@@ -878,7 +971,7 @@ bool s_GetZoomLevels(const CJsonNode& annot_data, CPSG_NamedAnnotInfo::TZoomLeve
 
         if (!zoom_level.IsInteger()) return false;
 
-        result.push_back(zoom_level.AsInteger());
+        result.push_back(static_cast<unsigned>(zoom_level.AsInteger()));
     }
 
     return true;
@@ -951,6 +1044,14 @@ CPSG_NamedAnnotInfo::TAnnotInfoList CPSG_NamedAnnotInfo::GetAnnotInfoList() cons
 }
 
 
+CPSG_PublicComment::CPSG_PublicComment(unique_ptr<CPSG_DataId> id, string text) :
+    CPSG_ReplyItem(ePublicComment),
+    m_Id(move(id)),
+    m_Text(move(text))
+{
+}
+
+
 CPSG_Reply::CPSG_Reply() :
     m_Impl(new SImpl)
 {
@@ -977,20 +1078,20 @@ shared_ptr<CPSG_ReplyItem> CPSG_Reply::GetNextItem(CDeadline deadline)
     assert(m_Impl->reply);
 
     auto& reply_item = m_Impl->reply->reply_item;
-    auto& state = reply_item.GetMTSafe().state;
+    auto& reply_state = reply_item.GetMTSafe().state;
 
     do {
-        bool was_in_progress = state.InProgress();
+        bool was_in_progress = reply_state.InProgress();
 
         if (auto items_locked = m_Impl->reply->items.GetLock()) {
             auto& items = *items_locked;
 
             for (auto& item_ts : items) {
-                const auto& state = item_ts.GetMTSafe().state;
+                const auto& item_state = item_ts.GetMTSafe().state;
 
-                if (state.Returned()) continue;
+                if (item_state.Returned()) continue;
 
-                if (state.Empty()) {
+                if (item_state.Empty()) {
                     auto item_locked = item_ts.GetLock();
                     auto& item = *item_locked;
 
@@ -999,7 +1100,11 @@ shared_ptr<CPSG_ReplyItem> CPSG_Reply::GetNextItem(CDeadline deadline)
                 }
 
                 // Do not hold lock on item_ts around this call!
-                return m_Impl->Create(&item_ts);
+                if (auto rv = m_Impl->Create(&item_ts)) {
+                    return rv;
+                }
+
+                continue;
             }
         }
 
@@ -1008,7 +1113,7 @@ shared_ptr<CPSG_ReplyItem> CPSG_Reply::GetNextItem(CDeadline deadline)
             return shared_ptr<CPSG_ReplyItem>(new CPSG_ReplyItem(CPSG_ReplyItem::eEndOfReply));
         }
     }
-    while (reply_item.WaitUntil(state.GetState(), deadline, SPSG_Reply::SState::eInProgress, true));
+    while (reply_item.WaitUntil(reply_state.GetState(), deadline, SPSG_Reply::SState::eInProgress, true));
 
     return {};
 }
@@ -1061,6 +1166,13 @@ bool CPSG_Queue::IsEmpty() const
     return m_Impl->Empty();
 }
 
+bool CPSG_Queue::RejectsRequests() const
+{
+    _ASSERT(m_Impl);
+    return m_Impl->RejectsRequests();
+}
+
+
 CPSG_Queue::TApiLock CPSG_Queue::GetApiLock()
 {
     return SImpl::GetApiLock();
