@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_TRANSPORT__HPP
 #define OBJTOOLS__PUBSEQ_GATEWAY__PSG_CLIENT_TRANSPORT__HPP
 
-/*  $Id: psg_client_transport.hpp 608075 2020-05-11 17:58:58Z ivanov $
+/*  $Id: psg_client_transport.hpp 628921 2021-04-07 18:46:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,15 +53,13 @@
 #include <utility>
 #include <condition_variable>
 #include <forward_list>
-#include <mutex>
 #include <chrono>
 #include <sstream>
 #include <random>
-
-#include <nghttp2/nghttp2.h>
-#include <uv.h>
+#include <type_traits>
 
 #include "mpmc_nw.hpp"
+#include <connect/impl/ncbi_uv_nghttp2.hpp>
 #include <connect/services/netservice_api.hpp>
 #include <corelib/ncbi_param.hpp>
 #include <corelib/ncbi_url.hpp>
@@ -70,68 +68,11 @@
 BEGIN_NCBI_SCOPE
 
 // Different TRACE macros allow turning off tracing in some classes
-#define PSG_UV_WRITE_TRACE(message)        _TRACE(message)
-#define PSG_UV_TCP_TRACE(message)          _TRACE(message)
-#define PSG_NGHTTP2_SESSION_TRACE(message) _TRACE(message)
 #define PSG_THROTTLING_TRACE(message)      _TRACE(message)
 #define PSG_IO_SESSION_TRACE(message)      _TRACE(message)
 #define PSG_IO_TRACE(message)              _TRACE(message)
 #define PSG_DISCOVERY_TRACE(message)       _TRACE(message)
 
-template <class TType>
-struct SPSG_ThreadSafe : SPSG_CV<0>
-{
-    template <class T>
-    struct SLock : private unique_lock<std::mutex>
-    {
-        T& operator*()  { return *m_Object; }
-        T* operator->() { return  m_Object; }
-
-        // A safe and elegant RAII alternative to explicit scopes or 'unlock' method.
-        // It allows locks to be declared inside 'if' condition.
-        using unique_lock<std::mutex>::operator bool;
-
-    private:
-        SLock(T* c, std::mutex& m) : unique_lock(m), m_Object(c) { _ASSERT(m_Object); }
-
-        T* m_Object;
-
-        friend struct SPSG_ThreadSafe;
-    };
-
-    template <class... TArgs>
-    SPSG_ThreadSafe(TArgs&&... args) : m_Object(forward<TArgs>(args)...) {}
-
-    SLock<      TType> GetLock()       { return { &m_Object, GetMutex() }; }
-    SLock<const TType> GetLock() const { return { &m_Object, GetMutex() }; }
-
-    // Direct access to the protected object (e.g. to access atomic members).
-    // All thread-safe members must be explicitly marked volatile to be available.
-          volatile TType& GetMTSafe()       { return m_Object; }
-    const volatile TType& GetMTSafe() const { return m_Object; }
-
-private:
-    TType m_Object;
-};
-
-struct SPSG_Error : string
-{
-    enum EError{
-        eNgHttp2Cb = 1,
-        eShutdown,
-        eException,
-        eTimeout,
-    };
-
-    template <class ...TArgs>
-    SPSG_Error(TArgs&&... args) : string(Build(forward<TArgs>(args)...)) {}
-
-private:
-    static string Build(EError error, const char* details);
-    static string Build(int error);
-    static string Build(int error, const char* details);
-};
-
 struct SPSG_Args : CUrlArgs
 {
     using CUrlArgs::CUrlArgs;
@@ -171,14 +112,12 @@ struct SPSG_Params
 {
     TPSG_DebugPrintout debug_printout;
     TPSG_RequestsPerIo requests_per_io;
-    TPSG_UseCache use_cache;
     TPSG_RequestRetries request_retries;
     TPSG_PsgClientMode client_mode;
 
     SPSG_Params() :
         debug_printout(TPSG_DebugPrintout::eGetDefault),
         requests_per_io(TPSG_RequestsPerIo::eGetDefault),
-        use_cache(TPSG_UseCache::eGetDefault),
         request_retries(TPSG_RequestRetries::eGetDefault),
         client_mode(TPSG_PsgClientMode::eGetDefault)
     {}
@@ -216,11 +155,11 @@ private:
 
     enum EType { eSend = 1000, eReceive, eClose, eRetry, eFail };
 
-    void Event(const char*, const string&)          { Event(eSend);    }
+    void Event(SSocketAddress, const string&)       { Event(eSend);    }
     void Event(const SPSG_Args&, const SPSG_Chunk&) { Event(eReceive); }
     void Event(uint32_t)                            { Event(eClose);   }
-    void Event(unsigned, const SPSG_Error&)         { Event(eRetry);   }
-    void Event(const SPSG_Error&)                   { Event(eFail);    }
+    void Event(unsigned, const SUvNgHttp2_Error&)   { Event(eRetry);   }
+    void Event(const SUvNgHttp2_Error&)             { Event(eFail);    }
 
     void Event(EType type)
     {
@@ -229,11 +168,11 @@ private:
         m_Events.emplace_back(ms, type, thread_id);
     }
 
-    void Print(const char* authority, const string& path);
+    void Print(SSocketAddress address, const string& path);
     void Print(const SPSG_Args& args, const SPSG_Chunk& chunk);
     void Print(uint32_t error_code);
-    void Print(unsigned retries, const SPSG_Error& error);
-    void Print(const SPSG_Error& error);
+    void Print(unsigned retries, const SUvNgHttp2_Error& error);
+    void Print(const SUvNgHttp2_Error& error);
 
     bool IsPerf() const
     {
@@ -344,7 +283,7 @@ struct SPSG_Reply
 
     struct SItem
     {
-        using TTS = SPSG_ThreadSafe<SItem>;
+        using TTS = SPSG_CV<0, SThreadSafe<SItem>>;
 
         vector<SPSG_Chunk> chunks;
         SPSG_Args args;
@@ -355,20 +294,13 @@ struct SPSG_Reply
         void SetSuccess();
     };
 
-    // Forbid signals on reply
-    class SItemsTS : public SPSG_ThreadSafe<list<SItem::TTS>>
-    {
-        using SPSG_ThreadSafe::NotifyOne;
-        using SPSG_ThreadSafe::NotifyAll;
-        using SPSG_ThreadSafe::WaitUntil;
-    };
-
-    SItemsTS items;
+    SThreadSafe<list<SItem::TTS>> items;
     SItem::TTS reply_item;
     SDebugPrintout debug_printout;
 
     SPSG_Reply(string id, const SPSG_Params& params) : debug_printout(move(id), params) {}
     void SetSuccess();
+    void AddError(string message);
 };
 
 struct SPSG_Request
@@ -379,7 +311,16 @@ struct SPSG_Request
 
     SPSG_Request(string p, shared_ptr<SPSG_Reply> r, CRef<CRequestContext> c, const SPSG_Params& params);
 
-    void OnReplyData(const char* data, size_t len) { while (len) (this->*m_State)(data, len); }
+    bool OnReplyData(const char* data, size_t len)
+    {
+        while (len) {
+            if (!(this->*m_State)(data, len)) {
+                return false;
+            }
+        }
+
+        return true;
+    }
 
     unsigned GetRetries()
     {
@@ -387,10 +328,10 @@ struct SPSG_Request
     }
 
 private:
-    void StatePrefix(const char*& data, size_t& len);
-    void StateArgs(const char*& data, size_t& len);
-    void StateData(const char*& data, size_t& len);
-    void StateIo(const char*& data, size_t& len) { data += len; len = 0; }
+    bool StatePrefix(const char*& data, size_t& len);
+    bool StateArgs(const char*& data, size_t& len);
+    bool StateData(const char*& data, size_t& len);
+    bool StateIo(const char*& data, size_t& len) { data += len; len = 0; return true; }
 
     void SetStatePrefix()  { Add(); m_State = &SPSG_Request::StatePrefix; }
     void SetStateArgs()           { m_State = &SPSG_Request::StateArgs;   }
@@ -399,7 +340,7 @@ private:
     void Add();
     void AddIo();
 
-    using TState = void (SPSG_Request::*)(const char*& data, size_t& len);
+    using TState = bool (SPSG_Request::*)(const char*& data, size_t& len);
     TState m_State;
 
     struct SBuffer
@@ -416,261 +357,6 @@ private:
     unsigned m_Retries;
 };
 
-template <typename THandle>
-struct SPSG_UvHandle : protected THandle
-{
-    SPSG_UvHandle(uv_close_cb cb = nullptr) : m_Cb(cb) {}
-
-    void Close()
-    {
-        uv_close(reinterpret_cast<uv_handle_t*>(this), m_Cb);
-    }
-
-private:
-    uv_close_cb m_Cb;
-};
-
-struct SPSG_UvWrite
-{
-    SPSG_UvWrite(void* user_data);
-
-    vector<char>& GetBuffer() { _ASSERT(m_CurrentBuffer); return m_CurrentBuffer->data; }
-    int Write(uv_stream_t* handle, uv_write_cb cb);
-    void OnWrite(uv_write_t* req);
-    void Reset();
-
-private:
-    struct SBuffer
-    {
-        uv_write_t request;
-        vector<char> data;
-        bool in_progress = false;
-    };
-
-    void NewBuffer();
-
-    void* const m_UserData;
-    const TPSG_WriteHiwater m_WriteHiwater;
-    forward_list<SBuffer> m_Buffers;
-    SBuffer* m_CurrentBuffer = nullptr;
-};
-
-struct SPSG_UvConnect
-{
-    SPSG_UvConnect(void* user_data, const CNetServer::SAddress& address);
-
-    int operator()(uv_tcp_t* handle, uv_connect_cb cb);
-
-private:
-    struct sockaddr_in m_Address;
-    uv_connect_t m_Request;
-};
-
-struct SPSG_UvTcp : SPSG_UvHandle<uv_tcp_t>
-{
-    using TConnectCb = function<void(int)>;
-    using TReadCb = function<void(const char*, ssize_t)>;
-    using TWriteCb = function<void(int)>;
-
-    SPSG_UvTcp(uv_loop_t *loop, const CNetServer::SAddress& address,
-            TConnectCb connect_cb, TReadCb read_cb, TWriteCb write_cb);
-
-    int Write();
-    void Close();
-
-    vector<char>& GetWriteBuffer() { return m_Write.GetBuffer(); }
-
-private:
-    enum EState {
-        eClosed,
-        eConnecting,
-        eConnected,
-        eClosing,
-    };
-
-    void OnConnect(uv_connect_t* req, int status);
-    void OnAlloc(uv_handle_t*, size_t suggested_size, uv_buf_t* buf);
-    void OnRead(uv_stream_t*, ssize_t nread, const uv_buf_t* buf);
-    void OnWrite(uv_write_t*, int status);
-    void OnClose(uv_handle_t*);
-
-    template <class THandle, class ...TArgs1, class ...TArgs2>
-    static void OnCallback(void (SPSG_UvTcp::*member)(THandle*, TArgs1...), THandle* handle, TArgs2&&... args)
-    {
-        auto that = static_cast<SPSG_UvTcp*>(handle->data);
-        (that->*member)(handle, forward<TArgs2>(args)...);
-    }
-
-    static void s_OnAlloc(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf) { OnCallback(&SPSG_UvTcp::OnAlloc, handle, suggested_size, buf); }
-    static void s_OnRead(uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf) { OnCallback(&SPSG_UvTcp::OnRead, stream, nread, buf); }
-    static void s_OnWrite(uv_write_t* req, int status) { OnCallback(&SPSG_UvTcp::OnWrite, req, status); }
-    static void s_OnConnect(uv_connect_t* req, int status) { OnCallback(&SPSG_UvTcp::OnConnect, req, status); }
-    static void s_OnClose(uv_handle_t* handle) { OnCallback(&SPSG_UvTcp::OnClose, handle); }
-
-    uv_loop_t* m_Loop;
-    EState m_State = eClosed;
-    vector<char> m_ReadBuffer;
-    SPSG_UvConnect m_Connect;
-    SPSG_UvWrite m_Write;
-    TConnectCb m_ConnectCb;
-    TReadCb m_ReadCb;
-    TWriteCb m_WriteCb;
-};
-
-struct SPSG_UvAsync : SPSG_UvHandle<uv_async_t>
-{
-    void Init(void* d, uv_loop_t* l, uv_async_cb cb)
-    {
-        if (auto rc = uv_async_init(l, this, cb)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_async_init failed " << uv_strerror(rc));
-        }
-
-        data = d;
-    }
-
-    void Signal()
-    {
-        if (auto rc = uv_async_send(this)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_async_send failed " << uv_strerror(rc));
-        }
-    }
-};
-
-struct SPSG_UvTimer : SPSG_UvHandle<uv_timer_t>
-{
-    SPSG_UvTimer(void* d, uv_timer_cb cb, uint64_t t, uint64_t r) :
-        m_Cb(cb),
-        m_Timeout(t),
-        m_Repeat(r)
-    {
-        data = d;
-    }
-
-    void Init(uv_loop_t* l)
-    {
-        if (auto rc = uv_timer_init(l, this)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_timer_init failed " << uv_strerror(rc));
-        }
-    }
-
-    void Start()
-    {
-        if (auto rc = uv_timer_start(this, m_Cb, m_Timeout, m_Repeat)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_timer_start failed " << uv_strerror(rc));
-        }
-    }
-
-    void Close()
-    {
-        if (auto rc = uv_timer_stop(this)) {
-            ERR_POST("uv_timer_stop failed " << uv_strerror(rc));
-        }
-
-        SPSG_UvHandle<uv_timer_t>::Close();
-    }
-
-private:
-    uv_timer_cb m_Cb;
-    const uint64_t m_Timeout;
-    const uint64_t m_Repeat;
-};
-
-struct SPSG_UvBarrier
-{
-    SPSG_UvBarrier(unsigned count)
-    {
-        if (auto rc = uv_barrier_init(&m_Barrier, count)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_barrier_init failed " << uv_strerror(rc));
-        }
-    }
-
-    void Wait()
-    {
-        auto rc = uv_barrier_wait(&m_Barrier);
-
-        if (rc > 0) {
-            uv_barrier_destroy(&m_Barrier);
-        } else if (rc < 0) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_barrier_wait failed " << uv_strerror(rc));
-        }
-    }
-
-private:
-    uv_barrier_t m_Barrier;
-};
-
-struct SPSG_UvLoop : uv_loop_t
-{
-    SPSG_UvLoop()
-    {
-        if (auto rc = uv_loop_init(this)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_loop_init failed " << uv_strerror(rc));
-        }
-    }
-
-    void Run()
-    {
-        if (auto rc = uv_run(this, UV_RUN_DEFAULT)) {
-            NCBI_THROW_FMT(CPSG_Exception, eInternalError, "uv_run failed " << uv_strerror(rc));
-        }
-    }
-
-    ~SPSG_UvLoop()
-    {
-        if (auto rc = uv_loop_close(this)) {
-            ERR_POST("uv_loop_close failed " << uv_strerror(rc));
-        }
-    }
-};
-
-struct SPSG_NgHttp2Session
-{
-    SPSG_NgHttp2Session(const string& authority, void* user_data,
-            nghttp2_on_data_chunk_recv_callback on_data,
-            nghttp2_on_stream_close_callback    on_stream_close,
-            nghttp2_on_header_callback          on_header,
-            nghttp2_error_callback              on_error);
-
-    void Del();
-
-    int32_t Submit(shared_ptr<SPSG_Request>& req);
-    ssize_t Send(vector<char>& buffer);
-    ssize_t Recv(const uint8_t* buffer, size_t size);
-
-    uint32_t GetMaxStreams() const { return m_MaxStreams; }
-
-private:
-    enum EHeaders { eMethod, eScheme, eAuthority, ePath, eUserAgent, eSessionID, eSubHitID, eClientIP, eSize };
-
-    struct SHeader : nghttp2_nv
-    {
-        template <size_t N, size_t V> SHeader(const char (&n)[N], const char (&v)[V]);
-        template <size_t N>           SHeader(const char (&n)[N], const string& v);
-        template <size_t N>           SHeader(const char (&n)[N], uint8_t f = NGHTTP2_NV_FLAG_NO_COPY_VALUE);
-        void operator=(const string& v);
-    };
-
-    ssize_t Init();
-    ssize_t x_DelOnError(ssize_t rv)
-    {
-        if (rv < 0) {
-            nghttp2_session_del(m_Session);
-            m_Session = nullptr;
-        }
-
-        return rv;
-    }
-
-    nghttp2_session* m_Session = nullptr;
-    array<SHeader, eSize> m_Headers;
-    void* m_UserData;
-    nghttp2_on_data_chunk_recv_callback m_OnData;
-    nghttp2_on_stream_close_callback    m_OnStreamClose;
-    nghttp2_on_header_callback          m_OnHeader;
-    nghttp2_error_callback              m_OnError;
-    uint32_t m_MaxStreams;
-};
-
 struct SPSG_TimedRequest
 {
     SPSG_TimedRequest(shared_ptr<SPSG_Request> r) : m_Request(move(r)) {}
@@ -684,7 +370,7 @@ private:
     unsigned m_Seconds = 0;
 };
 
-struct SPSG_AsyncQueue : SPSG_UvAsync
+struct SPSG_AsyncQueue : SUv_Async
 {
     using TRequest = shared_ptr<SPSG_Request>;
 
@@ -703,7 +389,7 @@ struct SPSG_AsyncQueue : SPSG_UvAsync
         }
     }
 
-    using SPSG_UvAsync::Signal;
+    using SUv_Async::Signal;
 
 private:
     CMPMCQueue<TRequest> m_Queue;
@@ -730,7 +416,7 @@ struct SPSG_ThrottleParams
 
 struct SPSG_Throttling
 {
-    SPSG_Throttling(const CNetServer::SAddress& address, SPSG_ThrottleParams p, uv_loop_t* l);
+    SPSG_Throttling(const SSocketAddress& address, SPSG_ThrottleParams p, uv_loop_t* l);
 
     bool Active() const { return m_Active != eOff; }
     bool AddSuccess() { return AddResult(true); }
@@ -757,7 +443,7 @@ private:
 
         SStats(SPSG_ThrottleParams p) : params(p) {}
 
-        bool Adjust(const CNetServer::SAddress& address, bool result);
+        bool Adjust(const SSocketAddress& address, bool result);
         void Reset();
     };
     enum EThrottling { eOff, eOnTimer, eUntilDiscovery };
@@ -783,124 +469,57 @@ private:
         }
     }
 
-    const CNetServer::SAddress& m_Address;
-    SPSG_ThreadSafe<SStats> m_Stats;
+    const SSocketAddress& m_Address;
+    SThreadSafe<SStats> m_Stats;
     atomic<EThrottling> m_Active;
-    SPSG_UvTimer m_Timer;
-    SPSG_UvAsync m_Signal;
+    SUv_Timer m_Timer;
+    SUv_Async m_Signal;
 };
 
 struct SPSG_Server
 {
-    const CNetServer::SAddress address;
+    const SSocketAddress address;
     atomic<double> rate;
     SPSG_Throttling throttling;
 
-    SPSG_Server(CNetServer::SAddress a, double r, SPSG_ThrottleParams p, uv_loop_t* l) :
+    SPSG_Server(SSocketAddress a, double r, SPSG_ThrottleParams p, uv_loop_t* l) :
         address(move(a)),
         rate(r),
         throttling(address, move(p), l)
     {}
 };
 
-struct SPSG_IoSession
+struct SPSG_IoSession : SUvNgHttp2_SessionBase
 {
     SPSG_Server& server;
 
-    SPSG_IoSession(SPSG_Server& s, SPSG_AsyncQueue& queue, uv_loop_t* loop);
-
-    void StartClose();
+    template <class... TNgHttp2Cbs>
+    SPSG_IoSession(SPSG_Server& s, SPSG_AsyncQueue& queue, uv_loop_t* loop, TNgHttp2Cbs&&... callbacks);
 
     bool ProcessRequest(shared_ptr<SPSG_Request>& req);
     void CheckRequestExpiration();
     bool IsFull() const { return m_Session.GetMaxStreams() <= m_Requests.size(); }
 
-    template <typename TReturn, class ...TArgs1, class ...TArgs2>
-    TReturn TryCatch(TReturn (SPSG_IoSession::*member)(TArgs1...), TReturn error, TArgs2&&... args)
-    {
-        try {
-            return (this->*member)(forward<TArgs2>(args)...);
-        }
-        catch(const CException& e) {
-            ostringstream os;
-            os << e.GetErrCodeString() << " - " << e.GetMsg();
-            Reset(SPSG_Error::eException, os.str().c_str());
-        }
-        catch(const std::exception& e) {
-            Reset(SPSG_Error::eException, e.what());
-        }
-        catch(...) {
-            Reset(SPSG_Error::eException, "Unexpected exception");
-        }
-
-        return error;
-    }
-
-private:
-    using TRequests = unordered_map<int32_t, SPSG_TimedRequest>;
-
-    SPSG_IoSession(SPSG_IoSession&&) = delete;
-    SPSG_IoSession& operator=(SPSG_IoSession&&) = delete;
-
-    void OnConnect(int status);
-    void OnWrite(int status);
-    void OnRead(const char* buf, ssize_t nread);
-
-    bool Send();
-    bool Write();
-    bool Retry(shared_ptr<SPSG_Request> req, const SPSG_Error& error);
-    void RequestComplete(TRequests::iterator& it);
-
-    void Reset(SPSG_Error error);
-
-    template <class ...TArgs>
-    void Reset(TArgs&&... args)
-    {
-        Reset(SPSG_Error(forward<TArgs>(args)...));
-    }
-
+protected:
     int OnData(nghttp2_session* session, uint8_t flags, int32_t stream_id, const uint8_t* data, size_t len);
     int OnStreamClose(nghttp2_session* session, int32_t stream_id, uint32_t error_code);
     int OnHeader(nghttp2_session* session, const nghttp2_frame* frame, const uint8_t* name, size_t namelen,
             const uint8_t* value, size_t valuelen, uint8_t flags);
-    int OnError(nghttp2_session* session, const char* msg, size_t len);
+    int OnError(nghttp2_session*, const char* msg, size_t) { Reset(msg); return 0; }
 
-    template <class ...TArgs1, class ...TArgs2>
-    static int OnNgHttp2(void* user_data, int (SPSG_IoSession::*member)(TArgs1...), TArgs2&&... args)
-    {
-        auto that = static_cast<SPSG_IoSession*>(user_data);
-        return that->TryCatch(member, (int)NGHTTP2_ERR_CALLBACK_FAILURE, forward<TArgs2>(args)...);
-    }
+private:
+    enum EHeaders { eMethod, eScheme, eAuthority, ePath, eUserAgent, eSessionID, eSubHitID, eClientIP, eSize };
 
-    static int s_OnData(nghttp2_session* session, uint8_t flags, int32_t stream_id, const uint8_t* data,
-            size_t len, void* user_data)
-    {
-        return OnNgHttp2(user_data, &SPSG_IoSession::OnData, session, flags, stream_id, data, len);
-    }
+    using TRequests = unordered_map<int32_t, SPSG_TimedRequest>;
 
-    static int s_OnStreamClose(nghttp2_session* session, int32_t stream_id, uint32_t error_code, void* user_data)
-    {
-        return OnNgHttp2(user_data, &SPSG_IoSession::OnStreamClose, session, stream_id, error_code);
-    }
+    bool Retry(shared_ptr<SPSG_Request> req, const SUvNgHttp2_Error& error);
+    void RequestComplete(TRequests::iterator& it);
 
-    static int s_OnHeader(nghttp2_session* session, const nghttp2_frame* frame, const uint8_t* name, size_t namelen,
-            const uint8_t* value, size_t valuelen, uint8_t flags, void* user_data)
-    {
-        return OnNgHttp2(user_data, &SPSG_IoSession::OnHeader, session, frame, name, namelen, value, valuelen, flags);
-    }
-
-    static int s_OnError(nghttp2_session*, const char* msg, size_t, void* user_data)
-    {
-        auto that = static_cast<SPSG_IoSession*>(user_data);
-        that->Reset(SPSG_Error::eNgHttp2Cb, msg);
-        return 0;
-    }
+    void OnReset(SUvNgHttp2_Error error) override;
 
+    array<SNgHttp2_Header<NGHTTP2_NV_FLAG_NO_COPY_NAME | NGHTTP2_NV_FLAG_NO_COPY_VALUE>, eSize> m_Headers;
     const TPSG_RequestTimeout m_RequestTimeout;
     SPSG_AsyncQueue& m_Queue;
-    SPSG_UvTcp m_Tcp;
-    SPSG_NgHttp2Session m_Session;
-
     TRequests m_Requests;
 };
 
@@ -908,7 +527,7 @@ template <class TImpl>
 struct SPSG_Thread : public TImpl
 {
     template <class... TArgs>
-    SPSG_Thread(SPSG_UvBarrier& barrier, uint64_t timeout, uint64_t repeat, TArgs&&... args) :
+    SPSG_Thread(SUv_Barrier& barrier, uint64_t timeout, uint64_t repeat, TArgs&&... args) :
         TImpl(forward<TArgs>(args)...),
         m_Timer(this, s_OnTimer, timeout, repeat),
         m_Thread(s_Execute, this, ref(barrier))
@@ -937,9 +556,9 @@ private:
         io->TImpl::OnTimer(handle);
     }
 
-    static void s_Execute(SPSG_Thread* io, SPSG_UvBarrier& barrier)
+    static void s_Execute(SPSG_Thread* io, SUv_Barrier& barrier)
     {
-        SPSG_UvLoop loop;
+        SUv_Loop loop;
 
         io->TImpl::OnExecute(loop);
         io->m_Shutdown.Init(io, &loop, s_OnShutdown);
@@ -953,22 +572,24 @@ private:
         io->TImpl::AfterExecute();
     }
 
-    SPSG_UvAsync m_Shutdown;
-    SPSG_UvTimer m_Timer;
+    SUv_Async m_Shutdown;
+    SUv_Timer m_Timer;
     thread m_Thread;
 };
 
 struct SPSG_Servers : protected deque<SPSG_Server>
 {
     using TBase = deque<SPSG_Server>;
-    using TTS = SPSG_ThreadSafe<SPSG_Servers>;
+    using TTS = SThreadSafe<SPSG_Servers>;
 
     // Only methods that do not use/change size can be used directly
     using TBase::begin;
     using TBase::end;
     using TBase::operator[];
 
-    SPSG_Servers() : m_Size(0) {}
+    atomic_bool fail_requests;
+
+    SPSG_Servers() : fail_requests(false), m_Size(0) {}
 
     template <class... TArgs>
     void emplace_back(TArgs&&... args)
@@ -1024,13 +645,14 @@ private:
     }
 
     SPSG_Servers::TTS& m_Servers;
-    deque<pair<SPSG_IoSession, double>> m_Sessions;
+    deque<pair<SUvNgHttp2_Session<SPSG_IoSession>, double>> m_Sessions;
     pair<uniform_real_distribution<>, default_random_engine> m_Random;
 };
 
 struct SPSG_DiscoveryImpl
 {
-    SPSG_DiscoveryImpl(CNetServiceDiscovery service, SPSG_Servers::TTS& servers) :
+    SPSG_DiscoveryImpl(CServiceDiscovery service, SPSG_Servers::TTS& servers) :
+        m_NoServers(servers),
         m_Service(move(service)),
         m_Servers(servers)
     {}
@@ -1042,7 +664,21 @@ protected:
     void AfterExecute() {}
 
 private:
-    CNetServiceDiscovery m_Service;
+    struct SNoServers
+    {
+        SNoServers(SPSG_Servers::TTS& servers);
+
+        bool operator()(bool discovered, SUv_Timer* timer);
+
+    private:
+        const uint64_t m_RetryDelay;
+        const uint64_t m_Timeout;
+        atomic_bool& m_FailRequests;
+        uint64_t m_Passed = 0;
+    };
+
+    SNoServers m_NoServers;
+    CServiceDiscovery m_Service;
     SPSG_Servers::TTS& m_Servers;
     SPSG_ThrottleParams m_ThrottleParams;
 };
@@ -1051,20 +687,21 @@ struct SPSG_IoCoordinator
 {
     SPSG_Params params;
 
-    SPSG_IoCoordinator(CNetServiceDiscovery service);
+    SPSG_IoCoordinator(CServiceDiscovery service);
     bool AddRequest(shared_ptr<SPSG_Request> req, const atomic_bool& stopped, const CDeadline& deadline);
     string GetNewRequestId() { return to_string(m_RequestId++); }
-    const string& GetClientId() const { return m_ClientId; }
+    const string& GetUrlArgs() const { return m_UrlArgs; }
+    bool RejectsRequests() const { return m_Servers.GetMTSafe().fail_requests; }
 
 private:
-    SPSG_UvBarrier m_Barrier;
+    SUv_Barrier m_Barrier;
     SPSG_IoImpl::TSpaceCV m_Space;
     SPSG_Servers::TTS m_Servers;
     SPSG_Thread<SPSG_DiscoveryImpl> m_Discovery;
     vector<unique_ptr<SPSG_Thread<SPSG_IoImpl>>> m_Io;
     atomic<size_t> m_RequestCounter;
     atomic<size_t> m_RequestId;
-    const string m_ClientId;
+    const string m_UrlArgs;
 };
 
 END_NCBI_SCOPE
