@@ -1,4 +1,4 @@
-/*  $Id: agp_validate_reader.cpp 606864 2020-04-28 11:00:04Z ivanov $
+/*  $Id: agp_validate_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -483,14 +483,13 @@ void CAgpValidateReader::OnScaffoldEnd()
 /*
     if (m_prev_orientation=='-' &&
         m_prev_component_beg==1) {
-        auto errCode = m_AgpErr->m_strict ? 
+        auto errCode = m_AgpErr->m_strict ?
             CAgpErrEx::E_SingletonCompBeginsAt1AndMinusOri :
             CAgpErrEx::W_SingletonCompBeginsAt1AndMinusOri;
         m_AgpErr->Msg(errCode, CAgpErr::fAtPrevLine);
     }
 */
 
-   
     const bool not_plus = m_prev_orientation && m_prev_orientation!='+';
     if (m_prev_component_beg==1 && not_plus) {
         m_AgpErr->Msg(CAgpErrEx::W_SingleOriNotPlus, CAgpErr::fAtPrevLine);
@@ -887,7 +886,7 @@ void CAgpValidateReader::x_PrintTotals(CNcbiOstream& out, bool use_xml) // witho
   }
 
   if (!m_AgpErr->m_strict && m_unplaced) {
-      const int singleton_ori_not_plus_count = 
+      const int singleton_ori_not_plus_count =
           m_AgpErr->GetCount(CAgpErrEx::W_SingleOriNotPlus);
       e_count += singleton_ori_not_plus_count;
       w_count -= singleton_ori_not_plus_count;
@@ -1404,7 +1403,7 @@ bool CAgpValidateReader::x_PrintPatterns(
   bool mixedCategories=(nucCount && otherCount);
   if(mixedCategories && wPattern<20) wPattern=20;
   // Print the total
-  //string xml_tag_for_pattern_and_count = "";
+  //string xml_tag_for_pattern_and_count;
   string xml_outer_tag;
   if(strHeader.size()) {
     if(use_xml) {
@@ -1470,7 +1469,7 @@ bool CAgpValidateReader::x_PrintPatterns(
             case fProtein      : acc_warning ="looks like protein accession"; break;
           }
         }
-        string xml_tag = "";
+        string xml_tag;
         if( acc_warning.size() ) {
           xprint.m_eol_text = string(" (") + acc_warning + ")\n";
         }
