@@ -1,4 +1,4 @@
-/*  $Id: gff2_data.cpp 610645 2020-06-22 11:31:02Z ivanov $
+/*  $Id: gff2_data.cpp 632531 2021-06-02 17:25:37Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include <objects/general/Dbtag.hpp>
 #include <objects/seq/so_map.hpp>
 #include <objects/seqloc/Seq_interval.hpp>
+#include <objects/seqloc/Seq_loc_mix_.hpp>
 #include <objects/seqfeat/Cdregion.hpp>
 #include <objects/seq/Seq_annot.hpp>
 #include <objects/seq/Annot_id.hpp>
@@ -76,7 +77,7 @@ CRef<CCode_break> s_StringToCodeBreak(
     const string cdstr_start = "(pos:";
     const string cdstr_div = ",aa:";
     const string cdstr_end = ")";
-    
+
     CRef<CCode_break> pCodeBreak;
     if (!NStr::StartsWith(str, cdstr_start)  ||  !NStr::EndsWith(str, cdstr_end)) {
         return pCodeBreak;
@@ -159,10 +160,10 @@ CBioSource::EGenome s_StringToGenome(
     }
     return CBioSource::eGenome_unknown;
 }
-    
+
 //  -----------------------------------------------------------------------------
 void CGff2Record::TokenizeGFF(
-    vector<CTempStringEx>& columns, 
+    vector<CTempStringEx>& columns,
     const CTempStringEx& in_line)
 //  -----------------------------------------------------------------------------
 {
@@ -176,11 +177,11 @@ void CGff2Record::TokenizeGFF(
     columns.clear();
 
     // better to be thread-safe static
-    const CTempString space_tab_delim("\t "); 
+    const CTempString space_tab_delim("\t ");
     const CTempString digits("0123456789");
 
     size_t current = 0;
-    while (current != CTempStringEx::npos && columns.size()<8 && 
+    while (current != CTempStringEx::npos && columns.size()<8 &&
             (index = in_line.find_first_of(space_tab_delim, current)) != CTempStringEx::npos) {
         CTempStringEx next = in_line.substr(current, index-current);
         current = in_line.find_first_not_of(space_tab_delim, index);
@@ -204,7 +205,7 @@ void CGff2Record::TokenizeGFF(
                 continue;
             }
         }
-        columns.push_back(next); 
+        columns.push_back(next);
     }
     if (current != CTempStringEx::npos)
         columns.push_back(in_line.substr(current));
@@ -215,7 +216,7 @@ bool CGff2Record::AssignFromGff(
 //  ----------------------------------------------------------------------------
 {
     vector< CTempStringEx > columns;
-    
+
     TokenizeGFF(columns, strRawInput);
     if ( columns.size() < 9 ) {
         AutoPtr<CObjReaderLineException> pErr(
@@ -231,10 +232,12 @@ bool CGff2Record::AssignFromGff(
     columns[0].Copy(mSeqId, 0, CTempString::npos);
     columns[1].Copy(m_strSource, 0, CTempString::npos);
     columns[2].Copy(m_strType, 0, CTempString::npos);
+    m_strNormalizedType = m_strType;
+    NStr::ToLower(m_strNormalizedType);
 
     try {
-    m_uSeqStart = NStr::StringToUInt( columns[3] ) - 1;
-    m_uSeqStop = NStr::StringToUInt( columns[4] ) - 1;
+        m_uSeqStart = NStr::StringToUInt( columns[3] ) - 1;
+        m_uSeqStop = NStr::StringToUInt( columns[4] ) - 1;
     }
     catch (const CException&) {
         string message = "Bad data line: Both \"start\" and \"stop\" must be positive integers.";
@@ -255,12 +258,12 @@ bool CGff2Record::AssignFromGff(
     }
 
     if ( columns[5] != "." ) {
-        m_pdScore = new double( NStr::StringToDouble( columns[5] ) );
+        m_pdScore = new double( NStr::StringToDouble( columns[5], NStr::fAllowLeadingSpaces ) );
     }
 
     enum ENa_strand strand;
     switch (columns[6][0]) {
-    default: 
+    default:
         strand = objects::eNa_strand_unknown;
         break;
     case '+':
@@ -294,12 +297,12 @@ bool CGff2Record::AssignFromGff(
         m_pePhase = new TFrame(frame);
 
     columns[8].Copy(m_strAttributes, 0, CTempString::npos);
-    
+
     return xAssignAttributesFromGff(m_strType, columns[8]);
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::GetAttribute( 
+bool CGff2Record::GetAttribute(
     const string& strKey,
     string& strValue ) const
 //  ----------------------------------------------------------------------------
@@ -314,7 +317,7 @@ bool CGff2Record::GetAttribute(
 }
 
 //  ----------------------------------------------------------------------------
-bool CGff2Record::GetAttribute( 
+bool CGff2Record::GetAttribute(
     const string& strKey,
     list<string>& values ) const
 //  ----------------------------------------------------------------------------
@@ -347,7 +350,7 @@ string CGff2Record::xNormalizedAttributeValue(
     }
     if ( NStr::EndsWith( strValue, "\"" ) ) {
         strValue = strValue.substr( 0, strValue.length() - 1 );
-    }   
+    }
     return NStr::URLDecode(strValue, NStr::eUrlDec_Percent);
 }
 
@@ -431,7 +434,7 @@ bool x_GetNextAttribute(CTempString& input, CTempString& key, CTempString& value
             break;
         }
     }
-    
+
     if (semicolon == CTempString::npos)
         semicolon = input.length();
 
@@ -467,38 +470,38 @@ bool CGff2Record::xAssignAttributesFromGff(
 //  ----------------------------------------------------------------------------
 bool CGff2Record::xSplitGffAttributes(
     const string& strRawAttributes,
-	vector< string >& attributes) const
+    vector< string >& attributes) const
 //  ----------------------------------------------------------------------------
 {
-	string strCurrAttrib;
-	bool inQuotes = false;
+    string strCurrAttrib;
+    bool inQuotes = false;
 
-	ITERATE (string, iterChar, strRawAttributes) {
-		if (inQuotes) {
-			if (*iterChar == '\"') {
-				inQuotes = false;
-			}  
-			strCurrAttrib += *iterChar;
-		} else { // not in quotes
-			if (*iterChar == ';') {
-				NStr::TruncateSpacesInPlace( strCurrAttrib );
-				if(!strCurrAttrib.empty())
-					attributes.push_back(strCurrAttrib);
-				strCurrAttrib.clear();
-			} else {
-				if(*iterChar == '\"') {
-					inQuotes = true;
-				}
-				strCurrAttrib += *iterChar;
-			}
-		}
-	}
+    ITERATE (string, iterChar, strRawAttributes) {
+        if (inQuotes) {
+            if (*iterChar == '\"') {
+                inQuotes = false;
+            }
+            strCurrAttrib += *iterChar;
+        } else { // not in quotes
+            if (*iterChar == ';') {
+                NStr::TruncateSpacesInPlace( strCurrAttrib );
+                if(!strCurrAttrib.empty())
+                    attributes.push_back(strCurrAttrib);
+                strCurrAttrib.clear();
+            } else {
+                if(*iterChar == '\"') {
+                    inQuotes = true;
+                }
+                strCurrAttrib += *iterChar;
+            }
+        }
+    }
 
-	NStr::TruncateSpacesInPlace( strCurrAttrib );
-	if (!strCurrAttrib.empty())
-		attributes.push_back(strCurrAttrib);
+    NStr::TruncateSpacesInPlace( strCurrAttrib );
+    if (!strCurrAttrib.empty())
+        attributes.push_back(strCurrAttrib);
 
-	return true;
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
@@ -522,31 +525,10 @@ bool CGff2Record::UpdateFeature(
     //  ----------------------------------------------------------------------------
 {
     auto subtype = pFeature->GetData().GetSubtype();
-    auto recType = Type();
-    NStr::ToLower(recType);
-
-    const CSeq_loc& target = pFeature->GetLocation();
+    auto recType = NormalizedType();
     CRef<CSeq_loc> pAddLoc = GetSeqLoc(flags, seqidresolve);
 
-    if (target.IsInt()  &&  target.GetInt().GetFrom() <= SeqStart()  &&
-        target.GetInt().GetTo() >= SeqStop() ) {
-        if (recType == "start_codon"  ||  recType == "stop_codon") {
-            return true;
-        }
-        // indicates current feature location is a placeholder interval to be
-        //  totally overwritten by the constituent sub-intervals
-        pFeature->SetLocation().SetMix().AddSeqLoc(*pAddLoc);
-    }
-    else {
-        // indicates the feature location is already under construction
-        pFeature->SetLocation(*pFeature->SetLocation().Add(
-            *pAddLoc, CSeq_loc::fSort | CSeq_loc::fMerge_Abutting, 0));
-        if (pFeature->GetLocation().IsInt()) {
-            CRef<CSeq_loc> pOld(new CSeq_loc);
-            pOld->Assign(pFeature->GetLocation());
-            pFeature->SetLocation().SetMix().AddSeqLoc(*pOld);
-        }
-    }
+    pFeature->SetLocation().SetMix().AddSeqLoc(*pAddLoc);
     if (!xUpdateFeatureData(flags, pFeature)) {
         return false;
     }
@@ -602,14 +584,14 @@ bool CGff2Record::xUpdateFeatureData(
     }
     return true;
 }
-    
+
 //  ----------------------------------------------------------------------------
 bool CGff2Record::xMigrateAttributes(
     int flags,
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
-    TAttributes attrs_left(m_Attributes.begin(), m_Attributes.end()); 
+    TAttributes attrs_left(m_Attributes.begin(), m_Attributes.end());
     TAttrIt it;
 
     it = attrs_left.find("Note");
@@ -651,10 +633,11 @@ bool CGff2Record::xMigrateAttributes(
 
     it = attrs_left.find("Name");
     if (it != attrs_left.end()) {
-        if (0 == NStr::CompareNocase(Type(), "cds")) {
-            attrs_left.erase(it);
-        }
-        if (0 == NStr::CompareNocase(Type(), "mRNA")) {
+        auto soType = NormalizedType();
+        string gbKey;
+        GetAttribute("gbkey", gbKey);
+        if (soType == "cds"  ||  soType == "mrna"  ||  soType == "biological_region"  ||
+                (soType == "region"  &&  gbKey != "Src")) {
             attrs_left.erase(it);
         }
     }
@@ -869,7 +852,7 @@ bool CGff2Record::xMigrateAttributes(
     if (it != attrs_left.end()) {
         if (pFeature->GetData().IsImp()  &&  pFeature->GetData().GetImp().GetKey() == "repeat_region") {
             attrs_left.erase(it);
-        }           
+        }
     }
 
     it = attrs_left.find("transl_except");
@@ -877,7 +860,7 @@ bool CGff2Record::xMigrateAttributes(
         if (pFeature->GetData().IsCdregion()) {
             vector<string> codebreaks;
             NStr::Split(it->second, ",", codebreaks, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
-            for (vector<string>::iterator it1 = codebreaks.begin(); 
+            for (vector<string>::iterator it1 = codebreaks.begin();
                     it1 != codebreaks.end(); ++it1 ) {
                 string breakData = xNormalizedAttributeValue(*it1);
                 CRef<CSeq_id> pBreakId = GetSeqId(flags);
@@ -903,10 +886,10 @@ bool CGff2Record::xMigrateAttributes(
     }
 
     if (!xMigrateAttributesGo(flags, pFeature, attrs_left)) {
-		return false;
-	}
+        return false;
+    }
 
-    if (pFeature->GetData().IsBiosrc()) { 
+    if (pFeature->GetData().IsBiosrc()) {
         if (!xMigrateAttributesSubSource(flags, pFeature, attrs_left)) {
             return false;
         }
@@ -1031,7 +1014,7 @@ bool CGff2Record::xMigrateAttributesOrgName(
     }
     list<CRef<COrgMod> >& orgMod =
         pFeature->SetData().SetBiosrc().SetOrg().SetOrgname().SetMod();
-    for ( ORGMOD_MAP::const_iterator sit = sOrgModMap.begin(); 
+    for ( ORGMOD_MAP::const_iterator sit = sOrgModMap.begin();
             sit != sOrgModMap.end(); ++sit) {
         TAttributes::iterator ait = attrs_left.find(sit->first);
         if (ait == attrs_left.end()) {
@@ -1128,7 +1111,7 @@ bool CGff2Record::xMigrateAttributesSubSource(
 
     list<CRef<CSubSource> >& subType =
         pFeature->SetData().SetBiosrc().SetSubtype();
-    for ( SUBSOURCE_MAP::const_iterator sit = sSubSourceMap.begin(); 
+    for ( SUBSOURCE_MAP::const_iterator sit = sSubSourceMap.begin();
             sit != sSubSourceMap.end(); ++sit) {
         TAttributes::iterator ait = attrs_left.find(sit->first);
         if (ait == attrs_left.end()) {
@@ -1149,28 +1132,31 @@ bool CGff2Record::xInitFeatureData(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
-    if (Type() == "region") {
+    auto recognizedType = NormalizedType();
+
+    if (recognizedType == "region"  ||  recognizedType == "biological_region") {
         string gbkey;
         if (GetAttribute("gbkey", gbkey)) {
             if (gbkey == "Src") {
                 pFeature->SetData().SetBiosrc();
                 return true;
             }
-            // regardless of gbkey (rw-1062)
-            pFeature->SetData().SetRegion();
-            return true;
-        }
+         }
+        // regardless of gbkey (rw-1062)
+        string name;
+        GetAttribute("Name", name);
+        pFeature->SetData().SetRegion(name);
+        return true;
     }
 
-    auto recognizedType = Type();
     if (recognizedType == "start_codon"  || recognizedType == "stop_codon") {
         recognizedType = "cds";
     }
 
     bool invalidFeaturesToRegion = !(flags & CGff2Reader::fGenbankMode);
     if (!CSoMap::SoTypeToFeature(
-        recognizedType, *pFeature, invalidFeaturesToRegion)) {
-        string message = "Bad data line: Invalid feature type \"" + recognizedType + "\"";
+        Type(), *pFeature, invalidFeaturesToRegion)) {
+        string message = "Bad data line: Invalid feature type \"" + Type() + "\"";
         CReaderMessage error(
             eDiag_Error,
             0,
@@ -1180,5 +1166,17 @@ bool CGff2Record::xInitFeatureData(
     return CGffBaseColumns::xInitFeatureData(flags, pFeature);
 }
 
+//  ============================================================================
+bool
+CGff2Record::IsMultiParent() const
+//  =============================================================================
+{
+    list<string> parentIds;
+    if (!GetAttribute("Parent", parentIds)) {
+        return false;
+    }
+    return (parentIds.size() > 1);
+}
+
 END_objects_SCOPE
 END_NCBI_SCOPE
