@@ -1,4 +1,4 @@
-/*  $Id: gff2_reader.cpp 610837 2020-06-24 15:29:29Z ivanov $
+/*  $Id: gff2_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -86,7 +86,8 @@ CGff2Reader::CGff2Reader(
     CReaderBase(iFlags, name, title, resolver, pRL),
     m_pErrors(0),
     mCurrentFeatureCount(0),
-    mParsingAlignment(false)
+    mParsingAlignment(false),
+    mAtSequenceData(false)
 {
 }
 
@@ -96,7 +97,7 @@ CGff2Reader::~CGff2Reader()
 {
 }
 
-//  --------------------------------------------------------------------------- 
+//  ---------------------------------------------------------------------------
 void
 CGff2Reader::ReadSeqAnnots(
     TAnnots& annots,
@@ -107,8 +108,8 @@ CGff2Reader::ReadSeqAnnots(
     CStreamLineReader lr( istr );
     ReadSeqAnnots( annots, lr, pMessageListener );
 }
- 
-//  ---------------------------------------------------------------------------                       
+
+//  ---------------------------------------------------------------------------
 void
 CGff2Reader::ReadSeqAnnots(
     TAnnots& annots,
@@ -117,7 +118,7 @@ CGff2Reader::ReadSeqAnnots(
 //  ----------------------------------------------------------------------------
 {
     xProgressInit(lr);
-    while (!lr.AtEOF()) {
+    while (!lr.AtEOF()  &&  !mAtSequenceData) {
         CRef<CSeq_annot> pNext = this->ReadSeqAnnot(lr, pEC);
         if (pNext) {
             annots.push_back(pNext);
@@ -126,18 +127,18 @@ CGff2Reader::ReadSeqAnnots(
     return;
 }
 
-//  ----------------------------------------------------------------------------                
+//  ----------------------------------------------------------------------------
 CRef< CSeq_entry >
 CGff2Reader::ReadSeqEntry(
     ILineReader& lr,
-    ILineErrorListener* pMessageListener ) 
-//  ----------------------------------------------------------------------------                
-{ 
+    ILineErrorListener* pMessageListener )
+//  ----------------------------------------------------------------------------
+{
     xProgressInit(lr);
 
     TAnnots annots;
     ReadSeqAnnots( annots, lr, pMessageListener );
-    
+
     CRef<CSeq_entry> pSeqEntry(new CSeq_entry());
     pSeqEntry->SetSet();
 
@@ -145,7 +146,7 @@ CGff2Reader::ReadSeqEntry(
             it != annots.end(); ++it) {
         CRef<CBioseq> pSeq( new CBioseq() );
         pSeq->SetAnnot().push_back(*it);
-        pSeq->SetId().push_back( CRef<CSeq_id>( 
+        pSeq->SetId().push_back( CRef<CSeq_id>(
             new CSeq_id(CSeq_id::e_Local, "gff-import") ) );
         pSeq->SetInst().SetRepr(CSeq_inst::eRepr_not_set);
         pSeq->SetInst().SetMol(CSeq_inst::eMol_not_set);
@@ -157,18 +158,18 @@ CGff2Reader::ReadSeqEntry(
     return pSeqEntry;
 }
 
-//  ----------------------------------------------------------------------------                
+//  ----------------------------------------------------------------------------
 CRef< CSerialObject >
 CGff2Reader::ReadObject(
     ILineReader& lr,
-    ILineErrorListener* pMessageListener ) 
-//  ----------------------------------------------------------------------------                
-{ 
-    CRef<CSerialObject> object( 
+    ILineErrorListener* pMessageListener )
+//  ----------------------------------------------------------------------------
+{
+    CRef<CSerialObject> object(
         ReadSeqEntry( lr, pMessageListener ).ReleaseOrNull() );
     return object;
 }
- 
+
 //  ----------------------------------------------------------------------------
 void CGff2Reader::xPostProcessAnnot(
     CSeq_annot& annot)
@@ -191,32 +192,46 @@ CGff2Reader::xGetData(
 {
     readerData.clear();
     string line;
-    if (xGetLine(lr, line)) {
-        if (xNeedsNewSeqAnnot(line)) {
-            return;
-        }
-        if (xIsTrackLine(line)) {
-            if (!mCurrentFeatureCount) {
-                xParseTrackLine(line);
-                xGetData(lr, readerData);
-                return;
-            }
-            m_PendingLine = line;
-            return;
-        }
-        if (xIsTrackTerminator(line)) {
-            if (!mCurrentFeatureCount) {
-                xParseTrackLine("track");
-                xGetData(lr, readerData);
-            }
-            return;
-        }
-        if (!xIsCurrentDataType(line)) {
-            xUngetLine(lr);
-            return;
-        }
-    readerData.push_back(TReaderLine{m_uLineNumber, line});
+    if (!xGetLine(lr, line)) {
+        return;
     }
+    if (xNeedsNewSeqAnnot(line)) {
+        return;
+    }
+    if (xIsTrackLine(line)) {
+        if (!mCurrentFeatureCount) {
+            xParseTrackLine(line);
+            xGetData(lr, readerData);
+            return;
+        }
+        m_PendingLine = line;
+        return;
+    }
+    if (xIsTrackTerminator(line)) {
+        if (!mCurrentFeatureCount) {
+            xParseTrackLine("track");
+            xGetData(lr, readerData);
+        }
+        return;
+    }
+    if (xIsSequenceRegion(line)) {
+        xProcessSequenceRegionPragma(line);
+        if (!mCurrentFeatureCount) {
+            xParseTrackLine("track");
+            xGetData(lr, readerData);
+        }
+        return;
+    }
+    if (xIsFastaMarker(line)) {
+        mAtSequenceData = true;
+        readerData.clear();
+        return;
+    }
+    if (!xIsCurrentDataType(line)) {
+        xUngetLine(lr);
+        return;
+    }
+    readerData.push_back(TReaderLine{m_uLineNumber, line});
     ++m_uDataCount;
 }
 
@@ -231,17 +246,17 @@ void CGff2Reader::xAssignAnnotId(
     }
 
     string annotId(givenId);
-    if (annotId.empty() && 
+    if (annotId.empty() &&
         (!(m_iFlags & fGenbankMode)) &&
         m_pTrackDefaults) {
         annotId = m_pTrackDefaults->Name();
     }
     if (annotId.empty()) {
         return;
-    }    
+    }
     CRef< CAnnot_id > pAnnotId(new CAnnot_id);
     pAnnotId->SetLocal().SetStr(annotId);
-    annot.SetId().push_back(pAnnotId);   
+    annot.SetId().push_back(pAnnotId);
 }
 
 
@@ -275,8 +290,8 @@ CGff2Reader::xParseFeature(
     shared_ptr<CGff2Record> pRecord(x_CreateRecord());
     try {
         if (!pRecord->AssignFromGff(line)) {
-			return false;
-		}
+            return false;
+        }
     }
     catch(CObjReaderLineException& err) {
         ProcessError(err, pEC);
@@ -303,7 +318,7 @@ CGff2Reader::xParseFeature(
 
 
 //  ----------------------------------------------------------------------------
-void CGff2Reader::x_GetAlignmentScores(const CSeq_align& alignment, 
+void CGff2Reader::x_GetAlignmentScores(const CSeq_align& alignment,
                                        TScoreValueMap& score_values) const
 //  ----------------------------------------------------------------------------
 {
@@ -330,12 +345,12 @@ void CGff2Reader::x_GetAlignmentScores(const CSeq_align& alignment,
 
 
 //  ----------------------------------------------------------------------------
-bool s_CompareValues(const CScore::TValue& score_val1, 
+bool s_CompareValues(const CScore::TValue& score_val1,
                      const CScore::TValue& score_val2)
 //  ----------------------------------------------------------------------------
 {
 
-    if (score_val1.IsInt() && 
+    if (score_val1.IsInt() &&
         score_val2.IsInt() &&
         score_val1.GetInt() == score_val2.GetInt()) {
         return true;
@@ -375,14 +390,14 @@ void CGff2Reader::x_FindMatchingScores(const TScoreValueMap& scores_1,
 //  ----------------------------------------------------------------------------
 void CGff2Reader::x_ProcessAlignmentsGff(const list<string>& id_list,
                             const map<string, list<CRef<CSeq_align>>>& alignments,
-                            CRef<CSeq_annot> pAnnot) 
+                            CRef<CSeq_annot> pAnnot)
 //  ----------------------------------------------------------------------------
 {
     if (pAnnot.IsNull()) {
         pAnnot = Ref(new CSeq_annot());
     }
 
-    for (const auto id : id_list) {
+    for (const string& id : id_list) {
         CRef<CSeq_align> pAlign = Ref(new CSeq_align());
         if (x_MergeAlignments(alignments.at(id), pAlign)) {
             // if available, add current browser information
@@ -404,7 +419,7 @@ void CGff2Reader::x_ProcessAlignmentsGff(const list<string>& id_list,
 
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::x_ParseAlignmentGff(
-    const string& strLine, 
+    const string& strLine,
     list<string>& id_list, // Add id to alignment
     map<string, list<CRef<CSeq_align>>>& alignments)
 //  ----------------------------------------------------------------------------
@@ -414,7 +429,7 @@ bool CGff2Reader::x_ParseAlignmentGff(
     if ( !pRecord->AssignFromGff(strLine) ) {
         return false;
     }
-    
+
     string id;
     if ( !pRecord->GetAttribute("ID", id) ) {
         id = pRecord->Id();
@@ -439,23 +454,23 @@ bool CGff2Reader::x_ParseAlignmentGff(
 
 
 //  ----------------------------------------------------------------------------
-void CGff2Reader::x_InitializeScoreSums(const TScoreValueMap score_values, 
+void CGff2Reader::x_InitializeScoreSums(const TScoreValueMap score_values,
         map<string, TSeqPos>& summed_scores) const
 //  ----------------------------------------------------------------------------
 {
     const list<string> score_names {"num_ident", "num_mismatch"};
-    
+
     for (const string& score_name : score_names) {
         if (score_values.find(score_name) != score_values.end()) {
             summed_scores[score_name] = score_values.at(score_name)->GetInt();
-        }    
+        }
     }
 }
 
 
 //  ----------------------------------------------------------------------------
-void CGff2Reader::x_ProcessAlignmentScores(const CSeq_align& alignment, 
-    map<string, TSeqPos>& summed_scores, 
+void CGff2Reader::x_ProcessAlignmentScores(const CSeq_align& alignment,
+    map<string, TSeqPos>& summed_scores,
     TScoreValueMap& common_scores) const
 //  ----------------------------------------------------------------------------
 {
@@ -474,11 +489,11 @@ void CGff2Reader::x_ProcessAlignmentScores(const CSeq_align& alignment,
     }
 
     set<string> matching_score_names;
-    x_FindMatchingScores(common_scores, 
-        new_scores, 
+    x_FindMatchingScores(common_scores,
+        new_scores,
         matching_score_names);
 
-    common_scores.clear(); 
+    common_scores.clear();
     for (string score_name : matching_score_names) {
         common_scores[score_name] = Ref(new CScore::TValue());
         common_scores[score_name]->Assign(*new_scores[score_name]);
@@ -509,7 +524,7 @@ bool CGff2Reader::x_MergeAlignments(
     TScoreValueMap score_values;
     x_GetAlignmentScores(**align_it, score_values);
 
-    x_InitializeScoreSums(score_values, 
+    x_InitializeScoreSums(score_values,
         summed_scores);
     ++align_it;
 
@@ -519,13 +534,13 @@ bool CGff2Reader::x_MergeAlignments(
         x_ProcessAlignmentScores(**align_it, summed_scores, score_values);
         ++align_it;
     }
-    // At this point, the score_values map should contain the scores that 
+    // At this point, the score_values map should contain the scores that
     // do not change over the rows
 
     const auto first_alignment = alignment_list.front();
-    if (first_alignment->IsSetSegs() && 
+    if (first_alignment->IsSetSegs() &&
         first_alignment->GetSegs().IsSpliced()) {
-        
+
         processed->SetType(CSeq_align::eType_global);
 
         if (first_alignment->IsSetDim()) {
@@ -613,18 +628,21 @@ CGff2Reader::xIsCurrentDataType(
 
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::xUpdateAnnotFeature(
-    const CGff2Record&,
-    CSeq_annot&,
+    const CGff2Record& record,
+    CSeq_annot& annot,
     ILineErrorListener*)
 //  ----------------------------------------------------------------------------
 {
-    return false;
+    CRef<CSeq_feat> pFeat(new CSeq_feat);
+    record.InitializeFeature(m_iFlags, pFeat);
+    xAddFeatureToAnnot(pFeat, annot);
+    return true;
 }
 
 
 bool CGff2Reader::x_CreateAlignment(
-        const CGff2Record& gff, 
-        CRef<CSeq_align>& pAlign ) 
+        const CGff2Record& gff,
+        CRef<CSeq_align>& pAlign )
 {
     pAlign = Ref(new CSeq_align());
     pAlign->SetType(CSeq_align::eType_partial);
@@ -685,13 +703,12 @@ bool CGff2Reader::xUpdateSplicedAlignment(const CGff2Record& gff,
 
 
 bool CGff2Reader::xUpdateSplicedSegment(
-        const CGff2Record& gff, 
+        const CGff2Record& gff,
         CSpliced_seg& segment) const
 {
     if (segment.IsSetProduct_type()) {
         segment.SetProduct_type(CSpliced_seg::eProduct_type_transcript);
     }
-    
 
     CRef<CSpliced_exon> pExon = Ref(new CSpliced_exon());
     if (!xSetSplicedExon(gff, pExon)) {
@@ -707,7 +724,7 @@ bool CGff2Reader::xUpdateSplicedSegment(
 
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::xSetSplicedExon(
-        const CGff2Record& gff, 
+        const CGff2Record& gff,
         CRef<CSpliced_exon> pExon) const
 //  ----------------------------------------------------------------------------
 {
@@ -754,13 +771,13 @@ bool CGff2Reader::xGetTargetParts(const CGff2Record& gff, vector<string>& target
     if (targetParts.size() != 4) {
         return false;
     }
-   
+
     return true;
 }
 
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::xGetStartsOnMinusStrand(TSeqPos offset, 
+bool CGff2Reader::xGetStartsOnMinusStrand(TSeqPos offset,
         const vector<string>& gapParts,
         const bool isTarget,
         vector<int>& starts) const
@@ -776,7 +793,7 @@ bool CGff2Reader::xGetStartsOnMinusStrand(TSeqPos offset,
         default:
             return false;
 
-        case 'M': 
+        case 'M':
             starts.push_back(offset+1-changeSize);
             offset -= changeSize;
             break;
@@ -805,7 +822,7 @@ bool CGff2Reader::xGetStartsOnMinusStrand(TSeqPos offset,
 
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::xGetStartsOnPlusStrand(TSeqPos offset, 
+bool CGff2Reader::xGetStartsOnPlusStrand(TSeqPos offset,
         const vector<string>& gapParts,
         const bool isTarget,
         vector<int>& starts) const
@@ -821,7 +838,7 @@ bool CGff2Reader::xGetStartsOnPlusStrand(TSeqPos offset,
         default:
             return false;
 
-        case 'M': 
+        case 'M':
             starts.push_back(offset);
             offset += changeSize;
             break;
@@ -850,7 +867,7 @@ bool CGff2Reader::xGetStartsOnPlusStrand(TSeqPos offset,
 
 
 //  ----------------------------------------------------------------------------
-bool CGff2Reader::xSetDensegStarts(const vector<string>& gapParts, 
+bool CGff2Reader::xSetDensegStarts(const vector<string>& gapParts,
                                    const ENa_strand identStrand,
                                    const ENa_strand targetStrand,
                                    const TSeqPos targetStart,
@@ -872,7 +889,7 @@ bool CGff2Reader::xSetDensegStarts(const vector<string>& gapParts,
         }
     }
     else {
-        if (!xGetStartsOnPlusStrand(targetStart, 
+        if (!xGetStartsOnPlusStrand(targetStart,
             gapParts,
             isTarget,
             targetStarts)) {
@@ -919,8 +936,8 @@ bool CGff2Reader::xAlignmentSetSegment(
 {
     const string& type = gff.Type();
 
-    if (type == "cDNA_match" || 
-        type == "EST_match"  || 
+    if (type == "cDNA_match" ||
+        type == "EST_match"  ||
         type == "translated_nucleotide_match") {
         return xAlignmentSetSpliced_seg(gff, pAlign);
     }
@@ -941,13 +958,13 @@ bool CGff2Reader::xAlignmentSetSpliced_seg(
     }
 
     CSeq_align::TSegs& segs = pAlign->SetSegs();
-    
+
     auto& spliced_seg = segs.SetSpliced();
 
     const string& type = gff.Type();
     if (type == "translated_nucleotide_match") {
         spliced_seg.SetProduct_type(CSpliced_seg::eProduct_type_protein);
-    } 
+    }
     else {
         spliced_seg.SetProduct_type(CSpliced_seg::eProduct_type_transcript);
     }
@@ -996,7 +1013,7 @@ bool CGff2Reader::xAlignmentSetSpliced_seg(
         default:
             return false;
 
-        case 'M': 
+        case 'M':
             chunk->SetMatch(changeSize);
             break;
 
@@ -1066,7 +1083,7 @@ bool CGff2Reader::xAlignmentSetDenseg(
     const TSeqPos targetStart = NStr::StringToInt(targetParts[1])-1;
     const TSeqPos targetEnd   = NStr::StringToInt(targetParts[2])-1;
 
-    if (!xSetDensegStarts(gapParts, 
+    if (!xSetDensegStarts(gapParts,
                           identStrand,
                           targetStrand,
                           targetStart,
@@ -1098,7 +1115,7 @@ bool CGff2Reader::xAlignmentSetScore(
 //  ----------------------------------------------------------------------------
 {
     if (gff.IsSetScore()) {
-        pAlign->SetNamedScore(CSeq_align::eScore_Score, 
+        pAlign->SetNamedScore(CSeq_align::eScore_Score,
             int(gff.Score()));
     }
 
@@ -1168,7 +1185,7 @@ bool CGff2Reader::x_ProcessQualifierSpecialCase(
 //  ----------------------------------------------------------------------------
 {
     return false;
-}  
+}
 
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::xFeatureSetQualifier(
@@ -1186,16 +1203,16 @@ bool CGff2Reader::xFeatureSetQualifier(
 
 //  ----------------------------------------------------------------------------
 bool CGff2Reader::x_GetFeatureById(
-    const string & strId, 
+    const string & strId,
     ncbi::CRef<CSeq_feat>& pFeature )
 //  ----------------------------------------------------------------------------
 {
     map< string, CRef< CSeq_feat > >::iterator it;
     it = m_MapIdToFeature.find(strId);
-	if(it != m_MapIdToFeature.end()) {
+    if(it != m_MapIdToFeature.end()) {
         pFeature = it->second;
-		return true;
-	}
+        return true;
+    }
     return false;
 }
 
@@ -1272,7 +1289,7 @@ bool CGff2Reader::xGenerateParentChildXrefs(
         PARENTS parents;
         NStr::Split(parentStr, ",", parents, 0);
         for (auto parentIt = parents.begin(); parentIt != parents.end(); ++parentIt) {
-            const string& parent = *parentIt; 
+            const string& parent = *parentIt;
             xSetAncestryLine(feat, parent);
         }
     }
@@ -1314,42 +1331,66 @@ bool sFeatureHasXref(
     if (!feat.IsSetXref()) {
         return false;
     }
-    int xrefId = featId.GetLocal().GetId();
-    const XREFS& xrefs = feat.GetXref();
-    for (XREFS::const_iterator cit = xrefs.begin(); cit != xrefs.end(); ++cit) {
-        const CSeqFeatXref& ref = **cit; 
-        int contentId = ref.GetId().GetLocal().GetId();
-        if (contentId == xrefId) {
-            return true;
+    if (!featId.IsLocal()) {
+        return false;
+    }
+    const auto& local = featId.GetLocal();
+    if (local.IsId()) {
+        auto xrefId = local.GetId();
+        const XREFS& xrefs = feat.GetXref();
+        for (XREFS::const_iterator cit = xrefs.begin(); cit != xrefs.end(); ++cit) {
+            const CSeqFeatXref& ref = **cit;
+            if (!ref.GetId().IsLocal()  ||  !ref.GetId().GetLocal().IsId()) {
+                continue;
+            }
+            auto contentId = ref.GetId().GetLocal().GetId();
+            if (contentId == xrefId) {
+                return true;
+            }
         }
+        return false;
+    }
+    if (local.IsStr()) {
+        auto xrefId = local.GetStr();
+        const XREFS& xrefs = feat.GetXref();
+        for (XREFS::const_iterator cit = xrefs.begin(); cit != xrefs.end(); ++cit) {
+            const CSeqFeatXref& ref = **cit;
+            if (!ref.GetId().IsLocal()  ||  !ref.GetId().GetLocal().IsStr()) {
+                continue;
+            }
+            auto contentId = ref.GetId().GetLocal().GetStr();
+            if (contentId == xrefId) {
+                return true;
+            }
+        }
+        return false;
     }
     return false;
 }
 
+//  ============================================================================
+void CGff2Reader::xSetXrefFromTo(
+    CSeq_feat& from,
+    CSeq_feat& to)
+//  ============================================================================
+{
+    if (!sFeatureHasXref(from, to.GetId())) {
+        CRef<CFeat_id> pToId(new CFeat_id);
+        pToId->Assign(to.GetId());
+        CRef<CSeqFeatXref> pToXref(new CSeqFeatXref);
+        pToXref->SetId(*pToId);
+        from.SetXref().push_back(pToXref);
+    }
+}
+
 //  ============================================================================
 void CGff2Reader::xSetAncestorXrefs(
     CSeq_feat& descendent,
     CSeq_feat& ancestor)
 //  ============================================================================
 {
-
-    //xref descendent->ancestor
-    if (!sFeatureHasXref(descendent, ancestor.GetId())) {
-        CRef<CFeat_id> pAncestorId(new CFeat_id);
-        pAncestorId->Assign(ancestor.GetId());
-        CRef<CSeqFeatXref> pAncestorXref(new CSeqFeatXref);
-        pAncestorXref->SetId(*pAncestorId);
-        descendent.SetXref().push_back(pAncestorXref);
-    }
-
-    //xref ancestor->descendent
-    if (!sFeatureHasXref(ancestor, descendent.GetId())) {
-        CRef<CFeat_id> pDescendentId(new CFeat_id);
-        pDescendentId->Assign(descendent.GetId());
-        CRef<CSeqFeatXref> pDescendentXref(new CSeqFeatXref);
-        pDescendentXref->SetId(*pDescendentId);
-        ancestor.SetXref().push_back(pDescendentXref);
-    }
+    xSetXrefFromTo(descendent, ancestor);
+    xSetXrefFromTo(ancestor, descendent);
 }
 
 //  ============================================================================
@@ -1362,7 +1403,7 @@ bool CGff2Reader::IsAlignmentData(
     if (columns.size() < 9) {
         return false;
     }
-    if (NStr::StartsWith(columns[2], "match") || 
+    if (NStr::StartsWith(columns[2], "match") ||
         NStr::EndsWith(columns[2], "_match")) {
         return true;
     }
@@ -1415,6 +1456,46 @@ bool CGff2Reader::IsInGenbankMode() const
     return (m_iFlags & CGff2Reader::fGenbankMode);
 }
 
+//  -------------------------------------------------------------------------------
+bool CGff2Reader::xIsSequenceRegion(
+    const string& line)
+//  -------------------------------------------------------------------------------
+{
+    string lineLowerCase(line);
+    NStr::ToLower(lineLowerCase);
+    return NStr::StartsWith(lineLowerCase, "##sequence-region");
+}
+
+//  -------------------------------------------------------------------------------
+bool CGff2Reader::xIsFastaMarker(
+    const string& line)
+//  -------------------------------------------------------------------------------
+{
+    string lineLowerCase(line);
+    NStr::ToLower(lineLowerCase);
+    return NStr::StartsWith(lineLowerCase, "##fasta");
+}
+
+//  ----------------------------------------------------------------------------
+void
+CGff2Reader::xProcessData(
+    const TReaderData& readerData,
+    CSeq_annot& annot) 
+//  ----------------------------------------------------------------------------
+{
+    for (const auto& lineData: readerData) {
+        const auto& line = lineData.mData;
+        if (xParseStructuredComment(line)) {
+            continue;
+        }
+        if (xParseBrowserLine(line, annot)) {
+            continue;
+        }
+        if (xParseFeature(line, annot, nullptr)) {
+            continue;
+        }
+    }
+}
 
 
 END_objects_SCOPE
