@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 610837 2020-06-24 15:29:29Z ivanov $
+/*  $Id: gff3_reader.cpp 632532 2021-06-02 17:25:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -57,6 +57,8 @@
 #include <objects/seqfeat/Feat_id.hpp>
 
 #include <objtools/readers/gff3_reader.hpp>
+#include "gff3_location_merger.hpp"
+
 #include "reader_message_handler.hpp"
 
 #include <algorithm>
@@ -66,6 +68,32 @@ BEGIN_objects_SCOPE
 
 unsigned int CGff3Reader::msGenericIdCounter = 0;
 
+//  ----------------------------------------------------------------------------
+string CGff3Reader::xMakeRecordId(
+    const CGff2Record& record)
+//  ----------------------------------------------------------------------------
+{
+    string id, parentId;
+    record.GetAttribute("ID", id);
+    record.GetAttribute("Parent", parentId);
+
+    auto recordType = record.NormalizedType();
+    if (recordType == "cds") {
+        string cdsId = parentId;
+        if (cdsId.empty()) {
+            cdsId = (id.empty() ? xNextGenericId() : id);
+        }
+        else {
+            cdsId += ":cds";
+        }
+        return cdsId;
+    }
+    if (id.empty()) {
+        return xNextGenericId();
+    }
+    return id;
+}
+
 //  ----------------------------------------------------------------------------
 string CGff3Reader::xNextGenericId()
 //  ----------------------------------------------------------------------------
@@ -81,37 +109,43 @@ bool CGff3ReadRecord::AssignFromGff(
     if (!CGff2Record::AssignFromGff(strRawInput)) {
         return false;
     }
+    string id, parent;
+    GetAttribute("ID", id);
+    GetAttribute("Parent", parent);
+    if (id.empty()  &&  parent.empty()) {
+        m_Attributes["ID"] = CGff3Reader::xNextGenericId();
+    }
     if (m_strType == "pseudogene") {
-        m_strType = "gene";
+        SetType("gene");
         m_Attributes["pseudo"] = "true";
         return true;
     }
     if (m_strType == "pseudogenic_transcript") {
-        m_strType = "transcript";
+        SetType("transcript");
         m_Attributes["pseudo"] = "true";
         return true;
-    }        
+    }
     if (m_strType == "pseudogenic_tRNA") {
-        m_strType = "tRNA";
+        SetType("tRNA");
         m_Attributes["pseudo"] = "true";
         return true;
     }
     if (m_strType == "pseudogenic_rRNA") {
-        m_strType = "rRNA";
+        SetType("rRNA");
         m_Attributes["pseudo"] = "true";
         return true;
     }
     if (m_strType == "pseudogenic_exon") {
-        m_strType = "exon";
+        SetType("exon");
         return true;
     }
     if (m_strType == "pseudogenic_CDS") {
-        m_strType = "CDS";
+        SetType("CDS");
         m_Attributes["pseudo"] = "true";
         return true;
     }
     if (m_strType == "transcript") {
-        m_strType = "misc_RNA";
+        SetType("misc_RNA");
         return true;
     }
     return true;
@@ -167,6 +201,7 @@ CGff3Reader::CGff3Reader(
 //  ----------------------------------------------------------------------------
     CGff2Reader( uFlags, name, title, resolver, pRL )
 {
+    mpLocations.reset(new CGff3LocationMerger(uFlags, resolver, 0));
     CGff2Record::ResetId();
 }
 
@@ -185,16 +220,17 @@ CGff3Reader::~CGff3Reader()
 {
 }
 
-//  ----------------------------------------------------------------------------                
+//  ----------------------------------------------------------------------------
 CRef<CSeq_annot>
 CGff3Reader::ReadSeqAnnot(
     ILineReader& lr,
-    ILineErrorListener* pEC ) 
-//  ----------------------------------------------------------------------------                
+    ILineErrorListener* pEC )
+//  ----------------------------------------------------------------------------
 {
- 
     mCurrentFeatureCount = 0;
     mParsingAlignment = false;
+    mAlignmentData.Reset();
+    mpLocations->Reset();
     auto pAnnot = CReaderBase::ReadSeqAnnot(lr, pEC);
     if (pAnnot  &&  pAnnot->GetData().Which() == CSeq_annot::TData::e_not_set) {
         return CRef<CSeq_annot>();
@@ -206,12 +242,12 @@ CGff3Reader::ReadSeqAnnot(
 void
 CGff3Reader::xProcessData(
     const TReaderData& readerData,
-    CSeq_annot& annot) 
+    CSeq_annot& annot)
 //  ----------------------------------------------------------------------------
 {
     for (const auto& lineData: readerData) {
         const auto& line = lineData.mData;
-        if (xParseStructuredComment(line)  &&  
+        if (xParseStructuredComment(line)  &&
                 !NStr::StartsWith(line, "##sequence-region") ) {
             continue;
         }
@@ -226,10 +262,10 @@ CGff3Reader::xProcessData(
 
 //  ----------------------------------------------------------------------------
 void CGff3Reader::xProcessAlignmentData(
-    CSeq_annot& annot) 
+    CSeq_annot& annot)
 //  ----------------------------------------------------------------------------
 {
-    for (const auto id : mAlignmentData.mIds) {
+    for (const string& id : mAlignmentData.mIds) {
         CRef<CSeq_align> pAlign = Ref(new CSeq_align());
         if (x_MergeAlignments(mAlignmentData.mAlignments.at(id), pAlign)) {
             // if available, add current browser information
@@ -261,11 +297,11 @@ CGff3Reader::xParseFeature(
     }
 
     //parse record:
-    shared_ptr<CGff2Record> pRecord(x_CreateRecord());
+    shared_ptr<CGff3ReadRecord> pRecord(x_CreateRecord());
     try {
         if (!pRecord->AssignFromGff(line)) {
-			return false;
-		}
+            return false;
+        }
     }
     catch(CObjReaderLineException& err) {
         ProcessError(err, pEC);
@@ -280,6 +316,16 @@ CGff3Reader::xParseFeature(
         return true;
     }
 
+    //no support for multiparented features in genbank mode:
+    if (this->IsInGenbankMode()  &&  pRecord->IsMultiParent()) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Fatal,
+            0,
+            "Multiparented features are not supported in Genbank mode"));
+        ProcessError(*pErr, pEC);
+    }
+
     //append feature to annot:
     if (!xUpdateAnnotFeature(*pRecord, annot, pEC)) {
         return false;
@@ -307,7 +353,7 @@ bool CGff3Reader::xParseAlignment(
     if ( !pRecord->AssignFromGff(strLine) ) {
         return false;
     }
-    
+
     string id;
     if ( !pRecord->GetAttribute("ID", id) ) {
         id = pRecord->Id();
@@ -329,36 +375,46 @@ bool CGff3Reader::xParseAlignment(
     return true;
 }
 
-
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotFeature(
-    const CGff2Record& record,
+    const CGff2Record& gffRecord,
     CSeq_annot& annot,
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
+    //if (gffRecord.Type() == "CDS") {
+    //    if (gffRecord.SeqStart() == 114392786) {
+    //        cerr << "";
+    //    }
+    //}
+
+    mpLocations->AddRecord(gffRecord);
+
     CRef< CSeq_feat > pFeature(new CSeq_feat);
 
-    string type = record.Type();
-    NStr::ToLower(type);
-    if (type == "exon" || type == "five_prime_utr" || type == "three_prime_utr") {
-        return xUpdateAnnotExon(record, pFeature, annot, pEC);
+    auto recType = gffRecord.NormalizedType();
+    if (recType == "exon" || recType == "five_prime_utr" || recType == "three_prime_utr") {
+        return xUpdateAnnotExon(gffRecord, pFeature, annot, pEC);
     }
-    if (type == "cds"  ||  type == "start_codon"  || type == "stop_codon") {
-        return xUpdateAnnotCds(record, pFeature, annot, pEC);
+    if (recType == "cds") {
+        return xUpdateAnnotCds(gffRecord, pFeature, annot, pEC);
     }
-    if (type == "gene") {
-        return xUpdateAnnotGene(record, pFeature, annot, pEC);
+    if (recType == "gene") {
+         return xUpdateAnnotGene(gffRecord, pFeature, annot, pEC);
     }
-    if (type == "mrna") {
-        return xUpdateAnnotMrna(record, pFeature, annot, pEC);
+    if (recType == "mrna") {
+        return xUpdateAnnotMrna(gffRecord, pFeature, annot, pEC);
     }
-    if (type == "region") {
-        return xUpdateAnnotRegion(record, pFeature, annot, pEC);
+    if (recType == "region") {
+        return xUpdateAnnotRegion(gffRecord, pFeature, annot, pEC);
     }
-    return xUpdateAnnotGeneric(record, pFeature, annot, pEC);
+    if (!xUpdateAnnotGeneric(gffRecord, pFeature, annot, pEC)) {
+        return false;
+    }
+    return true;
 }
 
+
 //  ----------------------------------------------------------------------------
 void CGff3Reader::xVerifyExonLocation(
     const string& mrnaId,
@@ -379,7 +435,7 @@ void CGff3Reader::xVerifyExonLocation(
     const CSeq_interval& containingInt = cit->second.GetObject();
     const CRef<CSeq_loc> pContainedLoc = exon.GetSeqLoc(m_iFlags, mSeqIdResolve);
     const CSeq_interval& containedInt = pContainedLoc->GetInt();
-    if (containedInt.GetFrom() < containingInt.GetFrom()  ||  
+    if (containedInt.GetFrom() < containingInt.GetFrom()  ||
             containedInt.GetTo() > containingInt.GetTo()) {
         string message = "Bad data line: ";
         message += exon.Type();
@@ -402,9 +458,9 @@ bool CGff3Reader::xUpdateAnnotExon(
 {
     list<string> parents;
     if (record.GetAttribute("Parent", parents)) {
-        for (list<string>::const_iterator it = parents.begin(); it != parents.end(); 
+        for (list<string>::const_iterator it = parents.begin(); it != parents.end();
                 ++it) {
-            const string& parentId = *it; 
+            const string& parentId = *it;
             CRef<CSeq_feat> pParent;
             if (!x_GetFeatureById(parentId, pParent)) {
                 xAddPendingExon(parentId, record);
@@ -417,12 +473,12 @@ bool CGff3Reader::xUpdateAnnotExon(
                 if  (!xInitializeFeature(record, pFeature)) {
                     return false;
                 }
-                return xAddFeatureToAnnot(pFeature, annot);            
+                return xAddFeatureToAnnot(pFeature, annot);
             }
             IdToFeatureMap::iterator fit = m_MapIdToFeature.find(parentId);
             if (fit != m_MapIdToFeature.end()) {
                 CRef<CSeq_feat> pParent = fit->second;
-                if (!pParent->GetData().IsGene()  &&  !record.UpdateFeature(m_iFlags, pParent)) {
+                if (!record.UpdateFeature(m_iFlags, pParent)) {
                     return false;
                 }
             }
@@ -432,6 +488,44 @@ bool CGff3Reader::xUpdateAnnotExon(
 }
 
 
+//  ----------------------------------------------------------------------------
+bool CGff3Reader::xJoinLocationIntoRna(
+    const CGff2Record& record,
+    //CRef<CSeq_feat> pFeature,
+    //CSeq_annot& annot,
+    ILineErrorListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    list<string> parents;
+    if (!record.GetAttribute("Parent", parents)) {
+        return true;
+    }
+    for (list<string>::const_iterator it = parents.begin(); it != parents.end();
+            ++it) {
+        const string& parentId = *it;
+        CRef<CSeq_feat> pParent;
+        if (!x_GetFeatureById(parentId, pParent)) {
+            // Danger:
+            // We don't know whether the CDS parent is indeed an RNA and it could
+            //  possible be a gene.
+            // If the parent is indeed a gene then gene construction will have to
+            //  purge this pending exon (or it will cause a sanity check to fail
+            //  during post processing).
+            xAddPendingExon(parentId, record);
+            continue;
+        }
+        if (!pParent->GetData().IsRna()) {
+            continue;
+        }
+        xVerifyExonLocation(parentId, record);
+        if (!record.UpdateFeature(m_iFlags, pParent)) {
+            return false;
+        }
+    }
+    return true;
+}
+
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::xUpdateAnnotCds(
     const CGff2Record& record,
@@ -440,102 +534,30 @@ bool CGff3Reader::xUpdateAnnotCds(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
+    if (!xJoinLocationIntoRna(record, pEC)) {
+        return false;
+    }
     xVerifyCdsParents(record);
 
-    list<string> parents;
-    record.GetAttribute("Parent", parents);
-    map<string, string> impliedCdsFeats;
+    string cdsId = xMakeRecordId(record);
+    mpLocations->AddRecordForId(cdsId, record);
 
-    // Preliminary:
-    //  We do not support multiparented CDS features in -genbank mode yet.
-    if (IsInGenbankMode()  &&  parents.size() > 1){
-        CReaderMessage error(
-            eDiag_Error,
-            m_uLineNumber,
-            "Unsupported: CDS record with multiple parents.");
-        throw error;
+    auto pExistingFeature = m_MapIdToFeature.find(cdsId);
+    if (pExistingFeature != m_MapIdToFeature.end()) {
+        return true;
     }
 
-    // Step 1:
-    // Locations of parent mRNAs are constructed on the fly, by joining in the 
-    //  locations of child exons and CDSs as we discover them. Do this
-    //  first.
-    // IDs for CDS features are derived from the IDs of their parent features.
-    //  Generate a list of CDS IDs this record pertains to as we cycle through
-    //  the parent list.
-    //
-    for (list<string>::const_iterator it = parents.begin(); it != parents.end(); 
-            ++it) {
-        string parentId = *it;
-        bool parentIsGene = false;
-        //update parent location:
-        IdToFeatureMap::iterator featIt = m_MapIdToFeature.find(parentId);
-        if (featIt != m_MapIdToFeature.end()) {
-            CRef<CSeq_feat> pParent = featIt->second;
-            parentIsGene = pParent->GetData().IsGene();
-            if (!parentIsGene  &&  !record.UpdateFeature(m_iFlags, pParent)) {
-                return false;
-            }
-            //rw-143:
-            // if parent type is miscRNA then change it to mRNA:
-            if (pParent->GetData().IsRna()  &&  
-                    pParent->GetData().GetRna().GetType()  ==  CRNA_ref::eType_other) {
-                pParent->SetData().SetRna().SetType(CRNA_ref::eType_mRNA);
-            }
-        }
+    m_MapIdToFeature[cdsId] = pFeature;
+    xInitializeFeature(record, pFeature);
+    xAddFeatureToAnnot(pFeature, annot);
 
-        //generate applicable CDS ID:
-        string siblingId("cds");
-        if (!record.GetAttribute("ID", siblingId)  ||  !parentIsGene) {
-            siblingId = string("cds:") + parentId;
-        }
-        impliedCdsFeats[siblingId] = parentId;
-    }
-    // deal with unparented cds
-    if (parents.empty()) {
-        string cdsId;
-        if (!record.GetAttribute("ID", cdsId)) {
-            if (IsInGenbankMode()) {
-                cdsId = xNextGenericId();
-            }
-            else {
-                cdsId = "cds";
-            }
-        }
-        impliedCdsFeats[cdsId] = "";
-    }
-
-    // Step 2:
-    // For every sibling CDS feature, look if there is already a feature with that
-    //  ID under construction.
-    // If there is, use record to update feature under construction.
-    // If there isn't, use record to initialize a brand new feature.
-    //
-    for (auto featIt = impliedCdsFeats.begin(); featIt != impliedCdsFeats.end(); ++featIt) {
-        auto cdsId = featIt->first;
-        auto parentId = featIt->second;
-        auto idIt = m_MapIdToFeature.find(cdsId);
-        if (idIt == m_MapIdToFeature.end()) {
-            auto altId =  string("cds:") + parentId;
-            idIt = m_MapIdToFeature.find(altId);
-        }
-        if (idIt != m_MapIdToFeature.end()) {
-            //found feature with ID in question: update
-            record.UpdateFeature(m_iFlags, idIt->second);
-        }
-        else {
-            //didn't find feature with that ID: create new one
-            pFeature.Reset(new CSeq_feat);
-            xInitializeFeature(record, pFeature);
-            if (!parentId.empty()) {
-                xFeatureSetQualifier("Parent", parentId, pFeature);
-                xFeatureSetXrefParent(parentId, pFeature);
-                if (m_iFlags & fGeneXrefs) {
-                    xFeatureSetXrefGrandParent(parentId, pFeature);
-                }
-            }
-            xAddFeatureToAnnot(pFeature, annot);
-            m_MapIdToFeature[cdsId] = pFeature;
+    string parentId;
+    record.GetAttribute("Parent", parentId);
+    if (!parentId.empty()) {
+        xFeatureSetQualifier("Parent", parentId, pFeature);
+        xFeatureSetXrefParent(parentId, pFeature);
+        if (m_iFlags & fGeneXrefs) {
+            xFeatureSetXrefGrandParent(parentId, pFeature);
         }
     }
     return true;
@@ -568,7 +590,7 @@ bool CGff3Reader::xFeatureSetXrefGrandParent(
         CRef<CFeat_id> pGrandParentId(new CFeat_id);
         pGrandParentId->Assign(pGrandParent->GetId());
         CRef<CSeqFeatXref> pGrandParentXref(new CSeqFeatXref);
-        pGrandParentXref->SetId(*pGrandParentId);  
+        pGrandParentXref->SetId(*pGrandParentId);
         pFeature->SetXref().push_back(pGrandParentXref);
 
         //xref grandparent->grandchild
@@ -597,7 +619,7 @@ bool CGff3Reader::xFeatureSetXrefParent(
     CRef<CFeat_id> pParentId(new CFeat_id);
     pParentId->Assign(pParent->GetId());
     CRef<CSeqFeatXref> pParentXref(new CSeqFeatXref);
-    pParentXref->SetId(*pParentId);  
+    pParentXref->SetId(*pParentId);
     pChild->SetXref().push_back(pParentXref);
 
     //xref parent->child
@@ -628,12 +650,6 @@ bool CGff3Reader::xFindFeatureUnderConstruction(
         eDiag_Fatal,
         m_uLineNumber,
         "Bad data line:  Duplicate feature ID \"" + id + "\".");
-    if (record.Id() != mIdToSeqIdMap[id]) {
-        throw fatal;
-    }
-    if (it->second->GetData().IsRna()) {
-        throw fatal;
-    }
     CSeq_feat tempFeat;
     if (CSoMap::SoTypeToFeature(record.Type(), tempFeat)) {
         if (it->second->GetData().GetSubtype() != tempFeat.GetData().GetSubtype()) {
@@ -677,8 +693,8 @@ bool CGff3Reader::xUpdateAnnotGeneric(
             throw error;
         }
 
-        CRef<CCode_break> pCodeBreak(new CCode_break); 
-        CSeq_interval& cbLoc = pCodeBreak->SetLoc().SetInt();        
+        CRef<CCode_break> pCodeBreak(new CCode_break);
+        CSeq_interval& cbLoc = pCodeBreak->SetLoc().SetInt();
         CRef< CSeq_id > pId = mSeqIdResolve(record.Id(), m_iFlags, true);
         cbLoc.SetId(*pId);
         cbLoc.SetFrom(static_cast<TSeqPos>(record.SeqStart()));
@@ -704,7 +720,7 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
-    if (pFeature->GetData().IsRna()  ||  
+    if (pFeature->GetData().IsRna()  ||
             pFeature->GetData().GetSubtype() == CSeqFeatData::eSubtype_misc_RNA) {
         CRef<CSeq_interval> rnaLoc(new CSeq_interval);
         rnaLoc->Assign(pFeature->GetLocation().GetInt());
@@ -746,7 +762,7 @@ bool CGff3Reader::xUpdateAnnotMrna(
         }
     }
 
-    string strId;   
+    string strId;
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
@@ -797,6 +813,13 @@ bool CGff3Reader::xUpdateAnnotGene(
     if ( record.GetAttribute("ID", strId)) {
         m_MapIdToFeature[strId] = pFeature;
     }
+    // address corner case:
+    // parent of CDS is a gene but the DCS is listed before the gene so at the
+    //  time we did not know the parent would be a gene.
+    // remedy: throw out any collected cds locations that were meant for RNA
+    //  construction.
+    list<CGff2Record> pendingExons;
+    xGetPendingExons(strId, pendingExons);
     return true;
 }
 
@@ -844,6 +867,13 @@ void CGff3Reader::xVerifyCdsParents(
         return;
     }
     record.GetAttribute("Parent", parents);
+    if (parents.empty()) {
+        CReaderMessage error(
+            eDiag_Error,
+            m_uLineNumber,
+            "Bad data line: CDS record is missing mandatory Parent attribute.");
+        throw error;
+    }
     map<string, string>::iterator it = mCdsParentMap.find(id);
     if (it == mCdsParentMap.end()) {
         mCdsParentMap[id] = parents;
@@ -880,9 +910,11 @@ bool CGff3Reader::xIsIgnoredFeatureType(
     string ftype(CSoMap::ResolveSoAlias(featureType));
 
     static const char* const ignoredTypesAlways_[] = {
-        "protein"
+        "protein",
+        "start_codon", // also part of a cds feature
+        "stop_codon", // in GFF3, also part of a cds feature
     };
-    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, ignoredTypesAlways, ignoredTypesAlways_);    
+    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, ignoredTypesAlways, ignoredTypesAlways_);
     STRINGARRAY::const_iterator cit = ignoredTypesAlways.find(ftype);
     if (cit != ignoredTypesAlways.end()) {
         return true;
@@ -915,7 +947,7 @@ bool CGff3Reader::xIsIgnoredFeatureType(
         "vault_RNA",
         "Y_RNA"
     };
-    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, specialTypesGenbank, specialTypesGenbank_);    
+    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, specialTypesGenbank, specialTypesGenbank_);
 
     static const char* const ignoredTypesGenbank_[] = {
         "apicoplast_chromosome",
@@ -949,7 +981,7 @@ bool CGff3Reader::xIsIgnoredFeatureType(
         "translated_nucleotide_match",
         "ultracontig",
     };
-    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, ignoredTypesGenbank, ignoredTypesGenbank_);    
+    DEFINE_STATIC_ARRAY_MAP(STRINGARRAY, ignoredTypesGenbank, ignoredTypesGenbank_);
 
     cit = specialTypesGenbank.find(ftype);
     if (cit != specialTypesGenbank.end()) {
@@ -1028,11 +1060,68 @@ void CGff3Reader::xPostProcessAnnot(
         CReaderMessage warning(
             eDiag_Warning,
             m_uLineNumber,
-            "Bad data line: Record references non-existant Parent=" + it.first);
+            "Bad data line: Record references non-existent Parent=" + it.first);
         m_pMessageHandler->Report(warning);
     }
+
+    //location fixup:
+    for (auto itLocation: mpLocations->LocationMap()) {
+        auto id = itLocation.first;
+        auto itFeature = m_MapIdToFeature.find(id);
+        if (itFeature == m_MapIdToFeature.end()) {
+            continue;
+        }
+        CRef<CSeq_loc> pNewLoc(new CSeq_loc);
+        CCdregion::EFrame frame;
+        mpLocations->MergeLocation(pNewLoc, frame, itLocation.second);
+        CRef<CSeq_feat> pFeature = itFeature->second;
+        pFeature->SetLocation(*pNewLoc);
+        if (pFeature->GetData().IsCdregion()) {
+            auto& cdrData = pFeature->SetData().SetCdregion();
+            cdrData.SetFrame(
+                frame == CCdregion::eFrame_not_set ? CCdregion::eFrame_one : frame);
+        }
+    }
+
     return CGff2Reader::xPostProcessAnnot(annot);
 }
 
+//  ----------------------------------------------------------------------------
+void CGff3Reader::xProcessSequenceRegionPragma(
+    const string& pragma)
+//  ----------------------------------------------------------------------------
+{
+    TSeqPos sequenceSize(0);
+    vector<string> tokens;
+    NStr::Split(pragma, " \t", tokens, NStr::fSplit_MergeDelimiters);
+    if (tokens.size() < 2) {
+        CReaderMessage warning(
+            ncbi::eDiag_Warning,
+            m_uLineNumber,
+            "Bad sequence-region pragma - ignored.");
+        throw warning;
+    }
+    if (tokens.size() >= 4) {
+        try {
+            sequenceSize = NStr::StringToNonNegativeInt(tokens[3]);
+        }
+        catch(exception&) {
+            CReaderMessage warning(
+                ncbi::eDiag_Warning,
+                m_uLineNumber,
+                "Bad sequence-region pragma - ignored.");
+            throw warning;
+        }
+    }
+    mpLocations->SetSequenceSize(tokens[1], sequenceSize);
+}
+
+//  ----------------------------------------------------------------------------
+TSeqPos CGff3Reader::SequenceSize() const
+//  ----------------------------------------------------------------------------
+{
+    return mpLocations->SequenceSize();
+}
+
 END_objects_SCOPE
 END_NCBI_SCOPE
