@@ -1,4 +1,4 @@
-/*  $Id: gtf_reader.cpp 610936 2020-06-25 16:26:53Z ivanov $
+/*  $Id: gtf_reader.cpp 632531 2021-06-02 17:25:37Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,6 +58,7 @@
 #include <objects/seqfeat/Feat_id.hpp>
 
 #include <objtools/readers/gtf_reader.hpp>
+#include "gtf_location_merger.hpp"
 #include "reader_message_handler.hpp"
 
 #include <algorithm>
@@ -65,26 +66,6 @@
 BEGIN_NCBI_SCOPE
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
-//  ----------------------------------------------------------------------------
-bool s_AnnotId(
-    const CSeq_annot& annot,
-    string& strId )
-//  ----------------------------------------------------------------------------
-{
-    if ( ! annot.CanGetId() || annot.GetId().size() != 1 ) {
-        // internal error
-        return false;
-    }
-    
-    CRef< CAnnot_id > pId = *( annot.GetId().begin() );
-    if ( ! pId->IsLocal() ) {
-        // internal error
-        return false;
-    }
-    strId = pId->GetLocal().GetStr();
-    return true;
-}
-
 //  ----------------------------------------------------------------------------
 bool CGtfReadRecord::xAssignAttributesFromGff(
     const string& strGtfType,
@@ -94,7 +75,7 @@ bool CGtfReadRecord::xAssignAttributesFromGff(
     vector< string > attributes;
     xSplitGffAttributes(strRawAttributes, attributes);
 
-	for ( size_t u=0; u < attributes.size(); ++u ) {
+    for ( size_t u=0; u < attributes.size(); ++u ) {
         string key, value;
         string attribute(attributes[u]);
         if (!NStr::SplitInTwo(attribute, "=", key, value)) {
@@ -119,9 +100,9 @@ bool CGtfReadRecord::xAssignAttributesFromGff(
         }
         key = xNormalizedAttributeKey(key);
         value = xNormalizedAttributeValue(value);
-		if ( key.empty()  &&  value.empty() ) {
+        if ( key.empty()  &&  value.empty() ) {
             // Probably due to trailing "; ". Sequence Ontology generates such
-            // things. 
+            // things.
             continue;
         }
         if (NStr::StartsWith(value, "\"")) {
@@ -136,7 +117,7 @@ bool CGtfReadRecord::xAssignAttributesFromGff(
 }
 
 //  ----------------------------------------------------------------------------
-CGtfReader::CGtfReader( 
+CGtfReader::CGtfReader(
     unsigned int uFlags,
     const string& strAnnotName,
     const string& strAnnotTitle,
@@ -145,9 +126,11 @@ CGtfReader::CGtfReader(
 //  ----------------------------------------------------------------------------
     CGff2Reader( uFlags, strAnnotName, strAnnotTitle, resolver, pRL)
 {
+    mpLocations.reset(new CGtfLocationMerger(uFlags, resolver));
 }
+
 //  ----------------------------------------------------------------------------
-CGtfReader::CGtfReader( 
+CGtfReader::CGtfReader(
     unsigned int uFlags,
     CReaderListener* pRL):
 //  ----------------------------------------------------------------------------
@@ -162,12 +145,12 @@ CGtfReader::~CGtfReader()
 {
 }
 
-//  ----------------------------------------------------------------------------                
+//  ----------------------------------------------------------------------------
 CRef<CSeq_annot>
 CGtfReader::ReadSeqAnnot(
     ILineReader& lineReader,
-    ILineErrorListener* pEC ) 
-//  ----------------------------------------------------------------------------                
+    ILineErrorListener* pEC )
+//  ----------------------------------------------------------------------------
 {
     mCurrentFeatureCount = 0;
     return CReaderBase::ReadSeqAnnot(lineReader, pEC);
@@ -177,7 +160,7 @@ CGtfReader::ReadSeqAnnot(
 void
 CGtfReader::xProcessData(
     const TReaderData& readerData,
-    CSeq_annot& annot) 
+    CSeq_annot& annot)
 //  ----------------------------------------------------------------------------
 {
     for (const auto& lineData: readerData) {
@@ -205,17 +188,17 @@ bool CGtfReader::xUpdateAnnotFeature(
 //  ----------------------------------------------------------------------------
 {
     const CGtfReadRecord& gff = dynamic_cast<const CGtfReadRecord&>(record);
-    string strType = gff.Type();
+    auto recType = gff.NormalizedType();
 
     using TYPEHANDLER = bool (CGtfReader::*)(const CGtfReadRecord&, CSeq_annot&);
     using HANDLERMAP = map<string, TYPEHANDLER>;
 
     HANDLERMAP typeHandlers = {
-        {"CDS",         &CGtfReader::xUpdateAnnotCds},
+        {"cds",         &CGtfReader::xUpdateAnnotCds},
         {"start_codon", &CGtfReader::xUpdateAnnotCds},
         {"stop_codon",  &CGtfReader::xUpdateAnnotCds},
-        {"5UTR",        &CGtfReader::xUpdateAnnotTranscript},
-        {"3UTR",        &CGtfReader::xUpdateAnnotTranscript},
+        {"5utr",        &CGtfReader::xUpdateAnnotTranscript},
+        {"3utr",        &CGtfReader::xUpdateAnnotTranscript},
         {"exon",        &CGtfReader::xUpdateAnnotTranscript},
         {"initial",     &CGtfReader::xUpdateAnnotTranscript},
         {"internal",    &CGtfReader::xUpdateAnnotTranscript},
@@ -226,21 +209,21 @@ bool CGtfReader::xUpdateAnnotFeature(
     //
     // Handle officially recognized GTF types:
     //
-    HANDLERMAP::iterator it = typeHandlers.find(strType);
+    HANDLERMAP::iterator it = typeHandlers.find(recType);
     if (it != typeHandlers.end()) {
         TYPEHANDLER handler = it->second;
         return (this->*handler)(gff, annot);
     }
- 
+
     //
     //  Every other type is not officially sanctioned GTF, and per spec we are
     //  supposed to ignore it. In the spirit of being lenient on input we may
     //  try to salvage some of it anyway.
     //
-    if ( strType == "gene" ) {
+    if (recType == "gene") {
         return xCreateParentGene(gff, annot);
     }
-    if (strType == "mRNA") {
+    if (recType == "mrna"  ||  recType == "transcript") {
         return xCreateParentMrna(gff, annot);
     }
     return true;
@@ -252,53 +235,10 @@ bool CGtfReader::xUpdateAnnotCds(
     CSeq_annot& annot )
 //  ----------------------------------------------------------------------------
 {
-    //
-    // From the spec, the stop codon has _not_ been accounted for in any of the 
-    //  coding regions. Hence, we will treat (pieces of) the stop codon just
-    //  like (pieces of) CDS.
-    //
-    //
-    // If there is no gene feature to go with this CDS then make one. Otherwise,
-    //  make sure the existing gene feature includes the location of the CDS.
-    //
-    CRef<CSeq_feat> pGene = xFindParentGene(gff);
-    if (!pGene) {
-        if (!xCreateParentGene(gff, annot)) {
-            return false;
-        }
-        pGene = xFindParentGene(gff);
-    }
-    else {
-        if (!xMergeParentGene(gff, *pGene)) {
-            return false;
-        }
-    }
-
-    //
-    // If there is no CDS feature with this gene_id|transcript_id then make one.
-    //  Otherwise, fix up the location of the existing one.
-    //
-    CRef<CSeq_feat> pCds = xFindParentCds(gff);
-    if (!pCds) {
-        //
-        // Create a brand new CDS feature:
-        //
-        if (!xCreateParentCds(gff, annot)) {
-            return false;
-        }
-        pCds = xFindParentCds(gff);
-    }
-    else {
-        //
-        // Update an already existing CDS features:
-        //
-        if (!xMergeFeatureLocationMultiInterval(gff, *pCds)) {
-            return false;
-        }
-    }
-
-    return true;
-}
+    auto featId = mpLocations->GetFeatureIdFor(gff, "cds");
+    mpLocations->AddRecordForId(featId, gff) ;
+    return (xFindFeatById(featId)  ||  xCreateParentCds(gff, annot));
+ }
 
 //  ----------------------------------------------------------------------------
 bool CGtfReader::xUpdateAnnotTranscript(
@@ -310,16 +250,16 @@ bool CGtfReader::xUpdateAnnotTranscript(
     // If there is no gene feature to go with this CDS then make one. Otherwise,
     //  make sure the existing gene feature includes the location of the CDS.
     //
-    CRef< CSeq_feat > pGene = xFindParentGene(gff);
+    auto geneFeatId = mpLocations->GetFeatureIdFor(gff, "gene");
+    CRef< CSeq_feat > pGene = xFindFeatById(geneFeatId);
     if (!pGene) {
         if (!xCreateParentGene(gff, annot)) {
             return false;
         }
+        mpLocations->AddRecordForId(geneFeatId, gff);
     }
     else {
-        if (!xMergeParentGene(gff, *pGene)) {
-            return false;
-        }
+        mpLocations->AddRecordForId(geneFeatId, gff);
         if (!xFeatureTrimQualifiers(gff, *pGene)) {
             return false;
         }
@@ -329,7 +269,8 @@ bool CGtfReader::xUpdateAnnotTranscript(
     // If there is no mRNA feature with this gene_id|transcript_id then make one.
     //  Otherwise, fix up the location of the existing one.
     //
-    CRef<CSeq_feat> pMrna = xFindParentMrna(gff);
+    auto transcriptFeatId = mpLocations->GetFeatureIdFor(gff, "transcript");
+    CRef<CSeq_feat> pMrna = xFindFeatById(transcriptFeatId);
     if (!pMrna) {
         //
         // Create a brand new CDS feature:
@@ -337,14 +278,13 @@ bool CGtfReader::xUpdateAnnotTranscript(
         if (!xCreateParentMrna(gff, annot)) {
             return false;
         }
+        mpLocations->AddRecordForId(transcriptFeatId, gff);
     }
     else {
         //
         // Update an already existing CDS features:
         //
-        if (!xMergeFeatureLocationMultiInterval(gff, *pMrna)) {
-            return false;
-        }
+        mpLocations->AddRecordForId(transcriptFeatId, gff);
         if (!xFeatureTrimQualifiers(gff, *pMrna)) {
             return false;
         }
@@ -371,178 +311,34 @@ bool CGtfReader::xCreateFeatureId(
     return true;
 }
 
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xCreateFeatureLocation(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_id> pId = mSeqIdResolve(
-        record.Id(), m_iFlags & fAllIdsAsLocal, true);
-
-    CSeq_interval& location = feature.SetLocation().SetInt();
-    location.SetId(*pId);
-    location.SetFrom(record.SeqStart());
-    if (record.Type() != "mRNA") {
-        location.SetTo(record.SeqStop());
-    }
-    else {
-        // placeholder
-        //  actual location will be computed from the exons and CDSs living on 
-        //  this feature.
-        location.SetTo(record.SeqStart());
-    }
-    if (record.IsSetStrand()) {
-        location.SetStrand(record.Strand());
-    }
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xCreateGeneXrefs(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_feat> pParent = xFindParentGene(record);
-    if (!pParent) {
-        return true;
-    }
-    
-    CRef<CSeqFeatXref> pXrefToParent(new CSeqFeatXref);
-    pXrefToParent->SetId(pParent->SetId());    
-    feature.SetXref().push_back(pXrefToParent);
-
-    if (m_iFlags & CGtfReader::fGenerateChildXrefs) {
-        CRef<CSeqFeatXref> pXrefToChild(new CSeqFeatXref);
-        pXrefToChild->SetId(feature.SetId());
-        pParent->SetXref().push_back(pXrefToChild);
-    }
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xCreateMrnaXrefs(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_feat> pParent = xFindParentMrna(record);
-    if (!pParent) {
-        return true;
-    }
-    
-    CRef<CSeqFeatXref> pXrefToChild(new CSeqFeatXref);
-    pXrefToChild->SetId(feature.SetId());
-    pParent->SetXref().push_back(pXrefToChild);
-
-    CRef< CSeqFeatXref > pXrefToParent(new CSeqFeatXref);
-    pXrefToParent->SetId(pParent->SetId());    
-    feature.SetXref().push_back(pXrefToParent);
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xCreateCdsXrefs(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_feat> pParent = xFindParentCds(record);
-    if (!pParent) {
-        return true;
-    }
-    
-    CRef< CSeqFeatXref > pXrefToParent(new CSeqFeatXref);
-    pXrefToParent->SetId(pParent->SetId());    
-    feature.SetXref().push_back(pXrefToParent);
-
-    if (m_iFlags & CGtfReader::fGenerateChildXrefs) {
-        CRef<CSeqFeatXref> pXrefToChild(new CSeqFeatXref);
-        pXrefToChild->SetId(feature.SetId());
-        pParent->SetXref().push_back(pXrefToChild);
-    }
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xMergeFeatureLocationSingleInterval(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    const CSeq_interval& gene_int = feature.GetLocation().GetInt();
-    if (gene_int.GetFrom() > record.SeqStart() -1) {
-        feature.SetLocation().SetInt().SetFrom( record.SeqStart() );
-    }
-    if (gene_int.GetTo() < record.SeqStop() - 1) {
-        feature.SetLocation().SetInt().SetTo( record.SeqStop() );
-    }
-    if (record.Type() == "CDS"  &&  feature.GetData().IsCdregion()) {
-        return xFeatureTrimQualifiers(record, feature);
-    }
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xMergeFeatureLocationMultiInterval(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_id> pId = mSeqIdResolve(
-        record.Id(), m_iFlags & fAllIdsAsLocal, true);
-
-    CRef<CSeq_loc> pLocation(new CSeq_loc);
-    pLocation->SetInt().SetId(*pId);
-    pLocation->SetInt().SetFrom(record.SeqStart());
-    pLocation->SetInt().SetTo(record.SeqStop());
-    if (record.IsSetStrand()) {
-        pLocation->SetInt().SetStrand(record.Strand());
-    }
-    pLocation = pLocation->Add( 
-        feature.SetLocation(), CSeq_loc::fSortAndMerge_All, 0);
-    feature.SetLocation(*pLocation);
-    return true;
-}
-
 //  -----------------------------------------------------------------------------
 bool CGtfReader::xCreateParentGene(
     const CGtfReadRecord& gff,
     CSeq_annot& annot )
 //  -----------------------------------------------------------------------------
 {
-    //
-    // Create a single gene feature:
-    //
+    auto featId = mpLocations->GetFeatureIdFor(gff, "gene");
+    if (m_MapIdToFeature.find(featId) != m_MapIdToFeature.end()) {
+        return true;
+    }
+
     CRef<CSeq_feat> pFeature( new CSeq_feat );
 
     if (!xFeatureSetDataGene(gff, *pFeature)) {
         return false;
     }
-    if (!xCreateFeatureLocation(gff, *pFeature)) {
-        return false;
-    }
     if (!xCreateFeatureId(gff, "gene", *pFeature)) {
         return false;
     }
     if ( !xFeatureSetQualifiersGene(gff, *pFeature)) {
         return false;
     }
-    m_GeneMap[gff.GeneKey()] = pFeature;
-    xAddFeatureToAnnot( pFeature, annot );
-    return true;
-}
-    
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xMergeParentGene(
-    const CGtfReadRecord& record,
-    CSeq_feat& feature )
-//  ----------------------------------------------------------------------------
-{
-    if (!xMergeFeatureLocationSingleInterval(record, feature)) {
-        return false;
-    }
+
+    (gff.Type() == "gene") ?
+        mpLocations->AddRecordForId(featId, gff) :
+        mpLocations->AddStubForId(featId);
+    m_MapIdToFeature[featId] = pFeature;
+    xAddFeatureToAnnot(pFeature, annot);
     return true;
 }
 
@@ -573,7 +369,7 @@ bool CGtfReader::xFeatureSetQualifiersGene(
 
         // turn everything else into a qualifier
         xFeatureAddQualifiers(it->first, it->second, feature);
-    } 
+    }
     return true;
 }
 
@@ -601,7 +397,7 @@ bool CGtfReader::xFeatureSetQualifiersRna(
 
         // turn everything else into a qualifier
         xFeatureAddQualifiers(it->first, it->second, feature);
-    } 
+    }
     return true;
 }
 
@@ -629,7 +425,7 @@ bool CGtfReader::xFeatureSetQualifiersCds(
 
         // turn everything else into a qualifier
         xFeatureAddQualifiers(it->first, it->second, feature);
-    } 
+    }
     return true;
 }
 
@@ -639,42 +435,23 @@ bool CGtfReader::xCreateParentCds(
     CSeq_annot& annot )
 //  -----------------------------------------------------------------------------
 {
-    //
-    // Create a single cds feature.
-	// This creation may either be triggered by an actual CDS feature found in the
-	//	gtf, or by a feature that would imply a CDS feature (such as a start codon 
-	//	or a stop codon). The latter is necessary because nothing the the gtf 
-	//	standard stipulates that gtf features have to be arranged in any particular
-	//	order.
-    //
+    auto featId = mpLocations->GetFeatureIdFor(gff, "cds");
+    if (m_MapIdToFeature.find(featId) != m_MapIdToFeature.end()) {
+        return true;
+    }
+
     CRef<CSeq_feat> pFeature(new CSeq_feat);
 
-    string strType = gff.Type();
-    if (strType != "CDS"  &&  strType != "start_codon"  &&  strType != "stop_codon") {
-        return false;
-    }
-
-    m_CdsMap[gff.FeatureKey()] = pFeature;
-
     if (!xFeatureSetDataCds(gff, *pFeature)) {
         return false;
     }
-    if (!xCreateFeatureLocation(gff, *pFeature)) {
-        return false;
-    }
     if (!xCreateFeatureId(gff, "cds", *pFeature)) {
         return false;
     }
-    if (!xCreateGeneXrefs(gff, *pFeature)) {
-        return false;
-    }
-    if (!xCreateMrnaXrefs(gff, *pFeature)) {
-        return false;
-    }
     if (!xFeatureSetQualifiersCds(gff, *pFeature)) {
         return false;
     }
-
+    m_MapIdToFeature[featId] = pFeature;
     return xAddFeatureToAnnot(pFeature, annot);
 }
 
@@ -684,69 +461,39 @@ bool CGtfReader::xCreateParentMrna(
     CSeq_annot& annot )
 //  -----------------------------------------------------------------------------
 {
-    //
-    // Create a single cds feature:
-    //
+    auto featId = mpLocations->GetFeatureIdFor(gff, "transcript");
+    if (m_MapIdToFeature.find(featId) != m_MapIdToFeature.end()) {
+        return true;
+    }
+
     CRef< CSeq_feat > pFeature( new CSeq_feat );
 
     if (!xFeatureSetDataMrna(gff, *pFeature)) {
         return false;
     }
-    if (!xCreateFeatureLocation(gff, *pFeature)) {
-        return false;
-    }
     if (!xCreateFeatureId(gff, "mrna", *pFeature)) {
         return false;
     }
-    if (!xCreateGeneXrefs(gff, *pFeature) ) {
-        return false;
-    }
-    if (!xCreateCdsXrefs(gff, *pFeature)) {
-        return false;
-    }
     if ( ! xFeatureSetQualifiersRna( gff, *pFeature ) ) {
         return false;
     }
 
-    m_MrnaMap[gff.FeatureKey()] = pFeature;
+    mpLocations->AddStubForId(featId);
+    m_MapIdToFeature[featId] = pFeature;
 
     return xAddFeatureToAnnot( pFeature, annot );
 }
 
 //  ----------------------------------------------------------------------------
-CRef<CSeq_feat> CGtfReader::xFindParentGene(
-    const CGtfReadRecord& gff)
+CRef<CSeq_feat> CGtfReader::xFindFeatById(
+    const string& featId)
 //  ----------------------------------------------------------------------------
 {
-    auto gene_it = m_GeneMap.find(gff.GeneKey());
-    if (gene_it == m_GeneMap.end()) {
+    auto featIt = m_MapIdToFeature.find(featId);
+    if (featIt == m_MapIdToFeature.end()) {
         return CRef<CSeq_feat>();
     }
-    return gene_it->second;
-}
-
-//  ----------------------------------------------------------------------------
-CRef<CSeq_feat> CGtfReader::xFindParentCds(
-    const CGtfReadRecord& gff)
-//  ----------------------------------------------------------------------------
-{
-    auto cds_it = m_CdsMap.find(gff.FeatureKey());
-    if (cds_it == m_CdsMap.end()) {
-        return CRef<CSeq_feat>();
-    }
-    return cds_it->second;
-}
-
-//  ----------------------------------------------------------------------------
-CRef<CSeq_feat> CGtfReader::xFindParentMrna(
-    const CGtfReadRecord& gff)
-//  ----------------------------------------------------------------------------
-{
-    TIdToFeature::iterator rna_it = m_MrnaMap.find(gff.FeatureKey());
-    if ( rna_it == m_MrnaMap.end() ) {
-        return CRef<CSeq_feat>();
-    }
-    return rna_it->second;
+    return featIt->second;
 }
 
 //  ----------------------------------------------------------------------------
@@ -777,7 +524,7 @@ bool CGtfReader::xFeatureSetDataMrna(
 {
     if (!xFeatureSetDataRna(record, feature, CSeqFeatData::eSubtype_mRNA)) {
         return false;
-    }    
+    }
     CRNA_ref& rna = feature.SetData().SetRna();
 
     string product = record.GtfAttributes().ValueOf("product");
@@ -847,7 +594,7 @@ bool CGtfReader::xFeatureTrimQualifiers(
 {
     typedef CSeq_feat::TQual TQual;
     //task:
-    // for each attribute of the new piece check if we already got a feature 
+    // for each attribute of the new piece check if we already got a feature
     //  qualifier
     // if so, and with the same value, then the qualifier is allowed to live
     // otherwise it is subfeature specific and hence removed from the feature
@@ -885,21 +632,6 @@ bool CGtfReader::xFeatureTrimQualifiers(
     return true;
 }
 
-//  ----------------------------------------------------------------------------
-bool CGtfReader::xCdsIsPartial(
-    const CGtfReadRecord& record )
-//  ----------------------------------------------------------------------------
-{
-    if (record.GtfAttributes().HasValue("partial")) {
-        return true;
-    }
-    CRef<CSeq_feat> pMrna = xFindParentMrna(record);
-    if (!pMrna) {
-        return false;
-    }
-    return (pMrna->IsSetPartial()  &&  pMrna->GetPartial());
-}
-
 //  ----------------------------------------------------------------------------
 bool CGtfReader::xProcessQualifierSpecialCase(
     const string& key,
@@ -919,8 +651,8 @@ bool CGtfReader::xProcessQualifierSpecialCase(
         feature.SetComment(NStr::Join(values, ";"));
         return true;
     }
-    if ( 0 == NStr::CompareNocase(key, "dbxref") || 
-        0 == NStr::CompareNocase(key, "db_xref")) 
+    if ( 0 == NStr::CompareNocase(key, "dbxref") ||
+        0 == NStr::CompareNocase(key, "db_xref"))
     {
         for (auto value: values) {
             vector< string > tags;
@@ -943,7 +675,7 @@ bool CGtfReader::xProcessQualifierSpecialCase(
         }
     }
     return false;
-}  
+}
 
 //  ----------------------------------------------------------------------------
 void CGtfReader::xFeatureAddQualifiers(
@@ -957,5 +689,76 @@ void CGtfReader::xFeatureAddQualifiers(
     }
 };
 
+//  ============================================================================
+void CGtfReader::xSetAncestorXrefs(
+    CSeq_feat& descendent,
+    CSeq_feat& ancestor)
+//  ============================================================================
+{
+    xSetXrefFromTo(descendent, ancestor);
+    if (m_iFlags & CGtfReader::fGenerateChildXrefs) {
+        xSetXrefFromTo(ancestor, descendent);
+    }
+}
+
+//  ----------------------------------------------------------------------------
+void CGtfReader::xPostProcessAnnot(
+    CSeq_annot& annot)
+//  ----------------------------------------------------------------------------
+{
+    //location fixup:
+    for (auto itLocation: mpLocations->LocationMap()) {
+        auto id = itLocation.first;
+        auto itFeature = m_MapIdToFeature.find(id);
+        if (itFeature == m_MapIdToFeature.end()) {
+            continue;
+        }
+        CRef<CSeq_feat> pFeature = itFeature->second;
+        auto featSubType = pFeature->GetData().GetSubtype();
+        CRef<CSeq_loc> pNewLoc = mpLocations->MergeLocation(
+            featSubType, itLocation.second);
+        pFeature->SetLocation(*pNewLoc);
+    }
+
+    //generate xrefs:
+    for (auto itLocation: mpLocations->LocationMap()) {
+        auto id = itLocation.first;
+        auto itFeature = m_MapIdToFeature.find(id);
+        if (itFeature == m_MapIdToFeature.end()) {
+            continue;
+        }
+        CRef<CSeq_feat> pFeature = itFeature->second;
+        auto featSubType = pFeature->GetData().GetSubtype();
+        switch(featSubType) {
+            default: {
+                break;
+            }
+            case CSeqFeatData::eSubtype_mRNA: {
+                auto parentGeneFeatId = string("gene:") + pFeature->GetNamedQual("gene_id");
+                CRef<CSeq_feat> pParentGene;
+                if (x_GetFeatureById(parentGeneFeatId, pParentGene)) {
+                    xSetAncestorXrefs(*pFeature, *pParentGene);
+                }
+                break;
+            }
+            case CSeqFeatData::eSubtype_cdregion: {
+                auto parentRnaFeatId = string("transcript:") + pFeature->GetNamedQual("gene_id") +
+                    "_" + pFeature->GetNamedQual("transcript_id");
+                CRef<CSeq_feat> pParentRna;
+                if (x_GetFeatureById(parentRnaFeatId, pParentRna)) {
+                    xSetAncestorXrefs(*pFeature, *pParentRna);
+                }
+                auto parentGeneFeatId = string("gene:") + pFeature->GetNamedQual("gene_id");
+                CRef<CSeq_feat> pParentGene;
+                if (x_GetFeatureById(parentGeneFeatId, pParentGene)) {
+                    xSetAncestorXrefs(*pFeature, *pParentGene);
+                }
+                break;
+            }
+        }
+    }
+    return CGff2Reader::xPostProcessAnnot(annot);
+}
+
 END_objects_SCOPE
 END_NCBI_SCOPE
