@@ -1,4 +1,4 @@
-/*  $Id: rm_reader.cpp 610834 2020-06-24 15:29:06Z ivanov $
+/*  $Id: rm_reader.cpp 625986 2021-02-22 21:26:47Z mozese2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -358,7 +358,7 @@ void CRepeatLibrary::Read(CNcbiIstream& stream)
                     if (i_specificity.second) {
                         i_specificity.first->second = m_Taxonomy->GetTaxId(repeat.m_RptSpecificityName);
                         if (! i_specificity.first->second) {
-                            LOG_POST(Warning
+                            ERR_POST(Warning
                                      << "RepeatMasker library species failed lookup to taxonomy: "
                                      << repeat.m_RptSpecificityName);
                         }
@@ -839,7 +839,7 @@ CRepeatMaskerReader::ReadSeqAnnot(ILineReader& lr, ILineErrorListener* pMessageL
 
 bool CRepeatMaskerReader::IsHeaderLine(const string& line)
 {
-    string labels_1st_line[] = { "SW", "perc", "query", "position", "matching", "" };
+    string labels_1st_line[] = { "perc", "query", "position", "matching", "" };
     string labels_2nd_line[] = { "score", "div.", "del.", "ins.", "sequence", "" };
 
     // try to identify 1st line of column labels:
