@@ -1,4 +1,4 @@
-/*  $Id: enumtype.cpp 563890 2018-05-16 15:50:20Z ivanov $
+/*  $Id: enumtype.cpp 624723 2021-02-03 18:51:43Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -38,9 +38,12 @@
 #include "srcutil.hpp"
 #include <serial/impl/enumerated.hpp>
 #include "stdstr.hpp"
+#include <common/ncbi_sanitizers.h>
+
 
 BEGIN_NCBI_SCOPE
 
+
 CEnumDataType::CEnumDataType(void)
 {
 }
@@ -443,6 +446,7 @@ string CEnumDataType::GetXmlValueName(const string& value) const
 
 CTypeInfo* CEnumDataType::CreateTypeInfo(void)
 {
+    NCBI_LSAN_DISABLE_GUARD;
     AutoPtr<CEnumeratedTypeValues>
         info(new CEnumeratedTypeValues(GlobalName(), IsInteger()));
     ITERATE ( TValues, i, m_Values ) {
