@@ -1,4 +1,4 @@
-/*  $Id: statictype.cpp 582016 2019-03-08 15:00:32Z gouriano $
+/*  $Id: statictype.cpp 624723 2021-02-03 18:51:43Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -42,9 +42,12 @@
 #include <serial/impl/autoptrinfo.hpp>
 #include <typeinfo>
 #include <vector>
+#include <common/ncbi_sanitizers.h>
+
 
 BEGIN_NCBI_SCOPE
 
+
 TObjectPtr CStaticDataType::CreateDefault(const CDataValue& ) const
 {
     NCBI_THROW(CDatatoolException, eNotImplemented,
@@ -306,7 +309,7 @@ AutoPtr<CTypeStrings> CStaticDataType::GetFullCType(void) const
         a->SetStorageType(storage);
     }
     if (!type.empty()) {
-        if (NStr::EndsWith(type, "CStrictId64")) {
+        if (NStr::EndsWith(type, "CStrictId")) {
             (dynamic_cast<CStdTypeStrings*>(a.get()))->SetBigInt();
         }
     }
@@ -851,6 +854,7 @@ TTypeInfo COctetStringDataType::GetRealTypeInfo(void)
     if (x_AsBitString()) {
         return CBitStringDataType::GetRealTypeInfo();
     }
+    NCBI_LSAN_DISABLE_GUARD;
     if ( HaveModuleName() )
         return UpdateModuleName(CStdTypeInfo<vector<char> >::CreateTypeInfo());
     return CStdTypeInfo< vector<char> >::GetTypeInfo();
