@@ -1,4 +1,4 @@
-/*  $Id: type.cpp 552037 2017-11-28 17:00:49Z gouriano $
+/*  $Id: type.cpp 624723 2021-02-03 18:51:43Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -49,6 +49,7 @@
 #include "datatool.hpp"
 #include <serial/error_codes.hpp>
 #include <algorithm>
+#include <common/ncbi_sanitizers.h>
 
 
 #define NCBI_USE_ERRCODE_X   Serial_DTType
@@ -79,6 +80,7 @@ private:
 
 TTypeInfo CAnyTypeSource::GetTypeInfo(void)
 {
+    NCBI_LSAN_DISABLE_GUARD;
     return m_Type->GetAnyTypeInfo();
 }
 
