@@ -1,4 +1,4 @@
-/*  $Id: objistr.cpp 604627 2020-03-31 13:43:05Z ivanov $
+/*  $Id: objistr.cpp 607093 2020-04-30 12:29:36Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1902,7 +1902,7 @@ TObjectPtr CObjectIStream::GetParentObjectPtr(TTypeInfo type,
 #ifdef NCBI_STRICT_GI
 void CObjectIStream::ReadGi(TGi& obj)
 {
-    obj = ReadInt8();
+    obj.Set() = ReadInt8();
 }
 
 
