@@ -1,4 +1,4 @@
-/*  $Id: objistrjson.cpp 568143 2018-07-31 17:03:56Z gouriano $
+/*  $Id: objistrjson.cpp 624386 2021-01-28 19:46:31Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -462,7 +462,7 @@ string CObjectIStreamJson::ReadFileHeader(void)
             }
         }
     }
-    if (StackIsEmpty() || TopFrame().GetTypeInfo()->GetDataSpec() == EDataSpec::eJSON) {
+    if (!StackIsEmpty() && TopFrame().GetTypeInfo()->GetDataSpec() == EDataSpec::eJSON) {
         return kEmptyStr;
     }
     m_FileHeader = true;
