@@ -1,4 +1,4 @@
-/*  $Id: objostrxml.cpp 580114 2019-02-11 15:08:11Z ucko $
+/*  $Id: objostrxml.cpp 608769 2020-05-20 19:18:29Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -895,7 +895,7 @@ void CObjectOStreamXml::WriteBitString(const CBitString& obj)
         CBitString::statistics st;
         obj.calc_stat(&st);
         char* buf = (char*)malloc(st.max_serialize_mem);
-        unsigned int len = bm::serialize(obj, (unsigned char*)buf, tmp_block);
+        size_t len = bm::serialize(obj, (unsigned char*)buf, tmp_block);
         WriteBytes(buf,len);
         free(buf);
         bm::aligned_free(tmp_block);
